/******************************************************************************
 * Copyright Codito Technologies (www.codito.com)
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/board/aa3/arc_emac.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */



#include <cmd_confdefs.h>	/* Command configs	*/
#include <asm/u-boot.h>		/* Board data structure	*/
#include <asm/arc_emac.h>	/* MDIO macros and ARC_EMAC register macros	*/
#include <config.h>		/* ARC700 clock freq.	*/
#include <asm/errno.h>
#include <malloc.h>
#include <common.h>
#include <net.h>
#include <linux/types.h>
#if (CONFIG_COMMANDS & CFG_CMD_NET)

/* rx buffer descriptor. We align descriptors to 32 so info and 
 * data lie on the same cache line. This also satisfies the 8 byte 
 * alignment required by the VMAC */
struct aa3_buffer_desc rxbd[RX_BDT_LEN] __attribute__((aligned(32)));

/* tx buffer descriptor */
struct aa3_buffer_desc txbd[RX_BDT_LEN] __attribute__((aligned(32)));

/* transmit buffer counter */
static int txbd_cntr;

char default_addr[10] = { 0x00 ,0x01 ,0x02 ,0x03 ,0x04 ,0x05  };
int eth_rx (void)
{
	unsigned int i,*recv_data,len;
	volatile unsigned int *stat_reg,*reg_base_addr = VMAC_REG_BASEADDR;
	stat_reg = reg_base_addr + STAT_OFFSET;
	while( !( *(stat_reg) & RXINT_MASK));
	*stat_reg = RXINT_MASK;
	
	printf("%s: start\n", __func__);

	for (i=0 ; i < RX_BDT_LEN;i++) {
		if( (rxbd[i].info & OWN_MASK) == 0) {
			if( (rxbd[i].info & FRST_MASK) && (rxbd[i].info & LAST_MASK) ) {
				recv_data = (unsigned int *)rxbd[i].data;
				len = rxbd[i].info & LEN_MASK;
				if( !(rxbd[i].data = malloc(ETH_MTU + VMAC_BUFFER_PAD))) {
					printf("Out of memory,dropping packet\n");
					rxbd[i].info = OWN_MASK | (ETH_MTU + VMAC_BUFFER_PAD);
				}
				else {
					rxbd[i].info = OWN_MASK | (ETH_MTU + VMAC_BUFFER_PAD); 
					NetReceive((uchar *)recv_data,len);
					free(recv_data);
				}					
				
			}
			else {
				printf("Rx chained, Packet bigger than device MTU\n");
				rxbd[i].info = OWN_MASK | (ETH_MTU + VMAC_BUFFER_PAD);
			}
				
		}	
	}
	return (0);
}
int eth_send (volatile void *packet, int length)
{

	volatile unsigned int *reg_base_addr = VMAC_REG_BASEADDR;
	volatile unsigned int *stat_reg = reg_base_addr + STAT_OFFSET;
	if (length > ETH_MTU) {
	     printf("eth_send:packet length greater than MTU\n");  		
	     return -EMSGSIZE;
	}
	if ( length < 64 )
		length = 64;

	if( (txbd[txbd_cntr].info & OWN_MASK) == 0) {
			txbd[txbd_cntr].data = packet;
			txbd[txbd_cntr].info = OWN_MASK | length | FRST_MASK | LAST_MASK;
			*stat_reg = TXPL_MASK;		
			
	} else {
		printf("Out of Tx Buffers\n");
		return -ENOMEM;
	}
	
	/* Poll for TXINT */
	while ( !(*(stat_reg) & TXINT_MASK));

/*	printf ("aa3: TXINT, sent packet\n");*/
	
	*stat_reg = TXINT_MASK;
	txbd[txbd_cntr].data = 0;
	txbd[txbd_cntr].info = 0;
	txbd_cntr = (txbd_cntr + 1)%TX_BDT_LEN;	
	return (0);
}

void eth_halt(void)
{
	volatile unsigned int *reg_base_addr = VMAC_REG_BASEADDR;
	volatile unsigned int *control_reg = reg_base_addr + CONTROL_OFFSET;
	txbd_cntr = 0;
	(*control_reg) = (*control_reg) & (~EN_MASK)  ;  	 	/* VMAC disabled  */
}

void aa3_emac_set_address(void *ptr)
{
	char *mac_addr = (char *)ptr ;
        volatile unsigned int *reg;
	volatile unsigned int *reg_base_addr = VMAC_REG_BASEADDR;
	
	reg = reg_base_addr + ADDRL_OFFSET;
	*reg = *((unsigned int *)mac_addr);
	reg = reg_base_addr + ADDRH_OFFSET;
	(*reg) = (*(unsigned int *) (mac_addr+4)) & 0x0000ffff;
}



int eth_init(bd_t *bd)
{	
	volatile unsigned int *reg;
	volatile unsigned int *stat_reg;
	volatile unsigned int *reg_base_addr;
	int i;
	unsigned int temp,duplex ;
	static int initialised = 0 ;

	reg_base_addr	= (unsigned int *) (VMAC_REG_BASEADDR);
	if(initialised) {
	  reg = reg_base_addr + CONTROL_OFFSET;
	  goto out;	
	}
	reg = reg_base_addr + ENABLE_OFFSET;	
	(*reg) = 0;	/* Disable all interrupts as we are not handling */

	reg =  reg_base_addr + MDIO_DATA_OFFSET;
	stat_reg = reg_base_addr + STAT_OFFSET;

	/* Reset the PHY */
	__mdio_write(stat_reg, reg, PHY_ID, LXT970A_CTRL_REG, LXT970A_CTRL_RESET);

	/* Wait till the PHY has finished resetting */
	do {
		__mdio_read (stat_reg, reg, PHY_ID, LXT970A_CTRL_REG, temp);
	} while (temp & LXT970A_CTRL_RESET);
 
	/* Advertize capabilities */
	temp = LXT970A_AUTONEG_ADV_10BTX_FULL | 
			LXT970A_AUTONEG_ADV_10BT | AUTONEG_ADV_IEEE_8023;
	
	/* If the system clock is greater than 25Mhz then advertize 100 */
#if (CONFIG_ARC_CLK > 25000000)
	temp = LXT970A_AUTONEG_ADV_100BTX_FULL | LXT970A_AUTONEG_ADV_100BTX;
#endif
	__mdio_write(stat_reg, reg, PHY_ID, LXT970A_AUTONEG_ADV_REG, temp);

	/* Start Auto-Negotioation */
	__mdio_write(stat_reg, reg, PHY_ID, LXT970A_CTRL_REG,
			 (LXT970A_CTRL_AUTONEG | LXT970A_CTRL_RESTART_AUTO));
	
	/* Wait for Auto Negotiation to complete */
	do {
		__mdio_read(stat_reg, reg, PHY_ID, LXT970A_STATUS2_REG, temp);
	} while (!(temp & LXT970A_STATUS2_COMPLETE));

	/* Check if full duplex is supported and set the vmac accordingly */
	if (temp & LXT970A_STATUS2_FULL) 
		duplex = ENFL_MASK;
	else
		duplex = 0;

	 
								 

								 
	/* Allocate and set buffers for rx BD's */
	for ( i=0 ; i<RX_BDT_LEN ; i++) {
		rxbd[i].data = malloc(ETH_MTU +VMAC_BUFFER_PAD);
		rxbd[i].info = OWN_MASK | (ETH_MTU + VMAC_BUFFER_PAD);
	}
	
	/* All TX BD's owned by CPU */
	for ( i=0 ; i<TX_BDT_LEN ; i++) {
		txbd[i].data = 0;
		txbd[i].info = 0;
	}
	
	/* Set EMAC hardware address */
	aa3_emac_set_address((void *)&default_addr);	

	/* Initialize logical address filter */
	reg = reg_base_addr + LAFL_OFFSET;
	(*reg ) = 0x0;
	reg = reg_base_addr + LAFH_OFFSET;
	(*reg ) = 0x0;

	/* Set rx BD ring pointer */
	reg = reg_base_addr + RXRINGPTR_OFFSET;
	(*reg) = (unsigned int )rxbd;

	/* Set tx BD ring pointer */
	reg = reg_base_addr + TXRINGPTR_OFFSET;
	(*reg) = (unsigned int )txbd;

	/* Set Poll rate so that it polls every 1 ms*/
	reg = reg_base_addr + POLLRATE_OFFSET;
	(*reg) = (CONFIG_ARC_CLK/1000000); 	


	/* Set CONTROL */
	reg = reg_base_addr + CONTROL_OFFSET;
	(*reg) = (RX_BDT_LEN << 24) |	/* RX buffer desc table len */
		 (TX_BDT_LEN << 16) |	/* TX buffer des tabel len */
		 TXRN_MASK |		/* TX enable */
		 RXRN_MASK |		/* RX enable */
		 duplex;		/* Full Duplex enable */

 out:
	(*reg) |= EN_MASK;  	 	/* VMAC enable */
	initialised = 1;
	return (0);
}

#endif /* (CONFIG_COMMANDS & CFG_CMD_NET) */
