#include <config.h>
#include <common.h>
#include <flash.h>
#include <asm/jaguar.h>

extern unsigned eeprom_read_id();
#define mmreg_base 0xf0000000
#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))

flash_info_t flash_info[CFG_MAX_FLASH_BANKS];

unsigned long flash_init (void)
{
    int bank,sect,size = 0;
    unsigned int temp;
    temp = readl(PPC_SFT_RSTN + mmreg_base) & (~CCSPI_SFT_RSTN_MASK);
    writel(temp, PPC_SFT_RSTN + mmreg_base);
    temp = readl(PPC_SFT_RSTN + mmreg_base) | CCSPI_SFT_RSTN_MASK;
    writel(temp, PPC_SFT_RSTN + mmreg_base);


    temp = eeprom_read_id();
    debug("EEPROM ID is %x\n", temp);
    for(bank=0; bank < CFG_MAX_FLASH_BANKS; ++bank)
    {
    
        memset(&flash_info[bank],UNSUPPORTED_FLASH_CMD,sizeof(flash_info_t));

        if(temp == 0x00202018)
        {
            printf("EEPROM Type STM25P128\n");
            flash_info[bank].size = 0x1000000; // 16M bytes
            flash_info[bank].sector_count = 64;
            flash_info[bank].flash_id = FLASH_M25P128;
            flash_info[bank].my_page_num_per_sector = 1024;
            flash_info[bank].my_page_size = 256;
        }
        if(temp == 0x00c22017)
        {
            printf("EEPROM Type MX25L6445E\n");
            flash_info[bank].size = 0x1000000; // 16M bytes
            flash_info[bank].sector_count = 128;
            flash_info[bank].flash_id = FLASH_MX25L6445E;
            flash_info[bank].my_page_num_per_sector = 16;
            flash_info[bank].my_page_size = 4096;
        }
        if(temp == 0x00c22018)
        {
            printf("EEPROM Type MX25L12845E\n");
            flash_info[bank].size = 0x1000000; // 16M bytes
            flash_info[bank].sector_count = 256;
            flash_info[bank].flash_id = FLASH_MX25L12845E;
            flash_info[bank].my_page_num_per_sector = 16;
            flash_info[bank].my_page_size = 4096;
        }
        else
        {
            printf("EEPROM Type STM25P80\n");
            flash_info[bank].size = 0x100000; // 1M bytes
            flash_info[bank].sector_count = 16;
            flash_info[bank].flash_id = FLASH_M25P80;
            flash_info[bank].my_page_num_per_sector = 256;
            flash_info[bank].my_page_size = 256;
        }

        for(sect=0; sect < flash_info[bank].sector_count; ++sect)
        {
            flash_info[bank].start[sect] = CFG_FLASH_BASE + sect * flash_info[bank].my_page_num_per_sector * flash_info[bank].my_page_size;
            flash_info[bank].protect[sect] = 0;
        }
        size += flash_info[bank].size;
    }
    return size;
}


#define ADDR_SIZE 3
#define WREN (0x06 << 24)
#define RDSR (0x05 << 24)
#define WRSR (0x01 << 24)
#define READ (0x03 << 24)
#define PP   (0x02 << 24)
#define PW   (0x02 << 24)
#define SERASE (0xD8 << 24)
#define BERASE (0xC7 << 24)

static unsigned int spi_shift(unsigned int xfer_size, unsigned int hold_en, unsigned int cap_edge, unsigned int wdat) 
{
    unsigned int tmpValue;
    unsigned int config    = (0xA<<24)|(cap_edge << 3) | (xfer_size << 1) | hold_en;
    
    writel(config, SPI_CFG + mmreg_base);
    writel(wdat, SPI_WDAT + mmreg_base);
    while(0 != (readl(SPI_STATUS +mmreg_base) & 0x00000001));
    tmpValue = readl(SPI_RDAT +mmreg_base);
    return tmpValue;
}

unsigned eeprom_read_id()
{
	unsigned xfer_size, hold_en, read_data;
	////write READID op-code
	xfer_size = 0;
	hold_en   = 1;
	spi_shift(xfer_size, hold_en, 1, 0x9f000000);

	//read data
	xfer_size = 2;
	hold_en   = 0;
	read_data = spi_shift(xfer_size, hold_en, 1, 0x0);
	return read_data;
}
unsigned read_status (void)
{
	unsigned read_data;
	read_data = spi_shift(1, 0, 1, RDSR);
	return read_data;
}

//enable write eeprom
void enable_eeprom_write(void)
{
    //enable write - op-code = 0x6
	unsigned xfer_size = 0;
	unsigned hold_en   = 0;
	unsigned cap_edge = 1;
	spi_shift(xfer_size, hold_en, cap_edge, WREN);	
	if ((read_status() & 0x00000002) != 0x00000002) {
        printf("Write enable could not be set!");
    }
}
void eeprom_write_address(unsigned byte_addr)
{
	//write op-code PP and byte_addr - write_eeprom
	//page_write opcde = PW (slower, but doesn't require erase first).
	unsigned hold_en   = 1;
#ifdef PRE_ERASE
		//eeprom_page_erase(byte_addr/PAGE_SIZE);
		enable_eeprom_write();
		spi_shift(ADDR_SIZE, hold_en, 1, PP + (byte_addr << (8*(3-ADDR_SIZE))));	
#else
		enable_eeprom_write();
		spi_shift(ADDR_SIZE, hold_en, 1, PW + (byte_addr << (8*(3-ADDR_SIZE))));
#endif
}

#define TIMEOUT 9000000
void wait_eeprom_rdy (int loud)
{
	int	read_data  = read_status() & 0x00000001;
	int loop_count = 0;
	while (read_data && (loop_count<TIMEOUT)) {
		read_data = read_status() & 0x00000001;
		loop_count++;
	}
    if(loop_count >= TIMEOUT) 	{
		printf("WAIT EEPROM TIMEOUT ERROR\n");
//		g_bFail = true;
	}
}
void eeprom_write_data(unsigned hold_en, unsigned xfer_size, unsigned data)
{
	//write data
	spi_shift(xfer_size, hold_en, 1, data);
    //if hold = 0, then wait
    if (hold_en == 0)
	    wait_eeprom_rdy(0);
}

//read address
void eeprom_read_address(unsigned byte_addr)
{
	unsigned hold_en   = 1;
	spi_shift(ADDR_SIZE, hold_en, 1, READ + (byte_addr << (8*(3-ADDR_SIZE))));
}

void eeprom_sector_erase(unsigned addr)
{
	unsigned hold_en = 0;

	enable_eeprom_write();
	spi_shift(ADDR_SIZE, hold_en, 1, SERASE + (addr&0x00ffffff));	
	wait_eeprom_rdy(0);
}

unsigned eeprom_dword_read(unsigned hold_en){
	unsigned xfer_size = 3;
	unsigned read_data = spi_shift(xfer_size, hold_en, 1, 0x0);
	return read_data;
}

unsigned eeprom_byte_read(unsigned byte_addr){
	unsigned xfer_size, hold_en, read_data;
	////write READ op-code and byte_addr
	xfer_size = ADDR_SIZE;
	hold_en   = 1;
	spi_shift(xfer_size, hold_en, 1, READ + byte_addr);

	//read byte
	xfer_size = 0;
	hold_en   = 0;
	read_data = spi_shift(xfer_size, hold_en, 1, 0x0);

	return read_data;
}


void flash_print_info(flash_info_t * info)
{
    int i;

    if((info->flash_id != FLASH_M25P80) && (info->flash_id != FLASH_M25P128)&& (info->flash_id != FLASH_MX25L6445E)&& (info->flash_id != FLASH_MX25L12845E))
    {
        puts("missing or unknown FLASH type\n");
        return;
    }

    if(info->flash_id == FLASH_M25P80)
    {
        printf("M25P80 FLASH\n");
    }
    else if(info->flash_id == FLASH_M25P128)
    {
        printf("M25P128 FLASH\n");
    }
    else if(info->flash_id == FLASH_MX25L6445E)
    {
        printf("MX25L6445E FLASH\n");
    }
    else if(info->flash_id == FLASH_MX25L12845E)
    {
        printf("MX25L12845E FLASH\n");
    }
    
    printf("size = %1d MB in %d sectors\n", info->size >> 20, info->sector_count);
    printf("page num per sector %d\n", info->my_page_num_per_sector);
    printf("  Sector Start Address:");
    for(i=0; i < info->sector_count; ++i)
    {
        if((i%5) == 0)
            printf("\n");
        printf (" %08lX%s",info->start[i], info->protect[i] ? " (RO)  " : "     ");
    }
    printf("\n");
    return;
}


int flash_erase (flash_info_t * info, int s_first, int s_last)
{
    int prot;
    int sect;
    if ((info->flash_id != FLASH_M25P80) && (info->flash_id != FLASH_M25P128)&& (info->flash_id != FLASH_MX25L6445E)&& (info->flash_id != FLASH_MX25L12845E))
    {
        puts ("Can't erase unknown flash type - aborted\n");
        return 1;
    }
	if ((s_first < 0) || (s_first > s_last))
    {
		puts ("- no sectors to erase\n");
		return 1;
	}

	prot = 0;
	for (sect = s_first; sect <= s_last; ++sect)
    {
		if (info->protect[sect]) {
			prot++;
		}
	}
	if (prot)
    {
		printf ("- Warning: %d protected sectors will not be erased!\n", prot);
	} 
    else 
	{
		putc ('\n');
	}
    enable_eeprom_write();
    //printf("read status = %x\n",read_status());
    spi_shift(1,0,1,WRSR+(0x02<<16));
    wait_eeprom_rdy(0);
    enable_eeprom_write();
    //printf("read status = %x\n",read_status());
    
    for(sect = s_first; sect <= s_last; sect++)
    {
        if(info->protect[sect] == 0)
        {
            eeprom_sector_erase(sect*info->my_page_num_per_sector*info->my_page_size);
            putc('.');
        }
    }    
     puts(" done\n");
 	return 0;
}


int write_buff (flash_info_t * info, uchar * src, ulong addr, ulong cnt)
{   
    unsigned int j = 0;
    unsigned int hold_en;
    addr = addr & ( ~0xf0000000);

    while(j < cnt)
    {
        if ( (addr + j) % info->my_page_size == 0)
        {
             eeprom_write_address(addr + j);
        }
        hold_en = (addr +j +1) % info->my_page_size !=0 ? 1:0;
        eeprom_write_data(hold_en, 0, src[j] << 24);
        j++;
    }

    //Write dummy byte to padding the remaining page size

    while( (addr + j) % info->my_page_size != 0)
    {
        hold_en = (addr +j +1) % info->my_page_size !=0 ? 1:0;
        eeprom_write_data(hold_en, 0, 0xff);        
        j++;
    }

    return 0;
}


#if 0
int read_buff ( uchar * src, ulong addr, ulong cnt)
{
    unsigned int j = 0;

    addr = addr & ( ~0xf0000000);
    eeprom_read_address(addr + j);
    wait_eeprom_rdy(0);
    while(j < cnt)
    {
        src[j] = eeprom_byte_read(j + addr);
        j++;
    }
    return 0;
}
#else
int read_buff ( uchar * src, ulong addr, ulong cnt)
{
    unsigned int bytes_readed = 0;
    unsigned int i;
    unsigned int write_data = 0,read_data;

    wait_eeprom_rdy(0);
    
    //set address
    //set cmd and address
    //Note:current just support address size 3
    addr = addr & ( ~0xf0000000);
    eeprom_read_address(addr);

    //read
    while(bytes_readed < cnt)
    {
        if(cnt < 4||bytes_readed >= (cnt -4))
        {
            read_data = spi_shift(cnt-bytes_readed-1, 0, 1, 0x0);
            for(i=0; i < cnt-bytes_readed; i++)
            {
                src[bytes_readed+i] = (read_data>>((cnt - bytes_readed -1-i)*8))&0xff;
            }
            bytes_readed = cnt;
        }
        else
        {
            //read 4 bytes
            read_data = spi_shift(3, 1, 1, 0x0);
            for(i = 0; i< 4; i++)
            {
                src[bytes_readed+i] = (read_data>>(8*(3-i)))&0xff; 
            }
            bytes_readed += 4;
        }
    }

    return 0;	
}
#endif

