/******************************************************************************
 * Copyright Codito Technologies (www.codito.com)
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/board/aa3/arc_gmac.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

/* Amber Lin: modified to run Synopsys GMAC on Eagle using Enhanced Descriptor
 *            structure                                          -- April 2009
 */

#include <cmd_confdefs.h>	/* Command configs	*/
#include <asm/u-boot.h>		/* Board data structure	*/
#include "arc_gmac.h"	/* MDIO macros and ARC_EMAC register macros	*/
#include <asm/jaguar.h>
#include <config.h>		/* ARC700 clock freq.	*/
#include <asm/errno.h>
#include <malloc.h>
#include <common.h>
#include <net.h>
#include <linux/types.h>

#define ENABLE_LOOPBACK 0

#define BUF_MULTIPLE 16	/* data bus width is 128 bits on eagle */
#define RX_BUF_ALIGN(X) (((X) + (BUF_MULTIPLE - 1)) & ~(BUF_MULTIPLE - 1))

#if (CONFIG_COMMANDS & CFG_CMD_NET)

/* rx buffer descriptor. We align descriptors to 32 so
 * satisfies that the descriptor addresses must be aligned to the bus-width 
 * used */
DmaDesc *rxbd;

/* tx buffer descriptor */
DmaDesc *txbd;

static struct eth_device eth_netdev;

//DmaDesc rxbd[RX_BDT_LEN] __attribute__((aligned(32)));= {{0}};

/* tx buffer descriptor */
//DmaDesc txbd[TX_BDT_LEN] __attribute__((aligned(32)));//= {{0}};

//#define DEBUG

#ifdef DEBUG
#define PDEBUG(fmt, args...) printf(  fmt, ## args)
#else
#define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))

static int txbd_cntr = 0; /* transmit buffer counter */
static int rxbd_cntr = 0; /* receive buffer counter  */

/****************************************************/
/* System-dependant register access functions       */
/****************************************************/

unsigned long reg_read(unsigned long addr)
{
	unsigned long data;
	unsigned long reg;

	/* eth0 */
	reg = MMREG_BASE + ETH_STATUS_REG;
	do {
		data = readl(reg);
	} while((data & CSR_BUSY_MASK) != 0);

	reg = MMREG_BASE + ETH_CSR_CTRL_REG;
	writel(addr, reg);

	reg = MMREG_BASE + ETH_STATUS_REG;
	do {
		data = readl(reg);
	} while((data  & CSR_BUSY_MASK) != 0);
        
	reg = MMREG_BASE + ETH_RDAT_REG;
	data = readl(reg);

	return data;
}

void reg_write(unsigned long int data, unsigned long int addr)
{
	unsigned long reg, temp;
    

	/* eth0 */
	reg = MMREG_BASE + ETH_STATUS_REG;
	do {
		temp = readl(reg);
	} while( (temp & CSR_BUSY_MASK) != 0);

	reg = MMREG_BASE + ETH_WDAT_REG;
	writel(data, reg);

	reg = MMREG_BASE + ETH_CSR_CTRL_REG;
	writel(addr | 0x80000000, reg);

	reg = MMREG_BASE + ETH_STATUS_REG;
	do {
		temp = readl(reg);
	} while( (temp & CSR_BUSY_MASK) != 0);
}

void reg_write_eth0(unsigned long int data, unsigned long int addr)
{
	unsigned long reg, temp;

	reg = MMREG_BASE + ETH_STATUS_REG;
	do {
		temp = readl(reg);
	} while( (temp & CSR_BUSY_MASK) != 0);

	reg = MMREG_BASE + ETH_WDAT_REG;
	writel(data, reg);

	reg = MMREG_BASE + ETH_CSR_CTRL_REG;
	writel(addr | 0x80000000, reg);

	reg = MMREG_BASE + ETH_STATUS_REG;
	do {
		temp = readl(reg);
	} while( (temp & CSR_BUSY_MASK) != 0);
}

unsigned long read_mac_reg(unsigned long reg)
{
	unsigned long data = reg_read(GmacRegisterBase + reg);
	return data;
}

void write_mac_reg(unsigned long reg, unsigned long data)
{
	reg_write(data,GmacRegisterBase + reg);
}

void set_mac_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = GmacRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t |= data;
	reg_write(data_t, addr);
}

void clear_mac_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = GmacRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t &= ~data;
	reg_write(data_t, reg);
}

unsigned long read_dma_reg(unsigned long reg)
{
	unsigned long data = reg_read(DmaRegisterBase + reg);
	return data;
}

void write_dma_reg(unsigned long reg, unsigned long data)
{
	reg_write(data, DmaRegisterBase + reg);
}

void set_dma_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = DmaRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t |= data;
	reg_write(data_t, addr);
}

void clear_dma_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = DmaRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t &= ~data;
	reg_write(data_t, reg);
}

/***********************/
/* MAC module function */
/***********************/

unsigned long read_mii(unsigned long reg)
{
	unsigned long addr;
	unsigned long data;
	addr = ((DefaultPhyBase << GmiiDevShift) & GmiiDevMask) | ((reg << GmiiRegShift) & GmiiRegMask) | GmiiRead;
	write_mac_reg(GmacGmiiAddr, (addr | GmiiAppClk2 | GmiiBusy));
	while( (read_mac_reg(GmacGmiiAddr) & GmiiBusy) == GmiiBusy);
	data = read_mac_reg(GmacGmiiData) & 0xFFFF;
	return data;
}

void write_mii(unsigned long reg, unsigned long data)
{
	unsigned long addr;
	write_mac_reg(GmacGmiiData, data);
	addr = ((DefaultPhyBase << GmiiDevShift) & GmiiDevMask) | ((reg << GmiiRegShift) & GmiiRegMask) | GmiiWrite;
	write_mac_reg(GmacGmiiAddr, (addr | GmiiAppClk2 | GmiiBusy));
	while( (read_mac_reg(GmacGmiiAddr) & GmiiBusy) == GmiiBusy);
}

void gmac_set_address(void *ptr)
{
	char *mac_addr = (char *)ptr;
	unsigned long int data;

	data = (mac_addr[5] << 8) | mac_addr[4];
	write_mac_reg(GmacAddr0High, data);
	data = (mac_addr[3] << 24) | (mac_addr[2] << 16) | (mac_addr[1] << 8) | mac_addr[0];
	write_mac_reg(GmacAddr0Low, data);
}


/*********************************************************/
/* reset ethernet block                                  */
/********************************************************/
void eth_reset(void)
{
	unsigned long data;

//	printf("Ethernet: reset\n");
    
	// this pin could be used as GPIO pins or for ethernet, 
	// set this pins work for ethernet, not for gpio purpose
	data = readl(MMREG_BASE + GPIO_B_CTRL);
	data = (data & (~GPIO_B_CTRL_GPIO_MODE_SEL_MASK));
	writel(data, MMREG_BASE + GPIO_B_CTRL);

	// reset ethernet
	// CG_RESET_REG\ETH0_RESET   = 0x1
	// CG_RESET_REG\ETH0_RESET   = 0x0
	data = readl(MMREG_BASE + CG_RESET_REG);
	data = data | ETH0_RESET_MASK;
	writel(data, MMREG_BASE + CG_RESET_REG);

	data = readl(MMREG_BASE + CG_RESET_REG);
	data = data & (~ETH0_RESET_MASK);
	writel(data, MMREG_BASE + CG_RESET_REG);

	// ETH_CTRL_REG\RESETN    = 0x0
	// ETH_CTRL_REG\RESETN    = 0x1
	data = readl(MMREG_BASE + ETH_CTRL_REG);
	data = data & ~ETH_CTRL_REG_RESETN_MASK;
	writel(data, MMREG_BASE + ETH_CTRL_REG);

	data = readl(MMREG_BASE + ETH_CTRL_REG);
	data = data | ETH_CTRL_REG_RESETN_MASK;
	writel(data, MMREG_BASE + ETH_CTRL_REG);
}

int DescDma( DmaDesc Desc )
{
	return (Desc.status & DescOwnByDma) != 0;   /* Owned by DMA */
}

int DmaRxValid( unsigned int Status )
{
	/* no errors, and whole frame is in the buffer */
	return ( (Status & DescError) == 0 )
		&& ( (Status & DescRxFirst) != 0 )
		&& ( (Status & DescRxLast) != 0 );
}

static int gmac_eth_recv(struct eth_device *netdev)
{
	unsigned int *recv_data, len;
	unsigned long status;
	int j;
	uchar *p;

	PDEBUG("gmac_eth_recv\n");
	j=0;
	do {
		status = read_dma_reg(DmaStatus);
		j++;
		if (status & DmaIntRxStopped)
			write_dma_reg(DmaRxPollDemand, 0);
	} while(!(status  & DmaIntRxCompleted)  && j<100);

	write_dma_reg(DmaStatus, status);
	//if(j>=100)
	//	return 0;

	status = arc_read_uncached_32(&(rxbd[rxbd_cntr].status));
	while(!(status & DescOwnByDma))
	{
		if(DmaRxValid(status))
		{
			len = ((status & DescFrameLengthMask) >> DescFrameLengthShift) - 4; /* ignore ethernet crc */
			recv_data = (unsigned int *)(arc_read_uncached_32(&rxbd[rxbd_cntr].buffer1));
			inv_dcache_all();
			p = (uchar *)recv_data;
			NetReceive((uchar *)recv_data, len);
		}
		else
			printf("DmaRx invalid status = %x\n", status);

		arc_write_uncached_32(&(rxbd[rxbd_cntr].status), DescOwnByDma);
		rxbd_cntr = (rxbd_cntr + 1) % RX_BDT_LEN;
		status = arc_read_uncached_32(&(rxbd[rxbd_cntr].status));
	}

	PDEBUG("gmac_eth_recv return\n");
	return 0;
}

static int gmac_eth_send(struct eth_device *netdev, volatile void *packet, int length)
{
	unsigned long  status;
	unsigned int i;
	
	PDEBUG("gmac_eth_send\n");

	if(length > ETH_MTU)
	{
		printf("gmac_eth_send: packet length %d greater than MTU\n", length);
		return -1;
	}
	if(length < 64)
		length = 64;

	/* set up the tx descriptor */
	if((arc_read_uncached_32(&(txbd[txbd_cntr].status)) & DescOwnByDma) == 0) //owned by the host
	{
		arc_write_uncached_32(&(txbd[txbd_cntr].buffer1), packet);
		arc_write_uncached_32(&(txbd[txbd_cntr].length), (length << DescSize1Shift) & DescSize1Mask);
		flush_dcache_all();

		status = arc_read_uncached_32(&txbd[txbd_cntr].status);
		status &= TxDescEndOfRing; //clear the descriptor
		status = status | TxDescChain | DescOwnByDma | DescTxFirst | DescTxLast | DescTxIntEnable;
		arc_write_uncached_32(&(txbd[txbd_cntr].status), status);
	} 
	else
	{
		printf("Out of Tx Buffers\n");
		return -ENOMEM;
	}

	/* issue a Poll Demand command to wake up TxDMA if it's in suspend mode */
	write_dma_reg(DmaTxPollDemand, 0);

	i=0;
	do {
		status = read_dma_reg(DmaStatus);
		i++;
	}
	while( !( status & DmaIntTxCompleted)  && i<100);

	if(i>=100)
	{
		printf("tx time out status = %x\n", status);
	}

	write_dma_reg(DmaStatus, status);
	arc_write_uncached_32(&(txbd[txbd_cntr].status), (txbd_cntr == TX_BDT_LEN-1) ? TxDescEndOfRing : 0 | TxDescChain);
	arc_write_uncached_32(&(txbd[txbd_cntr].buffer1), 0);

	txbd_cntr = (txbd_cntr + 1) % TX_BDT_LEN;	
	PDEBUG("gmac_eth_send return\n");

	return 0;
}

int DmaInit(void)
{
	unsigned long data;

	/* 1. Set Host bus access parameters */
	data = read_dma_reg(DmaBusMode);
	data |= DmaBurstLength8;
	write_dma_reg(DmaBusMode, data);

	/* 2. Mask uncessary interrupt causes: disable all as interrupt
	      mechanism is not implemented in u-boot */
	write_dma_reg(DmaInterrupt, 0);
	write_dma_reg(DmaStatus, read_dma_reg(DmaStatus));

	/* 3. Create Tx/Rx descriptor lists and write to DMA Reg 3 & Reg 4 
	      (descriptor lists have been created in eth_init) */
	write_dma_reg(DmaRxBaseAddr, (unsigned long int)rxbd);//rx buf desc ptr
	write_dma_reg(DmaTxBaseAddr, (unsigned long int)txbd);//tx buf desc ptr

//	printf(" rxbd = %x   txbd = %x\n", rxbd, txbd);

	return 0;
}

int MacInit(void)
{
	unsigned long data;	

	/* Step 4: write to GMAC Reg 1, 2, 3 for filtering options */
	/* GMAC filter register for receive all */
	write_mac_reg(GmacFrameFilter, GmacFilterOff);

//	/* Step 5: write to GMAC Reg 0 for Tx/Rx operating modes */
//	if(strcmp(getenv("ethact"), "eth1") == 0)
//		reg_write_eth0(GmacSelectMii, GmacConfig + GmacRegisterBase);
		// select MII (10/100Mbps)

	/* full duplex rx & tx eable disable receive own */
	data = read_mac_reg(GmacConfig);
	data |= GmacFullDuplex | GmacSelectMii | GmacTxEnable | GmacRxEnable | GmacDisableRxOwn;
	data &= ~GmacLoopbackOn;
	write_mac_reg(GmacConfig, data);

	return 0;
}

void gmac_eth_halt(void)
{
	unsigned long data;
	data = read_dma_reg(DmaControl);
	data = data & (~DmaTxStart) & (~DmaRxStart);
	write_dma_reg(DmaControl,data);
}

int reset_eth_phy(void)
{
	int i;
	unsigned long temp;

	if ((read_mii(GEN_sts) & AUTOCMPLT) == 0)
	{
		write_mii(GEN_ctl, 0x2100);
		i = 500;
		while(i--);
	}

	if( (read_mii(GEN_sts) & LINK) == 0)
	{
		printf("no link\n");
		return -1;
	}
	else
		printf("link up\n");

#if(ENABLE_LOOPBACK ==1)
	temp = read_mii(GEN_ctl);
	write_mii(GEN_ctl, temp | LOOPBACK);
#else /* disable loopback */
	temp = read_mii(GEN_ctl);
	write_mii(GEN_ctl, temp & ~(LOOPBACK));
#endif

	return 0;
}

/* Referring to Synopsys Databook v3.42a 3.4.1 Initialization */
int gmac_eth_init(struct eth_device *netdev, bd_t *bd)
{
	int i, size;
	static int initialised = 0 ;
	char *p;

//	printf("gmac:%s:%p: enter\n", __func__, gmac_eth_init);

	if(initialised)
		goto out;

	/* Reset ethernet block */
	eth_reset();

	rxbd = (DmaDesc *)(0xe00000);
	txbd = (DmaDesc *)(0xe00000) + RX_BDT_LEN * sizeof(DmaDesc);

//	PDEBUG("Version Register = %x\n", read_mac_reg(0x20));

	/* Reset the PHY */
//	if(reset_eth_phy() < 0)
//		return -1;

	/* Create Rx descriptor list : enhanced descriptor structure is
	   implemented on eagle */
	for(i=0; i<RX_BDT_LEN; i++)
	{
		/* RDES0 */
		arc_write_uncached_32(&(rxbd[i].status), DescOwnByDma);

		/* RDES1 */
		size = RX_BUF_ALIGN(ETH_MTU + GMAC_BUFFER_PAD);
		arc_write_uncached_32(&(rxbd[i].length), size | RxDescChain);

		/* RDES2 */
		p = (unsigned char *)malloc(size + 32);
		p = (uchar *)((unsigned long)(p + 32) & (0xfffffe0));
//		printf("malloc addr = %x\n", p);
		arc_write_uncached_32(&(rxbd[i].buffer1), (unsigned long *)p);
		if (arc_read_uncached_32(&(rxbd[i].buffer1)) == 0 )
		{
			printf("no memory for receive descriptor i = %d\n", i);
		}

		/* RDES3 */
		arc_write_uncached_32(&rxbd[i].buffer2, &rxbd[i+1]);
	}
	arc_write_uncached_32(&(rxbd[RX_BDT_LEN-1].length), arc_read_uncached_32(&rxbd[RX_BDT_LEN-1].length) | RxDescEndOfRing);
	arc_write_uncached_32(&rxbd[RX_BDT_LEN-1].buffer2, &rxbd[0]);

	/* Create Tx descriptor list with enhanced descriptor structure */
	/* All TX BD's owned by CPU */
	for( i = 0; i<TX_BDT_LEN; i++)
	{
		/* TDES0 */
		arc_write_uncached_32(&(txbd[i].status), TxDescChain);

		/* TDES1 */
		arc_write_uncached_32(&(txbd[i].length), 0);

		/* TDES2 */
		arc_write_uncached_32(&(txbd[i].buffer1), 0);

		/* TDES3 */
		arc_write_uncached_32(&txbd[i].buffer2, &txbd[i + 1]);
	}
	arc_write_uncached_32(&(txbd[TX_BDT_LEN-1].status), TxDescChain | TxDescEndOfRing);
	arc_write_uncached_32(&txbd[TX_BDT_LEN-1].buffer2, &txbd[0]);
//	printf(" rxbd = %x   txbd = %x\n", rxbd, txbd);

	/* set GMAC hardware address */
	gmac_set_address(bd->bi_enetaddr);

	/* Step 1~3 */
	DmaInit();

	/* Step 4~5 */
	MacInit();
	/* Clear Interrupt Status */
	write_dma_reg(DmaStatus, read_dma_reg(DmaStatus));

	initialised = 1;

out:
	/* Step 6 */
	write_dma_reg(DmaControl, DmaRxStart | DmaTxStart);

	return (0);
}

int gmac_eth_initialize(bd_t *bd)
{
	struct eth_device *dev = &eth_netdev;

	strcpy(dev->name,"eth0");

	dev->init = gmac_eth_init;
	dev->send = gmac_eth_send;
	dev->recv = gmac_eth_recv;
	dev->halt = gmac_eth_halt;

	eth_register(dev);

	return 0;
}  
#endif /* (CONFIG_COMMANDS & CFG_CMD_NET) */

