/******************************************************************************
 * Copyright Vixs Systems Inc. (www.vixs.com)
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/include/asm-arc/arc_sata.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Jerry Wang
 */



#ifndef __ARC_SATA_H
#define __ARC_SATA_H
#include <config.h>
/*
 * HSATA Registers
 */
#define HSATA_PIO_DATA_REG   0x0000
#define HSATA_SCR0_REG       0x0024
#define HSATA_SCR0_SPD_GET(v) (((v) & 0x000000f0) >> 4)
#define HSATA_SCR1_REG       0x0028
#define HSATA_SCR2_REG       0x002C
#define HSATA_SCR3_REG       0x0030
#define HSATA_SCR4_REG       0x0034

#define HSATA_SSTATUS_REG        HSATA_SCR0_REG
#define HSATA_SERROR_REG         HSATA_SCR1_REG
#define HSATA_SERROR_ERR_BITS    0x0000ffff
#define HSATA_SCONTROL_REG       HSATA_SCR2_REG
#define HSATA_SACTIVE_REG        HSATA_SCR3_REG
#define HSATA_SNOTIFICATION_REG  HSATA_SCR4_REG

#define HSATA_FPTAGR_REG         0x0064
#define HSATA_FPBOR_REG          0x0068
#define HSATA_FPTCR_REG          0x006C

#define HSATA_FEAT_REG           0x0004
#define HSATA_CMD_REG            0x001c
#define HSATA_STATUS_REG         0x001c
#define HSATA_CONTROL_REG        0x0020
#define HSATA_DMACR_REG          0x0070
#define HSATA_DBTSR_REG          0x0074
#define HSATA_INTPR_REG          0x0078
#define HSATA_INTPR_ERR_BIT      0x00000008
#define HSATA_INTPR_FP_BIT       0x00000002  /* new DMA setup FIS arrived */
#define HSATA_INTMR_REG          0x007C
#define HSATA_INTMR_ERRM_BIT     0x00000008
#define HSATA_INTMR_NEWFP_BIT    0x00000002
#define HSATA_ERRMR_REG          0x0080
#define HSATA_LLCR_REG           0x0084
#define HSATA_PHYCR_REG          0x0088
#define HSATA_PHYSR_REG          0x008C
#define HSATA_RXBISTPD_REG       0x0090
#define HSATA_RXBISTD1_REG       0x0094
#define HSATA_RXBISTD2_REG       0x0098
#define HSATA_TXBISTPD_REG       0x009C
#define HSATA_TXBISTD1_REG       0x00A0
#define HSATA_TXBISTD2_REG       0x00A4
#define HSATA_BISTCR_REG         0x00A8
#define HSATA_BISTFCTR_REG       0x00AC
#define HSATA_BISTSR_REG         0x00B0
#define HSATA_BISTDECR_REG       0x00B4

#define HSATA_TESTR_REG          0x00F4
#define HSATA_VERSONR_REG        0x00F8
#define HSATA_IDR_REG            0x00FC

#define HSATA_SCR_OFFSET (0x24)



#define HSATA_ADDR_OK(a) ( ((unsigned long)(a) < 0x00000800) && \
                           (((unsigned long)(a) & 0x03) == 0x0 ) )


//********************************************************
//SDUH register definition
//********************************************************
#define SATA_REG_BASE	0xd000d000
#define SATA_REG_SIZE		0x80
#define XCODE3_SATA_REG_BASE    SATA_REG_BASE

#define SATA_CTL_REG						0x00
#define SATA_STATUS_REG					0x04
#define SATA_WDATA_REG					0x08
#define SATA_RDATA_REG						0x0C
#define SATA0_HOST_CTL_REG				0x10
#define SATA1_HOST_CTL_REG				0x14
#define SATA_SFT_RESETS_REG				0x18
#define SATA_INT_STATUS_REG				0x1C
	#define SATA0_INT_MASK		0x1
	#define SATA1_INT_MASK		0x2
	#define DMAC_INT_MASK		0x4
	#define MCIF_INT_MASK		0x8
#define SATA_MIPS_INT_MASK_REG			0x20
#define SATA_HOST_INT_MASK_REG			0x24
#define SATA_PHY_COMMON_STATUS_REG 		0x28
	#define SATA0_PHY_OP_DONE            (1<<3)
	#define SATA1_PHY_OP_DONE            (1<<19)
#define SATA_PHY_TXRX_CTRL_REG	 		0x2C
#define SATA_PHY_MPLL_CTRL_REG			0x30
#define SATA_PHY_ANALOG_CTRL_REG			0x34
#define SATA0_PHY_CR_OUT_REG				0x38
#define SATA0_PHY_CR_IN_REG				0x3C
#define SATA1_PHY_CR_OUT_REG				0x40
#define SATA1_PHY_CR_IN_REG				0x44
#define SATA_DEBUG_REG						0x48

typedef enum {
	SATA0_REG,
	SATA1_REG,
	DMAC_REG
} SATA_reg_type;

#define SATA_SEL_SATA0		(1<<12)
#define SATA_SEL_SATA1		(1<<13)
#define SATA_SEL_DMAC		(1<<14)

#define readb(addr) (*(volatile unsigned char *) (addr))
#define readw(addr) (*(volatile unsigned short *) (addr))
#define readl(addr) (*(volatile unsigned int *) (addr))
#define writeb(b,addr) (*(volatile unsigned char *) (addr) = (b))
#define writew(b,addr) (*(volatile unsigned short *) (addr) = (b))
#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))

extern u32 SDSH_writel(void* SATA_reg_base, u32 sata_reg_addr, u32 value, SATA_reg_type type);
extern u32 SDSH_readl(void* SATA_reg_base, u32 sata_reg_addr, SATA_reg_type type);
extern u32 SDSH_writeb(void* SATA_reg_base, u32 sata_reg_addr, u8 value, SATA_reg_type type);
extern u8 SDSH_readb(void* SATA_reg_base, u32 sata_reg_addr, SATA_reg_type type);
extern u32 SDSH_writew(void* SATA_reg_base, u32 sata_reg_addr, u16 value, SATA_reg_type type);
extern u16 SDSH_readw(void* SATA_reg_base, u32 sata_reg_addr, SATA_reg_type type);
extern u32 SDSH_writesl(void * SATA_reg_base, u32 sata_reg_addr, u32*pbuf, u32 count, SATA_reg_type type);
extern u32 SDSH_readsl(void* SATA_reg_base, u32 sata_reg_addr, u32 *pbuf, u32 count, SATA_reg_type type);
extern u32 SDSH_writesb(void * SATA_reg_base, u32 sata_reg_addr, u8* pbuf , u32 count, SATA_reg_type type);
extern u32 SDSH_readsb(void* SATA_reg_base, u32 sata_reg_addr, u8 *pbuf, u32 count, SATA_reg_type type);
extern u32 SDSH_writesw(void * SATA_reg_base, u32 sata_reg_addr, u16* pbuf , u32 count, SATA_reg_type type);
extern u32 SDSH_readsw(void* SATA_reg_base, u32 sata_reg_addr, u16* pbuf, u32 count, SATA_reg_type type);

extern u32 XCode3_SATA_dev_available;

#endif /* __ARC_SATA_H */
