/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Aug 17 , 2005
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/cpu/arc700/serial.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

#include <common.h>
#include <asm/serial.h>
#include <asm/eagle.h>		/* for XC4 */

#define XC_SOC_PROC_MMREG_BASE 0xF0000000
#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))

aa3_uart *uart = (aa3_uart *) UART_BASEADDR;

/* Amber: Commenting out this line will use Register mode, but try not to do it. */
#define UART_DMA_MODE

#ifdef UART_DMA_MODE
#define TX_BUF_SIZE 1
#define RX_BUF_SIZE 4096
static char __attribute__((aligned(16))) tx_buf[TX_BUF_SIZE];
static char __attribute__((aligned(16))) rx_buf[RX_BUF_SIZE];
#endif

/*
 * Sets baudarate
 */
void serial_setbrg (void)
{
	int div_int;
	unsigned long uart_clk_speed, hueclk_sel;
	int console_baud;
	unsigned long tmp;
#ifdef UART_DMA_MODE
	volatile char *xmit_buf_ptr = tx_buf;
	volatile char *ring_buf_ptr = rx_buf;
#endif

	hueclk_sel = (readl(XC_SOC_PROC_MMREG_BASE+CG_CLK_SRC_SEL4) & HUECLK_SRC_SEL_MASK) >> HUECLK_SRC_SEL_SHIFT;
	switch( hueclk_sel>>4 ){
	case 0:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL2_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 3;
		break;
	case 1:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL3_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 5;
		break;
	case 2:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL7_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 6;
		break;
	case 3:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL1_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 4;
		break;
	case 4:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL4_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 4;
		break;
	case 5:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL5_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 3;
		break;
	case 6:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL6_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 8;
		break;
	case 7:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG_PLL0_FREQ) & 0x01F00000) >> 20;
		uart_clk_speed = (div_int * 27) / 6;
		break;
	default:
		uart_clk_speed = 130;
		break;
	}
	uart_clk_speed *= 1000000;

	/* When CG_CLK_SRC_EN0/HUECLK_SRC_EN is 0, clock speed is 27MHz */
	hueclk_sel = (readl(XC_SOC_PROC_MMREG_BASE+CG_CLK_SRC_EN0)) & HUECLK_SRC_EN_MASK;
        if( !hueclk_sel )
		uart_clk_speed = 27000000;

	console_baud = (unsigned long)uart_clk_speed/(CONSOLE_BAUD_RATE<<4);

	/* Set the baud rate */
	uart->linectrl |= UART0_LINE_CTRL_DIVISOR_ACCESS_MASK;
	tmp = uart->info;
	/* update divisor */
	uart->data = console_baud & 0x000000FF; /* rx_tx_divlsb reg */
	tmp = uart->info;
	uart->divmsb = (console_baud >> 8) & 0x000000FF;
	tmp = uart->info;
	uart->linectrl &= ~UART0_LINE_CTRL_DIVISOR_ACCESS_MASK;
	tmp = uart->info;

#ifdef UART_DMA_MODE
	/* set uart at DMA mode */
	uart->ctrl_reg &= ~UART0_CTRL_REG_CTRL_TX_MASK;
	uart->ctrl_reg &= ~UART0_CTRL_REG_CONTROL_RX_MASK;
	uart->xmit_base = (int)xmit_buf_ptr;
	uart->ring_base = (int)ring_buf_ptr;
	uart->ring_size = RX_BUF_SIZE - 1;
	/* set dma mode bit as DWORD */
	uart->fifoctrl &= ~DMA_MODE_BYTE;
	tmp = uart->info;
#else
	/* set uart at register mode */
	uart->ctrl_reg |= UART0_CTRL_REG_CTRL_TX_MASK;
	uart->ctrl_reg |= UART0_CTRL_REG_CONTROL_RX_MASK;
	/* set trigger level as 1 byte */
	uart->fifoctrl &= 0xffffff3f;  //only for register mode
	tmp = uart->info;
#endif
	/* character length is 8bits */
	uart->linectrl |= UART0_LINE_CTRL_CHAR_LEN_MASK;
	/* transmit 1 stop bit */
	uart->linectrl &= ~UART0_LINE_CTRL_STOP_BITS_MASK;
	/* no parity */
	uart->linectrl &= ~UART0_LINE_CTRL_PARITY_EN_MASK;
	/* enable parity: EVEN */
//	uart->linectrl |= UART0_LINE_CTRL_PARITY_EN_MASK;
	/* enable hardware flow control: RTS/CTS */
//	uart->ctrl_reg |= UART0_CTRL_REG_HW_FLOW_CTRL_EN_MASK;

	uart->int_mask = (0x00000000 | UART0_INT_STATUS_RECIEVE_DATA_RDY_MASK | UART0_INT_MASK_TRANSMIT_DONE_EMPTY_MASK | UART0_INT_MASK_TIMEOUT_MASK);
}

/*
 * Intialise the serial port with given baudrate
 */
int serial_init (void)
{
	/* reset and activate UART */
	writel(UART_RESET_MASK | readl(XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1), XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1);
	writel(~UART_RESET_MASK & readl(XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1), XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1);

	serial_setbrg ();
	return (0);
}


/*
 * Output a single byte to the serial port.
 */
void serial_putc (const char c)
{
	unsigned long tmp;
	if (c == '\n') 
		serial_putc('\r');	       		
#ifdef UART_DMA_MODE
	tx_buf[0] = c;
	flush_dcache_range((unsigned int)tx_buf, (unsigned int)tx_buf);
	uart->xmit_size = 0;
#else
	uart->data = c;
	tmp = uart->info;
#endif

	/* wait until the char is transmitted */
	while(!(uart->int_status & UART0_INT_MASK_TRANSMIT_DONE_EMPTY_MASK));
	uart->int_status |= UART0_INT_MASK_TRANSMIT_DONE_EMPTY_MASK;
	tmp = uart->info;
}

/*
 * Read a single byte from the serial port. Returns 1 on success, 0
 * otherwise 0.
 */
int serial_tstc (void)
{
	/* Wait till character is placed in fifo */
	if(uart->int_status & UART0_INT_STATUS_RECIEVE_DATA_RDY_MASK)
		return (1);
	return (0);
}

/* This function has nothing to do with UART. Because uboot is a single thread program, to
  * regularly monitor certain status, we have to add this feature in serial_getc as this is
  * the place holding the program.
  * Use init_xcode_monitor in lib_arc/board.c to initialize the status if needed.
  */
static void xcode_monitor(void)
{
	unsigned int temp;

	temp = readl(XC_SOC_PROC_MMREG_BASE+CG_DUMMY_REG);
	if((temp==0x2301) || (temp==0x2304)) {
		/* reset the hardware when USBD disconnection is detected */
		temp = readl(XC_SOC_PROC_MMREG_BASE+USBD_UDC20_STATUS0);
		if(temp & UDC20_SUSPEND_MASK) {
			/* set GPIO to output mode */
			temp = readl(XC_SOC_PROC_MMREG_BASE + GPIO_E_OE);
			temp |= 0x1;
			writel(temp, XC_SOC_PROC_MMREG_BASE + GPIO_E_OE);
			/* set high to hard reset xcode */
			temp = readl(XC_SOC_PROC_MMREG_BASE + GPIO_E_OUT);
			temp |= 0x1;
			writel(temp, XC_SOC_PROC_MMREG_BASE + GPIO_E_OUT);
		}
	}
}

/*
 * Read a single byte from the serial port. Returns 1 on success, 0
 * otherwise. When the function is succesfull, the character read is
 * written into its argument c.
 */
int serial_getc (void)
{
	unsigned char ch;
	unsigned long tmp;
#ifdef UART_DMA_MODE
	unsigned int head, tail;
#endif
	/* Wait till character is placed in fifo */
	while(!(uart->int_status & UART0_INT_STATUS_RECIEVE_DATA_RDY_MASK))
		xcode_monitor();

	/* Also check for overflow errors */
	if(uart->line_status & UART0_LINE_STATUS_OVERRUN_MASK){
		uart->line_status |= UART0_LINE_STATUS_OVERRUN_MASK;
		tmp = uart->info;
		ch = 0;
	}
	else{
#ifdef UART_DMA_MODE
		head = uart->ring_head;
		tail = uart->ring_tail;
		if(head != tail) {
			inv_dcache_range((unsigned int)rx_buf+tail, (unsigned int)rx_buf+tail);
			ch = rx_buf[uart->ring_tail];
			tail = (tail + 1) % RX_BUF_SIZE;
			uart->ring_tail = tail;
		}
#else
		ch = uart->data & 0xff;
#endif
	}
	uart->int_status |= UART0_INT_STATUS_RECIEVE_DATA_RDY_MASK;
	tmp = uart->info;

	return ((int)ch);
}

void serial_puts (const char *s)
{
	while (*s) {
		serial_putc (*s++);
	}
}

