/*
 * Copyright (C) Freescale Semiconductor, Inc. 2006. All rights reserved.
 * Author: Jason Jin<Jason.jin@freescale.com>
 *         Zhang Wei<wei.zhang@freescale.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * with the reference on libata and ahci drvier in kernel
 *
 */
#include <common.h>

#ifdef CONFIG_SCSI_AHCI

#include <command.h>
#include <pci.h>
#include <asm/processor.h>
#include <asm/errno.h>
#include <asm/io.h>
#include <malloc.h>
#include <scsi.h>
#include <ata.h>
#include <linux/ctype.h>
#include <ahci.h>
#ifdef PLATFORM_XCODE
#include <asm/viper.h>

#define VIXS_SATA_MAX_HOST		2
#define VIXS_SATA_HOST_SEL_MASK	0x80000000	// we pick bit 31 to select host. 0: host0, 1: host1

u32 g_xcode_ahci_clk = 50; //125Mhz by default
u32 g_xcode_ahci_clk_type = 1;  //0: internal clock 1: external clock
u32 g_xcode_ahci_ss_mode = 0;	//0: not spread spectrum 1: spred spectrum
u32 g_sata0_limit_speed = 0; /* 0: no limit, 1: limit SATA speed to Gen1 */
u32 g_sata1_limit_speed = 0;
u32 g_sata_hw_inited = 0;

void* SDSH_SATA_REG_BASE = 0xBFFFD000;
extern u32 SDSH_writel(void* SATA_reg_base, void * sata_reg_addr, u32 value);
extern u32 SDSH_readl(void* SATA_reg_base, void * sata_reg_addr);
extern u32 SDSH_writeb(void* SATA_reg_base, void * sata_reg_addr, u8 value);
extern u8 SDSH_readb(void* SATA_reg_base, void * sata_reg_addr);
extern u32 SDSH_writew(void* SATA_reg_base, void * sata_reg_addr, u16 value);
extern u16 SDSH_readw(void* SATA_reg_base, void * sata_reg_addr);
extern u32 SDSH_writesl(void * SATA_reg_base, void * sata_reg_addr, u32*pbuf, u32 count);
extern u32 SDSH_readsl(void* SATA_reg_base, void * sata_reg_addr, u32 *pbuf, u32 count);
extern u32 SDSH_writesb(void * SATA_reg_base, void * sata_reg_addr, u8* pbuf , u32 count);
extern u32 SDSH_readsb(void* SATA_reg_base, void * sata_reg_addr, u8 *pbuf, u32 count);
extern u32 SDSH_writesw(void * SATA_reg_base, void * sata_reg_addr, u16* pbuf , u32 count);
extern u32 SDSH_readsw(void* SATA_reg_base, void * sata_reg_addr, u16* pbuf, u32 count);


u32 SDSH_writel(void * SATA_reg_base, void * sata_reg_addr, u32 value)
{
	u32 regVal = 0;

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	writel(value, SATA_reg_base + 8);

	if(((u32)sata_reg_addr)&VIXS_SATA_HOST_SEL_MASK) //Second Host
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x80220000;        
	}
	else
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x80210000;
	}

	writel(regVal, SATA_reg_base + 0);

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(regVal&0x2) { return 0;};
	return 0;
}

    
u32 SDSH_readl(void* SATA_reg_base, void * sata_reg_addr)
{
	u32 regVal = 0;
		
	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(((u32)sata_reg_addr)&VIXS_SATA_HOST_SEL_MASK) //Second Host
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x00220000;
	}
	else
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x00210000;
	}

	writel(regVal, SATA_reg_base + 0);

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(regVal&0x2) 
	{
		return 0;
	}

	regVal = readl(SATA_reg_base + 0xc);

	return regVal;
}

u32 SDSH_writeb(void* SATA_reg_base, void * sata_reg_addr, u8 value)
{
	u32 regVal = 0;

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	writel(value, SATA_reg_base + 8);

	if(((u32)sata_reg_addr)&VIXS_SATA_HOST_SEL_MASK) //Second Host
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x80020000;        
	}
	else
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x80010000;
	}

	writel(regVal, SATA_reg_base + 0);

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(regVal&0x2) { return 0;};

	return 0;
}

u8 SDSH_readb(void* SATA_reg_base, void * sata_reg_addr)
{
	u32 regVal = 0;

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(((u32)sata_reg_addr)&VIXS_SATA_HOST_SEL_MASK) //Second Host
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x00020000;
	}
	else
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x00010000;
	}

	writel(regVal, SATA_reg_base + 0);

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(regVal&0x2) 
	{
		return 0;
	}

	regVal = (u8)readl(SATA_reg_base + 0xc);

	return (u8)regVal;
}


u32 SDSH_writew(void* SATA_reg_base, void * sata_reg_addr, u16 value)
{
    u32 regVal = 0;

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	writel(value, SATA_reg_base + 8);

	if(((u32)sata_reg_addr)&VIXS_SATA_HOST_SEL_MASK) //Second Host
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x80120000;        
	}
	else
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x80110000;
	}

	writel(regVal, SATA_reg_base + 0);

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(regVal&0x2) { return 0;};

	return 0;
}


u16 SDSH_readw(void* SATA_reg_base, void * sata_reg_addr)
{
	u32 regVal = 0;

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	regVal = ((u32)sata_reg_addr)|0x00110000;
	if(((u32)sata_reg_addr)&VIXS_SATA_HOST_SEL_MASK) //Second Host
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x00120000;
	}
	else
	{
		regVal = ((u32)sata_reg_addr&0xffff)|0x00110000;
	}

	writel(regVal, SATA_reg_base + 0);

	while(((regVal = readl(SATA_reg_base + 4))&0x1)== 1){};

	if(regVal&0x2) 
	{
		return 0;
	}

	regVal = (u16)readl(SATA_reg_base + 0xc);

	return (u16)regVal;
}



u32 SDSH_writesl(void * SATA_reg_base, void * sata_reg_addr, u32*pbuf, u32 count)
{
    u32 i;
    u32 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        SDSH_writel(SATA_reg_base, sata_reg_addr, (u32)*p);
        p++;
    }

    return 0;
}

u32 SDSH_readsl(void* SATA_reg_base, void * sata_reg_addr, u32 *pbuf, u32 count)
{
    u32 i;
    u32 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        *p = SDSH_readl(SATA_reg_base, sata_reg_addr);
        p++;
    }

    return 0;
}
    
u32 SDSH_writesb(void * SATA_reg_base, void * sata_reg_addr, u8* pbuf , u32 count)
{
    u32 i;
    u8 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        SDSH_writeb(SATA_reg_base, sata_reg_addr, (u8)*p);
        p++;
    }

    return 0;
}
    
u32 SDSH_readsb(void* SATA_reg_base, void * sata_reg_addr, u8 *pbuf, u32 count)
{
    u32 i;
    u8 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        *p = SDSH_readb(SATA_reg_base, sata_reg_addr);
        p++;
    }

    return 0;
}    

u32 SDSH_writesw(void * SATA_reg_base, void * sata_reg_addr, u16* pbuf , u32 count)
{
    u32 i;
    u16 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        SDSH_writew(SATA_reg_base, sata_reg_addr, (u16)*p);
        p++;
    }

    return 0;
}
    
u32 SDSH_readsw(void* SATA_reg_base, void * sata_reg_addr, u16* pbuf, u32 count)
{
    u32 i;
    u16 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        *p = SDSH_readw(SATA_reg_base, sata_reg_addr);
        p++;
    }

    return 0;
}



#define SATA_readb(a)	    SDSH_readb(SDSH_SATA_REG_BASE, (void*)a)
#define SATA_readw(a)	    SDSH_readw(SDSH_SATA_REG_BASE, (void*)a)
#define SATA_readl(a)	    SDSH_readl(SDSH_SATA_REG_BASE, (void*)a)
#define SATA_writeb(a,b)	    SDSH_writeb(SDSH_SATA_REG_BASE, (void*)b,a)
#define SATA_writew(a,b)	    SDSH_writew(SDSH_SATA_REG_BASE, (void*)b,a)
#define SATA_writel(a,b)	    SDSH_writel(SDSH_SATA_REG_BASE, (void*)b,a)

#endif


static int __ilog2(unsigned int x)
{
    int i = 0;

    while(x = (x>>1))
    {
        i++;
    }

    return i;
}

struct ahci_probe_ent *probe_ent[VIXS_SATA_MAX_HOST] = {NULL, NULL};
hd_driveid_t ataid[VIXS_SATA_MAX_HOST][AHCI_MAX_PORTS];

#define SATA_writel_with_flush(a,b)	do { SATA_writel(a,b); SATA_readl(b); } while (0)


//#define cpu_to_le32(x) ((__u32)(x))
//#define be32_to_cpu(x) ( (((x)&0xff)<<24)|(((x)&0xff00)<<8)|(((x)&0xff0000)>>8)|(((x)&0xff000000)>>24))
//#define be16_to_cpu(x) ( (((x)&0xff)<<8)|(((x)>>8)&0xff))


#ifndef MAX_SATA_BLOCKS_READ_WRITE
#define MAX_SATA_BLOCKS_READ_WRITE      0x80
#endif


static inline u32 ahci_port_base(u32 base, u32 port)
{
	return base + 0x100 + (port * 0x80);
}


static void ahci_setup_port(struct ahci_ioports *port, unsigned long base,
			    unsigned int port_idx)
{
	base = ahci_port_base(base, port_idx);

	port->cmd_addr = base;
	port->scr_addr = base + PORT_SCR;
}


#define msleep(a) udelay(a * 1000)
#define ssleep(a) msleep(a * 1000)

static int waiting_for_cmd_completed(volatile u8 *offset,
				     int timeout_msec,
				     u32 sign)
{
	int i;
	u32 status;

	for (i = 0; ((status = SATA_readl(offset)) & sign) && i < timeout_msec; i++)
		msleep(1);

	return (i < timeout_msec) ? 0 : -1;
}

#ifdef PLATFORM_XCODE
static void xcode_ahci_host_init(void )
{
	u32 temp;

	//Enable SATA clock
	temp = readl(XC_SOC_PROC_MMREG_BASE+ CG1_CLK_STOP0);
	temp &= ~SATACLK_STOP_MASK;
	writel(temp, XC_SOC_PROC_MMREG_BASE+ CG1_CLK_STOP0);

	// enable clk for low power mode 
	temp = readl(XC_SOC_PROC_MMREG_BASE+ CG1_BLK_CLK_STOP0);
	temp &= ~SATA_MCLK_STOP_MASK;
	writel(temp, XC_SOC_PROC_MMREG_BASE+ CG1_BLK_CLK_STOP0);

	/* Reset SATA */
	temp = readl(XC_SOC_PROC_MMREG_BASE+ CG_RESET_REG);
	temp |= SATA_RESET_MASK;
	writel(temp, XC_SOC_PROC_MMREG_BASE+ CG_RESET_REG);

	/* Take SATA out of reset */
	temp = readl(XC_SOC_PROC_MMREG_BASE+ CG_RESET_REG);
	temp &= ~SATA_RESET_MASK;
	writel(temp, XC_SOC_PROC_MMREG_BASE+ CG_RESET_REG);

	udelay(1000);

	temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_CONTROL);
//	temp &= ~SATA0_AHBM_MODE_MASK;
	temp &= ~(SATA0_LOCAL_CLK_GATE_MASK|SATA1_LOCAL_CLK_GATE_MASK);
	writel(temp, XC_SOC_PROC_MMREG_BASE+ SATA_CONTROL);

	writel(0, XC_SOC_PROC_MMREG_BASE + SATA_SFT_RESETS);

	//set mpll frequency
	temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);
	temp &= ~0x7f6;
	if(g_xcode_ahci_clk == 125)
	{
		printf("XCode AHCI clock 125MHZ\n");
		temp |= 0x54;
	}
	else if(g_xcode_ahci_clk == 50)
	{
		printf("XCode AHCI clock 50MHZ\n");
		temp |= 0x460;
	}
	else
	{
		printf("NOT supported AHCI clock %d MHZ, set to default 50MHZ\n", g_xcode_ahci_clk);
		temp |= 0x460;
	}

	if(g_xcode_ahci_ss_mode)
	{
		temp |= SATA_PHY_MPLL_SS_EN_MASK;
	}
	else
	{
		temp &= ~SATA_PHY_MPLL_SS_EN_MASK;	
	}
	writel(temp, XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);

	// limit speed?
	if(g_sata0_limit_speed == 1) { // limit speed to Gen1
		temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_CONTROL);
		temp &= ~SATA0_PHY_SPDMODE_MASK;
		writel(temp, XC_SOC_PROC_MMREG_BASE + SATA_CONTROL);
	}
	if(g_sata1_limit_speed == 1) {
		temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_CONTROL);
		temp &= ~SATA1_PHY_SPDMODE_MASK;
		writel(temp, XC_SOC_PROC_MMREG_BASE + SATA_CONTROL);
	}

	//set to external clock frequency
	temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);
	temp &= ~0x2000;
	writel(temp, XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);

	//trun off mp_ck
	temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);
	temp |= 0x1000;
	writel(temp, XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);

	//trun on mp_ck
	temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);
	temp &= ~0x1000;
	writel(temp, XC_SOC_PROC_MMREG_BASE + SATA_PHY_MPLL_CTRL);

	temp = readl(XC_SOC_PROC_MMREG_BASE + SATA_PHY_ANALOG_CTRL);
	temp &= ~0x1fffff;
	temp |= 0x911;
	temp = (temp & ~SATA_PHY_TX_LVL_MASK) | (0x8 << SATA_PHY_TX_LVL_SHIFT);
	writel(temp, XC_SOC_PROC_MMREG_BASE+ SATA_PHY_ANALOG_CTRL);

	writel(0x23354030 , SATA0_PHY_TXRX_CTRL + XC_SOC_PROC_MMREG_BASE);
	writel(0x23354030 , SATA1_PHY_TXRX_CTRL + XC_SOC_PROC_MMREG_BASE);

	writel(0xff, XC_SOC_PROC_MMREG_BASE + SATA_SFT_RESETS);
}

static void xcode_ahci_set_oob(void  *mmio, u32 clock)
{
        //set up OOB register here for CLK setting

        switch(clock)
        {
            case 25:
            case 100:
            case 50:
                SATA_writel(0x82060b13, mmio + HOST_OOB);
                SATA_writel(0x82060b13, mmio + HOST_OOB);
                break;
            case 75:
                SATA_writel(0x840a111e, mmio + HOST_OOB);                
                SATA_writel(0x840a111e, mmio + HOST_OOB);                
                break;
        }

}

#endif

static void sata_config(void)
{
	if(getenv("sata_clkfreq"))
	{
		printf("Clock Freq is %sMHZ\n", getenv("sata_clkfreq"));
		if(strcmp(getenv("sata_clkfreq"), "125") == 0)
		{
			g_xcode_ahci_clk = 125;
		}
		else if(strcmp(getenv("sata_clkfreq"), "50") == 0)
		{
			g_xcode_ahci_clk = 50;
		}
		else
		{
			printf("Not supported SATA clock, set to default 125MHZ\n");
			g_xcode_ahci_clk = 125;  
		}
	}
	else
	{
		printf("Can't find ENV sata_clkfreq, set it to default clock 50MHZ\n");
		g_xcode_ahci_clk = 50;
	}

	if(getenv("sata_ssmode"))
	{
		if(strcmp(getenv("sata_ssmode"), "0") == 0)
		{
			printf("Disable SATA SS mode\n");
			g_xcode_ahci_ss_mode = 0;
		}
		else
		{
			printf("Enable SATA SS mode\n");
			g_xcode_ahci_ss_mode = 1;
		}
	}
	else
	{
		printf("Can't find ENV sata_ssmode, set it to default: disabled\n");
		g_xcode_ahci_ss_mode = 0;
	}

	if(getenv("sata0_limit_speed"))
	{
		/* Limit SATA0 speed to Generation1 */
		if(strcmp(getenv("sata0_limit_speed"), "1") == 0)
			g_sata0_limit_speed = 1;
	}
	if(getenv("sata1_limit_speed"))
	{
		/* Limit SATA1 speed to Generation1 */
		if(strcmp(getenv("sata1_limit_speed"), "1") == 0)
			g_sata1_limit_speed = 1;
	}

}

static int ahci_host_init(struct ahci_probe_ent *pdev)
{

	volatile u8 *mmio = (volatile u8 *)pdev->mmio_base;
	u32 tmp, cap_save;
	int i, j;
	volatile u8 *port_mmio;

	if(!g_sata_hw_inited) {
		sata_config();
		xcode_ahci_host_init();
		g_sata_hw_inited = 1;
	}

	cap_save = SATA_readl(mmio + HOST_CAP);
	cap_save &= ((1 << 28) | (1 << 17));
	cap_save |= (1 << 27);

	/* global controller reset */
	tmp = SATA_readl(mmio + HOST_CTL);
	if ((tmp & HOST_RESET) == 0)
		SATA_writel_with_flush(tmp | HOST_RESET, mmio + HOST_CTL);

	/* reset must complete within 1 second, or
	 * the hardware should be considered fried.
	 */
	ssleep(1);

	tmp = SATA_readl(mmio + HOST_CTL);
	if (tmp & HOST_RESET) {
		debug("controller reset failed (0x%x)\n", tmp);
		return -1;
	}

	SATA_writel_with_flush(HOST_AHCI_EN, mmio + HOST_CTL);
	SATA_writel(cap_save, mmio + HOST_CAP);
	SATA_writel_with_flush(0xf, mmio + HOST_PORTS_IMPL);

	xcode_ahci_set_oob((void *)mmio, g_xcode_ahci_clk);

	pdev->cap = SATA_readl(mmio + HOST_CAP);
	pdev->port_map = SATA_readl(mmio + HOST_PORTS_IMPL);
	pdev->n_ports = (pdev->cap & 0x1f) + 1;

	debug("cap 0x%x  port_map 0x%x  n_ports %d\n",
	      pdev->cap, pdev->port_map, pdev->n_ports);

	for (i = 0; i < pdev->n_ports; i++) {
		pdev->port[i].port_mmio = ahci_port_base((u32) mmio, i);
		port_mmio = (u8 *)pdev->port[i].port_mmio;
		ahci_setup_port(&pdev->port[i], (unsigned long)mmio, i);

		/* make sure port is not active */
		tmp = SATA_readl(port_mmio + PORT_CMD);
		if (tmp & (PORT_CMD_LIST_ON | PORT_CMD_FIS_ON |
			   PORT_CMD_FIS_RX | PORT_CMD_START)) {
			tmp &= ~(PORT_CMD_LIST_ON | PORT_CMD_FIS_ON |
				 PORT_CMD_FIS_RX | PORT_CMD_START);
			SATA_writel_with_flush(tmp, port_mmio + PORT_CMD);

			/* spec says 500 msecs for each bit, so
			 * this is slightly incorrect.
			 */
			msleep(500);
		}

		// starts COMRESET (SPIN_UP bit is read/write when staggered spin-up is supported)
		SATA_writel(PORT_CMD_SPIN_UP, port_mmio + PORT_CMD);

		j = 0;
		while (j < 100) {
			msleep(1);
			tmp = SATA_readl(port_mmio + PORT_SCR_STAT);
			if ((tmp & 0xf) == 0x3) // device presence detected and PHY communication established
				break;
			j++;
		}

		tmp = SATA_readl(port_mmio + PORT_SCR_ERR);
		debug("PORT_SCR_ERR 0x%x\n", tmp);
		SATA_writel(tmp, port_mmio + PORT_SCR_ERR);

		/* ack any pending irq events for this port */
		tmp = SATA_readl(port_mmio + PORT_IRQ_STAT);
		debug("PORT_IRQ_STAT 0x%x\n", tmp);
		if (tmp)
			SATA_writel(tmp, port_mmio + PORT_IRQ_STAT);

		SATA_writel(1 << i, mmio + HOST_IRQ_STAT);

		/* set irq mask (enables interrupts) */
		SATA_writel(DEF_PORT_IRQ, port_mmio + PORT_IRQ_MASK);

		/*register linkup ports */
		tmp = SATA_readl(port_mmio + PORT_SCR_STAT);
		debug("Port %d status: 0x%x\n", i, tmp);
		if ((tmp & 0xf) == 0x03)
			pdev->link_port_map |= (0x01 << i);
	}

	if(pdev->link_port_map == 0){
		printf("No device detected on host %d\n", pdev->dev);
		return 1;
	}

	tmp = SATA_readl(mmio + HOST_CTL);
	debug("HOST_CTL 0x%x\n", tmp);
	SATA_writel(tmp | HOST_IRQ_EN, mmio + HOST_CTL);
	tmp = SATA_readl(mmio + HOST_CTL);
	debug("HOST_CTL 0x%x\n", tmp);

	return 0;
}


static void ahci_print_info(struct ahci_probe_ent *pdev)
{
	volatile u8 *mmio = (volatile u8 *)pdev->mmio_base;
	u32 vers, cap, impl, speed;
	const char *speed_s;
	const char *scc_s;

	vers = SATA_readl(mmio + HOST_VERSION);
	cap = pdev->cap;
	impl = pdev->port_map;

	speed = (cap >> 20) & 0xf;
	if (speed == 1)
		speed_s = "1.5";
	else if (speed == 2)
		speed_s = "3";
	else
		speed_s = "?";


	printf("AHCI %02x%02x.%02x%02x "
	       "%u slots %u ports %s Gbps 0x%x impl %s mode\n",
	       (vers >> 24) & 0xff,
	       (vers >> 16) & 0xff,
	       (vers >> 8) & 0xff,
	       vers & 0xff,
	       ((cap >> 8) & 0x1f) + 1, (cap & 0x1f) + 1, speed_s, impl, scc_s);
#if 0
	printf("flags: "
	       "%s%s%s%s%s%s"
	       "%s%s%s%s%s%s%s\n",
	       cap & (1 << 31) ? "64bit " : "",
	       cap & (1 << 30) ? "ncq " : "",
	       cap & (1 << 28) ? "ilck " : "",
	       cap & (1 << 27) ? "stag " : "",
	       cap & (1 << 26) ? "pm " : "",
	       cap & (1 << 25) ? "led " : "",
	       cap & (1 << 24) ? "clo " : "",
	       cap & (1 << 19) ? "nz " : "",
	       cap & (1 << 18) ? "only " : "",
	       cap & (1 << 17) ? "pmp " : "",
	       cap & (1 << 15) ? "pio " : "",
	       cap & (1 << 14) ? "slum " : "",
	       cap & (1 << 13) ? "part " : "");
#endif
}

static int ahci_init_one(pci_dev_t dev)
{
	int rc;

	memset((void *)ataid[dev], 0, sizeof(hd_driveid_t *) * AHCI_MAX_PORTS);

	probe_ent[dev] = malloc(sizeof(struct ahci_probe_ent));
	memset(probe_ent[dev], 0, sizeof(struct ahci_probe_ent));
	probe_ent[dev]->dev = dev;

	probe_ent[dev]->host_flags = ATA_FLAG_SATA
				| ATA_FLAG_NO_LEGACY
				| ATA_FLAG_MMIO
				| ATA_FLAG_PIO_DMA
				| ATA_FLAG_NO_ATAPI;
	probe_ent[dev]->pio_mask = 0x1f;
	probe_ent[dev]->udma_mask = 0x7f;	/*Fixme,assume to support UDMA6 */
	if(dev == 0) // host0
		probe_ent[dev]->mmio_base = 0;
	else // host1
		probe_ent[dev]->mmio_base = VIXS_SATA_HOST_SEL_MASK;

	/* initialize adapter */
	rc = ahci_host_init(probe_ent[dev]);
	if (rc)
		return rc;

	ahci_print_info(probe_ent[dev]);
	return 0;
}


#define MAX_DATA_BYTE_COUNT  (4*1024*1024)

static int ahci_fill_sg(u8 host, u8 port, unsigned char *buf, int buf_len)
{
	struct ahci_ioports *pp = &(probe_ent[host]->port[port]);
	struct ahci_sg *ahci_sg = pp->cmd_tbl_sg;
	u32 sg_count;
	int i;

	sg_count = ((buf_len - 1) / MAX_DATA_BYTE_COUNT) + 1;
	if (sg_count > AHCI_MAX_SG) {
		printf("Error:Too much sg!\n");
		return -1;
	}

	for (i = 0; i < sg_count; i++) {
		ahci_sg->addr =
		    cpu_to_le32((u32) buf + i * MAX_DATA_BYTE_COUNT);    //Use Physicaladdress, that's why mask using 0x3fffffff
		ahci_sg->addr_hi = 0;
		ahci_sg->flags_size = cpu_to_le32(
					  (buf_len < MAX_DATA_BYTE_COUNT
					   ? (buf_len - 1)
					   : (MAX_DATA_BYTE_COUNT - 1)));
		ahci_sg++;
		buf_len -= MAX_DATA_BYTE_COUNT;
	}

	return sg_count;
}


static void ahci_fill_cmd_slot(struct ahci_ioports *pp, u32 opts)
{
	pp->cmd_slot->opts = cpu_to_le32(opts);
	pp->cmd_slot->status = 0;
	pp->cmd_slot->tbl_addr = cpu_to_le32(pp->cmd_tbl);
	pp->cmd_slot->tbl_addr_hi = 0;
}


static void ahci_set_feature(u8 host, u8 port)
{
	struct ahci_ioports *pp = &(probe_ent[host]->port[port]);
	volatile u8 *port_mmio = (volatile u8 *)pp->port_mmio;
	u32 cmd_fis_len = 5;	/* five dwords */
	u8 fis[20];

	/*set feature */
	memset(fis, 0, 20);
	fis[0] = 0x27;
	fis[1] = 1 << 7;
	fis[2] = ATA_CMD_SETF;
	fis[3] = SETFEATURES_XFER;
//	fis[12] = __ilog2(probe_ent->udma_mask + 1) + 0x40 - 0x01;
        fis[12] = 0x6 + 0x40 - 0x01;        //hardcode it to UDMA 5 for now

	memcpy((unsigned char *)pp->cmd_tbl, fis, 20);
	ahci_fill_cmd_slot(pp, cmd_fis_len);

        flush_dcache_all(); //flush dcache here since every command will need to call this func
        
	SATA_writel(1, port_mmio + PORT_CMD_ISSUE);
	SATA_readl(port_mmio + PORT_CMD_ISSUE);
	if (waiting_for_cmd_completed(port_mmio + PORT_CMD_ISSUE, 15000, 0x1)) {
		printf("set feature error!\n");
	}
}


static int ahci_port_start(u8 host, u8 port)
{
	struct ahci_ioports *pp = &(probe_ent[host]->port[port]);
	volatile u8 *port_mmio = (volatile u8 *)pp->port_mmio;
	u32 port_status;
	u32 mem;

	debug("Enter start port: %d port_mmio is %x\n", port, port_mmio);
	port_status = SATA_readl(port_mmio + PORT_SCR_STAT);
	debug("Port %d status: %x\n", port, port_status);
	if ((port_status & 0xf) != 0x03) {
		printf("No Link on this port!\n");
		return -1;
	}

	mem = (u32) malloc(AHCI_PORT_PRIV_DMA_SZ + 2048);
	if (!mem) {
		free(pp);
		printf("No mem for table!\n");
		return -ENOMEM;
	}

       //Use uncached memory here
	mem = ((mem + 0x800) & (~0x7ff));	/* Aligned to 2048-bytes */
	memset((u8 *) mem, 0, AHCI_PORT_PRIV_DMA_SZ);

	/*
	 * First item in chunk of DMA memory: 32-slot command table,
	 * 32 bytes each in size
	 */
	pp->cmd_slot = (struct ahci_cmd_hdr *)mem;
	debug("cmd_slot = 0x%x\n", pp->cmd_slot);
	mem += (AHCI_CMD_SLOT_SZ + 224);

	/*
	 * Second item: Received-FIS area
	 */
	pp->rx_fis = mem;
	mem += AHCI_RX_FIS_SZ;

	/*
	 * Third item: data area for storing a single command
	 * and its scatter-gather table
	 */
	pp->cmd_tbl = mem;
	debug("cmd_tbl_dma = 0x%x\n", pp->cmd_tbl);

	mem += AHCI_CMD_TBL_HDR;
	pp->cmd_tbl_sg = (struct ahci_sg *)mem;

        //[jlwang] It has to be Physical Address for SATA Host, so here I unmask the top 2 bits
	SATA_writel_with_flush(((u32) pp->cmd_slot)&0x3FFFFFFF, port_mmio + PORT_LST_ADDR);

	SATA_writel_with_flush((pp->rx_fis)&0x3FFFFFFF, port_mmio + PORT_FIS_ADDR);

	SATA_writel_with_flush(PORT_CMD_ICC_ACTIVE | PORT_CMD_FIS_RX |
			  PORT_CMD_POWER_ON | PORT_CMD_SPIN_UP |
			  PORT_CMD_START, port_mmio + PORT_CMD);

	debug("Exit start port %d\n", port);

	return 0;
}


static int get_ahci_device_data(u8 host, u8 port, u8 *fis, int fis_len, u8 *buf,
				int buf_len)
{

	struct ahci_ioports *pp = &(probe_ent[host]->port[port]);
	volatile u8 *port_mmio = (volatile u8 *)pp->port_mmio;
	u32 opts;
	u32 port_status;
	int sg_count;

	debug("Enter get_ahci_device_data: for port %d\n", port);

	if (port > probe_ent[host]->n_ports) {
		printf("Invaild port number %d\n", port);
		return -1;
	}

	port_status = SATA_readl(port_mmio + PORT_SCR_STAT);
	if ((port_status & 0xf) != 0x03) {
		debug("No Link on port %d!\n", port);
		return -1;
	}

	memcpy((unsigned char *)pp->cmd_tbl, fis, fis_len);

	sg_count = ahci_fill_sg(host, port, buf, buf_len);
	opts = (fis_len >> 2) | (sg_count << 16);
	ahci_fill_cmd_slot(pp, opts);

       flush_dcache_all(); //flush dcache here since every command will need to call this func

	SATA_writel_with_flush(1, port_mmio + PORT_CMD_ISSUE);

	if (waiting_for_cmd_completed(port_mmio + PORT_CMD_ISSUE, 1500000, 0x1)) {
		printf("timeout exit!\n");
		return -1;
	}

      inv_dcache_all(); //flush dcache here since every command will need to call this func

      
	debug("get_ahci_device_data: %d byte transferred.\n",
	      pp->cmd_slot->status);

	return 0;
}


static int ahci_device_data_io(u8 host, u8 port, u8 *fis, int fis_len, u8 *buf,
                                int buf_len, u8 is_write)
{

	struct ahci_ioports *pp = &(probe_ent[host]->port[port]);
	int i;
	volatile u8 *port_mmio = (volatile u8 *)pp->port_mmio;
	u32 opts;
	u32 port_status;
	int sg_count;

	debug("Enter %s: for port %d\n", __func__, port);

	udelay(100);

	if (port > probe_ent[host]->n_ports) {
		printf("Invaild port number %d\n", port);
		return -1;
	}

	port_status = SATA_readl(port_mmio + PORT_SCR_STAT);
	if ((port_status & 0xf) != 0x03) {
		debug("No Link on port %d!\n", port);
		return -1;
	}

	memcpy((unsigned char *)pp->cmd_tbl, fis, fis_len);

	sg_count = ahci_fill_sg(host, port, buf, buf_len);

	opts = (fis_len >> 2) | (sg_count << 16) | (is_write << 6);
	ahci_fill_cmd_slot(pp, opts);

	flush_dcache_all(); //flush dcache here since every command will need to call this func

	SATA_writel_with_flush(1, port_mmio + PORT_CMD_ISSUE);

	if (waiting_for_cmd_completed(port_mmio + PORT_CMD_ISSUE, 150, 0x1)) {
		printf("timeout exit!\n");
		return -1;
	}
	inv_dcache_all();
	debug("%s: %d byte transferred.\n", __func__, pp->cmd_slot->status);

	return 0;
}


static char *ata_id_strcpy(u16 *target, u16 *src, int len)
{
	int i;
	for (i = 0; i < len / 2; i++)
		target[i] = be16_to_cpu(src[i]);
	return (char *)target;
}


static void dump_ataid(hd_driveid_t *ataid)
{    
	debug("(49)ataid->capability = 0x%x\n", ataid->capability);
	debug("(53)ataid->field_valid =0x%x\n", ataid->field_valid);
	debug("(63)ataid->dma_mword = 0x%x\n", ataid->dma_mword);
	debug("(64)ataid->eide_pio_modes = 0x%x\n", ataid->eide_pio_modes);
	debug("(75)ataid->queue_depth = 0x%x\n", ataid->queue_depth);
	debug("(80)ataid->major_rev_num = 0x%x\n", ataid->major_rev_num);
	debug("(81)ataid->minor_rev_num = 0x%x\n", ataid->minor_rev_num);
	debug("(82)ataid->command_set_1 = 0x%x\n", ataid->command_set_1);
	debug("(83)ataid->command_set_2 = 0x%x\n", ataid->command_set_2);
	debug("(84)ataid->cfsse = 0x%x\n", ataid->cfsse);
	debug("(85)ataid->cfs_enable_1 = 0x%x\n", ataid->cfs_enable_1);
	debug("(86)ataid->cfs_enable_2 = 0x%x\n", ataid->cfs_enable_2);
	debug("(87)ataid->csf_default = 0x%x\n", ataid->csf_default);
	debug("(88)ataid->dma_ultra = 0x%x\n", ataid->dma_ultra);
	debug("(93)ataid->hw_config = 0x%x\n", ataid->hw_config);
}


/*
 * SCSI INQUIRY command operation.
 */
static int ata_scsiop_inquiry(ccb *pccb)
{
	u8 hdr[] = {
		0,
		0,
		0x5,		/* claim SPC-3 version compatibility */
		2,
		95 - 4,
	};
	u8 fis[20];
	u8 port;	
	unsigned char host = pccb->lun;

	/* Clean ccb data buffer */
	memset(pccb->pdata, 0, pccb->datalen);

	memcpy(pccb->pdata, hdr, sizeof(hdr));

	if (pccb->datalen <= 35)
		return 0;

	memset(fis, 0, 20);
	/* Construct the FIS */
	fis[0] = 0x27;		/* Host to device FIS. */
	fis[1] = 1 << 7;	/* Command FIS. */
	fis[2] = ATA_CMD_IDENT;	/* Command byte. */

	/* Read id from sata */
	port = pccb->target;

	if (get_ahci_device_data(host, port, (u8 *) &fis, 20,
				 (u8 *)&ataid[host][port], sizeof(hd_driveid_t))) {
		debug("scsi_ahci: SCSI inquiry command failure.\n");
		return -EIO;
	}

	memcpy(&pccb->pdata[8], "ATA     ", 8);
	ata_id_strcpy((u16 *) &pccb->pdata[16], (u16 *)(&ataid[host][port].model), 16);
	ata_id_strcpy((u16 *) &pccb->pdata[32], (u16 *)(&ataid[host][port].fw_rev), 4);

    //JLWANG: There is some SATA devices which doesn't support ULTRADMA 6
    //              1. identify device to determine Ultradma mode 2. set feature using correct ultra dma mode 3. identify device ...
    //copy dma_ultra (WORD 88) 

	ata_id_strcpy((u16 *) &pccb->pdata[88*2], (u16 *)(&ataid[host][port].dma_ultra), 4);

	dump_ataid(&ataid[host][port]);
	return 0;
}


/*
 * SCSI READ10 command operation.
 */
static int ata_scsiop_read10(ccb * pccb)
{
	u64 lba = 0;
	u32 len = 0;
	u8 fis[20];

	lba = (((u64) pccb->cmd[2]) << 24) | (((u64) pccb->cmd[3]) << 16)
	    | (((u64) pccb->cmd[4]) << 8) | ((u64) pccb->cmd[5]);
	len = (((u32) pccb->cmd[7]) << 8) | ((u32) pccb->cmd[8]);

	/* For 10-byte and 16-byte SCSI R/W commands, transfer
	 * length 0 means transfer 0 block of data.
	 * However, for ATA R/W commands, sector count 0 means
	 * 256 or 65536 sectors, not 0 sectors as in SCSI.
	 *
	 * WARNING: one or two older ATA drives treat 0 as 0...
	 */
	if (!len)
		return 0;
	memset(fis, 0, 20);

	/* Construct the FIS */
	fis[0] = 0x27;		/* Host to device FIS. */
	fis[1] = 1 << 7;	/* Command FIS. */
	fis[2] = ATA_CMD_RD_DMA;	/* Command byte. */

	/* LBA address, only support LBA28 in this driver */
	fis[4] = pccb->cmd[5];
	fis[5] = pccb->cmd[4];
	fis[6] = pccb->cmd[3];
	fis[7] = (pccb->cmd[2] & 0x0f) | 0xe0;

	/* Sector Count */
	fis[12] = pccb->cmd[8];
	fis[13] = pccb->cmd[7];

	/* Read from ahci */
	if (get_ahci_device_data(pccb->lun, pccb->target, (u8 *) & fis, 20,
				 pccb->pdata, pccb->datalen)) {
		debug("scsi_ahci: SCSI READ10 command failure.\n");
		return -EIO;
	}

	return 0;
}



/*
 * SCSI READ10/WRITE10 command operation.
 */
static int ata_scsiop_read_write(ccb *pccb, u8 is_write)
{
	u32 lba = 0;
	u16 blocks = 0;
	u8 fis[20];
	u8 *user_buffer = pccb->pdata;
	u32 user_buffer_size = pccb->datalen;
	unsigned char host = pccb->lun;
	int i;

	/* Retrieve the base LBA number from the ccb structure. */
	//memcpy(&lba, pccb->cmd + 2, sizeof(lba));
	//lba = be32_to_cpu(lba);

	lba = (((u64) pccb->cmd[2]) << 24) | (((u64) pccb->cmd[3]) << 16)
	    | (((u64) pccb->cmd[4]) << 8) | ((u64) pccb->cmd[5]);

	/*
	 * And the number of blocks.
	 *
	 * For 10-byte and 16-byte SCSI R/W commands, transfer
	 * length 0 means transfer 0 block of data.
	 * However, for ATA R/W commands, sector count 0 means
	 * 256 or 65536 sectors, not 0 sectors as in SCSI.
	 *
	 * WARNING: one or two older ATA drives treat 0 as 0...
	 */
	blocks = (((u16)pccb->cmd[7]) << 8) | ((u16) pccb->cmd[8]);

	debug("scsi_ahci: %s %d blocks starting from lba 0x%x\n",
			is_write ?  "write" : "read", (unsigned)lba, blocks);

	/* Preset the FIS */
	memset(fis, 0, 20);
	fis[0] = 0x27;           /* Host to device FIS. */
	fis[1] = 1 << 7;         /* Command FIS. */
	/* Command byte (read/write). */
	fis[2] = is_write ? ATA_CMD_WR_DMA : ATA_CMD_RD_DMA;

	while (blocks) {
		u16 now_blocks; /* number of blocks per iteration */
		u32 transfer_size; /* number of bytes per iteration */

		now_blocks = min(MAX_SATA_BLOCKS_READ_WRITE, blocks);

		transfer_size = ATA_BLOCKSIZE * now_blocks;
		if (transfer_size > user_buffer_size) {
			printf("scsi_ahci: Error: buffer too small.\n");
			return -EIO;
		}

#if	0
		/* LBA address, only support LBA28 in this driver */
		fis[4] = (lba >> 0) & 0xff;
		fis[5] = (lba >> 8) & 0xff;
		fis[6] = (lba >> 16) & 0xff;
		fis[7] = ((lba >> 24) & 0xf) | 0xe0;

		/* Block (sector) count */
		fis[12] = (now_blocks >> 0) & 0xff;
		fis[13] = (now_blocks >> 8) & 0xff;

#endif

		/* LBA address, only support LBA28 in this driver */
		fis[4] = pccb->cmd[5];
		fis[5] = pccb->cmd[4];
		fis[6] = pccb->cmd[3];
		fis[7] = (pccb->cmd[2] & 0x0f) | 0xe0;

		/* Sector Count */
		fis[12] = pccb->cmd[8];
		fis[13] = pccb->cmd[7];


		/* Read/Write from ahci */
		if (ahci_device_data_io(host, pccb->target, (u8 *) &fis, sizeof(fis),
								user_buffer, user_buffer_size, 
								is_write)) {
			debug("scsi_ahci: SCSI %s10 command failure.\n",
					is_write ? "WRITE" : "READ");
			return -EIO;
		}
		user_buffer += transfer_size;
		user_buffer_size -= transfer_size;
		blocks -= now_blocks;
		lba += now_blocks;
	}

	return 0;
}



/*
 * SCSI SECURITY command operation.
 */
static int ata_scsiop_security(ccb *pccb)
{
	u32 lba = 0;
	u16 blocks = 0;
	u8 fis[20];
	u8 *user_buffer = pccb->pdata;
	u32 user_buffer_size = pccb->datalen;
	unsigned char host = pccb->lun;


	/* Retrieve the base LBA number from the ccb structure. */
	//memcpy(&lba, pccb->cmd + 2, sizeof(lba));
	//lba = be32_to_cpu(lba);

	lba = (((u64) pccb->cmd[2]) << 24) | (((u64) pccb->cmd[3]) << 16)
	    | (((u64) pccb->cmd[4]) << 8) | ((u64) pccb->cmd[5]);

	/*
	 * And the number of blocks.
	 *
	 * For 10-byte and 16-byte SCSI R/W commands, transfer
	 * length 0 means transfer 0 block of data.
	 * However, for ATA R/W commands, sector count 0 means
	 * 256 or 65536 sectors, not 0 sectors as in SCSI.
	 *
	 * WARNING: one or two older ATA drives treat 0 as 0...
	 */
	blocks = (((u16)pccb->cmd[7]) << 8) | ((u16) pccb->cmd[8]);

	debug("scsi_ahci: %s %d blocks starting from lba 0x%x\n",
			1 ?  "write" : "read", (unsigned)lba, blocks);

	/* Preset the FIS */
	memset(fis, 0, 20);
	fis[0] = 0x27;           /* Host to device FIS. */
	fis[1] = 1 << 7;         /* Command FIS. */
	fis[2] = ATA_CMD_SECURITY_UNLOCK ;


	/* LBA address, only support LBA28 in this driver */
	fis[4] = pccb->cmd[5];
	fis[5] = pccb->cmd[4];
	fis[6] = pccb->cmd[3];
	fis[7] = (pccb->cmd[2] & 0x0f) | 0x40;

	/* Sector Count */
	fis[12] = pccb->cmd[8];
	fis[13] = pccb->cmd[7];

	fis[14] = 0;
	fis[15] = 0x08;


	/* Read/Write from ahci */
	if (ahci_device_data_io(host, pccb->target, (u8 *) &fis, sizeof(fis),
							user_buffer, user_buffer_size, 
							1)) {
		debug("scsi_ahci: SCSI %s10 command failure.\n",
				1 ? "WRITE" : "READ");
		return -EIO;
	}
	return 0;
}




/*
 * SCSI READ CAPACITY10 command operation.
 */
static int ata_scsiop_read_capacity10(ccb *pccb)
{
	u8 buf[8];
	u8 host = pccb->lun;

/* ToDo
	if (!ataid[pccb->target]) {
		printf("scsi_ahci: SCSI READ CAPACITY10 command failure. "
		       "\tNo ATA info!\n"
		       "\tPlease run SCSI commmand INQUIRY firstly!\n");
		return -EPERM;
	}
*/
	memset(buf, 0, 8);

	if(ataid[host][pccb->target].command_set_2 & 0x0400)//lba48 support
	{
		*(u32 *) buf = be32_to_cpu((ataid[host][pccb->target].lba48_capacity[1]<<16)|ataid[host][pccb->target].lba48_capacity[0]);
	}
	else
	{
		*(u32 *) buf = be32_to_cpu(ataid[host][pccb->target].lba_capacity);
	}
	buf[6] = 512 >> 8;
	buf[7] = 512 & 0xff;

	memcpy(pccb->pdata, buf, 8);

	return 0;
}


/*
 * SCSI TEST UNIT READY command operation.
 */
static int ata_scsiop_test_unit_ready(ccb *pccb)
{
	// ToDo
	return (ataid[pccb->target]) ? 0 : -EPERM;
}


int scsi_exec(ccb *pccb)
{
	int ret;

	switch (pccb->cmd[0]) {
	case SCSI_READ10:
		ret = ata_scsiop_read_write(pccb, 0);
		break;
	case SCSI_WRITE10:
		ret = ata_scsiop_read_write(pccb, 1);
		break;
	case SCSI_SECURITY:
		ret = ata_scsiop_security(pccb);
		break;
	case SCSI_RD_CAPAC:
		ret = ata_scsiop_read_capacity10(pccb);
		break;
	case SCSI_TST_U_RDY:
		ret = ata_scsiop_test_unit_ready(pccb);
		break;
	case SCSI_INQUIRY:
		ret = ata_scsiop_inquiry(pccb);
		break;
	default:
		printf("Unsupport SCSI command 0x%02x\n", pccb->cmd[0]);
		return FALSE;
	}

	if (ret) {
		debug("SCSI command 0x%02x ret errno %d\n", pccb->cmd[0], ret);
		return FALSE;
	}
	return TRUE;

}

#ifdef PLATFORM_XCODE
void scsi_low_level_init(unsigned char dev)
{
	int i;
	u32 linkmap;
	
	ahci_init_one(dev);
	linkmap = probe_ent[dev]->link_port_map;
	
	for (i = 0; i < CFG_SCSI_MAX_SCSI_ID; i++) {
		if (((linkmap >> i) & 0x01)) {
			if (ahci_port_start(dev, (u8) i)) {
				printf("Can not start port %d\n", i);
				continue;
			}
		}
	}
}
#else
void scsi_low_level_init(int busdevfunc)
{
	int i;
	u32 linkmap;

	ahci_init_one(1);

	linkmap = probe_ent->link_port_map;

	for (i = 0; i < CFG_SCSI_MAX_SCSI_ID; i++) {
		if (((linkmap >> i) & 0x01)) {
			if (ahci_port_start((u8) i)) {
				printf("Can not start port %d\n", i);
				continue;
			}

#if 0            
//JLWANG: There is some SATA devices which doesn't support ULTRADMA 6
//              1. identify device to determine Ultradma mode 2. set feature using correct ultra dma mode 3. identify device ...

                        printf("Start identify device\n");
                        memset(&tempccb, 0, sizeof(ccb));
                        tempccb.target  = i;
			tempccb.lun=0;
			tempccb.pdata=(unsigned char *)&tempbuff;
			tempccb.datalen=512;
			scsi_setup_inquiry(&tempccb);
			if(scsi_exec(&tempccb)!=TRUE)
                        {
                            printf("Identify device failed\n");
                        }
                        else
                        {
                            printf("Identify device success\n");
                        }

                        probe_ent->udma_mask = (u32)((tempbuff[88]&0xff00)>>8);
#endif
			ahci_set_feature((u8) i);
		}
	}
}
#endif

void scsi_bus_reset(void)
{
	/*Not implement*/
}


void scsi_print_error(ccb * pccb)
{
	/*The ahci error info can be read in the ahci driver*/
}

unsigned int sata_read_phy_reg(int addr)
{
	int readval;

	//Write address to read
	writel(addr|0x10000, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Wait for ack signal
	while(!(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK));
	//Clear address from data in
	writel(0, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	// Wait for ack signal to deassert
	while(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK);
	//Send cr read signal
	writel(0x80000, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal and get read data...
	while(!(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK));
	//clear read signal
	readval = readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN);
	writel(0, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal to deassert
	while(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK);

	return (readval & 0xffff);
}

void sata_write_phy_reg(int addr, int data)
{
	//Write address to READ
	writel(addr|0x10000, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal
	while(!(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK));
	//Clear address from data in
	writel(0, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal to deassert
	while(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK);
	//Write DATA to PHY
	writel(data|0x20000, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal
	while(!(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK));
	//clear cap data signal
	writel(0, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal to deassert
	while(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK);
	//Clear data from data in and send cr write
	writel(0x40000, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal
	while(!(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK));
	//Clear cr write
	writel(0, XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_OUT);
	//Poll for ack signal to deassert
	while(readl(XC_SOC_PROC_MMREG_BASE+SATA_PHY_CR_IN)&SATA_PHY_CR_IN_ACK_MASK);
}

void ahci_bist_op(int host, int speed, char *pattern)
{
	unsigned int bistcr_addr;
	unsigned int data;

	bistcr_addr = probe_ent[host]->mmio_base + HOST_BISTCR;
		
	if((strncmp(pattern, "HFTP", 4)==0) || (strncmp(pattern, "hftp", 4)==0))
		SATA_writel(0x7,bistcr_addr);
	else if((strncmp(pattern, "LBP", 3)==0) || (strncmp(pattern, "lbp", 3)==0))
		SATA_writel(0x5,bistcr_addr);
	else if((strncmp(pattern, "MFTP", 4)==0) || (strncmp(pattern, "mftp", 4)==0))
		SATA_writel(0x6,bistcr_addr);
	else if((strncmp(pattern, "LFTP", 4)==0) || (strncmp(pattern, "lftp", 4)==0))
		SATA_writel(0x8,bistcr_addr);
	else
		printf("%s is not a valid pattern. Please select HFTP, LBP, MFTP, or LFTP\n", pattern);
		
	if(speed == 1){
		data = sata_read_phy_reg(0x2002);
		data |= 0x4001;
		sata_write_phy_reg(0x2005, data);
	}
	else
		sata_write_phy_reg(0x2005, 0);
				
	ssleep(1);
				
	data = SATA_readl(bistcr_addr);
	data |= 0x40700;
	SATA_writel(data,bistcr_addr);
	ssleep(3);
	sata_write_phy_reg(0x2006, 0x3b);
}


#endif
