#include <config.h>
#include <common.h>

#if defined(CFG_ENV_IS_IN_FLASH) || (CONFIG_COMMANDS & CFG_CMD_FLASH)

#include <flash.h>
#include <asm/viper.h>
#include <asm/vixstypes.h>

#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))

flash_info_t flash_info[CFG_MAX_FLASH_BANKS];

static inline void get_bus(void)
{
	unsigned int temp, reg;
	temp = readl(XC_SOC_PROC_MMREG_BASE + RBM_PADU_CTRL);
	reg = readl(XC_SOC_PROC_MMREG_BASE + RBM_BOARD_ID) & 0x1;
	if(reg)
		temp |= PADU_CTRL_FLASH_MASK;
	else
		temp &= ~PADU_CTRL_FLASH_MASK;
	writel(temp, XC_SOC_PROC_MMREG_BASE + RBM_PADU_CTRL);
	return;
}

#define ADDR_SIZE 3
#define WREN (0x06 << 24)
#define RDSR (0x05 << 24)
#define WRSR (0x01 << 24)
#define READ (0x03 << 24)
#define PP   (0x02 << 24)
#define PW   (0x02 << 24)
#define SERASE (0xD8 << 24)
#define BERASE (0xC7 << 24)

/* Command for density over 16MB SPI Flash */
/* - Spansion S25FL256S - Bank Address Register*/
#define BRAC	(0xB9 << 24)
#define BRRD	(0x16 << 24)
#define BRWR	(0x17 << 24)

/* - Numonyx N25Q256 - Extended Address Register*/
#define RDEAR	(0xC8 << 24)
#define WREAR	(0xC5 << 24)

/* Address Space Marco */
#define BAR16		(0x0)
#define BAR16TO32	(0x01)
#define BAR32TO48	(0x02)
#define BAR48TO64	(0x03)
#define STAT_ERR	(0x60)

/* For N25Q256 a EAR - extended address register 
 *
 */
static unsigned int spi_shift(unsigned int xfer_size, unsigned int hold_en, unsigned int cap_edge, unsigned int wdat) 
{
	unsigned int tmpValue;
	unsigned int config    = (0xA<<24)|(cap_edge << 3) | (xfer_size << 1) | hold_en;

	get_bus();

	writel(config, SPI_CFG + XC_SOC_PROC_MMREG_BASE);
	writel(wdat, SPI_WDAT + XC_SOC_PROC_MMREG_BASE);
	while(0 != (readl(SPI_STATUS +XC_SOC_PROC_MMREG_BASE) & 0x00000001));
	tmpValue = readl(SPI_RDAT +XC_SOC_PROC_MMREG_BASE);
	return tmpValue;
}

/* read 16 byte chip id include the ip and extended id */
static int eeprom_read_id(uint32 *id)
{
	unsigned xfer_size, hold_en;

	if(!id){
		printf("read id fail\n");
		return -1;
	}
	////write READID op-code
	xfer_size = 0;
	hold_en   = 1;
	spi_shift(xfer_size, hold_en, 1, 0x9f000000);

	//read data
	xfer_size = 3;
	hold_en   = 0;
	id[0] = spi_shift(xfer_size, hold_en, 1, 0x0);

	xfer_size = 3;
	hold_en   = 0;
	id[1] = spi_shift(xfer_size, hold_en, 1, 0x0);

	return 0;
}

unsigned read_status (void)
{
	unsigned read_data;
	read_data = spi_shift(1, 0, 1, RDSR);
	return read_data;
}

//enable write eeprom
void enable_eeprom_write(void)
{
	//enable write - op-code = 0x6
	unsigned xfer_size = 0;
	unsigned hold_en   = 0;
	unsigned cap_edge = 1;
	spi_shift(xfer_size, hold_en, cap_edge, WREN);	
	if ((read_status() & 0x00000002) != 0x00000002) {
		printf("Write enable could not be set!");
	}
}

void eeprom_write_address(unsigned byte_addr)
{
	//write op-code PP and byte_addr - write_eeprom
	//page_write opcde = PW (slower, but doesn't require erase first).
	unsigned hold_en   = 1;
#ifdef PRE_ERASE
	//eeprom_page_erase(byte_addr/PAGE_SIZE);
	enable_eeprom_write();
	spi_shift(ADDR_SIZE, hold_en, 1, PP + (byte_addr << (8*(3-ADDR_SIZE))));	
#else
	enable_eeprom_write();
	spi_shift(ADDR_SIZE, hold_en, 1, PW + (byte_addr << (8*(3-ADDR_SIZE))));
#endif
}

#define TIMEOUT 9000000
void wait_eeprom_rdy (int loud)
{
	int	read_data  = read_status() & 0x00000001;
	int loop_count = 0;
	while (read_data && (loop_count<TIMEOUT)) {
		read_data = read_status() & 0x00000001;
		loop_count++;
	}
	if(loop_count >= TIMEOUT) 	{
		printf("WAIT EEPROM TIMEOUT ERROR\n");
		//		g_bFail = true;
	}
}
void eeprom_write_data(unsigned hold_en, unsigned xfer_size, unsigned data)
{
	//write data
	spi_shift(xfer_size, hold_en, 1, data);
	//if hold = 0, then wait
	if (hold_en == 0)
		wait_eeprom_rdy(0);
}

/* For Flash density larger than 128M bits 
 * the address bit will be extended from 24 bits to 
 * 25 bits - 256 Mbit
 * 26 bits - 512 Mbit
 * This  function will set the bank address bit on Band Address Register
 */
static unsigned eeprom_get_bank(void)
{
	unsigned read_data = 0;

	if(flash_info[0].flash_id == FLASH_S25FL256S)
		read_data = spi_shift(1, 0, 1, BRRD) & 0xFF;
	else if(flash_info[0].flash_id == FLASH_N25Q256)
		read_data = spi_shift(1, 0, 1, RDEAR) & 0xFF;

	return read_data;
}

static int eeprom_set_bank_0(void)
{
	unsigned tmp, cmd;

	wait_eeprom_rdy(0);
	if(flash_info[0].flash_id == FLASH_S25FL256S){
		tmp = read_status();
		if(tmp & STAT_ERR){
			printf("Flash status ERR: %x\n", tmp);
			return -1;
		}
		cmd = BRWR;
	}else if(flash_info[0].flash_id == FLASH_N25Q256){
		enable_eeprom_write();
		cmd = WREAR;
	}

	spi_shift(1, 0, 1, cmd | (BAR16 << 16));

	return 0;
}

static int eeprom_set_bank_1(void)
{
	unsigned tmp, cmd;

	wait_eeprom_rdy(0);
	if(flash_info[0].flash_id == FLASH_S25FL256S){
		tmp = read_status();	
		if(tmp & STAT_ERR){
			printf("Flash status ERR: %x\n", tmp);
			return -1;
		}
		cmd = BRRD;
	}else if(flash_info[0].flash_id == FLASH_N25Q256){
		enable_eeprom_write();
		cmd = WREAR;
	}

	spi_shift(1, 0, 1, cmd | (BAR16TO32 << 16));
	return 0;
}

//read address
void eeprom_read_address(unsigned byte_addr)
{
	unsigned hold_en = 1;
	spi_shift(ADDR_SIZE, hold_en, 1, READ + (byte_addr << (8*(3-ADDR_SIZE))));
}

void eeprom_sector_erase(unsigned addr)
{
	unsigned hold_en = 0;
	addr &= ~0xF0000000;
	if(flash_info[0].size > 0x1000000){
		if(addr >= 0x1000000){
			eeprom_set_bank_1();			
		} else {
			eeprom_set_bank_0();
		}
		debug("Erase offset %x at bank [%d]\n", (unsigned)addr, (int)eeprom_get_bank());
	}

	enable_eeprom_write();
	spi_shift(ADDR_SIZE, hold_en, 1, SERASE + (addr&0x00ffffff));	
	wait_eeprom_rdy(0);
}

unsigned eeprom_dword_read(unsigned hold_en){
	unsigned xfer_size = 3;
	unsigned read_data = spi_shift(xfer_size, hold_en, 1, 0x0);
	return read_data;
}

unsigned eeprom_byte_read(unsigned byte_addr){
	unsigned xfer_size, hold_en, read_data;

	byte_addr &= ~0xF0000000;
	if(flash_info[0].size > 0x1000000){
		if(byte_addr >= 0x1000000)
			eeprom_set_bank_1();
		else
			eeprom_set_bank_0();
		debug("Read offset %x at bank [%d]\n", (unsigned)byte_addr, (int)eeprom_get_bank());
	}

	////write READ op-code and byte_addr
	xfer_size = ADDR_SIZE;
	hold_en   = 1;
	spi_shift(xfer_size, hold_en, 1, READ + byte_addr);

	//read byte
	xfer_size = 0;
	hold_en   = 0;
	read_data = spi_shift(xfer_size, hold_en, 1, 0x0);

	return read_data;
}

unsigned long flash_init (void)
{
	uint32 chip_id[2], id;
	int bank,sect,size = 0;
	uint32 temp;
	uint8 edi_len, ext_id;

	temp = readl(PPC_SFT_RSTN + XC_SOC_PROC_MMREG_BASE) & (~CCSPI_SFT_RSTN_MASK);
	writel(temp, PPC_SFT_RSTN + XC_SOC_PROC_MMREG_BASE);
	temp = readl(PPC_SFT_RSTN + XC_SOC_PROC_MMREG_BASE) | CCSPI_SFT_RSTN_MASK;
	writel(temp, PPC_SFT_RSTN + XC_SOC_PROC_MMREG_BASE);


	if(eeprom_read_id(chip_id)){
		printf("invalid id %08x %08x\n", chip_id[0], chip_id[1]);
	}else {
		id = chip_id[0] >> 8;
		edi_len = (uint8)(chip_id[0] & 0xff);
		ext_id = (uint8)(chip_id[1] >> 24);
		debug("EEPROM ID is %08x extended id len %d, ext id %02x\n", id, edi_len, ext_id);
	}

	for(bank=0; bank < CFG_MAX_FLASH_BANKS; ++bank)
	{

		memset(&flash_info[bank],UNSUPPORTED_FLASH_CMD,sizeof(flash_info_t));

		if(id == 0x00202018)
		{
			printf("EEPROM Type STM25P128\n");
			flash_info[bank].size = 0x1000000; // 16M bytes
			flash_info[bank].sector_count = 64;
			flash_info[bank].flash_id = FLASH_M25P128;
			flash_info[bank].my_page_num_per_sector = 1024;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00202015)
		{
			printf("EEPROM Type STM25P16\n");
			flash_info[bank].size = 0x200000; // 2M bytes
			flash_info[bank].sector_count = 32;
			flash_info[bank].flash_id = FLASH_M25P16;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x001f4501)
		{
			printf("EEPROM Type AT25DF81A\n");
			flash_info[bank].size = 0x100000; // 1M bytes
			flash_info[bank].sector_count = 16;
			flash_info[bank].flash_id = FLASH_AT25DF81A;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x0020ba18)
		{
			printf("EEPROM Type N25Q128\n");
			flash_info[bank].size = 0x1000000; // 16M bytes
			flash_info[bank].sector_count = 256;
			flash_info[bank].flash_id = FLASH_N25Q128;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x0020ba19)
		{
			printf("EEPROM Type N25Q256\n");
			flash_info[bank].size = 0x2000000; // 32M bytes
			flash_info[bank].sector_count = 512;
			flash_info[bank].flash_id = FLASH_N25Q256;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00c22014)
		{
			printf("EEPROM Type MX25L8006E\n");
			flash_info[bank].size = 0x100000; // 1M bytes
			flash_info[bank].sector_count = 16;
			flash_info[bank].flash_id = FLASH_MX25L8006E;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00c22015)
		{
			printf("EEPROM Type MX25L1606E\n");
			flash_info[bank].size = 0x200000; // 2M bytes
			flash_info[bank].sector_count = 32;
			flash_info[bank].flash_id = FLASH_MX25L1606E;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00c22017)
		{
			printf("EEPROM Type MX25L6445E\n");
			flash_info[bank].size = 0x1000000; // 16M bytes
			flash_info[bank].sector_count = 128;
			flash_info[bank].flash_id = FLASH_MX25L6445E;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00c22018)
		{
			printf("EEPROM Type MX25L12845E\n");
			flash_info[bank].size = 0x1000000; // 16M bytes
			flash_info[bank].sector_count = 256;
			flash_info[bank].flash_id = FLASH_MX25L12845E;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00202014)
		{
			printf("EEPROM Type STM25P80\n");
			flash_info[bank].size = 0x100000; // 1M bytes
			flash_info[bank].sector_count = 16;
			flash_info[bank].flash_id = FLASH_M25P80;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00EF4014)
		{
			printf("EEPROM Type W25Q80PV\n");
			flash_info[bank].size = 0x100000; // 1M bytes
			flash_info[bank].sector_count = 16;
			flash_info[bank].flash_id = FLASH_W25Q80PV;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00EF4015)
		{
			printf("EEPROM Type W25Q16CV\n");
			flash_info[bank].size = 0x200000; // 1M bytes
			flash_info[bank].sector_count = 32;
			flash_info[bank].flash_id = FLASH_W25Q16CV;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00EF4016)
		{
			printf("EEPROM Type W25Q32BV\n");
			flash_info[bank].size = 0x400000; // 1M bytes
			flash_info[bank].sector_count = 64;
			flash_info[bank].flash_id = FLASH_W25Q32BV;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00EF4017)
		{
			printf("EEPROM Type W25Q64BV\n");
			flash_info[bank].size = 0x800000; // 8M bytes
			flash_info[bank].sector_count = 128;
			flash_info[bank].flash_id = FLASH_W25Q64BV;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x00012018)
		{
			if(ext_id & 0x01)
			{
				printf("EEPROM Type S25FL129P\n");
				flash_info[bank].size = 0x1000000; // 16M bytes
				flash_info[bank].sector_count = 256; //64K sector
				flash_info[bank].flash_id = FLASH_S25FL129P;
				flash_info[bank].my_page_num_per_sector = 256;
				flash_info[bank].my_page_size = 256;
			}
			else
			{
				printf("EEPROM Type S25FL128P\n");
				flash_info[bank].size = 0x1000000; // 16M bytes
				flash_info[bank].sector_count = 64;
				flash_info[bank].flash_id = FLASH_S25FL128P;
				flash_info[bank].my_page_num_per_sector = 1024;
				flash_info[bank].my_page_size = 256;
			}
		}
		else if(temp == 0x00010219)
		{					
			if(ext_id & 0x01){			
				printf("EEPROM Type S25FL256SIH30\n");
				flash_info[bank].size = 0x2000000; // 32M bytes
				flash_info[bank].sector_count = 512; //64KiB Sector
				flash_info[bank].flash_id = FLASH_S25FL256S;
				flash_info[bank].my_page_num_per_sector = 256;
				flash_info[bank].my_page_size = 256;
			} else {
				printf("EEPROM Type S25FL256SIH31\n");
				flash_info[bank].size = 0x2000000; // 32M bytes
				flash_info[bank].sector_count = 128; //256KiB Sector
				flash_info[bank].flash_id = FLASH_S25FL256S;
				flash_info[bank].my_page_num_per_sector = 1024;
				flash_info[bank].my_page_size = 256;
			}
		}
		else if(id == 0x008C2014)
		{
			printf("EEPROM Type F25L80P\n");
			flash_info[bank].size = 0x100000; // 1M bytes
			flash_info[bank].sector_count = 16;
			flash_info[bank].flash_id = FLASH_F25L80P;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x008C2015)
		{
			printf("EEPROM Type F25L16P\n");
			flash_info[bank].size = 0x200000; // 2M bytes
			flash_info[bank].sector_count = 32;
			flash_info[bank].flash_id = FLASH_F25L16P;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else if(id == 0x001C5115)
		{
			printf("EEPROM Type EN25T16\n");
			flash_info[bank].size = 0x200000; // 2M bytes
			flash_info[bank].sector_count = 32;
			flash_info[bank].flash_id = FLASH_EN25T16;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}
		else
		{
			printf("Unrecognized SPI Flash ID:0x%x\n",id);
			printf("Using default EEPROM Type STM25P80\n");
			flash_info[bank].size = 0x100000; // 1M bytes
			flash_info[bank].sector_count = 16;
			flash_info[bank].flash_id = FLASH_M25P80;
			flash_info[bank].my_page_num_per_sector = 256;
			flash_info[bank].my_page_size = 256;
		}

		for(sect=0; sect < flash_info[bank].sector_count; ++sect)
		{
			flash_info[bank].start[sect] = CFG_FLASH_BASE + sect * flash_info[bank].my_page_num_per_sector * flash_info[bank].my_page_size;
			flash_info[bank].protect[sect] = 0;
		}
		size += flash_info[bank].size;
	}
	return size;
}

void flash_print_info(flash_info_t * info)
{
	int i;

	if(info->flash_id >= FLASH_MAX_INDEX)
	{
		puts("missing or unknown FLASH type\n");
		return;
	}

	if(info->flash_id == FLASH_M25P80)
	{
		printf("M25P80 FLASH\n");
	}
	else if(info->flash_id == FLASH_M25P128)
	{
		printf("M25P128 FLASH\n");
	}
	else if(info->flash_id == FLASH_MX25L6445E)
	{
		printf("MX25L6445E FLASH\n");
	}
	else if(info->flash_id == FLASH_MX25L12845E)
	{
		printf("MX25L12845E FLASH\n");
	}

	printf("size = %1d MB in %d sectors\n", info->size >> 20, info->sector_count);
	printf("page num per sector %d\n", info->my_page_num_per_sector);
	printf("  Sector Start Address:");
	for(i=0; i < info->sector_count; ++i)
	{
		if((i%5) == 0)
			printf("\n");
		printf (" %08lX%s",info->start[i], info->protect[i] ? " (RO)  " : "     ");
	}
	printf("\n");
	return;
}


int flash_erase (flash_info_t * info, int s_first, int s_last)
{
	int prot;
	int sect;
	if (info->flash_id >= FLASH_MAX_INDEX)
	{
		puts ("Can't erase unknown flash type - aborted\n");
		return 1;
	}
	if ((s_first < 0) || (s_first > s_last))
	{
		puts ("- no sectors to erase\n");
		return 1;
	}

	prot = 0;
	for (sect = s_first; sect <= s_last; ++sect)
	{
		if (info->protect[sect]) {
			prot++;
		}
	}
	if (prot)
	{
		printf ("- Warning: %d protected sectors will not be erased!\n", prot);
	} 
	else 
	{
		putc ('\n');
	}
	enable_eeprom_write();
	//printf("read status = %x\n",read_status());
	spi_shift(1,0,1,WRSR+(0x02<<16));
	wait_eeprom_rdy(0);
	enable_eeprom_write();
	//printf("read status = %x\n",read_status());

	for(sect = s_first; sect <= s_last; sect++)
	{
		if(info->protect[sect] == 0)
		{
			eeprom_sector_erase(sect*info->my_page_num_per_sector*info->my_page_size);
			putc('.');
		}
	}    
	puts(" done\n");
	return 0;
}


int write_buff (flash_info_t * info, uchar * src, ulong addr, ulong cnt)
{   
	unsigned int j = 0;
	unsigned int hold_en;
	addr = addr & ( ~0xf0000000);

	while(j < cnt)
	{
		if ( (addr + j) % info->my_page_size == 0)
		{
			if(flash_info[0].size > 0x1000000){
				if((addr + j) >= 0x1000000) {
					eeprom_set_bank_1();					
				} else {
					eeprom_set_bank_0();					
				}
				debug("Write offset %x at bank [%d]\n", (unsigned)(addr + j), (int)eeprom_get_bank());
			}			
			debug("Write Address %x\n", (addr + j)& 0xFFFFFF);
			eeprom_write_address((addr + j) & 0xFFFFFF);
		}
		hold_en = (addr +j +1) % info->my_page_size !=0 ? 1:0;
		eeprom_write_data(hold_en, 0, src[j] << 24);
		j++;
	}

	//Write dummy byte to padding the remaining page size

	while( (addr + j) % info->my_page_size != 0)
	{
		hold_en = (addr +j +1) % info->my_page_size !=0 ? 1:0;
		eeprom_write_data(hold_en, 0, 0xff);        
		j++;
	}

	return 0;
}


#if 0
int read_buff ( uchar * src, ulong addr, ulong cnt)
{
	unsigned int j = 0;

	addr = addr & ( ~0xf0000000);
	eeprom_read_address(addr + j);
	wait_eeprom_rdy(0);
	while(j < cnt)
	{
		src[j] = eeprom_byte_read(j + addr);
		j++;
	}
	return 0;
}
#else
int read_buff ( uchar * src, ulong addr, ulong cnt)
{
	unsigned int bytes_readed = 0;
	unsigned int bank0_cnt = 0, bank1_cnt = 0;
	unsigned int i, flag = 0;
	unsigned int read_data;

	wait_eeprom_rdy(0);

	/*set address
	 * set cmd and address
	 * Check the address is great than 16MB, 
	 * If yes for chip have larger density, set the bank address first
	 * bank1_bytes contained the byte count on bank1
	 */
	addr = addr & ( ~0xf0000000);

	debug("Request to read from offset %x, conut: %x\n",(unsigned)addr, (unsigned)cnt);

	if(addr  < 0x1000000){
		if((addr + cnt) <= 0x1000000){
			bank0_cnt = cnt;
		} else {
			bank0_cnt = 0x1000000 - addr;
			bank1_cnt = addr + cnt - 0x1000000;
		}
	} else {
		bank0_cnt = 0;
		bank1_cnt = cnt;
	}
	debug("read %x bytes on bank0, read %x on bank1\n", bank0_cnt, bank1_cnt);

	if(flash_info[0].size > 0x1000000){
		if(bank0_cnt){
			eeprom_set_bank_0(); 		
		} else {
			eeprom_set_bank_1();
			flag = 1;			
		}
		debug("Read offset %x at bank [%d]\n", (unsigned)addr, (int)eeprom_get_bank());
	}else if (bank1_cnt) {
		printf("Invalid request from addr: %x size: %x, Flash chip size: %x\n", 
				(unsigned)addr, (unsigned)cnt, (unsigned)flash_info[0].size);
		return -1;
	}

	debug("Set read address %x\n", (unsigned)(addr));

	// Use only low 24 Bits address for each bank


	eeprom_read_address(addr & 0xFFFFFF);

	//read
	while(bytes_readed < cnt)
	{
		if((bytes_readed <= bank0_cnt)&&(bank0_cnt != 0)){
			if(cnt < 4||bytes_readed >= (cnt - 4))
			{
				read_data = spi_shift(cnt-bytes_readed-1, 0, 1, 0x0);
				for(i=0; i < cnt-bytes_readed; i++)
				{
					src[bytes_readed+i] = (read_data>>((cnt - bytes_readed -1-i)*8))&0xff;
				}
				bytes_readed = cnt;
			}
			else
			{
				//read 4 bytes
				read_data = spi_shift(3, 1, 1, 0x0);
				for(i = 0; i< 4; i++)
				{
					src[bytes_readed+i] = (read_data>>(8*(3-i)))&0xff; 
				}
				bytes_readed += 4;
			}
		} else {
			if(!flag){
				eeprom_set_bank_1();
				debug("Switch to  offset 0 at bank [%d]\n", (int)eeprom_get_bank());
				eeprom_read_address(0x0);
				flag = 1;
			}

			if((cnt < 4) || (bytes_readed >= (cnt - 4)))
			{
				read_data = spi_shift(cnt-bytes_readed-1, 0, 1, 0x0);
				for(i=0; i < cnt-bytes_readed; i++)
				{
					src[bytes_readed+i] = (read_data>>((cnt - bytes_readed -1-i)*8))&0xff;
				}
				bytes_readed = cnt;
			} else {
				//read 4 bytes
				read_data = spi_shift(3, 1, 1, 0x0);
				for(i = 0; i< 4; i++)
				{
					src[bytes_readed+i] = (read_data>>(8*(3-i)))&0xff; 
				}
				bytes_readed += 4;
			}
		}
	}

	return 0;	
}
#endif

#endif /*CFG_ENV_IS_IN_FLASH*/

