/******************************************************************************
 * Copyright Codito Technologies (www.codito.com)
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/board/aa3/arc_gmac.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

/* Amber Lin: modified to run Synopsys GMAC on Eagle using Enhanced Descriptor
 *            structure                                          -- April 2009
 */

#include <cmd_confdefs.h>	/* Command configs	*/
#include <asm/u-boot.h>		/* Board data structure	*/
#include "arc_gmac.h"	/* MDIO macros and ARC_EMAC register macros	*/
#include <asm/viper.h>
#include <config.h>		/* ARC700 clock freq.	*/
#include <asm/errno.h>
#include <malloc.h>
#include <common.h>
#include <net.h>
#include <linux/types.h>

/* The forced speed, 10Mb, 100Mb, gigabit  */
#define SPEED_10		10
#define SPEED_100		100
#define SPEED_1000		1000

/* Duplex, half or full. */
#define DUPLEX_HALF		0x00
#define DUPLEX_FULL		0x01

/* 1000BASE-T Control register */
#define ADVERTISE_1000FULL      0x0200  /* Advertise 1000BASE-T full duplex */
#define ADVERTISE_1000HALF      0x0100  /* Advertise 1000BASE-T half duplex */
#define LPA_1000FULL            0x0800  /* Link partner 1000BASE-T full duplex */
#define LPA_1000HALF            0x0400  /* Link partner 1000BASE-T half duplex */

/* Link partner ability register. */
#define LPA_SLCT                0x001f  /* Same as advertise selector  */
#define LPA_10HALF              0x0020  /* Can do 10mbps half-duplex   */
#define LPA_1000XFULL           0x0020  /* Can do 1000BASE-X full-duplex */
#define LPA_10FULL              0x0040  /* Can do 10mbps full-duplex   */
#define LPA_1000XHALF           0x0040  /* Can do 1000BASE-X half-duplex */
#define LPA_100HALF             0x0080  /* Can do 100mbps half-duplex  */
#define LPA_1000XPAUSE          0x0080  /* Can do 1000BASE-X pause     */
#define LPA_100FULL             0x0100  /* Can do 100mbps full-duplex  */
#define LPA_1000XPAUSE_ASYM     0x0100  /* Can do 1000BASE-X pause asym*/
#define LPA_100BASE4            0x0200  /* Can do 100mbps 4k packets   */
#define LPA_PAUSE_CAP           0x0400  /* Can pause                   */
#define LPA_PAUSE_ASYM          0x0800  /* Can pause asymetrically     */
#define LPA_RESV                0x1000  /* Unused...                   */
#define LPA_RFAULT              0x2000  /* Link partner faulted        */
#define LPA_LPACK               0x4000  /* Link partner acked us       */
#define LPA_NPAGE               0x8000  /* Next page bit               */

#define LPA_DUPLEX		(LPA_10FULL | LPA_100FULL)
#define LPA_100			(LPA_100FULL | LPA_100HALF | LPA_100BASE4)


#define ENABLE_LOOPBACK 0

#define BUF_MULTIPLE 16	/* data bus width is 128 bits on eagle */
#define RX_BUF_ALIGN(X) (((X) + (BUF_MULTIPLE - 1)) & ~(BUF_MULTIPLE - 1))

#if (CONFIG_COMMANDS & CFG_CMD_NET)

#define GMAC_DESC_ALIGN 0x40
#define GMAC_DESC_MASK  (~(GMAC_DESC_ALIGN - 1))

#define GMAC_DATA_ALIGN 0x40
#define GMAC_DATA_MASK  (~(GMAC_DATA_ALIGN - 1))

#define GMAC_DMA_ADDR(addr) (((unsigned int)(addr)) & 0xfffffff )

/* rx buffer descriptor. We align descriptors to 32 so
 * satisfies that the descriptor addresses must be aligned to the bus-width 
 * used */
DmaDesc *rxbd;

/* tx buffer descriptor */
DmaDesc *txbd;

unsigned static int rxbd_org = 0;
unsigned static int txbd_org = 0;

static struct eth_device eth_netdev;
static unsigned long PhyBase = (unsigned long)DefaultPhyBase;
static int act_eth_port = 0;
static int gmii = 0;
static int eth_speed = SPEED_100;
static int eth_duplex = DUPLEX_FULL;

//DmaDesc rxbd[RX_BDT_LEN] __attribute__((aligned(32)));= {{0}};

/* tx buffer descriptor */
//DmaDesc txbd[TX_BDT_LEN] __attribute__((aligned(32)));//= {{0}};

//#define DEBUG

#ifdef DEBUG
#define PDEBUG(fmt, args...) printf(  fmt, ## args)
#else
#define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))

static int txbd_cntr = 0; /* transmit buffer counter */
static int rxbd_cntr = 0; /* receive buffer counter  */

/****************************************************/
/* System-dependant register access functions       */
/****************************************************/

static unsigned long reg_read(unsigned long addr)
{
	unsigned long data;
	unsigned long reg;

	if(0 == act_eth_port) /* eth0 */
	{
		reg = MMREG_BASE + ETH_STATUS_REG;
		do {
			data = readl(reg);
		} while((data & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);

		reg = MMREG_BASE + ETH_CSR_CTRL_REG;
		writel(addr, reg);

		reg = MMREG_BASE + ETH_STATUS_REG;
		do {
			data = readl(reg);
		} while((data  & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);
        
		reg = MMREG_BASE + ETH_RDAT_REG;
		data = readl(reg);
		return data;
	}
	else if(1 == act_eth_port) /* eth1 */
	{
		reg = MMREG_BASE + ETH_STATUS_REG1;
		do {
			data = readl(reg);
		} while((data & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);

		reg = MMREG_BASE + ETH_CSR_CTRL_REG1;
		writel(addr, reg);

		reg = MMREG_BASE + ETH_STATUS_REG1;
		do {
			data = readl(reg);
		} while((data  & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);
        
		reg = MMREG_BASE + ETH_RDAT_REG1;
		data = readl(reg);
		return data;
	}
}

static void reg_write(unsigned long int data, unsigned long int addr)
{
	unsigned long reg, temp;
    
	if(0 == act_eth_port) /* eth0 */
	{
		reg = MMREG_BASE + ETH_STATUS_REG;
	    do {
		    temp = readl(reg);
	    } while( (temp & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);

	    reg = MMREG_BASE + ETH_WDAT_REG;
	    writel(data, reg);

	    reg = MMREG_BASE + ETH_CSR_CTRL_REG;
	    writel(addr | 0x80000000, reg);

	    reg = MMREG_BASE + ETH_STATUS_REG;
	    do {
		   temp = readl(reg);
	    } while( (temp & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);
	}
	else if(1 == act_eth_port) /* eth1 */
	{
		reg = MMREG_BASE + ETH_STATUS_REG1;
		do {
			temp = readl(reg);
		} while( (temp & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);
		
		reg = MMREG_BASE + ETH_WDAT_REG1;
		writel(data, reg);
		
		reg = MMREG_BASE + ETH_CSR_CTRL_REG1;
		writel(addr | 0x80000000, reg);
		
		reg = MMREG_BASE + ETH_STATUS_REG1;
		do {
		   temp = readl(reg);
		} while( (temp & ETH_STATUS_REG_CSR_BUSY_MASK) != 0);
	}
}

static unsigned long read_mac_reg(unsigned long reg)
{
	unsigned long data = reg_read(GmacRegisterBase + reg);
	return data;
}

static void write_mac_reg(unsigned long reg, unsigned long data)
{
	reg_write(data,GmacRegisterBase + reg);
}

static void set_mac_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = GmacRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t |= data;
	reg_write(data_t, addr);
}

static void clear_mac_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = GmacRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t &= ~data;
	reg_write(data_t, reg);
}

static unsigned long read_dma_reg(unsigned long reg)
{
	unsigned long data = reg_read(DmaRegisterBase + reg);
	return data;
}

static void write_dma_reg(unsigned long reg, unsigned long data)
{
	reg_write(data, DmaRegisterBase + reg);
}

static void set_dma_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = DmaRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t |= data;
	reg_write(data_t, addr);
}

static void clear_dma_reg(unsigned long reg, unsigned long data)
{
	unsigned long addr = DmaRegisterBase + reg;
	unsigned long data_t = reg_read(addr);
	data_t &= ~data;
	reg_write(data_t, reg);
}

/***********************/
/* MAC module function */
/***********************/

static unsigned long read_mii(unsigned long reg)
{
	unsigned long addr;
	unsigned long data;
	addr = ((PhyBase << GmiiDevShift) & GmiiDevMask) | ((reg << GmiiRegShift) & GmiiRegMask) | GmiiRead;
	write_mac_reg(GmacGmiiAddr, (addr | GmiiAppClk1 | GmiiBusy));
	while( (read_mac_reg(GmacGmiiAddr) & GmiiBusy) == GmiiBusy);
	data = read_mac_reg(GmacGmiiData) & 0xFFFF;
	return data;
}

static void write_mii(unsigned long reg, unsigned long data)
{
	unsigned long addr;
	write_mac_reg(GmacGmiiData, data);
	addr = ((PhyBase << GmiiDevShift) & GmiiDevMask) | ((reg << GmiiRegShift) & GmiiRegMask) | GmiiWrite;
	write_mac_reg(GmacGmiiAddr, (addr | GmiiAppClk1 | GmiiBusy));
	while( (read_mac_reg(GmacGmiiAddr) & GmiiBusy) == GmiiBusy);
}

static void gmac_set_address(void *ptr)
{
	char *mac_addr = (char *)ptr;
	unsigned long int data;

	data = (mac_addr[5] << 8) | mac_addr[4];
	write_mac_reg(GmacAddr0High, data);
	data = (mac_addr[3] << 24) | (mac_addr[2] << 16) | (mac_addr[1] << 8) | mac_addr[0];
	write_mac_reg(GmacAddr0Low, data);
}

#if (CONFIG_COMMANDS & CFG_CMD_MII)
#include <miiphy.h>

#ifdef CONFIG_TERSE_MII

int miiphy_read (unsigned char addr, unsigned char reg, unsigned short *value)
{
    unsigned long addr_reg_val;

    addr_reg_val = ((addr << GmiiDevShift) & GmiiDevMask) | ((reg << GmiiRegShift) & GmiiRegMask) | GmiiRead;
    write_mac_reg(GmacGmiiAddr, (addr_reg_val | GmiiAppClk1 | GmiiBusy));
    while( (read_mac_reg(GmacGmiiAddr) & GmiiBusy) == GmiiBusy);
    *value = read_mac_reg(GmacGmiiData) & 0xFFFF;	
    return 0;

}

int miiphy_write (unsigned char addr, unsigned char reg, unsigned short value)
{
    unsigned long addr_reg_val;

    write_mac_reg(GmacGmiiData, value);
    addr_reg_val = ((addr << GmiiDevShift) & GmiiDevMask) | ((reg << GmiiRegShift) & GmiiRegMask) | GmiiWrite;
    write_mac_reg(GmacGmiiAddr, (addr_reg_val | GmiiAppClk1 | GmiiBusy));
    while( (read_mac_reg(GmacGmiiAddr) & GmiiBusy) == GmiiBusy);

}

#endif //CONFIG_TERSE_MII

#endif

static int board_required_xcode_clk(unsigned int board_id)
{
       if((board_id == 0x2100)||(board_id == 0x2101))
               return 1;

       return 0;
}


/*********************************************************/
/* reset ethernet block                                  */
/********************************************************/
static void eth_reset(void)
{
	unsigned long data, boardid;

//	printf("Ethernet: reset\n");
    
	// this pin could be used as GPIO pins or for ethernet, 
	// set this pins work for ethernet, not for gpio purpose
	data = readl(MMREG_BASE + GPIO_B_CTRL);
	data = (data & (~GPIO_B_CTRL_GPIO_MODE_SEL_MASK));
	writel(data, MMREG_BASE + GPIO_B_CTRL);

	data = readl(MMREG_BASE + GPIO_A_CTRL);
	data = (data & (~GPIO_A_CTRL_GPIO_MODE_SEL_MASK));
	writel(data, MMREG_BASE + GPIO_A_CTRL);

	boardid=readl(MMREG_BASE + CG_DUMMY_REG);

    // When using Gigabit ETHERNET
	writel(readl((unsigned long *)(CG_CLK_STOP0 + MMREG_BASE)) & ~ETHR125CLK_STOP_MASK, (unsigned long *)(CG_CLK_STOP0 + MMREG_BASE));

    // When using ETHERNET0
	writel(readl((unsigned long *)(CG_CLK_STOP0 + MMREG_BASE)) & ~ETHR0CLK_STOP_MASK, (unsigned long *)(CG_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH0_MCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH0_SCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));

   	// When using ETHERNET1
	writel(readl((unsigned long *)(CG_CLK_STOP0 + MMREG_BASE)) & ~ETHR1CLK_STOP_MASK, (unsigned long *)(CG_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH1_MCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH1_SCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));
	
	// reset ethernet0
	// CG_RESET_REG\ETH0_RESET   = 0x1
	// CG_RESET_REG\ETH0_RESET   = 0x0
	data = readl(MMREG_BASE + CG_RESET_REG);
	data = data | ETH0_RESET_MASK;
	writel(data, MMREG_BASE + CG_RESET_REG);

	data = readl(MMREG_BASE + CG_RESET_REG);
	data = data & (~ETH0_RESET_MASK);
	writel(data, MMREG_BASE + CG_RESET_REG);

	// reset ethernet1
	// CG_RESET_REG\ETH0_RESET   = 0x1
	// CG_RESET_REG\ETH0_RESET   = 0x0
	data = readl(MMREG_BASE + CG_RESET_REG);
	data = data | ETH1_RESET_MASK;
	writel(data, MMREG_BASE + CG_RESET_REG);

	data = readl(MMREG_BASE + CG_RESET_REG);
	data = data & (~ETH1_RESET_MASK);
	writel(data, MMREG_BASE + CG_RESET_REG);


	if(0==gmii)
	{
		data = ETH_CTRL_REG_RESETN_MASK;

        if(board_required_xcode_clk(boardid))
	        data |= ETH_CTRL_REG_GTXCLK_SOURCE_MASK;

		writel(data, MMREG_BASE + ETH_CTRL_REG);
		writel(data, MMREG_BASE + ETH_CTRL_REG1);
	}
	else
	{
		data = TX_NEGEDGE_MASK|ETH_CTRL_REG_RESETN_MASK;
		writel(data, MMREG_BASE + ETH_CTRL_REG);
		writel(data, MMREG_BASE + ETH_CTRL_REG1);
	}

}

static int DescDma( DmaDesc Desc )
{
	return (Desc.status & DescOwnByDma) != 0;   /* Owned by DMA */
}

static int DmaRxValid( unsigned int Status )
{
	/* no errors, and whole frame is in the buffer */
	return ( (Status & DescError) == 0 )
		&& ( (Status & DescRxFirst) != 0 )
		&& ( (Status & DescRxLast) != 0 );
}

static int gmac_eth_recv(struct eth_device *netdev)
{
	unsigned int *recv_data, len;
	unsigned long status;
	int j;
	uchar *p;

	PDEBUG("gmac_eth_recv\n");
	j=0;
	do {
		status = read_dma_reg(DmaStatus);
		j++;
		if (status & DmaIntRxStopped)
			write_dma_reg(DmaRxPollDemand, 0);
	} while(!(status  & DmaIntRxCompleted)  && j<100);

	write_dma_reg(DmaStatus, status);

	status = arc_read_uncached_32(&(rxbd[rxbd_cntr].status));
	while(!(status & DescOwnByDma))
	{
		if(DmaRxValid(status))
		{
			len = ((status & DescFrameLengthMask) >> DescFrameLengthShift) - 4; /* ignore ethernet crc */
			recv_data = (unsigned int *)(arc_read_uncached_32(&rxbd[rxbd_cntr].tag1));
			inv_dcache_range(recv_data,len);
			p = (uchar *)recv_data;
			PDEBUG("recv addr:0x%x,len:%d\n",p,len);
	
			NetReceive((uchar *)recv_data, len);
			
		}
		else
			printf("DmaRx invalid status = %x\n", status);

		arc_write_uncached_32(&(rxbd[rxbd_cntr].status), DescOwnByDma);
		rxbd_cntr = (rxbd_cntr + 1) % RX_BDT_LEN;
		status = arc_read_uncached_32(&(rxbd[rxbd_cntr].status));
	}

	PDEBUG("gmac_eth_recv return\n");
	return 0;
}

static int gmac_eth_send(struct eth_device *netdev, volatile void *packet, int length)
{
	unsigned long  status;
	unsigned int i;
	
	PDEBUG("gmac_eth_send\n");
	PDEBUG("gmac_eth_send: addr 0x%x,length:%d\n", packet,length);

	if(length > ETH_MTU)
	{
		printf("gmac_eth_send: packet length %d greater than MTU\n", length);
		return -1;
	}
	if(length < 64)
		length = 64;

	/* set up the tx descriptor */
	if((arc_read_uncached_32(&(txbd[txbd_cntr].status)) & DescOwnByDma) == 0) //owned by the host
	{
		arc_write_uncached_32(&(txbd[txbd_cntr].buffer1), GMAC_DMA_ADDR(packet));
	
		arc_write_uncached_32(&(txbd[txbd_cntr].length), (length << DescSize1Shift) & DescSize1Mask);
		flush_dcache_range((unsigned long)packet,(unsigned long)packet + length);

		status = arc_read_uncached_32(&txbd[txbd_cntr].status);
		status &= TxDescEndOfRing; //clear the descriptor
		status = status | TxDescChain | DescOwnByDma | DescTxFirst | DescTxLast | DescTxIntEnable;
		arc_write_uncached_32(&(txbd[txbd_cntr].status), status);
	} 
	else
	{
		printf("Out of Tx Buffers\n");
		return -ENOMEM;
	}

	/* issue a Poll Demand command to wake up TxDMA if it's in suspend mode */
	write_dma_reg(DmaTxPollDemand, 0);

	i=0;
	do {
		status = read_dma_reg(DmaStatus);
		i++;
	}
	while( !( status & DmaIntTxCompleted)  && i<100);

	if(i>=100)
	{
		printf("tx time out status = %x\n", status);
	}

	write_dma_reg(DmaStatus, status);
	arc_write_uncached_32(&(txbd[txbd_cntr].status), (txbd_cntr == TX_BDT_LEN-1) ? TxDescEndOfRing : 0 | TxDescChain);
	arc_write_uncached_32(&(txbd[txbd_cntr].buffer1), 0);

	txbd_cntr = (txbd_cntr + 1) % TX_BDT_LEN;	
	PDEBUG("gmac_eth_send return\n");

	return 0;
}

static int DmaInit(void)
{
	unsigned long data;

	/* 1. Set Host bus access parameters */
	data = read_dma_reg(DmaBusMode);
	data |= DmaBurstLength8;
	write_dma_reg(DmaBusMode, data);

	/* 2. Mask uncessary interrupt causes: disable all as interrupt
	      mechanism is not implemented in u-boot */
	write_dma_reg(DmaInterrupt, 0);
	write_dma_reg(DmaStatus, read_dma_reg(DmaStatus));

	/* 3. Create Tx/Rx descriptor lists and write to DMA Reg 3 & Reg 4 
	      (descriptor lists have been created in eth_init) */
	write_dma_reg(DmaRxBaseAddr, GMAC_DMA_ADDR(rxbd));//rx buf desc ptr
	write_dma_reg(DmaTxBaseAddr, GMAC_DMA_ADDR(txbd));//tx buf desc ptr

//	printf(" rxbd = %x   txbd = %x\n", rxbd, txbd);

	return 0;
}

static int MacInit(void)
{
	unsigned long data;	


	write_mac_reg(GmacFrameFilter, GmacFilterOff);

	/* full duplex rx & tx eable disable receive own */
	data = read_mac_reg(GmacConfig);
	data |=  GmacTxEnable | GmacRxEnable | GmacDisableRxOwn;
	if(SPEED_1000 == eth_speed)
		data &= ~GmacSelectMii;
	else
		data |= GmacSelectMii;

	if(DUPLEX_FULL == eth_duplex) 
		data |= GmacFullDuplex;
	else
		data &= ~GmacFullDuplex;
	write_mac_reg(GmacConfig, data);

	return 0;
}

static void MacPauseControl(void)
{
	u32 omr_reg;
	u32 mac_flow_control_reg;
	
	omr_reg = read_dma_reg(DmaControl);	
	omr_reg |= DmaRxFlowCtrlAct4K | DmaRxFlowCtrlDeact5K |DmaEnHwFlowCtrl;	
	write_dma_reg(DmaControl, omr_reg);	
	
	mac_flow_control_reg = read_mac_reg(GmacFlowControl);
	mac_flow_control_reg |= GmacRxFlowControl | GmacTxFlowControl | 0xFFFF0000;
	write_mac_reg(GmacFlowControl,mac_flow_control_reg);	
	return;
}

static void gmac_eth_halt(void)
{
	unsigned long data;
	data = read_dma_reg(DmaControl);
	data = data & (~DmaTxStart) & (~DmaRxStart);
	write_dma_reg(DmaControl,data);
}

static int reset_eth_phy(void)
{

	int i;
	unsigned long temp;
	int boardid;

	boardid = readl(MMREG_BASE + CG_DUMMY_REG) & 0xffff;
	if((boardid == 0x1103) || (boardid == 0x1104) || (boardid == 0x1105)
		|| (boardid == 0x1106))
	{
		eth_speed = SPEED_100;
		eth_duplex = DUPLEX_FULL;
		printf("Skip phy setting phy \n");
		goto probe_end;
	}
	
	PhyBase = 0;
	while(PhyBase < 32)
	{			
		temp = (read_mii(0x02) << 16)|(read_mii(0x03));
		if (temp != 0xffffffff)
			break;
		PhyBase++;
	}
	
	if (PhyBase == 32)
	{
		printf("Non Valid PHY Index Base detected\n");
		return -1;
	} 
	else 
	{
		printf("Detected Ethernet PHY Index Base %d, PHY id: %x\n",(int)PhyBase, temp);
	}

	temp = read_mii(Giga_ctrl);
	if(0 == gmii)	
	{
		temp &= ~(ADVERTISE_1000FULL|ADVERTISE_1000HALF);
	}
	else
	{
		temp |= (ADVERTISE_1000FULL|ADVERTISE_1000HALF);
	}

	write_mii(Giga_ctrl, temp);	// diable 1000 Base-T
	
	temp = read_mii(GEN_ctl);
	write_mii(GEN_ctl, temp | 0x8000);	// reset the PHY to restart auto-negotiation	

	i = 0;
	while(((read_mii(GEN_sts) & AUTOCMPLT) != AUTOCMPLT)&&(i < 50000))
	{
		udelay(100);
		i++;
	}

	i = 0;
	while(((read_mii(GEN_sts) & LINK) == 0)&&(i < 50000))
	{
		udelay(100);
		i++;
	}

	if( (read_mii(GEN_sts) & LINK) == 0)
	{
		printf("No link\n");
		return -1;
	}
	else
	{
		printf("Linked up:");
		if(1==gmii)
		{
			temp = read_mii(Giga_stat);
			if(temp & LPA_1000FULL)
			{
				eth_speed = SPEED_1000;
				eth_duplex = DUPLEX_FULL;
				goto probe_end;
			}
			else if(temp & LPA_1000HALF)
			{
				eth_speed = SPEED_1000;
				eth_duplex = DUPLEX_HALF;
				goto probe_end;
			}
	
		}
	
		temp = read_mii(AN_lpa);
		if((temp & LPA_100FULL)||(temp & LPA_100HALF))
			eth_speed = SPEED_100;
		else
			eth_speed = SPEED_10;

		if((temp & LPA_100FULL)||(temp & LPA_10FULL))
			eth_duplex = DUPLEX_FULL;
		else
			eth_duplex = DUPLEX_HALF;
				
	probe_end:
		printf("%s,%s\n",(eth_speed == SPEED_1000)?"1000M":(eth_speed == SPEED_100)?"100M":"10M",
			(eth_duplex == DUPLEX_FULL)? "duplex":"half duplex");

	}


	
#if(ENABLE_LOOPBACK ==1)
	temp = read_mii(GEN_ctl);
	write_mii(GEN_ctl, temp | LOOPBACK);
#endif

	return 0;
}

/* Referring to Synopsys Databook v3.42a 3.4.1 Initialization */
static int gmac_eth_init(struct eth_device *netdev, bd_t *bd)
{
	int i, size;
	static int initialised = 0 ;
	char *p;

	PDEBUG("Version Register = %x\n", read_mac_reg(0x20));

	if(initialised)
		goto out;

	/* Reset ethernet block */
	eth_reset();

	rxbd_org = (unsigned int)malloc( RX_BDT_LEN * sizeof(DmaDesc) + GMAC_DESC_ALIGN);
	txbd_org = (unsigned int)malloc( TX_BDT_LEN * sizeof(DmaDesc) + GMAC_DESC_ALIGN);

	if((NULL == rxbd_org) || (NULL == txbd_org))
		return -1;

	rxbd = (DmaDesc *)((rxbd_org + GMAC_DESC_ALIGN) & (~(GMAC_DESC_ALIGN - 1)));
	txbd = (DmaDesc *)((txbd_org + GMAC_DESC_ALIGN) & (~(GMAC_DESC_ALIGN - 1)));

	PDEBUG("rxbd_org = 0x%x,rxbd = 0x%x\n", rxbd_org,rxbd);
	PDEBUG("txbd_org = 0x%x,txbd = 0x%x\n", txbd_org,txbd);
	
	/* Reset the PHY */
	if(reset_eth_phy() < 0)
		return -1;

	/* Create Rx descriptor list : enhanced descriptor structure is
	   implemented on eagle */
	for(i=0; i<RX_BDT_LEN; i++)
	{
		/* RDES0 */
		arc_write_uncached_32(&(rxbd[i].status), DescOwnByDma);

		/* RDES1 */
		size = RX_BUF_ALIGN(ETH_MTU + GMAC_BUFFER_PAD);
		arc_write_uncached_32(&(rxbd[i].length), size | RxDescChain);

		/* RDES2 */
		p = (unsigned char *)malloc(size + GMAC_DATA_ALIGN);
		if(NULL == p)
		{
			return -1;
		}

		
		arc_write_uncached_32(&(rxbd[i].tag2), p);
		PDEBUG("malloc addr = %x\n", p);
		
		p = (char *)((unsigned int)(p + 32) & (~(GMAC_DATA_ALIGN - 1)));
		arc_write_uncached_32(&(rxbd[i].tag1), p);
		PDEBUG("-------->tag1malloc addr = %x\n", p);
		
		p = (char *)(((unsigned int)p) & 0xfffffff);
		arc_write_uncached_32(&(rxbd[i].buffer1), (unsigned long *)p);
		/*if (arc_read_uncached_32(&(rxbd[i].buffer1)) == 0 )
		{
			printf("no memory for receive descriptor i = %d\n", i);
		}*/

		/* RDES3 */
		if(i != (RX_BDT_LEN - 1))
			arc_write_uncached_32(&rxbd[i].buffer2, GMAC_DMA_ADDR(&rxbd[i+1]));
	}
	
	arc_write_uncached_32(&(rxbd[RX_BDT_LEN-1].length), arc_read_uncached_32(&rxbd[RX_BDT_LEN-1].length) | RxDescEndOfRing);
	arc_write_uncached_32(&rxbd[RX_BDT_LEN-1].buffer2, GMAC_DMA_ADDR(&rxbd[0]));

	/* Create Tx descriptor list with enhanced descriptor structure */
	/* All TX BD's owned by CPU */
	for( i = 0; i<TX_BDT_LEN; i++)
	{
		/* TDES0 */
		arc_write_uncached_32(&(txbd[i].status), TxDescChain);

		/* TDES1 */
		arc_write_uncached_32(&(txbd[i].length), 0);

		/* TDES2 */
		arc_write_uncached_32(&(txbd[i].buffer1), 0);

		/* TDES3 */
		if( i != (TX_BDT_LEN) )
			arc_write_uncached_32(&txbd[i].buffer2, GMAC_DMA_ADDR(&txbd[i + 1]));
		
	}
	arc_write_uncached_32(&(txbd[TX_BDT_LEN-1].status), TxDescChain | TxDescEndOfRing);
	arc_write_uncached_32(&txbd[TX_BDT_LEN-1].buffer2, GMAC_DMA_ADDR(&txbd[0]));
//	printf(" rxbd = %x   txbd = %x\n", rxbd, txbd);

	/* set GMAC hardware address */
	gmac_set_address(bd->bi_enetaddr);

	/* Step 1~3 */
	DmaInit();

	/* Step 4~5 */
	MacInit();
	MacPauseControl();
	/* Clear Interrupt Status */
	write_dma_reg(DmaStatus, read_dma_reg(DmaStatus));

	initialised = 1;

out:
	/* Step 6 */
	write_dma_reg(DmaControl, DmaRxStart | DmaTxStart);

	return (0);
}

int gmac_eth_initialize(bd_t *bd)
{
	char *tmp;
	
	struct eth_device *dev = &eth_netdev;

	tmp = getenv("ethact");
	if(tmp)
	{
		if(strcmp(tmp, "eth0") == 0)
		{
			act_eth_port=0;
			strcpy(dev->name,"eth0");
		}
		else if(strcmp(tmp, "eth1") == 0)
		{
			act_eth_port=1;
			strcpy(dev->name,"eth1");
		}
	}
	else
	{
		act_eth_port=0;
		strcpy(dev->name,"eth0");
	}

	tmp = getenv("gmii");
	if(tmp && (strcmp(tmp,"yes") == 0))
	{
		gmii=1;
		printf("1000M/100M/10M Ethernet is used.\n");
	}
	else
		printf("100M/10M Ethernet is used.\n");
	
	dev->init = gmac_eth_init;
	dev->send = gmac_eth_send;
	dev->recv = gmac_eth_recv;
	dev->halt = gmac_eth_halt;

	eth_register(dev);

	return 0;
}
#endif /* (CONFIG_COMMANDS & CFG_CMD_NET) */

