/******************************************************************************
 * Copyright Vixs Systems Inc. (www.vixs.com)
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/board/aa3/arc_sata.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Jerry Wang
 */



#include <cmd_confdefs.h>	/* Command configs	*/
#if (CONFIG_COMMANDS & CFG_CMD_IDE)
#include <asm/u-boot.h>		/* Board data structure	*/
#include "arc_sata.h"	/* MDIO macros and ARC_EMAC register macros	*/
#include "asm/jaguar.h"
#include <config.h>		/* ARC700 clock freq.	*/
#include <asm/errno.h>
#include <malloc.h>
#include <common.h>
#include <ide.h>
#include <ata.h>
#include <linux/types.h>
#include "arc_sata.h"

//#define CONFIG_XC3_SATA_SS_MODE_ENABLED
u32 XCode3_SATA_dev_available = 0;
u32 g_XC3_Sata_Initialized = 0;


u32 SDSH_writel(void * SATA_reg_base, u32 sata_reg_addr, u32 value, SATA_reg_type type)
{
    u32 regVal = 0;
    u32 tmpVal = 0;

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};

    writel(value, SATA_reg_base + SATA_WDATA_REG);

    regVal = sata_reg_addr|0x80000000;
    switch(type)
    {
        case SATA0_REG:
            regVal |= SATA_SEL_SATA0;
            tmpVal = readl(SATA_reg_base + SATA0_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|2, SATA_reg_base + SATA0_HOST_CTL_REG);
            break;
        case SATA1_REG:
            regVal |= SATA_SEL_SATA1;
            tmpVal = readl(SATA_reg_base + SATA1_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|2, SATA_reg_base + SATA1_HOST_CTL_REG);
            break;
        case DMAC_REG:
        default:
            regVal |= SATA_SEL_DMAC;
            break;
    }

    writel(regVal, SATA_reg_base + SATA_CTL_REG);

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};

    if(regVal&0x2) { return 0;};

    return 0;
};


u32 SDSH_readl(void* SATA_reg_base, u32 sata_reg_addr, SATA_reg_type type)
{
    u32 regVal = 0;
    u32 tmpVal = 0;
		
    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};


    regVal = sata_reg_addr;
    switch(type)
    {
        case SATA0_REG:
            regVal |= SATA_SEL_SATA0;
            tmpVal = readl(SATA_reg_base + SATA0_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|2, SATA_reg_base + SATA0_HOST_CTL_REG);
            break;
        case SATA1_REG:
            regVal |= SATA_SEL_SATA1;
            tmpVal = readl(SATA_reg_base + SATA1_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|2, SATA_reg_base + SATA1_HOST_CTL_REG);
            break;
        case DMAC_REG:
        default:
            regVal |= SATA_SEL_DMAC;				
            break;
    }

    writel(regVal, SATA_reg_base + SATA_CTL_REG);

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};

    if(regVal&0x2) { return 0;};

    regVal = readl(SATA_reg_base + SATA_RDATA_REG);

    return regVal;
};


u32 SDSH_writeb(void* SATA_reg_base, u32 sata_reg_addr, u8 value, SATA_reg_type type)
{
    u32 regVal = 0;
    u32 tmpVal = 0;
		
    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};


    writel(value, SATA_reg_base + SATA_WDATA_REG);

    regVal = sata_reg_addr|0x80000000;
    switch(type)
    {
        case SATA0_REG:
            regVal |= SATA_SEL_SATA0;
            tmpVal = readl(SATA_reg_base + SATA0_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal, SATA_reg_base + SATA0_HOST_CTL_REG);
            break;
        case SATA1_REG:
            regVal |= SATA_SEL_SATA1;
            tmpVal = readl(SATA_reg_base + SATA1_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal, SATA_reg_base + SATA1_HOST_CTL_REG);
            break;
        case DMAC_REG:
        default:
            regVal |= SATA_SEL_DMAC;
            printf("Error!No byte access to DMA registers\n");
            while(1);
            break;
    }

    writel(regVal, SATA_reg_base + SATA_CTL_REG);

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};

    if(regVal&0x2) {debug("Writeb error at end\n"); return 0;};

    return 0;
}

u8 SDSH_readb(void* SATA_reg_base, u32 sata_reg_addr, SATA_reg_type type)
{
    u32 regVal = 0;
    u32 tmpVal = 0;

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};


    regVal = sata_reg_addr;
    switch(type)
    {
        case SATA0_REG:
            regVal |= SATA_SEL_SATA0;
            tmpVal = readl(SATA_reg_base + SATA0_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal, SATA_reg_base + SATA0_HOST_CTL_REG);
            break;
        case SATA1_REG:
            regVal |= SATA_SEL_SATA1;
            tmpVal = readl(SATA_reg_base + SATA1_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal, SATA_reg_base + SATA1_HOST_CTL_REG);
            break;
        case DMAC_REG:
        default:
            regVal |= SATA_SEL_DMAC;				
            printf("Error!No byte access to DMA registers\n");
            while(1);
            break;
    }

    writel(regVal, SATA_reg_base + SATA_CTL_REG);

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};

    if(regVal&0x2) { debug("Readb error at end\n");return 0;};

    regVal = (u8)readl(SATA_reg_base + SATA_RDATA_REG);

    return (u8)regVal;
}



u32 SDSH_writew(void* SATA_reg_base, u32 sata_reg_addr, u16 value, SATA_reg_type type)
{
    u32 regVal = 0;
    u32 tmpVal = 0;

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};


    writel(value, SATA_reg_base + SATA_WDATA_REG);

    regVal = sata_reg_addr|0x80000000;
    switch(type)
    {
        case SATA0_REG:
            regVal |= SATA_SEL_SATA0;
            tmpVal = readl(SATA_reg_base + SATA0_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|1, SATA_reg_base + SATA0_HOST_CTL_REG);
            break;
        case SATA1_REG:
            regVal |= SATA_SEL_SATA1;
            tmpVal = readl(SATA_reg_base + SATA1_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|1, SATA_reg_base + SATA1_HOST_CTL_REG);
            break;
        case DMAC_REG:
        default:
            regVal |= SATA_SEL_DMAC;
            printf("Error!No word access to DMA registers\n");
            while(1);
            break;
    }

    writel(regVal, SATA_reg_base + SATA_CTL_REG);

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};

    if(regVal&0x2) { return 0;};

    return 0;
}

u16 SDSH_readw(void* SATA_reg_base, u32 sata_reg_addr, SATA_reg_type type)
{
    u32 regVal = 0;
    u32 tmpVal = 0;

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};


    regVal = sata_reg_addr;
    switch(type)
    {
        case SATA0_REG:
            regVal |= SATA_SEL_SATA0;
            tmpVal = readl(SATA_reg_base + SATA0_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|1, SATA_reg_base + SATA0_HOST_CTL_REG);
            break;
        case SATA1_REG:
            regVal |= SATA_SEL_SATA1;
            tmpVal = readl(SATA_reg_base + SATA1_HOST_CTL_REG)&0xfffffffc;
            writel(tmpVal|1, SATA_reg_base + SATA1_HOST_CTL_REG);
            break;
        case DMAC_REG:
        default:
            regVal |= SATA_SEL_DMAC;				
            printf("Error!No word access to DMA registers\n");
            while(1);
            break;
    }

    writel(regVal, SATA_reg_base + SATA_CTL_REG);

    while(((regVal = readl(SATA_reg_base + SATA_STATUS_REG))&0x1)== 1){};

    if(regVal&0x2) { return 0;};

    regVal = (u16)readl(SATA_reg_base + SATA_RDATA_REG);

    return (u16)regVal;
}




u32 SDSH_writesl(void * SATA_reg_base, u32 sata_reg_addr, u32*pbuf, u32 count, SATA_reg_type type)
{
    u32 i;
    u32 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        SDSH_writel(SATA_reg_base, sata_reg_addr, (u32)*p, type);
        p++;
    }

    return 0;
}

u32 SDSH_readsl(void* SATA_reg_base, u32 sata_reg_addr, u32 *pbuf, u32 count, SATA_reg_type type)
{
    u32 i;
    u32 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        *p = SDSH_readl(SATA_reg_base, sata_reg_addr, type);
        p++;
    }

    return 0;
}

u32 SDSH_writesb(void * SATA_reg_base, u32 sata_reg_addr, u8* pbuf , u32 count, SATA_reg_type type)
{
    u32 i;
    u8 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        SDSH_writeb(SATA_reg_base, sata_reg_addr, (u8)*p, type);
        p++;
    }

    return 0;
}

u32 SDSH_readsb(void* SATA_reg_base, u32 sata_reg_addr, u8 *pbuf, u32 count, SATA_reg_type type)
{
    u32 i;
    u8 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        *p = SDSH_readb(SATA_reg_base, sata_reg_addr, type);
        p++;
    }

    return 0;
}


u32 SDSH_writesw(void * SATA_reg_base, u32 sata_reg_addr, u16* pbuf , u32 count, SATA_reg_type type)
{
    u32 i;
    u16 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        SDSH_writew(SATA_reg_base, sata_reg_addr, (u16)*p, type);
        p++;
    }

    return 0;
}

u32 SDSH_readsw(void* SATA_reg_base, u32 sata_reg_addr, u16* pbuf, u32 count, SATA_reg_type type)
{
    u32 i;
    u16 *p = pbuf;

    for(i = 0; i < count; i++)
    {
        *p = SDSH_readw(SATA_reg_base, sata_reg_addr, type);
        p++;
    }

    return 0;
}



int sata_init(bd_t *bd)
{
   u32 temp,i;
   u32 phy_op_done_mask = 0;
   u32 max_retry_num = 1000;
   u32 bSSMode = 0;
   u32 bClkTypeExt = 1;
   u32 bClkFreq625 = 0;
   

    printf("SATA Host Controller Initializing\n");

    if(getenv("sata_ssmode"))
    {
        printf("SATA SS mode is %s\n", getenv("sata_ssmode"));
        if(strcmp(getenv("sata_ssmode"), "yes") == 0)
        {
            bSSMode = 1;
            printf("SS mode\n");
        }
    }

    if(getenv("sata_clktype"))
    {
        printf("SATA Clock Type is %s\n", getenv("sata_clktype"));
        if(strcmp(getenv("sata_clktype"), "int") == 0)
        {
            bClkTypeExt = 0;
            printf("Internal clock\n");
        }
    }

    if(getenv("sata_clkfreq"))
    {
        printf("Clock Freq is %sMHZ\n", getenv("sata_clkfreq"));
        if(strcmp(getenv("sataclkfreq"), "62.5") == 0)
        {
            bClkFreq625 = 1;
            printf("62.5Mhz\n");
        }
    }

    
    //Reset Hardware Module
    temp = readl(CG_RESET_REG + 0xd0000000);
    temp |= SATA_RESET_MASK;
    writel(temp, CG_RESET_REG + 0xd0000000);
    udelay(1000);
    temp &= ~SATA_RESET_MASK;
    writel(temp, CG_RESET_REG + 0xd0000000);	
    udelay(1000);

    //bring up SATA module on Xcode3
    //pull phy out of reset
#if 1//def XCODE3_PANTHER

    writel(0x0, XCODE3_SATA_REG_BASE + SATA_SFT_RESETS_REG);
    writel(0x3, CG_IP_PHY_CLK_CTRL + 0xd0000000);
    if(bSSMode)
        writel(0x3Ad93Ad9, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
    else
    {
        if(bClkFreq625)
            writel(0x3Ad93Ad9, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
        else
            writel(0x1A091A09, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
        
    }
    udelay(500);

    //reset the mpll and turn on the vixs pll
    if(bClkTypeExt)
    {
        if(bSSMode)
            writel(0x005c005c, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );
        else
        {
            if(bClkFreq625)
                writel(0x00500050, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );
            else
                writel(0x00540054, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );

        }

        writel(0x02002003, CG_PLL4_CTRL  + 0xd0000000);
        writel(0x0, CG_IP_PHY_CLK_CTRL + 0xd0000000);
    }
    else
    {
        temp = readl(CG_CLK_SRC_SEL9  + 0xd0000000);
        temp &= 0xFF00FFFF;
        if(bClkFreq625)
            temp |= 0x00600000;
        else
            temp |= 0x0000000;    
    
        writel(temp, CG_CLK_SRC_SEL9  + 0xd0000000);
    
        if(bSSMode)
            writel(0x005c005c, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );
        else
            writel(0x00540054, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );

        writel(0xf, CG_IP_PHY_CLK_CTRL + 0xd0000000);
        writel(0x02000003, CG_PLL4_CTRL  + 0xd0000000);
        writel(0xC, CG_IP_PHY_CLK_CTRL + 0xd0000000);
    }

    udelay(500);
    writel(0x3, XCODE3_SATA_REG_BASE + SATA_SFT_RESETS_REG);
    udelay(1600);    

    if(bSSMode)
    {
        writel(0x005d005d, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );
        udelay(800);
        writel(0xbAd9bAd9, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
        udelay(800);
    }
    else
    {
        if(bClkFreq625)
        {
            writel(0x00510051, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );
            udelay(800);
            writel(0xbAd9bAd9, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
            udelay(800);
        }
        else
        {
            writel(0x00550055, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG );
            udelay(800);
            writel(0x9A099A09, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
            udelay(800);
        }
    }
    writel(0xff, XCODE3_SATA_REG_BASE + SATA_SFT_RESETS_REG);
    udelay(200);
    writel(0xf3, XCODE3_SATA_REG_BASE + SATA_SFT_RESETS_REG);
    udelay(200);
    writel(0xff, XCODE3_SATA_REG_BASE + SATA_SFT_RESETS_REG);
    udelay(800);
#else//Jaguar
    writel(0x430c430c, XCODE3_SATA_REG_BASE + SATA_PHY_ANALOG_CTRL_REG);
    writel(0x1A091A09, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
    writel(3, XCODE3_SATA_REG_BASE + SATA_SFT_RESETS_REG);
    udelay(1000);
	//setup analog part
    writel(0x00550055, XCODE3_SATA_REG_BASE + SATA_PHY_MPLL_CTRL_REG);
    udelay(1000);

    writel(0x1A091A09, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
    udelay(1000);
    writel(0x9A099A09, XCODE3_SATA_REG_BASE + SATA_PHY_TXRX_CTRL_REG);
    udelay(1000);
    writel(0xff, XCODE3_SATA_REG_BASE + SATA_SFT_RESETS_REG);
    udelay(1000);

#endif

    //for U-boot, don't need to enable SATA interrupt mask
    if(0)
    {   
        u32 temp;
        temp = readl(MIPS4_INTERRUPT_MASK  + 0xd0000000);
        temp |= MIPS4_INTERRUPT_MASK_SATA_INT_MASK;
        writel(temp, MIPS4_INTERRUPT_MASK + 0xd0000000);
    }
   
   //[JLWANG] TODO: specify the SATA speed( Gen1 or Gen2): reset port

    udelay(1000);
    SDSH_writeb((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, 0x21, SATA0_REG);
    udelay(1000);
    SDSH_writeb((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, 0x20, SATA0_REG);
    udelay(1000);


     //do COMRESET here
     temp = SDSH_readl((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, SATA0_REG);
     temp = (temp&0x0f0)|0x301;
     SDSH_writel((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, temp, SATA0_REG);
     //flush
     SDSH_readl((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, SATA0_REG);

     udelay(10000);

     temp = SDSH_readl((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, SATA0_REG);
     temp = (temp&0x0f0)|0x300;
     SDSH_writel((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, temp, SATA0_REG);

     udelay(100000);
     
    i = 0;

    do 
    {
        temp = (u32)SDSH_readb((void*)XCODE3_SATA_REG_BASE,  HSATA_SCR0_REG, SATA0_REG);
        if((temp & 0x3) == 0x3)
        {
            printf("SATA0: Device exists\n");
            XCode3_SATA_dev_available |= 1;
            break;
        }
        udelay(5000);
        i++;
    }while(i < max_retry_num);

    udelay(1000);
    SDSH_writeb((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, 0x21, SATA1_REG);
    udelay(1000);
    SDSH_writeb((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, 0x20, SATA1_REG);
    udelay(1000);

     //do COMRESET here
     temp = SDSH_readl((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, SATA1_REG);
     temp = (temp&0x0f0)|0x301;
     SDSH_writel((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, temp, SATA1_REG);
     //flush
     SDSH_readl((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, SATA1_REG);

     udelay(10000);

     temp = SDSH_readl((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, SATA1_REG);
     temp = (temp&0x0f0)|0x300;
     SDSH_writel((void*)XCODE3_SATA_REG_BASE, HSATA_SCR2_REG, temp, SATA1_REG);

     udelay(100000);

    i = 0;

    do
    {
        temp = (u32)SDSH_readb((void*)XCODE3_SATA_REG_BASE,  HSATA_SCR0_REG, SATA1_REG);
        if((temp & 0x3) == 0x3)
        {
            printf("SATA1: Device exists\n");
            XCode3_SATA_dev_available |= (1<<1);
            break;
        }
        udelay(5000);
        i++;
    }while(0/*i < max_retry_num*/);
    printf("Do Ide initialization\n");

    ide_init();

    printf("SATA Host Controller Initialization End\n");
    g_XC3_Sata_Initialized = 1;
    return (0);
}
#endif /* (CONFIG_COMMANDS & CFG_CMD_SATA) */
