#include <config.h>

#if defined(CFG_PARTITION_TABLE_IN_NAND)

#include <common.h>
#include <malloc.h>
#include <linux/mtd/nand.h>
#include <asm/vixs_nand.h>
//#include <console.h>

#define STR_LEN_MAX 32
#define PARTITION_MAX_BID3300 64

typedef struct device_info_bid3300
{
	char name[STR_LEN_MAX];
	unsigned int size;
	unsigned int phys;
	void *virt;
	void *cached;
	int bankwidth;
	unsigned int used;
} device_info_bid3300;

typedef struct partition_info_bid3300
{
	char name[STR_LEN_MAX]; /* identifier string */
	unsigned int offset;    /* offset within the master MTD space. */
	unsigned int size;      /* partition size */
	char filename[STR_LEN_MAX];  /*file name */
	unsigned int filesize;        /* file size */
	unsigned int sw_ver;          /* software version */
	unsigned char used;           /* Is used ? */
	unsigned char valid;          /* Is valid ? */
	unsigned int mask_flags;       /* master MTD flags to mask out for this partition */
} partition_info_bid3300;

struct partmap_info_bid3300
{
	unsigned int magic;
	unsigned int cur_epk_ver;
	unsigned int old_epk_ver;
	unsigned char npartition;
	struct device_info_bid3300 dev;
	struct partition_info_bid3300 partition[PARTITION_MAX_BID3300];
};


extern struct nand_chip nand_dev_desc[];

extern int nand_rw (struct nand_chip* nand, int cmd,
	    size_t start, size_t len,
	    size_t * retlen, u_char * buf);

struct partmap_info_bid3300 *nand_read_partition_table(void)
{
    u_char *buf = NULL;
    size_t retlen;
    struct partmap_info_bid3300 *part;
    int ret;

    buf = malloc(CFG_PARTITION_TABLE_SIZE);
    if(!buf) {
        printf("Can't allocate buffer for partition table, size: 0x%x\n", CFG_PARTITION_TABLE_SIZE);
        return NULL;
    }

    ret = nand_rw(nand_dev_desc+0, NANDRW_READ, CFG_PARTITION_TABLE_OFFSET, 
                CFG_PARTITION_TABLE_SIZE, &retlen, buf);
    if(ret) {
        printf("Read nand failed, offset 0x%x\n", CFG_PARTITION_TABLE_OFFSET);
        free(buf);
        return NULL;
    }

    part = (struct partmap_info_bid3300 *)buf;
    if (part->magic != CFG_PARTITION_MAGIC) {
        free(buf);
        printf("Partition magic doesn't match. Partition table not available\n");
        return NULL;
    }

    return part;
}

int nand_print_partition_table(void)
{
    int i;
    struct partmap_info_bid3300 *part; 

    part = nand_read_partition_table();
    if(!part) {
        return 1;
    }

	printf("nPartition = %d magic %x\n", part->npartition, part->magic);
	for (i =0 ; i< part->npartition; i++)
	{
		printf("partition[%d] : %s offset (%x) size(%x)\n", i, part->partition[i].name,
							part->partition[i].offset, part->partition[i].size);

	}

    free(part);
    return 0;
}

int get_partition_address_by_name(struct partmap_info_bid3300 *part, char*name, ulong *offset, ulong *size)
{
    int i;

    for (i =0 ; i< part->npartition; i++)
	{
        if(strcmp(name,part->partition[i].name) == 0) {
            *offset = part->partition[i].offset;
            *size = part->partition[i].size;
            return 0;
        }
	}

    return 1;
}

int nand_read_partition(char *name,ulong addr)
{
    int ret;
    struct partmap_info_bid3300 *part;
    ulong offset, size;
    size_t retlen;

    part = nand_read_partition_table();
    if(!part) {
        return 1;
    }

    ret = get_partition_address_by_name(part, name, &offset, &size);
    if(ret) {
        printf("Can't find partition %s\n", name);
        free(part);
        return 1;
    }

    ret = nand_rw(nand_dev_desc+0, NANDRW_READ, offset, 
                size, &retlen, (u_char *)addr);
    if(ret)
        printf("read partition failed\n");

    free(part);
    return ret;
}

int nand_write_partition(char *name,ulong addr)
{
    int ret;
    struct partmap_info_bid3300 *part;
    ulong offset, size;
    size_t retlen;

    part = nand_read_partition_table();
    if(!part) {
        return 1;
    }

    ret = get_partition_address_by_name(part, name, &offset, &size);
    if(ret) {
        printf("Can't find partition %s\n", name);
        free(part);
        return 1;
    }

    ret = nand_erase (nand_dev_desc + 0, offset, size, 0);
    if(ret) {
        printf("Nand erase failed: offset %x size %x\n", offset, size);
        free(part);
        return 1;
    }


    printf("Write offset %x with addr %x\n", offset, addr);
    ret = nand_rw(nand_dev_desc+0, NANDRW_WRITE, offset, 
                size, &retlen, (u_char *)addr);

    if(ret)
        printf("write partition failed\n");

    free(part);
    return ret;
}
#endif //CFG_PARTITION_TABLE_IN_NAND
