/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Aug 17 , 2005
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/cpu/arc700/serial.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

#include <common.h>
#include <asm/serial.h>
#include <asm/viper.h>		/* for XC4 */


#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))

static xcode_uart *uart_addr[NUM_PORTS] = UART_BASEADDR;

int DEFAULT_CONSOLE_PORT = 0;
static char printbuf[1024];
static int printbuf_len = 0;
int stdlog = -1;

/* Commenting out this line will use Register mode, but try not to do it. */
//#define UART_DMA_MODE

#ifdef UART_DMA_MODE
#define TX_BUF_SIZE 1
#define RX_BUF_SIZE 4096
static char __attribute__((aligned(16))) tx_buf[TX_BUF_SIZE];
static char __attribute__((aligned(16))) rx_buf[RX_BUF_SIZE];
#endif

unsigned int get_uart_clk(void)
{
	int hueclk_sel, div_int;
	unsigned int freq=144;

	/* When CG1_CLK_SRC_EN0/HUECLK_SRC_EN is 0, clock speed is 27MHz */
	hueclk_sel = (readl(XC_SOC_PROC_MMREG_BASE+CG1_CLK_SRC_EN0)) & HUECLK_SRC_EN_MASK;
	if( !hueclk_sel )
		return 27000000;

	hueclk_sel = (readl(XC_SOC_PROC_MMREG_BASE+CG1_CLK_SRC_SEL3) & HUECLK_SRC_SEL_MASK) >> HUECLK_SRC_SEL_SHIFT;

	switch( hueclk_sel>>4 ){
	case 0:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG1_PLL3_FREQ) & CG1_PLL3_FREQ_FB_DIV_INT_MASK) >> CG1_PLL3_FREQ_FB_DIV_INT_SHIFT;
		freq = (div_int * 27) / 3;
		break;
	case 1:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG1_PLL4_FREQ) & CG1_PLL4_FREQ_FB_DIV_INT_MASK) >> CG1_PLL4_FREQ_FB_DIV_INT_SHIFT;
		freq = (div_int * 27) / 5;
		break;
	case 2:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG1_PLL5_FREQ) & CG1_PLL5_FREQ_FB_DIV_INT_MASK) >> CG1_PLL5_FREQ_FB_DIV_INT_SHIFT;
		freq = (div_int * 27) / 4;
		break;
	case 3:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG1_PLL4_FREQ) & CG1_PLL4_FREQ_FB_DIV_INT_MASK) >> CG1_PLL4_FREQ_FB_DIV_INT_SHIFT;
		freq = (div_int * 27) / 6;
		break;
	case 4:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG1_PLL5_FREQ) & CG1_PLL5_FREQ_FB_DIV_INT_MASK) >> CG1_PLL5_FREQ_FB_DIV_INT_SHIFT;
		freq = (div_int * 27) / 5;
		break;
	case 5:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG1_PLL5_FREQ) & CG1_PLL5_FREQ_FB_DIV_INT_MASK) >> CG1_PLL5_FREQ_FB_DIV_INT_SHIFT;
		freq = (div_int * 27) / 10;
		break;
	case 6:
		div_int = (readl(XC_SOC_PROC_MMREG_BASE+CG1_PLL5_FREQ) & CG1_PLL5_FREQ_FB_DIV_INT_MASK) >> CG1_PLL5_FREQ_FB_DIV_INT_SHIFT;
		freq = (div_int * 27) / 12;
		break;
	case 0xf9:
	case 0xfa:
	case 0xfb:
		/* in real chip, those values shouldn't be used, but if they are, fix them */
		arc_write_uncached_32((unsigned int *)0x80000000, 0xdeadbeef);
		break;
	}

	return (freq * 1000000);
}

/*
 * Sets baudarate
 */
void serial_setbrg (int port, int baudrate)
{
	xcode_uart *uart = uart_addr[port];
	unsigned int divisor, clk_speed=30000000;
	int tmp;
#ifdef UART_DMA_MODE
	volatile char *xmit_buf_ptr = tx_buf[port];
	volatile char *ring_buf_ptr = rx_buf[port];
#endif

	/* take UART out of soft reset */
	uart->ctrl_reg &= ~UART0_CTRL_REG_SOFT_RESET_MASK;
	
#ifndef VIPER_FPGA
	clk_speed = get_uart_clk();
#endif
	divisor = clk_speed*10/((baudrate << 4)*UART_CLK_DIVIDER);
	tmp = (clk_speed/(baudrate<<4)*UART_CLK_DIVIDER) * 10;
	if(divisor - tmp >= 5){
		divisor /= 10;
		++divisor;
	}
	else
		divisor /= 10;

	/* enable divisor access */
	uart->linectrl |= UART0_LINE_CTRL_DIVISOR_ACCESS_MASK;
	/* update divisor */
	uart->data = divisor & 0x000000FF; /* rx_tx_divlsb reg */
	uart->divmsb = (divisor >> 8) & 0x000000FF;
	/* lock divisor access */
	uart->linectrl &= ~UART0_LINE_CTRL_DIVISOR_ACCESS_MASK;
#ifdef UART_DMA_MODE
	/* set uart at DMA mode */
	uart->ctrl_reg &= ~UART0_CTRL_REG_CTRL_TX_MASK;
	uart->ctrl_reg &= ~UART0_CTRL_REG_CONTROL_RX_MASK;
	uart->xmit_base = (int)xmit_buf_ptr;
	uart->ring_base = (int)ring_buf_ptr;
	uart->ring_size = RX_BUF_SIZE - 1;
	/* set dma mode bit as DWORD */
	uart->fifoctrl &= ~DMA_MODE_BYTE;
	uart->info;
#else
	/* set uart at register mode */
	uart->ctrl_reg |= UART0_CTRL_REG_CTRL_TX_MASK;
	uart->ctrl_reg |= UART0_CTRL_REG_CONTROL_RX_MASK;
	/* set trigger level as 1 byte */
	uart->fifoctrl &= 0xffffff3f;  //only for register mode
	uart->info;
#endif
	/* character length is 8bits */
	uart->linectrl |= UART0_LINE_CTRL_CHAR_LEN_MASK;
	/* transmit 1 stop bit */
	uart->linectrl &= ~UART0_LINE_CTRL_STOP_BITS_MASK;
	/* no parity */
	uart->linectrl &= ~UART0_LINE_CTRL_PARITY_EN_MASK;
	/* enable parity: EVEN */
//	uart->linectrl |= UART0_LINE_CTRL_PARITY_EN_MASK;
	/* enable hardware flow control: RTS/CTS */
//	uart->ctrl_reg |= UART0_CTRL_REG_HW_FLOW_CTRL_EN_MASK;
}

/*
 * Intialise the serial port with given baudrate
 */
int serial_init (void)
{
	unsigned int reg;
	int i;

	arc_write_uncached_32((unsigned int *)0x80000004, 0x31);
#if NUM_PORTS > 1
	/* set GPIO_H GPIO_MODE_SEL0~3 to HW mode for UART */
	reg = readl(XC_SOC_PROC_MMREG_BASE+GPIO_H_CTRL);
	reg &= 0x0fffffff;
	writel(reg, XC_SOC_PROC_MMREG_BASE+GPIO_H_CTRL);
#else
	writel((~(GPIO_H_CTRL_GPIO_MODE_SEL0_MASK|GPIO_H_CTRL_GPIO_MODE_SEL1_MASK)) & readl(XC_SOC_PROC_MMREG_BASE+GPIO_H_CTRL), XC_SOC_PROC_MMREG_BASE+GPIO_H_CTRL);
#endif

	/* reset and activate UART */
	writel(UART_RESET_MASK | readl(XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1), XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1);
	writel(~UART_RESET_MASK & readl(XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1), XC_SOC_PROC_MMREG_BASE+CG_RESET_REG1);
	/* workaround for EPR 367: set UART to register mode and put into soft reset till the real use */
	for(i=0; i<4; i++){
		if(i<2)
			reg = XC_SOC_PROC_MMREG_BASE + (0x50*i);
		else
			reg = XC_SOC_PROC_MMREG_BASE + (0x50*i) + 0x60;
		writel(UART0_CTRL_REG_CTRL_TX_MASK | readl(reg+UART0_CTRL_REG), reg+UART0_CTRL_REG);
		writel(UART0_CTRL_REG_CONTROL_RX_MASK | readl(reg+UART0_CTRL_REG), reg+UART0_CTRL_REG);
		writel(UART0_CTRL_REG_SOFT_RESET_MASK | readl(reg+UART0_CTRL_REG), reg+UART0_CTRL_REG);
	}

	serial_setbrg(DEFAULT_CONSOLE_PORT, DEFAULT_CONSOLE_BAUD_RATE);

	return (0);
}


void xcode_putc(char c)
{
	xcode_uart *uart = uart_addr[DEFAULT_CONSOLE_PORT];

	if (c == '\n') 
		xcode_putc('\r');

#ifdef UART_DMA_MODE
	tx_buf[0] = c;
	flush_dcache_range((unsigned int)tx_buf, (unsigned int)tx_buf);
	uart->xmit_size = 0;
#else
	uart->data = c;
	uart->info;
#endif
	
	/* wait until the char is transmitted */
	while(!(uart->int_status & UART0_INT_MASK_TRANSMIT_DONE_EMPTY_MASK));
	uart->int_status |= UART0_INT_MASK_TRANSMIT_DONE_EMPTY_MASK;
	uart->info;
}

/*
 * Output a single byte to the serial port.
 */
void serial_putc (const char c)
{
	/* JIRA 1168 */
	if(stdlog == 0)
		return;

	if(stdlog < 0) {
		printbuf[printbuf_len++] = c;
		printbuf[printbuf_len] = 0;
		return;
	}

	if(printbuf_len > 0) {
		int i;
		for(i=0; i<printbuf_len; i++)
			xcode_putc(printbuf[i]);
		printbuf_len = 0;
	}

	xcode_putc(c);
	return;
}

/*
 * Read a single byte from the serial port. Returns 1 on success, 0
 * otherwise 0.
 */
int serial_tstc (void)
{
	xcode_uart *uart = uart_addr[DEFAULT_CONSOLE_PORT];

	/* Wait till character is placed in fifo */
	if(uart->int_status & UART0_INT_STATUS_RECIEVE_DATA_RDY_MASK)
		return (1);
	return (0);
}

/* This function has nothing to do with UART. Because uboot is a single thread program, to
  * regularly monitor certain status, we have to add this feature in serial_getc as this is
  * the place holding the program.
  * Use init_xcode_monitor in lib_arc/board.c to initialize the status if needed.
  */
static void xcode_monitor(void)
{

}

/*
 * Read a single byte from the serial port. Returns 1 on success, 0
 * otherwise. When the function is succesfull, the character read is
 * written into its argument c.
 */
int serial_getc (void)
{
	xcode_uart *uart = uart_addr[DEFAULT_CONSOLE_PORT];
	unsigned char ch;
#ifdef UART_DMA_MODE
	unsigned int head, tail;
#endif
	/* Wait till character is placed in fifo */
	while(!(uart->int_status & UART0_INT_STATUS_RECIEVE_DATA_RDY_MASK))
		xcode_monitor();

	/* Also check for overflow errors */
	if(uart->line_status & UART0_LINE_STATUS_OVERRUN_MASK){
		uart->line_status |= UART0_LINE_STATUS_OVERRUN_MASK;
		uart->info;
		ch = 0;
	}
	else{
#ifdef UART_DMA_MODE
		head = uart->ring_head;
		tail = uart->ring_tail;
		if(head != tail) {
			inv_dcache_range((unsigned int)rx_buf+tail, (unsigned int)rx_buf+tail);
			ch = rx_buf[uart->ring_tail];
			tail = (tail + 1) % RX_BUF_SIZE;
			uart->ring_tail = tail;
		}
#else
		ch = uart->data & 0xff;
#endif
	}
	uart->int_status |= UART0_INT_STATUS_RECIEVE_DATA_RDY_MASK;
	uart->info;

	return ((int)ch);
}

void serial_puts (const char *s)
{
	while (*s) {
		serial_putc (*s++);
	}
}

