#include <common.h>
#include <i2c.h>
#include <asm/vixs_hmi.h>

#define I2C_MAX_SIZE	64
#define NUM_OF_MENU		12
#define ITEM_LEN		16
#define DATA_LEN		(NUM_OF_MENU * ITEM_LEN)


unsigned int menu_rd_crc(void)
{	
	int ret;	
	u32	value;	

	ret = i2c_read(0x50,0x3c00, 2,&value,4);
	if (ret)
		return ret;
	else
		return value;	
}

int menu_create(menu_t *menu)
{
	int i;
	u8 *temp = NULL;
	item_t item;
	u8 name[13][12] = {
		{"bootname"},
		{"bootname_1"},
		{"bootname_2"},
		{"bootname_3"},		
		{"bootname_4"},
		{"bootname_5"},
		{"bootname_6"},
		{"bootname_7"},
		{"bootname_8"},
		{"bootname_9"},
		{"bootname_10"},
		{"bootname_11"},		
		{NULL},
	};
	menu->menu_ptr = malloc(DATA_LEN);
	if (!menu->menu_ptr) {
		debug("MENU BUFFER ALLOCATE FAIL\n");
		return -1;
	}		
			
	for(i=0; i<12; i++) {
		memset((void*)item.name, 0x20, 14);
		temp = getenv(name[i]);
		if (temp != NULL) {			
			if(strlen(temp) > 11) {
				printf("Boot name should less than 11 characters\n");
				item.status = 0;				
			} else {		
				strcpy(item.name, temp);
				item.status = 2;
			}
		} else {			
			item.status = 0;			
		}
		item.number = i;
		temp = NULL;
		memcpy((void*)(menu->menu_ptr + i*ITEM_LEN), (void*)&item, ITEM_LEN);
		debug("&item: %p; buffer:%p\n", &item, menu->menu_ptr);
		debug("Menu %d - %s, status: %x\n",i, item.name, item.status);
	}

	
	//crc32(menu->crc32, (u8*)menu->menu_ptr, 12*16);
	debug("CRC32: %x\n", menu->crc32);	
	return 0;
}

int menu_write(menu_t *menu)
{
	u8* buffer = NULL;
	int ret,i,j;
	u32 temp;
		
	temp = menu->crc32;
	ret = i2c_write(0x50,0x3c00,2,&temp,4);
	if(ret) {
		printf("Fail to write CRC data\n");
		goto write_exit;
	}
	j = 100;
	while(j){
		udelay(1000);
		j--;
	}
	
	if ((buffer = malloc(I2C_MAX_SIZE/2)) == NULL) {
		debug("MENU WRITE BUFFER ALLOC FAIL\n");
		goto write_exit;
	}

	for(i=0; i<DATA_LEN; i+=32) {
		memcpy(buffer,&menu->menu_ptr[i], 32);
		ret = i2c_write(0x50,(0x4000+(i<<8)),2,buffer,I2C_MAX_SIZE/2);
		
		if(ret) {
			printf("Fail to write data\n");
			goto write_exit;
		}
		// wait for write finish 
		j = 100;
		while(j){
			udelay(1000);
			j--;
		}
	}

write_exit:
	if (menu->menu_ptr != NULL)
		free(menu->menu_ptr);
	if (buffer != NULL)
		free(buffer);
	return ret;
}

int menu_read(menu_t *menu)
{
	u8* buffer = NULL;
	int ret,i;
	u32 temp;
	/*	
	temp = menu->crc32;
	ret = i2c_write(0x50,0x3c00,2,temp,4);
	if(ret)
		goto write_exit;
	*/
	buffer = malloc(DATA_LEN);
	printf("READ BUFFER: %p\n", buffer);
	for (i=0; i<DATA_LEN; i+=64) {
		ret = i2c_read(0x50,(0x4000 + (i<<8)),2,buffer+i,I2C_MAX_SIZE);
		if(ret)
			goto read_exit;
	}
read_exit:
	if (buffer != NULL)
		free(buffer);
	return ret;
}

