/*
 * ViXS System Inc. 
 * Copyright 2011-05-24
 * Emerson Chi echi@vixs.com
 */
#include <malloc.h>
#include <exports.h>
#include <common.h>	
#include <asm/errno.h>
#include <asm/viper.h>
#include <asm/vixs_secu.h>

//#define DEBUG
#ifdef DEBUG
#define debug(fmt, args... )	printf("[%s]"fmt, __func__, ##args)
#else
#define debug(fmt, args... )
#endif

#define  RSA_CLIENT_CTRL 0x1800    
#define MOD_WEN_MASK 0x00000001    
#define MOD_WEN_SHIFT 0    
#define EXP_WEN_MASK 0x00000002    
#define EXP_WEN_SHIFT 1    
#define DAT_WEN_MASK 0x00000004    
#define DAT_WEN_SHIFT 2    
#define REG_IDX_MASK 0x000003F0    
#define REG_IDX_SHIFT 4

#define  RSA_CLIENT_READ 0x1808    
#define RSA_CLIENT_READ_DATA_MASK 0xFFFFFFFF    
#define RSA_CLIENT_READ_DATA_SHIFT 0

#define  RSA_CLIENT_WRITE 0x1804    
#define RSA_CLIENT_WRITE_DATA_MASK 0xFFFFFFFF    
#define RSA_CLIENT_WRITE_DATA_SHIFT 0

#define  RSA_ENGINE_CTRL 0x1814
#define RSA_ENGINE_CTRL_BUSY_MASK 0x00000008    
#define RSA_ENGINE_CTRL_BUSY_SHIFT 3

#define  RSA_INT_STATUS 0x181C
#define RSA_DONE_MASK 0x00000002    
#define RSA_DONE_SHIFT 1

#define readl(addr)			(*((volatile unsigned int*)(MMREG_BASE + (addr))))
#define writel(addr, data) 	(*((volatile unsigned int*)(MMREG_BASE + (addr))) = (data))
#define reverse_endian(x)	(((x&0xff)<<24) | ((x&0xff00)<<8) | ((x&0xff0000)>>8) | ((x&0xff000000)>>24))

#define MAX_DMASIZE				(64 * 1024)

typedef struct _dmadesc
{
    uint64 src_addr;
    uint64 dst_addr;
    uint32 cmd;
    uint32 cw;
    uint32 idxs;
    uint32 ts_cw;
    uint32 ubase_addr;
    uint32 vbase_addr;
    uint64 ext1;
    uint64 ext2;
    uint32 reserved1;
    uint32 reserved2;
} dmadesc __attribute__((aligned(32)));

// RSA decrypt signature using public key
// blocks until RSA calculation done
static int  calc_sig(const uint32* const sig, uint32 len)
{
    uint32 v, i;
    uint32 modulus[RSA2048>>2];
	uint32 public_key[8>>2];
	
	if ((sig == NULL) ||((len != RSA1024) && (len != RSA2048))){
		debug("Invalid input %p = %08x\n", sig, *sig);
		return -EINVAL;			
	}
	memset ((void*)modulus, 0x0, sizeof(modulus));
	// Public Exponent
    {
        public_key[0] = 0x00010001;
        public_key[1] = 0;

        modulus[0] = 0x1799f765;
        modulus[1] = 0x9a76b2ae;
        modulus[2] = 0x552c4213;
        modulus[3] = 0x7ebc9dd2;
        modulus[4] = 0xb234b978;
        modulus[5] = 0xe0603119;
        modulus[6] = 0x011f4094;
        modulus[7] = 0xac748157;
        modulus[8] = 0xf9ad7ca4;
        modulus[9] = 0xd1d7882a;
        modulus[10] = 0x890f282e;
        modulus[11] = 0xc9e94ce1;
        modulus[12] = 0xb3e025dc;
        modulus[13] = 0x772a9fb4;
        modulus[14] = 0x113481da;
        modulus[15] = 0xfcf2b920;
        modulus[16] = 0xba7098a8;
        modulus[17] = 0xa642a551;
        modulus[18] = 0x43d93817;
        modulus[19] = 0xfd4537be;
        modulus[20] = 0xcedbf96e;
        modulus[21] = 0xfca58cfc;
        modulus[22] = 0x117ee7be;
        modulus[23] = 0x80894a5a;
        modulus[24] = 0xec47707e;
        modulus[25] = 0x60735e5b;
        modulus[26] = 0xffbd643b;
        modulus[27] = 0xcb9fc040;
        modulus[28] = 0x52f88394;
        modulus[29] = 0x09ec060e;
        modulus[30] = 0x0ef91682;
        modulus[31] = 0x2e79207d;
        modulus[32] = 0x17470f68;
        modulus[33] = 0x89fc6e0d;
        modulus[34] = 0x523c8f19;
        modulus[35] = 0xdba31255;
        modulus[36] = 0x63d88c5f;
        modulus[37] = 0x9012d618;
        modulus[38] = 0xd141490a;
        modulus[39] = 0x2e897648;
        modulus[40] = 0x92008284;
        modulus[41] = 0x8effca5d;
        modulus[42] = 0x04909996;
        modulus[43] = 0xd285786d;
        modulus[44] = 0xb7b00e45;
        modulus[45] = 0xbc548650;
        modulus[46] = 0x727ef5e4;
        modulus[47] = 0x2ad2e931;
        modulus[48] = 0xb8dee209;
        modulus[49] = 0xea989eea;
        modulus[50] = 0x442aa014;
        modulus[51] = 0xbeb596c8;
        modulus[52] = 0x51de92dd;
        modulus[53] = 0xa95ae7d5;
        modulus[54] = 0xb112567b;
        modulus[55] = 0x94fb035b;
        modulus[56] = 0x0e5097ad;
        modulus[57] = 0x04d5257f;
        modulus[58] = 0xe8e92a0a;
        modulus[59] = 0x17ed639e;
        modulus[60] = 0x1dc7dc9f;
        modulus[61] = 0xd7c2e252;
        modulus[62] = 0x7e8005db;
        modulus[63] = 0x28d85bbe;
    }
    
    writel(RSA_CLIENT_CTRL, 0);
    
    // modulus
    for (i = 0; i < 64; i++)
    {
        v = MOD_WEN_MASK | i << REG_IDX_SHIFT;
        writel(RSA_CLIENT_CTRL, v);
        while (readl(RSA_CLIENT_CTRL) != v);
		
        writel(RSA_CLIENT_WRITE, modulus[i]);
    }        
    
    // public key
    for (i = 0; i < 64; i++)
    {
        v = EXP_WEN_MASK | i << REG_IDX_SHIFT;
        writel(RSA_CLIENT_CTRL, v);
        while (readl(RSA_CLIENT_CTRL) != v);
        
        if (i < 2)
            writel(RSA_CLIENT_WRITE, public_key[i]);
        else
            writel(RSA_CLIENT_WRITE, 0);
    }
            
    // write sig into RSA data regs
    for (i = 0; i < (len>>2); i++)
    {
        v = DAT_WEN_MASK | i << REG_IDX_SHIFT;
        writel(RSA_CLIENT_CTRL, v);
        while (readl(RSA_CLIENT_CTRL) != v);
        //debug ("mod[%d] = %08x <-> sig[%d] = %08x \n",i, modulus[i], 63 - i, reverse_endian(sig[63-i]));
        writel(RSA_CLIENT_WRITE, reverse_endian(sig[63-i]));		
    }        
            
    writel(RSA_CLIENT_CTRL, 0);
    while (readl(RSA_CLIENT_CTRL) != 0);
    
    // go go
    while (readl(RSA_ENGINE_CTRL) & RSA_ENGINE_CTRL_BUSY_MASK);
    writel(RSA_ENGINE_CTRL, 0);

    while (readl(RSA_ENGINE_CTRL) & RSA_ENGINE_CTRL_BUSY_MASK);
    while (! (readl(RSA_INT_STATUS) & RSA_DONE_MASK));
    writel(RSA_INT_STATUS, RSA_DONE_MASK);

	return 0;    
}


// pad the end of the image to SHA-256 specifications
// destroys data past the end of the image
// assume the length is at least DWORD-aligned
// returns the amount of padding added
static uint32 pad_for_SHA(uint32* const p, const uint32 len)
{
    uint8* const buf = (uint8* const)p;
    void* porig;
    uint32 n, i;
    
    // length of message in bits
    i = len;
    n = i << 3;

    porig = &buf[i];
    // now pad -- 1b and then 0b until at bit 448 mod 512 (56/64 in bytes)
    buf[i++] = 0x80;
    while ((i % (512>>3)) != (448>>3))
        buf[i++] = 0x00;

    // write length field -- 8 bytes listing the length of the message in bits
    // assume a 32-bit length val (only need more if image > ~512MB)
    buf[i++] = 0x00;
    buf[i++] = 0x00;
    buf[i++] = 0x00;
    buf[i++] = 0x00;
    buf[i++] = (n >> 24) & 0xFF;
    buf[i++] = (n >> 16) & 0xFF;
    buf[i++] = (n >>  8) & 0xFF;
    buf[i++] = (n      ) & 0xFF;
    
    flush_dcache_range((uint32)porig, (uint32)porig + (512>>3));
	debug ("Pad lenth %d\n", i-len);
    return(i - len);
}

#if 0 /* HW SHA256 */
static dmadesc desc __attribute__((aligned(32)));
// calculate SHA-256 hash of image
// blocks until done
// image comes in as a pointer to start of image and length (in bytes) of image
static void calc_hash(uint8* pbuf, int32 len)
{
	uint32 dma_size = 0;	
	uint64 tmp = 0;
	tmp += (uint32)pbuf;
	

	

	while (len > 0) {
		if (len > MAX_DMASIZE)
			dma_size = MAX_DMASIZE;
		else
			dma_size = len;
    	memset(&desc, 0, sizeof(dmadesc));
		
    	desc.src_addr = tmp;
    	// dest is don't care
    	if ((uint32)tmp == (uint32)pbuf)
    		desc.cmd = (1 << 31) | DMA_DIRECT_DMACMD_A_SRCSPACE_MASK | DMA_DIRECT_DMACMD_A_STREAMCIPHER_START_MASK | 
    					((dma_size & DMA_DIRECT_DMACMD_A_BYTECNT_MASK) << DMA_DIRECT_DMACMD_A_BYTECNT_SHIFT);
		else
			desc.cmd = (1 << 31) | DMA_DIRECT_DMACMD_A_SRCSPACE_MASK | 
						((dma_size & DMA_DIRECT_DMACMD_A_BYTECNT_MASK) << DMA_DIRECT_DMACMD_A_BYTECNT_SHIFT);		
    	desc.cw = (1 << 20);    // SHA-256
    	debug ("DMA desc cmd: %x CW: %x\n", desc.cmd, desc.cw);
		flush_dcache_range((uint32)(&desc), (uint32)(&desc + sizeof(dmadesc)));
    	debug ("DMA decs: %p Src: %x size %08x rem %08x\n", (uint32)(&desc) & ~0x80000000, (uint32)desc.src_addr, dma_size, len);
    	writel(DMA_DQ_PTR0, (uint32)(&desc) & ~0x80000000);  
    	while (readl(DMA_STATUS) & INDIRECT0_MASK);
		tmp += dma_size;
		len -= dma_size;
	}

}
#else /* Use Software SHA256 */
/* 
 * Calculate target image hash code -SHA256 
 * SHA256 block size is 512 bits /64 bytes
 * the size should be aligned with 64 bytes include padding.
 */
 // SHA-256
// operation macros
#define TRUNC32(x) ((x) & 0xFFFFFFFFUL)
#define Ch(x,y,z)  (((x) & (y)) ^ ((~(x)) & (z)))
#define Maj(x,y,z) (((x) & (y)) ^ ((x) & (z)) ^ ((y) & (z)))
#define SHRn(x,n)  TRUNC32(((x)>>n))
#define RORn(x,n)  (((x)>>n)|((x)<<(32-n)))
#define ROLn(x,n)  (((x)<<n)|((x)>>(32-n)))
#define SIG0(x)    ((RORn((x),2)  ^ RORn((x),13) ^ RORn((x),22)))
#define SIG1(x)    ((RORn((x),6)  ^ RORn((x),11) ^ RORn((x),25)))
#define WRO0(x)    ((RORn((x),7)  ^ RORn((x),18) ^ SHRn((x),3)))
#define WRO1(x)    ((RORn((x),17) ^ RORn((x),19) ^ SHRn((x),10)))
//constants
const uint32 SHA256_Constant[64] = {
    0x428a2f98UL,0x71374491UL,0xb5c0fbcfUL,0xe9b5dba5UL,
    0x3956c25bUL,0x59f111f1UL,0x923f82a4UL,0xab1c5ed5UL,
    0xd807aa98UL,0x12835b01UL,0x243185beUL,0x550c7dc3UL,
    0x72be5d74UL,0x80deb1feUL,0x9bdc06a7UL,0xc19bf174UL,
    0xe49b69c1UL,0xefbe4786UL,0x0fc19dc6UL,0x240ca1ccUL,
    0x2de92c6fUL,0x4a7484aaUL,0x5cb0a9dcUL,0x76f988daUL,
    0x983e5152UL,0xa831c66dUL,0xb00327c8UL,0xbf597fc7UL,
    0xc6e00bf3UL,0xd5a79147UL,0x06ca6351UL,0x14292967UL,
    0x27b70a85UL,0x2e1b2138UL,0x4d2c6dfcUL,0x53380d13UL,
    0x650a7354UL,0x766a0abbUL,0x81c2c92eUL,0x92722c85UL,
    0xa2bfe8a1UL,0xa81a664bUL,0xc24b8b70UL,0xc76c51a3UL,
    0xd192e819UL,0xd6990624UL,0xf40e3585UL,0x106aa070UL,
    0x19a4c116UL,0x1e376c08UL,0x2748774cUL,0x34b0bcb5UL,
    0x391c0cb3UL,0x4ed8aa4aUL,0x5b9cca4fUL,0x682e6ff3UL,
    0x748f82eeUL,0x78a5636fUL,0x84c87814UL,0x8cc70208UL,
    0x90befffaUL,0xa4506cebUL,0xbef9a3f7UL,0xc67178f2UL };

void SHA256_Init (uint32* const H)
{
    H[0]=0x6a09e667UL;
    H[1]=0xbb67ae85UL;
    H[2]=0x3c6ef372UL;
    H[3]=0xa54ff53aUL;
    H[4]=0x510e527fUL;
    H[5]=0x9b05688cUL;
    H[6]=0x1f83d9abUL;
    H[7]=0x5be0cd19UL;
}

static	uint32 hash[8];

static int calc_hash(const uint8* const buf, uint32 size)
{
	uint32 W[64],T1,T2;
    uint32 a,b,c,d,e,f,g,h;
    int i,n,t;
    	
	if(buf == NULL) {
		printf("Invalid buffer address!\n");
		return -1;
	}
	
	debug("size after padding: %x\n", (unsigned)size);

	size = size << 3;		// process size as bit

   	SHA256_Init(hash);
    n=0;
    	
	for(i = 0;i < size;i+=512)
    	{
        	for (t = 0;t < 16;t++)
        	{
            	W[t]=reverse_endian(((uint32*)buf)[n]);
            	n++;
        	}
        	for (;t < 64;t++)
        	{
           	 W[t]=TRUNC32(WRO1(W[t-2])+TRUNC32(W[t-7]+TRUNC32(TRUNC32(WRO0(W[t-15]))+W[t-16])));
        	}
        	a = hash[0];   
        	b = hash[1];   
	        c = hash[2];   
        	d = hash[3];
	        e = hash[4];   
        	f = hash[5];   
	        g = hash[6];   
        	h = hash[7];
	        for(t=0;t<64;t++)
        	{
	            T1=TRUNC32(h+SIG1(e)+Ch(e,f,g)+SHA256_Constant[t]+W[t]);
        	    T2=TRUNC32(SIG0(a)+Maj(a,b,c));
	            h=g;
        	    g=f;
	            f=e;
        	    e=TRUNC32(d+T1);
	            d=c;
        	    c=b;
	            b=a;
        	    a=TRUNC32(T1+T2);
        	}
	        hash[0]=a+hash[0];    
        	hash[1]=b+hash[1];    
	        hash[2]=c+hash[2];    
        	hash[3]=d+hash[3];
	        hash[4]=e+hash[4];    
        	hash[5]=f+hash[5];    
	        hash[6]=g+hash[6];    
        	hash[7]=h+hash[7];
	}    	

#ifdef DEBUG
	for(i=0;i<8;i++)
		debug("the Hash[%d] is %x\n",i,hash[i]);
#endif
	return 0;
}


#endif


// get a dword from RSA output reg of the specified index
static uint32 RSA_result(const uint8 idx)
{
    uint32 v;

    v = (uint32)idx << REG_IDX_SHIFT;
    writel(RSA_CLIENT_CTRL, v);
    while (readl(RSA_CLIENT_CTRL) != v);

    return(readl(RSA_CLIENT_READ));
}

// calculate SHA-256 hash of image and compare it to supplied hash
// image comes in as a pointer to start of image and length (in bytes) of image
// hash is still in RSA regs
// assume RSA output is still available from regs
// return 0 if they match, -EAGAIN if they don't, -EINVAL if input is wrong
static int check_hash(uint32* const pbuf, uint32 len)
{
    uint32 i;
	uint32 rsa = 0, sha = 0; 
	if ((pbuf == NULL)||(len > MAX_DATA_SIZE)) {
		debug("Invalid input\n");
		return -EINVAL;
	}
	debug ("buf: %p length: %x\n", pbuf, len);
	flush_dcache_range ((uint32)pbuf, (uint32)(pbuf + len));
    calc_hash((uint8*)pbuf, len);
    
    for (i = 0; i < 8; i++)     // PKCSv1.5-style signature -- 8-DWORD SHA-256 hash plus padding.  Ignore pad altogether
    {
        // hash was already calculated in calc_hash() and is waiting for us in the regs
        //writel(DMA_SHA_CTRL, 7-i);
		
		rsa = RSA_result(i);		
		debug ("RSA %08x <-> %08x SHA\n", rsa, hash[7-i]);
		
		if (rsa != hash[7-i])
            return(-EAGAIN);                      // hash didn't match sig         	
    }
    
    return(0);
}
void dma_init(void)
{
	uint32 tmp;
	tmp = readl(CG1_CLK_STOP0);
	tmp &= ~DMACLK_STOP_MASK;
	tmp &= ~DCLK_STOP_MASK;
	writel(CG1_CLK_STOP0, tmp);
	
	tmp = readl(CG1_BLK_CLK_STOP0);
	tmp &= ~DMA_MCLK_STOP_MASK;
	writel(CG1_BLK_CLK_STOP0, tmp);
	
	tmp = readl(CG1_BLK_CLK_STOP1);
	tmp &= ~DMA_DMACLK_STOP_MASK;
	writel(CG1_BLK_CLK_STOP1, tmp);
	return;
}

int valid_single_image(uint8* const pheader, uint32 len)
{
	void* psig;
	dma_init();
	psig = (void*)(pheader + (len & ~0x3F) + 0x40);				// Signature address header + length/512* 512 +512 
	
	debug ("Image header: %p Length: %x Signature address: %p\n", pheader, len, psig);
    calc_sig((uint32*)psig, RSA2048);                                         // put the decrypted value in the RSA output regs    
	
	len += pad_for_SHA((uint32*)pheader, len);                       // this destroys data in the sig, which is OK since we already RSAed it    	
	if (check_hash((uint32*)pheader, len))                         // gets the decrypted hash from RSA output regs
        return (-1);                                          // hash mismatch
    
    return(0);
}

/* 
 * DMA decrypt boot code in-place in shadow.  Blocks until done.
 * "direction" = 0 for decryption, "direction" = 1 for encryption
 * ONLY WORKS FOR SIZE < 64kB!!
 */
void aes_ecb_fb(volatile void* src, volatile void* dst, ulong size, char direction)
{	
	ulong cw_index = 0x02;
	ulong len;
	ulong MAXDMASIZE = 0x8000;

	//printf("src: %08lx dst: %08lx size: %08lx dir: %d\n",src,dst,size,direction);
	
	if (direction == 0)
			direction = 2;
		else
			direction = 1;
		
	while (size) {
		if (size > MAXDMASIZE)
			len = MAXDMASIZE;
		else
			len = size;
	    
	    writel(DMA_DIRECT_USRC_A, 0); 
	    writel(DMA_DIRECT_LSRC_A, (ulong)src);    // Source
	    writel(DMA_DIRECT_UDST_A, 0); 
	    writel(DMA_DIRECT_LDST_A, (ulong)dst);   // Destination
	    
	    writel(DMA_DIRECT_CRYPT_INDICES_A, (cw_index << DMA_DIRECT_CRYPT_INDICES_A_CW_SRC_INDEX_SHIFT));
	    writel(DMA_DIRECT_DMACMD_A, DMA_DIRECT_DMACMD_A_DSTSPACE_MASK |                      // to FB
	            DMA_DIRECT_DMACMD_A_SRCSPACE_MASK |                     // from FB
	            (direction << DMA_DIRECT_DMACMD_A_ENCRYPT_EN_SHIFT) | 
	            (3 << DMA_DIRECT_DMACMD_A_ENCRYPT_TYPE_SEL_SHIFT) |     // AES
	            len);
	    		
		size -= len;
		src += len;
		dst += len;		
		while ( readl(DMA_STATUS) & DIRECT_A_MASK);		
	}
}
