/*
 * (C) Copyright 2004
 * esd gmbh <www.esd-electronics.com>
 * Reinhard Arlt <reinhard.arlt@esd-electronics.com>
 *
 * made from cmd_reiserfs by
 *
 * (C) Copyright 2003 - 2004
 * Sysgo Real-Time Solutions, AG <www.elinos.com>
 * Pavel Bartusek <pba@sysgo.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

/*
 * Ext2fs support
 */
#include <common.h>

#if (CONFIG_COMMANDS & CFG_CMD_EXT2)
#include <config.h>
#include <command.h>
#include <image.h>
#include <linux/ctype.h>
#include <asm/byteorder.h>
#include <ext2fs.h>
#if ((CONFIG_COMMANDS & CFG_CMD_USB) && defined(CONFIG_USB_STORAGE))
#include <usb.h>
#endif

#if((CONFIG_COMMANDS & CFG_CMD_MMC) && defined(CONFIG_GENERIC_MMC))
#include <mmc.h>
#endif

#ifndef CONFIG_DOS_PARTITION
#error DOS partition support must be selected
#endif

#define	EXT2_DEBUG

#ifdef	EXT2_DEBUG
#define	PRINTF(fmt,args...)	printf (fmt ,##args)
#else
#define PRINTF(fmt,args...)
#endif

#if defined(PLATFORM_XCODE) 
u32 g_XC4_Sata_Initialized = 0; 
#endif

#if (CONFIG_COMMANDS & CFG_CMD_SCSI)
extern void scsi_init(void);
#endif

#if((CONFIG_COMMANDS & CFG_CMD_MMC) && defined(CONFIG_GENERIC_MMC))
extern struct mmc mmc_dev[2];
#endif

#if ((CONFIG_COMMANDS & CFG_CMD_USB)&&defined(CONFIG_USB_STORAGE))
extern char usb_started;
extern int usb_stor_curr_dev;
extern int usb_stor_scan(int);
#endif

#if (CONFIG_COMMANDS & CFG_CMD_I2C)
extern int i2c_read (uchar chip, uint addr, int alen, uchar * buffer, int len);
#endif

static block_dev_desc_t *get_dev (char* ifname, int dev)
{
#if (CONFIG_COMMANDS & CFG_CMD_IDE)
	if (strncmp(ifname,"ide",3)==0) {
		extern block_dev_desc_t * ide_get_dev(int dev);
		return((dev >= CFG_IDE_MAXDEVICE) ? NULL : ide_get_dev(dev));
	}
#endif
#if (CONFIG_COMMANDS & CFG_CMD_SCSI)
	if (strncmp(ifname,"scsi",4)==0) {
		extern block_dev_desc_t * scsi_get_dev(int dev);
		return((dev >= CFG_SCSI_MAXDEVICE) ? NULL : scsi_get_dev(dev));
	}
#endif
#if ((CONFIG_COMMANDS & CFG_CMD_USB) && defined(CONFIG_USB_STORAGE))
	if (strncmp(ifname,"usb",3)==0) {
		extern block_dev_desc_t * usb_stor_get_dev(int dev);
		return((dev >= USB_MAX_STOR_DEV) ? NULL : usb_stor_get_dev(dev));
	}
#endif
#if defined(CONFIG_MMC)
	if (strncmp(ifname,"mmc",3)==0) {
		extern block_dev_desc_t *  mmc_get_dev(int dev);
		return((dev >= 1) ? NULL : mmc_get_dev(dev));
	}
#endif
#if defined(CONFIG_SYSTEMACE)
	if (strcmp(ifname,"ace")==0) {
		extern block_dev_desc_t *  systemace_get_dev(int dev);
		return((dev >= 1) ? NULL : systemace_get_dev(dev));
	}
#endif
	return(NULL);
}

int do_ext2ls (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *filename = "/";
	int dev=0;
	int part=1;
	int ret;
	char *ep;
	block_dev_desc_t *dev_desc=NULL;
	int part_length;

	if (argc < 3) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return(1);
	}

	if(strncmp(argv[1], "scsi", 4) == 0){
#if(CONFIG_COMMANDS & CFG_CMD_SCSI)
		if(!g_XC4_Sata_Initialized)
		{
            scsi_init();
            scsi_scan(0, 0);
            scsi_scan(0, 1);
			g_XC4_Sata_Initialized = 1;
		}
#else
			printf("Interface %s was not enabled, please check config file\n", argv[1]);
#endif
	} else if(strncmp(argv[1], "mmc", 3) == 0) {
#if(CONFIG_COMMANDS & CFG_CMD_MMC)
			if(!mmc_dev[0].has_init)
				mmc_init(&mmc_dev[0]);
#else
			printf("Interface %s was not enabled, please check config file\n", argv[1]);
#endif
	} else if(strncmp(argv[1], "usb", 3) == 0) {
#if((CONFIG_COMMANDS & CFG_CMD_USB)&&defined(CONFIG_USB_STORAGE))
			if(!usb_started)
				if(usb_init() >= 0);
					usb_stor_curr_dev = usb_stor_scan(1);
#else
			printf("Interface %s was not enabled, please check config file\n", argv[1]);
#endif
	} else {
			printf("%s was not supported!\n", argv[1]);
	}

  
	dev = (int)simple_strtoul (argv[2], &ep, 16);
	dev_desc=get_dev(argv[1],dev);

	if (dev_desc == NULL) {
		printf ("\n** Block device %s %d not supported\n", argv[1], dev);
		return(1);
	}

	if (*ep) {
		if (*ep != ':') {
			puts ("\n** Invalid boot device, use `dev[:part]' **\n");
			return(1);
		}
		part = (int)simple_strtoul(++ep, NULL, 16);
	}

	if (argc == 4) {
	    filename = argv[3];
	}

	PRINTF("Using device %s %d:%d, directory: %s\n", argv[1], dev, part, filename);

	if ((part_length = ext2fs_set_blk_dev(dev_desc, part)) == 0) {
		printf ("** Bad partition - %s %d:%d **\n",  argv[1], dev, part);
		ext2fs_close();
		return(1);
	}

	if (!ext2fs_mount(part_length)) {
		printf ("** Bad ext2 partition or disk - %s %d:%d **\n",  argv[1], dev, part);
		ext2fs_close();
		return(1);
	}

	if (ext2fs_ls (filename)) {
		printf ("** Error ext2fs_ls() **\n");
		ext2fs_close();
		return(1);
	};

	ext2fs_close();

	return(0);
}

U_BOOT_CMD(
	ext2ls,	4,	1,	do_ext2ls,
	"ext2ls- list files in a directory (default /)\n",
	"<interface> <dev[:part]> [directory]\n"
	"    - list files from 'dev' on 'interface' in a 'directory'\n"
);

/******************************************************************************
 * Ext2fs boot command intepreter. Derived from diskboot
 */
#if defined(PLATFORM_XCODE) 
extern u32 g_XC3_Sata_Initialized; 
#endif
int do_ext2load (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *filename = NULL;
	char *ep;
	int dev, ret, part = 1;
	ulong addr = 0, part_length, filelen;
	disk_partition_t info;
	block_dev_desc_t *dev_desc = NULL;
	char buf [12];
	unsigned long count, boot_mode=0, multi_boot=0;
	char *addr_str = NULL, *env_str = NULL;	

	if(strncmp(argv[1], "scsi", 4) == 0){
#if(CONFIG_COMMANDS & CFG_CMD_SCSI)
		if(!g_XC4_Sata_Initialized)
		{
            dev = (int)simple_strtoul (argv[2], &ep, 16);
            scsi_init();
            scsi_scan(0, dev);
			g_XC4_Sata_Initialized = 1;
		}
#else
		printf("Interface %s was not enabled, please check config file\n", argv[1]);

#endif
	} else if(strncmp(argv[1], "mmc", 3) == 0) {
#if(CONFIG_COMMANDS & CFG_CMD_MMC)
		if(!mmc_dev[0].has_init)
			mmc_init(&mmc_dev[0]);
#else
		printf("Interface %s was not enabled, please check config file\n", argv[1]);
#endif
	} else if(strncmp(argv[1], "usb", 3) == 0) {
#if((CONFIG_COMMANDS & CFG_CMD_USB)&&defined(CONFIG_USB_STORAGE))
		if(!usb_started)
			if(usb_init() >= 0);
				usb_stor_curr_dev = usb_stor_scan(1);
#else
		printf("Interface %s was not enabled, please check config file\n", argv[1]);
#endif
	} else {
		printf("%s was not supported!\n", argv[1]);
	}

	env_str = getenv("multiboot");
	if (env_str != NULL)
		multi_boot = simple_strtoul(env_str, NULL,16);
	debug ("-- Multi_boot Value: %x\n",(unsigned)multi_boot);
	if (multi_boot == 0) {
		printf("Non-multi-boot Mode\n");
		switch (argc) {
		case 3:
			addr_str = getenv("loadaddr");
			if (addr_str != NULL) {
				addr = simple_strtoul (addr_str, NULL, 16);
			} else {
				addr = CFG_LOAD_ADDR;
			}
			filename = getenv ("bootfile");
			count = 0;
			break;
		case 4:
			addr = simple_strtoul (argv[3], NULL, 16);
			filename = getenv ("bootfile");
			count = 0;
			break;
		case 5:
			addr = simple_strtoul (argv[3], NULL, 16);
			filename = argv[4];
			count = 0;
			break;
		case 6:
			addr = simple_strtoul (argv[3], NULL, 16);
			filename = argv[4];
			count = simple_strtoul (argv[5], NULL, 16);
			break;

		default:
			printf ("Usage:\n%s\n", cmdtp->usage);
			return(1);
		}
	}else {		
#if (CONFIG_COMMANDS&CFG_CMD_I2C)	
		i2c_read(0x50, 0x3000, 0x02, (uchar*)(&boot_mode), 0x04);
#endif
		boot_mode &= 0xff;
		printf("Multi-boot, Mode:%d\n",boot_mode);		
		switch (boot_mode) {
			case 0x0:
				filename = getenv("bootfile");
				break;
			
			case 0x1:
				filename = getenv("bootfile_1");			
				break;
			
			case 0x2:				
				filename = getenv("bootfile_2");
				break;
			
			case 0x3:
				filename = getenv("bootfile_3");
				break;
			
			case 0x4:
				filename = getenv("bootfile_4");
				break;
			
			case 0x5:
				filename = getenv("bootfile_5");
				break;
			
			case 0x6:
				filename = getenv("bootfile_6");
				break;
			
			case 0x7:
				filename = getenv("bootfile_7");
				break;

			case 0x8:
				filename = getenv("bootfile_8");
				break;
			
			case 0x9:
				filename = getenv("bootfile_9");
				break;
			
			case 0xa:
				filename = getenv("bootfile_10");
				break;
			
			case 0xb:
				filename = getenv("bootfile_11");
				break;
			
			default:
				printf("Using Default bootfile\n");
				filename = getenv("bootfile");
				break;
			}		
		printf ("Boot file: %s\n", filename);
		switch (argc) {
		case 3:
			addr_str = getenv("loadaddr");
			if (addr_str != NULL) {
				addr = simple_strtoul (addr_str, NULL, 16);
			} else {
				addr = CFG_LOAD_ADDR;
			}			
			count = 0;
			break;
		case 4:
			addr = simple_strtoul (argv[3], NULL, 16);			
			count = 0;
			break;
		case 5:
			addr = simple_strtoul (argv[3], NULL, 16);
			filename = argv[4];
			count = 0;
			break;
		case 6:
			addr = simple_strtoul (argv[3], NULL, 16);
			filename = argv[4];
			count = simple_strtoul (argv[5], NULL, 16);
			break;

		default:
			printf ("Usage:\n%s\n", cmdtp->usage);
			return(1);
		}
	}
	if (!filename) {
		puts ("\n** No boot file defined, use default bootfile **\n");
		char kernel[25] = "/boot/uImage_mips_gz";
		filename = kernel;
		//return(1);
	}

    if(addr < 0x80000000)
    {
         printf("Wrong load address 0x%x which should be 0x80000000 + offset\n", addr);
         return(1);
    }
	dev = (int)simple_strtoul (argv[2], &ep, 16);
	dev_desc=get_dev(argv[1],dev);
	if (dev_desc==NULL) {
		printf ("\n** Block device %s %d not supported\n", argv[1], dev);
		return(1);
	}
	if (*ep) {
		if (*ep != ':') {
			puts ("\n** Invalid boot device, use `dev[:part]' **\n");
			return(1);
		}
		part = (int)simple_strtoul(++ep, NULL, 16);
	}

	PRINTF("Using device %s%d, partition %d\n", argv[1], dev, part);

	if (part != 0) {
		if (get_partition_info (dev_desc, part, &info)) {
			printf ("** Bad partition %d **\n", part);
			return(1);
		}

		if (strncmp(info.type, BOOT_PART_TYPE, sizeof(info.type)) != 0) {
			printf ("\n** Invalid partition type \"%.32s\""
				" (expect \"" BOOT_PART_TYPE "\")\n",
				info.type);
			return(1);
		}
		PRINTF ("\nLoading from block device %s device %d, partition %d: "
			"Name: %.32s  Type: %.32s  File:%s\n",
			argv[1], dev, part, info.name, info.type, filename);
	} else {
		PRINTF ("\nLoading from block device %s device %d, File:%s\n",
			argv[1], dev, filename);
	}


	if ((part_length = ext2fs_set_blk_dev(dev_desc, part)) == 0) {
		printf ("** Bad partition - %s %d:%d **\n",  argv[1], dev, part);
		ext2fs_close();
		return(1);
	}

	if (!ext2fs_mount(part_length)) {
		printf ("** Bad ext2 partition or disk - %s %d:%d **\n",  argv[1], dev, part);
		ext2fs_close();
		return(1);
	}

	filelen = ext2fs_open(filename);
	if (filelen < 0) {
		printf("** File not found %s\n", filename);
		ext2fs_close();
		return(1);
	}
	if ((count < filelen) && (count != 0)) {
	    filelen = count;
	}

	if (ext2fs_read((char *)addr, filelen) != filelen) {
		printf("\n** Unable to read \"%s\" from %s %d:%d **\n", filename, argv[1], dev, part);
		ext2fs_close();
		return(1);
	}

	ext2fs_close();

	/* Loading ok, update default load address */
	load_addr = addr;

	printf ("\n%ld bytes read\n", filelen);
	sprintf(buf, "%lX", filelen);
	setenv("filesize", buf);

	return(filelen);
}

U_BOOT_CMD(
	ext2load,	6,	0,	do_ext2load,
	"ext2load- load binary file from a Ext2 filesystem\n",
	"<interface> <dev[:part]> [addr] [filename] [bytes]\n"
	"    - load binary file 'filename' from 'dev' on 'interface'\n"
	"      to address 'addr' from ext2 filesystem\n"
);

#endif	/* CONFIG_COMMANDS & CFG_CMD_EXT2 */
