/*
 * (C) Copyright 2002
 * Detlev Zundel, DENX Software Engineering, dzu@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * LZO compress and uncompress routines
 */

#include <common.h>
#include <command.h>
#include <zlib.h>
#include <image.h>
#include <linux/string.h>
#include <linux/lzo.h>

#if (CONFIG_COMMANDS & CFG_CMD_LZO)
int do_lzo(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong src, dst;
	size_t cnt, ret;
	int sts;

	switch (argc) {
	case 5:
		src = simple_strtoul(argv[2], NULL, 16);
		dst = simple_strtoul(argv[3], NULL, 16);
		cnt = simple_strtoul(argv[4], NULL, 16);
		break;

	default:
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	if (src == dst){
		printf("Error, source and destion address are same\n");
		return 1;
	}

	if (!(src & 0x80000000)|| !(dst & 0x80000000)){
		printf("Error, invalid address%x %x\n", src, dst);
		return 1;
	}
	printf("lzo decompressing from %p to %p, length %x\n", src, dst, cnt);

	if(strncmp(argv[1], "decompress", 5) == 0) {	
		sts = lzop_decompress((u8*)src, cnt, (u8*)dst, &ret);
		if(sts){
			printf("fail to decompress image, error num:%d\n", sts);
			return sts;
		}
	} else {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	printf("%d of data decompressed\n", ret);
	flush_dcache_range(dst, dst + ret);
	return ret;
}

U_BOOT_CMD(
	lzo,	5,	1,	do_lzo,
	"lzo	- decompress the size of image from source to destion\n",
	"\tdecompress <src> <dst> <size>\n"
);

#endif
