/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Aug 17 , 2005
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/cpu/arc700/cpu.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

#include <command.h>
#include <common.h>
#include <asm/arcregs.h>
#include <asm/eagle.h>

#define readl(addr) (*(volatile unsigned int *) (addr))
#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))

 
/* cache register masks,shifts */
#define CACHE_CONFIG_MASK 0x00000f00
#define CACHE_CONFIG_SHIFT 8
#define CACHE_SIZE_MASK 0x0000f000
#define CACHE_SIZE_SHIFT 12
/* Sameer: Need to rectify this macro definition.
           Its just for the sake of satisfying build need of a macro
            in asm-generic/dma-mapping.h */
#define L1_CACHE_SHIFT CACHE_SIZE_SHIFT
#define CACHE_LINE_LENGTH_MASK 0x000f0000
#define CACHE_LINE_LENGTH_SHIFT 16

int icache_size_value[4] = { 8192, 16384, 32768, 65536 };
int dcache_size_value[4] = { 8192, 16384, 32768, 65536 };

/* needed to ensure correct number of blocks get invalidated/flushed */
#define DCACHE_LINE_MASK  (~(cpuinfo_arc700.dcache_line_length - 1))
#define ICACHE_LINE_MASK  (~(cpuinfo_arc700.icache_line_length - 1))

#define ICACHE_ALIASING_HW_FIX
#ifdef ICACHE_ALIASING_HW_FIX
#define EXTRACT_VIRT_BITS(vaddr) (((unsigned long )(vaddr) >> 13) & 0x1F)
#else
#define EXTRACT_VIRT_BITS(vaddr)	0
#endif

short unsigned int icache = 0, dcache = 0;
struct cpuinfo_arc {
	 unsigned int processor_id;
	 unsigned int processor_version;
	 unsigned int timers;
	 unsigned int extensions;
	 unsigned int vec_base;
	 unsigned int peripheral_base;
	 unsigned int dcache;
	 unsigned int dcache_size;
	 unsigned int dcache_type;
	 unsigned int dcache_line_length;
	 unsigned int icache;
	 unsigned int icache_size;
	 unsigned int icache_type;
	 unsigned int icache_line_length;
	 unsigned int mmu;
};
struct cpuinfo_arc cpuinfo_arc700;

int do_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	
#if CFG_UBOOT_ON_FLASH  	
	/* if uboot is on flash then boot agian from flash */
	void (*reset)();
	reset = (void (*)())0x1000;
	(*reset)();
#endif	

}

void disable_icache(void)
{
	__asm__ __volatile (	"lr  r0,[0x11] \n\t"
				"or  r0,r0,1 \n\t"
				"sr  r0,[0x11] \n\t"
				::
				:"r0");	
}

void disable_dcache(void)
{
	/* Flush D cache */
	__asm__ __volatile (	
			"sr  1, [0x47] \n\t"
			::
			);
	/* Disable D cache */
	__asm__ __volatile (	
			"lr  r0,[0x48] \n\t"
			"or  r0,r0,1 \n\t"
			"sr  r0,[0x48] \n\t"
			::
			:"r0");	
}

void  flush_cache (unsigned long dummy1, unsigned long dummy2)
{
#if 0
	/* flush instruction cache */
	write_new_aux_reg(ARC_REG_IC_IVIC, 1);

	/*
	 * lr will not complete till the icache inv operation is not over
	 */
	read_new_aux_reg(ARC_REG_IC_CTRL);
	
#ifdef ARC5		/* as data cache is only enabled for A5 */
	
	/* flush data cache */	
	__asm__ __volatile (	
			"sr  1, [0x47] \n\t"
			::
			);

#endif	
		
#endif	
	return;
}	

/*
 * enable icahce and dcache
*/
int arc_cache_init(void)
{
	unsigned int temp;
	int offset;

    icache = dcache = 1;    // enable dcache and icache
    
	/* load the icache build register */
	cpuinfo_arc700.icache = read_new_aux_reg(ARC_REG_I_CACHE_BUILD_REG);
	/* get the icache type */
	cpuinfo_arc700.icache_type =
	    ((cpuinfo_arc700.icache & CACHE_CONFIG_MASK) >> CACHE_CONFIG_SHIFT);

	/* get the icache size */
	offset = (cpuinfo_arc700.icache & CACHE_SIZE_MASK) >> CACHE_SIZE_SHIFT;
	cpuinfo_arc700.icache_size = icache_size_value[offset - 4];

	cpuinfo_arc700.icache_line_length = 32;
	if (cpuinfo_arc700.icache_type == 0x3) {
		/*printf("Detected I-cache : \n");
		printf("  Type=2 way set-associative, ");
		printf("Line length=%u, Size=%uK",
		       cpuinfo_arc700.icache_line_length,
		       (cpuinfo_arc700.icache_size / 1024));*/

		if (icache) {
			//printf(" (enabled)\n");
			temp = read_new_aux_reg(ARC_REG_IC_CTRL);
			write_new_aux_reg(ARC_REG_IC_CTRL, temp & 0xfffffffe);
		} else {
			//printf(" (disabled)\n");
			/* Disable I-cache */
			temp = read_new_aux_reg(ARC_REG_IC_CTRL);
			write_new_aux_reg(ARC_REG_IC_CTRL, (temp | 0x1));
		}
	}

	/* load the dcache build register */
	cpuinfo_arc700.dcache = read_new_aux_reg(ARC_REG_D_CACHE_BUILD_REG);

	/* load the dcache build register and 
	   initialize the dcache control register */
	/* set IM and set replacement policy as round robin */
	__asm__ __volatile__("lr %0, [%1]\n\t"
			     "sr 0x40, [%2]\n\t":"=r"(cpuinfo_arc700.dcache)
			     :"i"(ARC_REG_D_CACHE_BUILD_REG),
			     "i"(ARC_REG_DC_CTRL)
	    );
	/* get the dcache type */
	cpuinfo_arc700.dcache_type =
	    ((cpuinfo_arc700.dcache & CACHE_CONFIG_MASK) >> CACHE_CONFIG_SHIFT);

	/* get the dcache size */
	offset = (cpuinfo_arc700.dcache & CACHE_SIZE_MASK) >> CACHE_SIZE_SHIFT;
	cpuinfo_arc700.dcache_size = dcache_size_value[offset - 4];

	cpuinfo_arc700.dcache_line_length = 32;
	if (cpuinfo_arc700.dcache_type == 0x2) {
		/*printf("Detected D-cache : \n");
		printf("  Type=4 way set-associative, ");
		printf("Line length=%u, Size=%uK",
		       cpuinfo_arc700.dcache_line_length,
		       (cpuinfo_arc700.dcache_size / 1024));*/

		if (dcache) {
			//*printf(" (enabled)\n");
			temp = read_new_aux_reg(ARC_REG_DC_CTRL);
			write_new_aux_reg(ARC_REG_DC_CTRL, temp & 0xfffffffe);
		} else {
			//printf(" (disabled)\n");
			/* Set Invalidate mode so that the dirty pages are 
			   flushed out */
			temp = read_new_aux_reg(ARC_REG_DC_CTRL);
			write_new_aux_reg(ARC_REG_DC_CTRL,
					  (temp | IM_SET_DC_CTRL));
			/* Flush D cache */
			write_new_aux_reg(ARC_REG_DC_IVDC, 0x1);
			/* Disable D cache */
			write_new_aux_reg(ARC_REG_DC_CTRL, 0x1);
		}
	}
}

static inline void invalidate_PFB()
{
    // invalidate PFB
    writel(0x10052006, 0xF0000000+PROC3_PFB_INVALID_ALL);
    readl(0xF0000000+PROC3_PFB_INVALID_ALL);
    
}


void flush_and_inv_dcache_all(void)
{
	unsigned long dc_ctrl_org = read_new_aux_reg(ARC_REG_DC_CTRL);
	unsigned long dc_ctrl = dc_ctrl_org;

	if (!dcache)
		goto out;

	/*
	 * Set the IM bit in the ctrl reg so that the cache is flushed before
	 * it is invalidated
	 */
	dc_ctrl |= IM_SET_DC_CTRL;
	write_new_aux_reg(ARC_REG_DC_CTRL, dc_ctrl);

	write_new_aux_reg(ARC_REG_DC_IVDC, 0x1);

        // invalidate PFB
        invalidate_PFB();
    
	/* 
	 * wait for the flush to complete, poll on the FS Bit 
	 */
	while (read_new_aux_reg(ARC_REG_DC_CTRL) & FS_FLAG) ;

	write_new_aux_reg(ARC_REG_DC_CTRL, dc_ctrl_org);

      out:
	return;
}

void flush_dcache_range(unsigned long start, unsigned long end)
{
	if (!dcache)
		goto out;

	start &= DCACHE_LINE_MASK;

	while (end >= start) {
		write_new_aux_reg(ARC_REG_DC_FLDL,
				  start | EXTRACT_VIRT_BITS(start));
		start = start + cpuinfo_arc700.dcache_line_length;
	}

	/* 
	 * wait for the flush to complete, poll on the FS Bit 
	 */
	while (read_new_aux_reg(ARC_REG_DC_CTRL) & FS_FLAG) ;

      out:
	return;
}

void flush_dcache_all(void)
{
	unsigned long value;

	if (!dcache)
		goto out;

	write_new_aux_reg(ARC_REG_DC_FLSH, 1);

	/* poll till the flush is over */
	do {
		value = read_new_aux_reg(ARC_REG_DC_CTRL);
	} while (value & 0x100);	/* Check with the FS bit */

out:
	return;
}

/*
 * Fixme :: Might need to be corrected so that dma works correctly, leaving
 * for now.
 * All the addrs must be physical addrs, that means the addr
 * translation must be done before writing the addr to the cache line.
 */

void flush_and_inv_dcache_range(unsigned long start, unsigned long end)
{
	unsigned long dc_ctrl_org = read_new_aux_reg(ARC_REG_DC_CTRL);
	unsigned long dc_ctrl = dc_ctrl_org;

	if (!dcache)
		goto out;

	start &= DCACHE_LINE_MASK;

	/*
	 * Set the IM bit in the ctrl reg so that the cache is flushed before
	 * it is invalidated
	 */
	dc_ctrl |= IM_SET_DC_CTRL;
	write_new_aux_reg(ARC_REG_DC_CTRL, dc_ctrl);

	while (end >= start) {
		write_new_aux_reg(ARC_REG_DC_IVDL,
				  start | EXTRACT_VIRT_BITS(start));
		start = start + cpuinfo_arc700.dcache_line_length;
	}

        // invalidate PFB
        invalidate_PFB();

	/* 
	 * wait for the flush to complete, poll on the FS Bit 
	 */
	while (read_new_aux_reg(ARC_REG_DC_CTRL) & FS_FLAG) ;

	write_new_aux_reg(ARC_REG_DC_CTRL, dc_ctrl_org);

out:
	return;
}

void inv_dcache_range(unsigned long start, unsigned long end)
{
	if (!dcache)
		goto out;

	start &= DCACHE_LINE_MASK;
	while (end >= start) {
		write_new_aux_reg(ARC_REG_DC_IVDL,
				  start | EXTRACT_VIRT_BITS(start));
		start = start + cpuinfo_arc700.dcache_line_length;
	}

        // invalidate PFB
        invalidate_PFB();

out:
	return;
}

void inv_dcache_all(void)
{
	if (!dcache)
		goto out;

	write_new_aux_reg(ARC_REG_DC_IVDC, 0x1);

        // invalidate PFB
        invalidate_PFB();

out:
	return;
}

/*
 * disable dcache and icahce
 */

int cpu_init(void)
{
	disable_interrupts();
  
#if 0	
	/* Disable I cache */
	//disable_icache(); 
	/* Disable D cache */
	disable_dcache();  
#endif

    // enable dcache and icache
   arc_cache_init();

	return 0;
}


