/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Aug 17 , 2005
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/cpu/arc700/serial.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

#include <common.h>
#include <asm/serial.h>

aa3_uart *uart = (aa3_uart *) UART_BASEADDR;

/* The clock speed is set at config-time in CONFIG_ARC700_CLK */
int arc_console_baud = ((unsigned long)CONFIG_ARC700_CLK/(CONSOLE_BAUD_RATE*4))-1;


/*
 * Sets baudarate
 */
void serial_setbrg (void)
{
		
	/* DECLARE_GLOBAL_DATA_PTR; */

	int baud_h,baud_l;
	/* Get the baudrate from global structure or from configuration file as for now we get it from configuration file */
	

	/* Set the baud rate */
	baud_l = (arc_console_baud & 0xff);
        baud_h = (arc_console_baud & 0xff00) >> 8;

	uart->baudl = baud_l;
	uart->baudh = baud_h;
	      
}


/*
 * Intialise the serial port with given baudrate
 */
int serial_init (void)
{
	serial_setbrg ();

	return (0);
}


/*
 * Output a single byte to the serial port.
 */
void serial_putc (const char c)
{
	if (c == '\n') 
		serial_putc('\r');	       		
	
	/* Wait till dataTx register is empty */
	while (!(uart->status & UART_TXEMPTY));
	
	uart->data = c;

}

/*
 * Read a single byte from the serial port. Returns 1 on success, 0
 * otherwise. When the function is succesfull, the character read is
 * written into its argument c.
 */
int serial_tstc (void)
{
	/* Wait till character is placed in fifo */
	if (!(uart->status & UART_RXEMPTY))
		return (1);
	return (0);
}

/*
 * Read a single byte from the serial port. Returns 1 on success, 0
 * otherwise. When the function is succesfull, the character read is
 * written into its argument c.
 */
int serial_getc (void)
{
	unsigned char ch;	
	
	/* Wait till character is placed in fifo */
	while (uart->status & UART_RXEMPTY);
	
	/* Also check for overflow errors */
	if (uart->status & UART_OVERFLOW_ERR)
	       return 0;	
	
	ch = uart->data & 0x00ff;
		return ((int)ch);
}

void serial_puts (const char *s)
{
	while (*s) {
		serial_putc (*s++);
	}
}

