/* 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <common.h>
#include <mmc.h>
#include <asm/viper.h>
#include <asm/xcode_mmc.h>
#include <asm/addrspace.h>

//#define MMC_DEBUG
#ifdef MMC_DEBUG
#define mmc_debug(fmt, args...) printf(fmt, ##args) 
#else
#define mmc_debug(fmt, args...)
#endif

/* support 2 mmc hosts */
struct mmc mmc_dev[2];
struct mmc_host mmc_host[2] ____cacheline_aligned;

#define readl(addr) (*(volatile unsigned int *) (addr))
#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))

static int mmc_support_dual_host(void)
{
	return 0;
}

/*
 * XCode SMCC Block internal register access
 */
static u32 xcsdh_ubusy_wait(u32 core, int time_out_cnt)
{
	u32 csr_status = 0;
	u32 busy = 0, err = 0;
	int i = 0;
	u32 timeout = 0;

	csr_status = readl(MMREG_BASE + (CORE_REG(core, SMCC_CSR_STATUS_REG)));
	busy = (csr_status & SMCC_CSR_STATUS_REG_CSR_BUSY_MASK) >> SMCC_CSR_STATUS_REG_CSR_BUSY_SHIFT;
	err = (csr_status & CSR_ERR_MASK) >> CSR_ERR_SHIFT;

	while (busy || err) {

		udelay(10);
		i ++;
		if (i > time_out_cnt) {
			timeout = 1;
			break;
		}

		csr_status = readl(MMREG_BASE + (CORE_REG(core, SMCC_CSR_STATUS_REG)));
		busy = (csr_status & SMCC_CSR_STATUS_REG_CSR_BUSY_MASK) >> 
			SMCC_CSR_STATUS_REG_CSR_BUSY_SHIFT;
		err = (csr_status & CSR_ERR_MASK) >> 
			CSR_ERR_SHIFT;
		if(busy || err)
			printf("SMCC internal register access error\n");
	}

	return(timeout);
}

static int xcsdh_access_smcc_reg(u32 core, u32 reg, volatile u32 *rdata, u32 wdata, int write)
{
	u32 beef = 0;
	int time_out_cnt = 100;
	int err = 0;
	u32 csr_ctrl_reg = 0;

	err = xcsdh_ubusy_wait(core, time_out_cnt);
	if (err)
		goto out;

	switch (core) {
		case 0:
			csr_ctrl_reg = (write << SMCC_CSR_CTRL_REG_WOP_SHIFT) | reg;
			break;

		case 1:
			csr_ctrl_reg = (write << SMCC_CSR_CTRL_REG1_WOP1_SHIFT) | reg;
			break;
	}

	if (!write) {
		/* READ data */
		writel(csr_ctrl_reg, MMREG_BASE + (CORE_REG(core, SMCC_CSR_CTRL_REG)));
		err = xcsdh_ubusy_wait(core, time_out_cnt);
		if (err)
			goto out;

		/* XXX:
		 * Assume the RDAT is ready instantly
		 */
		udelay(10);
		beef = readl(MMREG_BASE + (CORE_REG(core, SMCC_RDAT_REG)));
		*rdata = beef;

	} else {
		/* WRITE data */
		writel(wdata, MMREG_BASE + (CORE_REG(core, SMCC_WDAT_REG)));
		writel(csr_ctrl_reg, MMREG_BASE + (CORE_REG(core, SMCC_CSR_CTRL_REG)));
		err = xcsdh_ubusy_wait(core, time_out_cnt);
		if (err)
			goto out;
	}

out:
	return (err);
}

/* Check Card present */
static void xcsdh_check_card_present(struct mmc_host *host)
{
	volatile u32 cd_reg = 0;
	u32 err = 0;
	
	/* Before enable irq, lets check do we have a card inserted */
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CDETECT, &cd_reg, 0, 0);
	/* We only have 1 slot. 0 in register means PRESENT... */
	host->card_present = (cd_reg & 0x01) ? 0 : 1;
}

/* Enable internal DMA */
static int xcsdh_enable_dma(struct mmc_host *host)
{
	volatile u32 reg;
	int err;


	/* Software reset the DMA controller internal registers */
	reg = 1 << SMCC_BMOD_SWR_SHIFT;
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_BMOD, NULL, reg, 1);
	if (err)
		goto out;

	reg = 1 << SMCC_BMOD_DE_SHIFT;

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_BMOD, NULL, reg, 1);
	if (err)
		goto out;


	reg = (1 << SMCC_CTRL_IDMAC_SHIFT) | (1 << SMCC_CTRL_INT_EN_SHIFT);	
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CTRL, NULL, reg, 1);
	if (err)
		goto out;

	/* bit8: normal IDMAC irq on 
	 * bit0,1: TI, RI irq on
	 */
	//	reg = 0x00000103;	
	//	reg = 0x00000103 | (1 << 5) | (1 << 4) | (1<<9);
	/* only report us the errors
	 * bit9: Abormal Interrupt Summary
	 * bit5: Card Error Summary
	 * bit4: Descriptor Unavailable
	 * bit2: Fatal Bus Error
	 * */
	reg = (1 << 2) | (1 << 5) | (1 << 4) | (1<<9);
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_IDINTEN, NULL, reg, 1);
	if (err)
		goto out;

	/* fix this! size not correct */
	memset(&(host->desc[0].d0), 0xffff, (4*sizeof(u32)));

	/* nolonger need buf pointer ourself */

out:
	return(err);
}

static int xcsdh_setup_polling_mode(struct mmc_host *host)
{
	int err = 0;
	volatile u32 irq_reg = 0;
	u32 mask;
	/* disable  irq on PROC5*/
	irq_reg = readl(MMREG_BASE + MIPS5_INTERRUPT1_MASK);
	irq_reg &= ~MIPS5_INTERRUPT1_MASK_SMCC_INT_MASK;
	writel(irq_reg, MMREG_BASE + MIPS5_INTERRUPT1_MASK);

	/* Core 0 */
	if(host->core_id == 0){
		irq_reg = readl(MMREG_BASE + SMCC_INT_EN_REG);
		irq_reg &= ~SMCC_INT_EN_REG_INT_EN_MASK;
		writel(irq_reg, MMREG_BASE + SMCC_INT_EN_REG);
	}
	/* Core 1 */
	if(host->core_id == 1){
		irq_reg = readl(MMREG_BASE + SMCC_INT_EN_REG1);
		irq_reg &= ~SMCC_INT_EN_REG_INT_EN_MASK;
		writel(irq_reg, MMREG_BASE + SMCC_INT_EN_REG1);
	}

	/* no int mask enabled polling raw int status  */
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_INT_MASK, NULL, 0x0, 1);

	if(err)
		printf(" [%s] failed err = %d\n",__func__, err);

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_INT_MASK, &mask, 0, 0);	
	return (err);
}

/*
 *  Wait for SMCC core takehe last command
 */
static int xcsdh_wait_ciu(u32 core, int time_out_cnt)
{
	u32 reg = 0;
	int err = 0;
	u32 start_cmd_bmask = 1 << SMCC_CMD_START_CMD_SHIFT;
	int i = 0;

	while (i < time_out_cnt) {
		err = xcsdh_access_smcc_reg(core, SMCC_CMD, &reg, 0, 0);
		if (err)
			break;

		/* start_cmd bit will be 0 if taken by CIU */
		if ((reg & start_cmd_bmask) == 0)
			break;
		i++;
		udelay(10);
	}

	/* XXX: set better ERRNO! */
	if (i >= time_out_cnt) {
		return (-1);
	}

	if (err)
		return (-1);

	return(0);

}

static int mmc_prepare_data(struct mmc_host *host, struct mmc_data *data)
{
	int err = 0;
	int count = 0;
	int i = 0, j;	

	mmc_debug(" [%s] data->dest: %08X, data->blocks: %u, data->blocksize: %u\n",
			__func__, (u32)data->dest, data->blocks, data->blocksize);

	count = data->blocks * data->blocksize;

	if(count & 0x3){
		printf(" [%s] size requested is not 4 byte aligned\n", __func__);
		err = -1;
		goto out;
	}

	/* can not handle request data size greater than 512KiB */
	if(count > (MMC_DESC_NUM * DESC_BUF_SIZE)){
		printf(" [%s] requested size %d greater than dma buffer size %d\n",
			__func__, count, MMC_DESC_NUM * DESC_BUF_SIZE);
		err = -1;
		goto out;
	}
	
	/* Prepare the DMA descriptor list
	 * D0 - 31 OWN; 30 CES; 5 END of RING; 4 CHAINED; 3 First Desc; 2 Last Desc; 1 DIC; 
	 * D1 - 25:13 Buffer2 size not used in chain mode; 12:0 Buffer1 size, 0 will caused discard.
	 * D2 - 31:0 Buffer physical Address Pointer 1, 
	 * D3 -  31:0 Next Desc Physical address
	 */
	j = (count / DESC_BUF_SIZE) + (((count % DESC_BUF_SIZE) == 0) ? 0 : 1);
	for(i = 0; i < j; i++){
		if(i == 0)			
			host->desc[i].d0 = (SMCC_DESC0_FS_MASK | SMCC_DESC0_DIC_MASK);
		
		if(i == (j-1)){
			host->desc[i].d0 &= ~SMCC_DESC0_CH_MASK;
			host->desc[i].d0 |= (SMCC_DESC0_ER_MASK|SMCC_DESC0_LD_MASK|SMCC_DESC0_DIC_MASK);
		} else {
			host->desc[i].d0 |= SMCC_DESC0_CH_MASK;
		}
		
		host->desc[i].d0 |= SMCC_DESC0_OWN_MASK;
		host->desc[i].d1 = (i < (j - 1)) ? DESC_BUF_SIZE : (count % DESC_BUF_SIZE);
		if(!(count % DESC_BUF_SIZE))
			host->desc[i].d1 = DESC_BUF_SIZE;
		host->desc[i].d2 = ((u32)data->dest + DESC_BUF_SIZE * i) & ~KSEG0;
		if( i == (j - 1))
			host->desc[i].d3 = 0;
		else
			host->desc[i].d3 = ((u32)host->desc + sizeof(host->desc[0]) * (i + 1)) & ~KSEG0;

		mmc_debug(" [%s] IDMA desc[%d] at %p, D0= %08x, D1= %08x, D2= %08x, D3= %08x\n",
			__func__, i, &(host->desc[i].d0), host->desc[i].d0, host->desc[i].d1, host->desc[i].d2, host->desc[i].d3);
	}
	asm("sync");

	/* does read need flush dcache? */
	flush_dcache_range((unsigned)data->src, (unsigned)(data->src + data->blocks * data->blocksize));		
	flush_dcache_range((unsigned)(&host->desc[0]), (unsigned)((u32)host->desc + j * sizeof(struct xcsdh_idmac_desc)));
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_DBADDR, NULL, (u32)host->desc & ~KSEG0, 1);

out:
	return err;
}

static void mmc_reset(struct mmc_host *host)
{	
	volatile unsigned int reg; 	

	if(host->core_id == 0){
		/* enable clock for SMCC0 block */
		reg = readl(MMREG_BASE + CG1_CLK_STOP0);
		reg &= ~SMCC0CLK_STOP_MASK;
		writel(reg, MMREG_BASE + CG1_CLK_STOP0);
		reg = readl(MMREG_BASE + CG1_BLK_CLK_STOP0);
		reg &= ~SMCC0_CLK_STOP_MASK;
		writel(reg, MMREG_BASE + CG1_BLK_CLK_STOP0);

#ifdef CFG_MMC_HIGHSPEED		
		/* enable clock 50MHz for SMCC host */
		reg = readl(MMREG_BASE + CG1_CLK_SRC_SEL6);
		reg &= ~SMCC0CLK_SRC_SEL_MASK;
		reg |= (0x1 << SMCC0CLK_SRC_SEL_SHIFT);
		writel(reg, MMREG_BASE + CG1_CLK_SRC_SEL6);
#else	
		reg = readl(MMREG_BASE + CG1_CLK_SRC_SEL6);
		reg &= ~SMCC0CLK_SRC_SEL_MASK;
		reg |= (0x0 << SMCC0CLK_SRC_SEL_SHIFT);
		writel(reg, MMREG_BASE + CG1_CLK_SRC_SEL6);
#endif
		/* set IO pin for SMCC0 */
		reg = readl(MMREG_BASE + GPIO_C_CTRL);
		reg &= ~GPIO_C_CTRL_GPIO_MODE_SEL0_MASK;
		writel(reg, MMREG_BASE + GPIO_C_CTRL);

		/* take out of reset */
		reg = readl(MMREG_BASE + CG_RESET_REG1);
		reg |= SMCC0_RESET_MASK;
		writel(reg, MMREG_BASE + CG_RESET_REG1);
		udelay(1000);
		reg = readl(MMREG_BASE + CG_RESET_REG1);
		reg &= ~SMCC0_RESET_MASK;
		writel(reg, MMREG_BASE + CG_RESET_REG1);

		reg = readl(MMREG_BASE + SMCC_CTRL_REG);
		reg |= (SMCC_CTRL_REG_SOFT_RST_MASK);
		writel(reg, MMREG_BASE + SMCC_CTRL_REG);
		udelay(100);
	
		reg = readl(MMREG_BASE + SMCC_CTRL_REG);
		reg &= ~(SMCC_CTRL_REG_SOFT_RST_MASK);
		writel(reg, MMREG_BASE + SMCC_CTRL_REG);
		udelay(100);
	
		/* Write to clear the all interrupt, disable the interrupt now*/
		xcsdh_access_smcc_reg(0, SMCC_PWREN, NULL, 0x1, 1);
		xcsdh_access_smcc_reg(0, SMCC_PWREN, &reg, 0, 0);		

		xcsdh_access_smcc_reg(0, SMCC_RINTSTS, NULL, 0xffffffff, 1);
		xcsdh_access_smcc_reg(0, SMCC_IDSTS, NULL, 0xffffffff, 1);	
		writel(0xffff, MMREG_BASE + SMCC_STATUS_REG);
		writel(0, MMREG_BASE + SMCC_INT_EN_REG);
	}
	
	if(host->core_id == 1){
		reg = readl(MMREG_BASE + CG1_CLK_STOP0);
		reg &= ~SMCC1CLK_STOP_MASK;
		writel(reg, MMREG_BASE + CG1_CLK_STOP0);
		reg = readl(MMREG_BASE + CG1_BLK_CLK_STOP0);
		reg &= ~SMCC1_CLK_STOP_MASK;
		writel(reg, MMREG_BASE + CG1_BLK_CLK_STOP0);

		/* Change the pin from ETH0 giga bit to MMC1 */
		reg = readl(MMREG_BASE + RBM_PADU_CTRL);
		reg |= PADU_CTRL_ETH0_MASK;
		writel(reg, MMREG_BASE + RBM_PADU_CTRL);

		/* take out of reset */
		reg = readl(MMREG_BASE + CG_RESET_REG1);
		reg |= SMCC0_RESET_MASK;
		writel(reg, MMREG_BASE + CG_RESET_REG1);
		udelay(1000);
		reg = readl(MMREG_BASE + CG_RESET_REG1);
		reg &= ~SMCC1_RESET_MASK;
		writel(reg, MMREG_BASE + CG_RESET_REG1);

		reg = readl(MMREG_BASE + SMCC_CTRL_REG1);
		reg |= (SMCC_CTRL_REG_SOFT_RST_MASK);
		writel(reg, MMREG_BASE + SMCC_CTRL_REG1);
		udelay(100);
	
		reg = readl(MMREG_BASE + SMCC_CTRL_REG1);
		reg &= ~(SMCC_CTRL_REG_SOFT_RST_MASK);
		writel(reg, MMREG_BASE + SMCC_CTRL_REG1);
		udelay(100);
	
		/* Write to clear the all interrupt, disable the interrupt now*/
		xcsdh_access_smcc_reg(1, SMCC_PWREN, NULL, 0x1, 1);
		xcsdh_access_smcc_reg(1, SMCC_PWREN, &reg, 0, 0);	

		xcsdh_access_smcc_reg(1, SMCC_RINTSTS, NULL, 0xffffffff, 1);
		xcsdh_access_smcc_reg(1, SMCC_IDSTS, NULL, 0xffffffff, 1);	
		writel(0xffff, MMREG_BASE + SMCC_STATUS_REG1);
		writel(0, MMREG_BASE + SMCC_INT_EN_REG1);
	}	
	return;
}

static int mmc_wait_cmd_done(struct mmc *mmc,
			    struct mmc_cmd *cmd,
			    struct mmc_data *data,
			    unsigned int timeout)
{
	volatile u32 rintsts;
	volatile u32 idsts;
	volatile u32 sts;
	volatile u32 dscaddr;
	volatile u32 bufaddr;
	volatile u32 tcbcnt;
	volatile u32 tbbcnt;
	volatile u32 *dummy;
	int err = 0, i, count;
	struct mmc_host *host = mmc->priv;

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_RINTSTS, &rintsts, 0, 0);
	if(err)
		goto error;

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_IDSTS, &idsts, 0, 0);
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_STATUS, &sts, 0, 0);
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_DSCADDR, &dscaddr, 0, 0); 
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_BUFADDR, &bufaddr, 0, 0);
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_TCBCNT, &tcbcnt, 0, 0); 
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_TBBCNT, &tbbcnt, 0, 0);

	mmc_debug("core id %d, rintsts 0x%08x, idsts 0x%08x, status 0x%08x DSCADDR 0x%08x, BUFADDR 0x%08x, tcbcnt %d, tbbcnt %d last opcode %d\n",
				host->core_id,
				rintsts, idsts, sts, 
				dscaddr, bufaddr,
				tcbcnt, tbbcnt,
				cmd->cmdidx);

	if(host->core_id == 0){
		
		/* wait for cmd done */
		while(!(rintsts & SMCC_RINTSTS_CD_MASK)){
			xcsdh_access_smcc_reg(host->core_id, SMCC_RINTSTS, &rintsts, 0, 0);			
			udelay(1000);
			timeout --;
			if(timeout == 0){
				printf(" [%s] command %d time out\n", __func__, (unsigned)cmd->cmdidx);			
				return TIMEOUT;	
			}		
		}
		mmc_debug(" [%s] core_id %d cmd: %d command done\n", __func__, host->core_id, cmd->cmdidx);
		

	
		/* Command Done */
		if(data){ 
			/* RW command wait for DTO */
			mmc_debug(" [%s] a data %s command\n", __func__, (data->flags & MMC_DATA_READ) ? "READ" : "WRITE");
			while(!(rintsts & SMCC_RINTSTS_DTO_MASK)){				
				if(rintsts & SMCC_RINTSTS_ALL_ERR_MASK){
					err = COMM_ERR;
					goto error;
				}
				
				if(rintsts & SMCC_RINTSTS_CD_MASK){
					rintsts = 1 << SMCC_RINTSTS_CD_SHIFT;
					xcsdh_access_smcc_reg(host->core_id,SMCC_RINTSTS, NULL, rintsts, 1);
				}
				
				/* Card Remove, reset host if card removed */
				if(rintsts & SMCC_RINTSTS_CARDDET_MASK){					
					xcsdh_check_card_present(host);
					if(!host->card_present){
						printf("card removed, reset host controller\n");
						mmc->has_init = 0;
						err = NO_CARD_ERR;
						mmc_reset(host);
						goto done;
					}
				}
				xcsdh_access_smcc_reg(host->core_id, SMCC_RINTSTS, &rintsts, 0, 0);	
			}
			
			/* wait for all desc take by HW */			
			count = data->blocks * data->blocksize/DESC_BUF_SIZE;
			if(count % DESC_BUF_SIZE)
				count ++;
			for(i = 0; i < count; i++){
				dummy = (volatile u32 *)((u32)&(host->desc[i].d0) | 0xA0000000);
				while(*dummy & SMCC_DESC0_OWN_MASK);
			}
			mmc_debug(" [%s] all dma descriptor were taken by controller\n", __func__);

			xcsdh_access_smcc_reg(host->core_id,SMCC_IDSTS, NULL, 0, 1);

			if(data->flags & MMC_DATA_READ){
				inv_dcache_range((unsigned)data->dest, (unsigned)(data->dest + data->blocks * data->blocksize));
			} else {
				mmc_debug(" [%s] write data done\n", __func__);
			}			
		} else {
			if(rintsts & SMCC_RINTSTS_ALL_ERR_MASK){
				err=COMM_ERR;
				goto error;
			}
			
			if(rintsts & SMCC_RINTSTS_DTO_MASK){
				mmc_debug(" [%s] impossible DTO should not be here \n", __func__);
			}

			/* Card Remove, reset host if card removed */
			if(rintsts & SMCC_RINTSTS_CARDDET_MASK){
				mmc_debug(" [%s] Detected card present status change\n", __func__);
				xcsdh_check_card_present(host);			
				if(!host->card_present){
					printf("card removed, reset host controller\n");
					mmc->has_init = 0;
					err = NO_CARD_ERR;
					mmc_reset(host);
					goto done;					
				}
			}
		}		
	} else {
		printf("Warning, not support host controller #1\n");
	}

error:
	if(cmd->resp_type & MMC_RSP_PRESENT){
		if(cmd->resp_type & MMC_RSP_136){
			xcsdh_access_smcc_reg(host->core_id,SMCC_RESP0, &cmd->response[3], 0, 0);
			xcsdh_access_smcc_reg(host->core_id,SMCC_RESP1, &cmd->response[2], 0, 0);
			xcsdh_access_smcc_reg(host->core_id,SMCC_RESP2, &cmd->response[1], 0, 0);
			xcsdh_access_smcc_reg(host->core_id,SMCC_RESP3, &cmd->response[0], 0, 0);
		} else {
			xcsdh_access_smcc_reg(host->core_id,SMCC_RESP0, &cmd->response[0], 0, 0);
		}
		mmc_debug(" [%s] cmd: %d response 0=%x 1=%x 2=%x 3=%x\n", __func__, cmd->cmdidx, 
			cmd->response[0], cmd->response[1], cmd->response[2], cmd->response[3]);
	}


done:
	if(err){
		mmc_debug(" [%s] cmd: %d error, errno: %d\n", __func__, cmd->cmdidx, err);
	}
	xcsdh_access_smcc_reg(host->core_id,SMCC_RINTSTS, &rintsts, 0, 0);
	xcsdh_access_smcc_reg(host->core_id,SMCC_RINTSTS, NULL, rintsts, 1);
	mmc_debug(" [%s] exit\n", __func__);
	return err;
}

static int mmc_send_cmd(struct mmc *mmc, struct mmc_cmd *cmd,
			struct mmc_data *data)
{
	struct mmc_host *host = (struct mmc_host *)mmc->priv;
	unsigned int flags = 0, args, reg = 0;
	int err;

	mmc_debug(" mmc_send_cmd called\n");
	
	/* wait for last transaction done */
	err = xcsdh_wait_ciu(host->core_id, 1000);
	if (err){
		mmc_debug("wait for controller receiving command timed out\n");
		return err;
	}
	
	
	if(cmd->cmdidx == MMC_CMD_GO_IDLE_STATE){
		/* CMD0 need init bit */
		reg = (1 << SMCC_CMD_START_CMD_SHIFT) | (1 << SMCC_CMD_SEND_INIT_SHIFT);		
	} else {
		reg = 1 << SMCC_CMD_START_CMD_SHIFT;
	}

	if(cmd->resp_type & MMC_RSP_PRESENT){
		flags = 1 << SMCC_CMD_RSP_EXP_SHIFT;
	}

	if(cmd->resp_type & MMC_RSP_136){
		flags |= (1 << SMCC_CMD_RSP_LEN_SHIFT);
	}
	
	if(cmd->resp_type & MMC_RSP_CRC){
		flags |= (1 << SMCC_CMD_CHK_RSP_CRC_SHIFT);
	}

	if((cmd->cmdidx == 17)||(cmd->cmdidx == 18)||(cmd->cmdidx == 24)||(cmd->cmdidx == 25)){
		xcsdh_access_smcc_reg(host->core_id, SMCC_CTRL, NULL, 0x2000010 | (0x3<<9), 1);
	}
	reg |= (flags | (1 << SMCC_CMD_WAIT_PRVDATA_CMP_SHIFT) | cmd->cmdidx);
	args = cmd->cmdarg;

	mmc_debug(" [%s] opcode %d args: %08x flags: %08x reg %08x\n", __func__, cmd->cmdidx, cmd->cmdarg, cmd->flags, reg);

	if (data){
		/* RW command */
		if(data->flags & MMC_DATA_READ){
			mmc_debug(" [%s] read data command blksize: %08x blks: %08x \n", __func__, data->blocksize, data->blocks);		
		} else if (data->flags & MMC_DATA_WRITE){
			mmc_debug(" [%s] write data command blksize: %08x blks: %08x \n", __func__, data->blocksize, data->blocks);
		} else {
			printf("impossible received data command without any flag\n");
			err = -1;
			goto out;
		}

		/* Set up iDMA Desc & Buffer*/
		mmc_prepare_data(host, data);
		
		/* write */
		//mmc_set_transfer_mode(host, data);

		err = xcsdh_access_smcc_reg(host->core_id, SMCC_BYTCNT, NULL, (data->blocksize * data->blocks), 1);
		if (err)
			goto out;

		err = xcsdh_access_smcc_reg(host->core_id, SMCC_BLKSIZ, NULL, data->blocksize, 1);
		if (err)
			goto out;

		if((data->blocks > 1)&&(cmd->cmdidx != MMC_CMD_APP_CMD))
			reg |= 1 << SMCC_CMD_SEND_AUTO_STOP_SHIFT;

		reg |= 1 << SMCC_CMD_DATA_EXP_SHIFT;

		if(data->flags & MMC_DATA_WRITE)
			reg |= 1 << SMCC_CMD_DATA_RW_SHIFT;
		else
			reg &= ~(1 << SMCC_CMD_DATA_RW_SHIFT);
#if 0
		/* not sure */
		if(mmc->ocr & OCR_HCS){
			mmc_debug(" [%s] detect a HC card mmc->ocr: %08x\n", __func__, mmc->ocr);
			/* SDHC, count with block (512) */
			args = cmd->cmdarg >> 9;
		}
#endif

	} else {
		xcsdh_access_smcc_reg(host->core_id, SMCC_BYTCNT, NULL, 0, 1);		
		xcsdh_access_smcc_reg(host->core_id, SMCC_BLKSIZ, NULL, 0, 1);
	}
	
	/* SEND MMC COMMAND */
	mmc_debug(" [%s] write cmd reg: 0x%08x, opcode: %d, arg: 0x%08x \n",__func__, reg, cmd->cmdidx, args);

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CMDARG, NULL, args, 1);
	if (err)
		goto out;
	
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CMD, NULL, reg, 1);
	if (err)
		goto out;

	err = xcsdh_wait_ciu(host->core_id, 1000);
	if (err)
		goto out;

	/* wait for command done, timeout 10 ms */
	err = mmc_wait_cmd_done(mmc, cmd, data, 100);

out:
	if(err)
		mmc_debug(" [%s] ====> error!!!\n", __func__);
	else
		mmc_debug(" [%s] success\n\n", __func__);
	return err;	
}

/* DB p177, pre clock command */
static inline int mmc_load_clock_reg(struct mmc_host *host)
{
	int err = 0;	
	volatile u32 reg = 	1 << SMCC_CMD_START_CMD_SHIFT 	| \
						1 << SMCC_CMD_UPDATE_CLK_SHIFT 	| \
						1 << SMCC_CMD_WAIT_PRVDATA_CMP_SHIFT;

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CMD, NULL, reg, 1);
	asm("sync");
	return err;
}

/* take input clock divider, it is increment on the 2 power of clk_div*/
static int mmc_change_clock(struct mmc *mmc, uint clk_div)
{
	volatile u32 reg;
	int err = 0;
	struct mmc_host *host = mmc->priv;

	/* disable clock */
	reg = 0;
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CLKENA, NULL, reg, 1);
	if (err)
		goto out;
	if(mmc_load_clock_reg(host))
		goto out;
	
	/* set clock */
	reg = clk_div;
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CLKDIV, NULL, reg, 1);
	if (err)
		goto out;
	/* Set clock source (always 0) */
	reg = 0;
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CLKSRC, NULL, reg, 1);
	if (err)
		goto out;
	if(mmc_load_clock_reg(host))
		goto out;

	reg = 1;
	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CLKENA, NULL, reg, 1);

	if(mmc_load_clock_reg(host))
		goto out;

	/* Enable clock */
	if (clk_div == 0) 
		host->clock = mmc->f_max;
	else 
		host->clock = mmc->f_max / (clk_div * 2);

#ifdef MMC_DEBUG
	xcsdh_access_smcc_reg(host->core_id, SMCC_CLKDIV, &reg, 0, 0);
	mmc_debug(" [%s] clock divder = %08x\n", __func__, reg);
	xcsdh_access_smcc_reg(host->core_id, SMCC_CLKSRC, &reg, 0, 0);
	mmc_debug(" [%s] clock source = %08x\n", __func__, reg);
#endif
	
	out:
		if(err)
			mmc_debug(" [%s] fail\n", __func__);
		return (err);
}

static int xcsdh_set_bus_width(struct mmc_host *host, u32 w)
{
	volatile u32 reg;
	int err;

	switch(w) {
		case 1:
			reg = 0x0;
			break;
		case 4:
			reg = 0x01;	/* only 1 card... */
			break;
		default:
			reg = 0x0;
			break;
	}

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_CTYPE, NULL, reg, 1);
	xcsdh_access_smcc_reg(host->core_id, SMCC_CTYPE, &reg, 0, 0);
	return(err);
}

static void mmc_set_ios(struct mmc *mmc)
{
	struct mmc_host *host = mmc->priv;
	unsigned int clk_div = 0;
	
	mmc_debug(" [%s] bus_width: %d, clock: %d\n", __func__, mmc->bus_width, mmc->clock);

	/* Change clock first */
	if (mmc->clock != 0) {
		clk_div = mmc->f_max / mmc->clock;
		clk_div = (int) (clk_div / 2); /* its the clock devider */
		mmc_debug("[%s] requested mmc->clock = %d, clk divider = %d\n",
				__func__, mmc->clock, clk_div);
	
		if(mmc_change_clock(mmc, clk_div))
			printf("mmc set clock fail\n");			
	}
	
	/* Change the bus width */
	if(mmc->bus_width == 8) {
		printf("Host does not support 8 bit bus width\n");
	} else if(mmc->bus_width == 4) {
		if(xcsdh_set_bus_width(host, 4))
			printf("mmc set bus width %d fail\n", mmc->bus_width);
	} else {
		if(xcsdh_set_bus_width(host, 1))
			printf("mmc set bus width %d fail\n", mmc->bus_width);
	}
	
	return;
}

static int mmc_core_init(struct mmc *mmc)
{
	struct mmc_host *host = (struct mmc_host *)mmc->priv;
	unsigned int version = 0;
	int err;

	err = xcsdh_access_smcc_reg(host->core_id, SMCC_VERID, &version, 0, 0);
	if(err)
		goto init_fail;
	mmc_debug("[%s] mmc host controller version: %x \n", __func__, version);

	xcsdh_check_card_present(host);

	mmc_debug(" [%s] detect card in slot %d\n", __func__, host->core_id);

	if(xcsdh_enable_dma(host))
		goto init_fail;

	if(xcsdh_setup_polling_mode(host))
		goto init_fail;

	return 0;

init_fail:
	mmc_debug(" [%s] failed!!!\n", __func__);
	return -1;
}

static int xcode_mmc_getcd(struct mmc *mmc)
{
	struct mmc_host *host = (struct mmc_host *)mmc->priv;	

	xcsdh_check_card_present(host);		

	return (host->card_present);
}

/*
  * dev_index	- Host Controller index
  * bus_width	- support 1/4 bits data bus width
  * pwr_gpio 	- Output: Host slot power control gpio, on SDK board is always on
  * oc_gpio	- Input: Over Current detect gpio, on SDK board is discard
  */
int xcode_mmc_init(int dev_index, int bus_width, int pwr_gpio, int oc_gpio)
{
	struct mmc_host *host;
	struct mmc *mmc;	

	mmc_debug(" [%s]: index %d, bus width %d "
		"pwr_gpio %d oc_gpio %d\n",
		__func__, dev_index, bus_width, pwr_gpio, oc_gpio);

	if(mmc_support_dual_host() && (dev_index > 0)){
		printf(" [%s]: board does not support dual host.\n");
		return -1;
	}

	host = &mmc_host[dev_index];
	host->core_id = dev_index; 
	//init host HW

	mmc = &mmc_dev[dev_index];
	memset((void *)host, 0, sizeof(struct mmc_host));
	memset((void *)mmc, 0, sizeof(struct mmc));

	mmc_reset(host);

	sprintf(mmc->name, "XCode SD/MMC");
	mmc->priv = host;
	mmc->send_cmd = mmc_send_cmd;
	mmc->set_ios = mmc_set_ios;
	mmc->init = mmc_core_init;
	mmc->getcd = xcode_mmc_getcd;


	/* set up voltage */
	mmc->voltages = MMC_VDD_32_33 | MMC_VDD_33_34 | MMC_VDD_34_35 | MMC_VDD_35_36;
	
	/* set up bus width */
	if (bus_width == 4){
		mmc->host_caps = MMC_MODE_4BIT;
	} else if(bus_width == 8) {
		printf("Warning mmc host does not support 8-bit bus\n");
		return -1;
	} else {
		printf("Default mmc host bus width 1-bit\n");
	}
	
	/* setup host capability */
	mmc->host_caps |= (MMC_MODE_HS_52MHz | MMC_MODE_HS | MMC_MODE_HC);

	/*
	 * setup frequency
	 * min freq is for card identification, it is 400KHz
	 * max freq is 50MHz
	 */
	mmc->f_min = 400 * 1000;
	mmc->f_max = 50 * 1000 * 1000;	
		
	mmc_register(mmc);	
	return 0;
}

int board_mmc_init(bd_t *bis)
{
	int err = 0;
	
	err = xcode_mmc_init(0, 4, -1, -1);
	if(err)
		puts("board mmc init fail\n");
	return err;
}

