/*-
 * Copyright (c) 2001-2010, Vixs Systems, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#if (CONFIG_COMMANDS&CFG_CMD_USB)&&defined(CONFIG_USB_EHCI)

#include <usb.h>

#include "ehci.h"
#include "ehci-core.h"
#include "asm/viper.h"
#include <asm/viper_gpio.h>
#include <asm/io.h>


unsigned int SDUH_HCD_REG_BASE = 0;

//TODO: [JLWANG] Add debug message if failed in case
unsigned int SDUH_writel(unsigned int ehci_reg_addr, unsigned int value, SDUH_HC_reg_type type)
{
	unsigned int regVal = 0;

	//    printf("Try write ehci %x\n", ehci_reg_addr);
	while(((regVal = readl(XC_SOC_PROC_MMREG_BASE+USBH_CSR_STATUS_REG))&0x1)== 1){};


	writel(value, XC_SOC_PROC_MMREG_BASE+USBH_WDAT_REG);
	switch(type)
	{
		case EHCI_REG:
			writel(ehci_reg_addr|0x80000000, XC_SOC_PROC_MMREG_BASE+USBH_EHCI_CSR_CTRL_REG);
			break;
		case OHCI_REG:
			writel(ehci_reg_addr|0x80000000, XC_SOC_PROC_MMREG_BASE+USBH_OHCI_OP_CSR_CTRL_REG);
			break;
		default:
			return 0;
			break;
	}
	while(((regVal = readl(XC_SOC_PROC_MMREG_BASE+USBH_CSR_STATUS_REG))&0x1)== 1){};

	if(regVal&0x2) { return 0;};

	//   printf("end\n");
	return 0;
};


unsigned int SDUH_readl(unsigned int ehci_reg_addr, SDUH_HC_reg_type type)
{
	unsigned int regVal = 0;
	//    printf("Try read ehci %x\n", ehci_reg_addr);

	while(((regVal = readl(XC_SOC_PROC_MMREG_BASE+USBH_CSR_STATUS_REG))&0x1)== 1)
	{
		//    			printk("<1>"" regVal value is %x\n", regVal);
	};


	switch(type)
	{
		case EHCI_REG:
			writel(ehci_reg_addr, XC_SOC_PROC_MMREG_BASE+USBH_EHCI_CSR_CTRL_REG);
			break;
		case OHCI_REG:
			writel(ehci_reg_addr, XC_SOC_PROC_MMREG_BASE+USBH_OHCI_OP_CSR_CTRL_REG);
			break;
		default:
			return 0;
			break;
	}
	while(((regVal = readl(XC_SOC_PROC_MMREG_BASE+USBH_CSR_STATUS_REG))&0x1)== 1)
	{
		//    			printk("<1>"" regVal value is %x\n", regVal);
	};

	if(regVal&0x2) { return 0;};

	regVal = readl(XC_SOC_PROC_MMREG_BASE+USBH_RDAT_REG);

	//    		printk("<1>"" regVal value is %x\n", regVal);

	//    printf("end %x\n", regVal);

	return regVal;
};

/*
 * Create the appropriate control structures to manage
 * a new EHCI host controller.
 */

static void ehci_sduh_gpio_setup(void)
{
	u32 bid, reg;
	//	unsigned long flags;
	bid = readl((void*)(CG_DUMMY_REG + XC_SOC_PROC_MMREG_BASE));

	switch(bid) {
		case 0x1701:
			/* GPIO_F PIN 1, 3 OUT: USB Connector power; PIN 0 IN: USB hosr Over Current; */
			//			local_irq_save(flags);
			reg = readl((void*)(GPIO_F_CTRL + XC_SOC_PROC_MMREG_BASE));
			writel(reg | GPIO_F_CTRL_GPIO_MODE_SEL_MASK, (void*)(GPIO_F_CTRL + XC_SOC_PROC_MMREG_BASE));
			reg = readl((void*)(GPIO_F_OE + XC_SOC_PROC_MMREG_BASE));
			reg |= ((1 << 1) | (1 << 3));
			reg &= ~0x01;
			writel(reg, (void*)(GPIO_F_OE + XC_SOC_PROC_MMREG_BASE));
			//			local_irq_restore(flags);
			udelay(10);
			break;

		case 0x1C00:
		case 0x1C01:
		case 0x1C02:
		case 0x1C03:
			/* GPIO_DEDICATED PIN 5 OUT: USB Connector power; PIN 6 IN: USB hosr Over Current; */
			//			local_irq_save(flags);
			reg = readl((void*)(GPIO_DEDICATED_OUTEN + XC_SOC_PROC_MMREG_BASE));
			reg |= (1 << 5);
			reg &= ~(1 << 6);
			writel(reg, (void*)(GPIO_DEDICATED_OUTEN + XC_SOC_PROC_MMREG_BASE));
			//			local_irq_restore(flags);
			udelay(10);
			break;

		default:
			/* GPIO_J PIN 12, 13 OUT: USB Connector power; */
			//			local_irq_save(flags);
			reg = readl((void*)(GPIO_J_CTRL + XC_SOC_PROC_MMREG_BASE));
			writel(reg | GPIO_J_CTRL_GPIO_MODE_SEL_MASK, (void*)(GPIO_F_CTRL + XC_SOC_PROC_MMREG_BASE));    
			reg = readl((void*)(GPIO_F_OE + XC_SOC_PROC_MMREG_BASE));
			reg |= ((1 << 12) | (1 << 13));
			writel(reg, (void*)(GPIO_F_OE + XC_SOC_PROC_MMREG_BASE));
			//			local_irq_restore(flags);
			udelay(10);
			break;
	}
	return;
}

static void usb_connector_pwr_on(void)
{
	u32 bid;
	bid = readl((void*)(CG_DUMMY_REG + XC_SOC_PROC_MMREG_BASE));

	/* Togle the power connector power GPIO to reset the connected USB device first
	 * then turn on the connector power
	 */
	switch(bid) {
		case 0x1C00:
		case 0x1C01:
		case 0x1C02:
		case 0x1C03:
			viper_gpio_clear_prot(GPIO_DEDICATED_OUT, 5);
			udelay(10);
			viper_gpio_set_prot(GPIO_DEDICATED_OUT, 5);
			break;

		case 0x1701:
			viper_gpio_clear_prot(GPIO_F_OUT, 1);
			viper_gpio_clear_prot(GPIO_F_OUT, 3);
			udelay(10);
			viper_gpio_set_prot(GPIO_F_OUT, 1);
			viper_gpio_set_prot(GPIO_F_OUT, 3);
			break;

		default:
			viper_gpio_clear_prot(GPIO_J_OUT, 12);
			viper_gpio_clear_prot(GPIO_J_OUT, 13);
			udelay(10);
			viper_gpio_set_prot(GPIO_J_OUT, 12);
			viper_gpio_set_prot(GPIO_J_OUT, 13);
			break;
	}
	return;
}

int ehci_hcd_init(void)
{
	unsigned int temp;

	//Do HW initialization first
	//TODO: move the following parts into ehci_cpu_init


	ehci_sduh_gpio_setup();
	usb_connector_pwr_on();

	// enable USBH and USBD clk
	temp = readl((void *)(CG1_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));
	temp &= ~UCLK_STOP_MASK;
	writel(temp, (void *)(CG1_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));

	// When using Device mode
	temp = readl((void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));
	temp &= ~USBD_MCLK_STOP_MASK;
	writel(temp, (void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));

	// When using Host mode
	temp = readl((void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));
	temp &= ~USBH_MCLK_STOP_MASK;
	writel(temp, (void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));
	temp = readl((void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));
	temp &= ~USBH_SCLK_STOP_MASK;
	writel(temp, (void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));


	//reset USB D firstl
	debug("try reset SDUH controller\n");
	temp = readl((void *)(CG_RESET_REG + XC_SOC_PROC_MMREG_BASE));
	//temp |= USBD_RESET_MASK;
	//writel(temp, (void *)(CG_RESET_REG + XC_SOC_PROC_MMREG_BASE));
	//udelay(1000);
	temp &= ~USBD_RESET_MASK;
	writel(temp, (void *)(CG_RESET_REG + XC_SOC_PROC_MMREG_BASE));  
	udelay(1000);

	//Configure the clock for PHY 
	temp = 0;
	temp = readl((void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));
	temp |= UCLK_STOP_MASK;
	writel(temp, (void *)(CG1_BLK_CLK_STOP0 + XC_SOC_PROC_MMREG_BASE));
	temp = (2<<PHY_REFCLKSEL_SHIFT);
	temp |= (2<<PHY_REFCLKDIV_SHIFT);
	writel(temp , (void*)(USBD_PHY_CLK_CTRL+ XC_SOC_PROC_MMREG_BASE));
	udelay(1000);

	//Select USB PHY in host mode
	temp = readl((void *)(USBD_PHY_CTRL + XC_SOC_PROC_MMREG_BASE));
	temp &= ~0xc;
	writel(temp|0x3, (void *)(USBD_PHY_CTRL + XC_SOC_PROC_MMREG_BASE));
	udelay(1000);

	//Add settings for viper
	temp = readl((void *)(USBD_PHY_CTRL + XC_SOC_PROC_MMREG_BASE));
	writel(temp|0x3, (void *)(USBD_PHY_CTRL+ XC_SOC_PROC_MMREG_BASE));
	//udelay(1000);

	udelay(1000);

	//reset host controll
	temp = readl((void *)(CG_RESET_REG + XC_SOC_PROC_MMREG_BASE));
	temp |= USBH_RESET_MASK;
	writel(temp, (void *)(CG_RESET_REG + XC_SOC_PROC_MMREG_BASE));
	udelay(1000);
	temp &= ~USBH_RESET_MASK;
	writel(temp, (void *)(CG_RESET_REG + XC_SOC_PROC_MMREG_BASE));  
	udelay(1000);

	temp = readl((void *)(XC_SOC_PROC_MMREG_BASE + USBH_CTRL_REG));
	temp |= USBH_CTRL_REG_SOFT_RESET_MASK;
	writel(temp, (void *)(XC_SOC_PROC_MMREG_BASE + USBH_CTRL_REG));

	// reset USBH PHY
	temp = readl((void *)(XC_SOC_PROC_MMREG_BASE + USBH_PHY_CTRL_REG));
	temp |= USBH_PHY_POR_MASK;
	writel(temp, (void *)(XC_SOC_PROC_MMREG_BASE + USBH_PHY_CTRL_REG));
	udelay(1000);
	temp = readl((void *)(XC_SOC_PROC_MMREG_BASE + USBH_PHY_CTRL_REG));
	temp &= ~USBH_PHY_POR_MASK;
	writel(temp, (void *)(XC_SOC_PROC_MMREG_BASE + USBH_PHY_CTRL_REG));
	udelay(1000);

	// Take the USBH out of reset
	temp = readl((void *)(XC_SOC_PROC_MMREG_BASE + USBH_CTRL_REG));
	temp &= ~USBH_CTRL_REG_SOFT_RESET_MASK;
	writel(temp, (void *)(XC_SOC_PROC_MMREG_BASE + USBH_CTRL_REG));    
	udelay(1000);
	debug("reset SDUH controller end\n");


	//setup the phy to fix port power problem

	temp = readl((void *)(XC_SOC_PROC_MMREG_BASE + USBH_PHY_CTRL_REG));
	temp |= 0x10;
	writel(temp, (void *)(XC_SOC_PROC_MMREG_BASE + USBH_PHY_CTRL_REG));
	udelay(1000);
	temp &= ~0x10;
	writel(temp, (void *)(XC_SOC_PROC_MMREG_BASE + USBH_PHY_CTRL_REG));    
	udelay(1000);



	hccr = (struct ehci_hccr *)0;
	hcor = (struct ehci_hcor *)((uint32_t) hccr +
			HC_LENGTH(ehci_readl(&hccr->cr_capbase)));

	return 0;
}

/*
 * Destroy the appropriate control structures corresponding
 * the the EHCI host controller.
 */
int ehci_hcd_stop(void)
{
	return 0;
}


#endif

