#include <common.h>
#include <asm/errno.h>
#include <linux/usb/ch9.h>
#include <linux/usb/cdc.h>
#include <linux/usb/gadget.h>
#include <net.h>
#include <linux/ctype.h>

//#include "usb_gadget/gadget_chips.h"
#include "gadget_chips.h"
#include "ic_debug.h"
#include <linux/usb/xvif.h>
#include <linux/usb/vixsdma.h>
#include <linux/usb/xvethioctl.h>
#include <linux/usb/xvdrv_if.h>
#include <ic_debug.h>

#define XVDEV_IOCTL_SEND_TIMEOUT  (10*HZ)
#ifndef CDC_HOST
#define XC3ARC_BUILD
#include <linux/usb/gadget.h>
#endif
#ifdef VIXS_CDC

static int vixs_eth_start_xmit (struct para_send_req *skb, struct net_device *net, int len);

static void xvdev_release_handle(PVDEV pVdev, int handle)
{
    int i;
	if(pVdev->xvdev_ioctl_param[handle].send_buf_user)
    {
        pVdev->xvdev_ioctl_param[handle].send_buf_user= 0;
    }

    if(pVdev->xvdev_ioctl_param[handle].send_buf)
    {
        free(pVdev->xvdev_ioctl_param[handle].send_buf);
        pVdev->xvdev_ioctl_param[handle].send_buf = 0;
    }


        
    for(i=0;i<RECV_BUF_NUM;i++)
    {
	    if(pVdev->xvdev_ioctl_param[handle].recv_buf_user[i])
        {
            pVdev->xvdev_ioctl_param[handle].recv_buf_user[i] = 0;
        }

        if(pVdev->xvdev_ioctl_param[handle].recv_buf[i])
        {
            free(pVdev->xvdev_ioctl_param[handle].recv_buf[i]);
            pVdev->xvdev_ioctl_param[handle].recv_buf[i] = 0;
        }

     
    }

    pVdev->xvdev_ioctl_param[handle].flag = NULL_HANDLE;
    
}

static unsigned int map_kernel_to_user(unsigned int kaddr,unsigned size)
{
    return kaddr;

}


int xvdev_allocate_kernel_mem(PVETHDEV pVdev,void *parg)
{
    struct para_alloc_kernel_mem *preq = (struct para_alloc_kernel_mem *)parg;
    
    preq->memaddr_kernel  = malloc(preq->memsize);
    if(!preq->memaddr_kernel)
    {
        preq->return_value = VDEV_IOCTL_NO_MEM;    
        return -1;
    }

    PDEBUG("Allocated Kernel Mem vAddr %x pAddr %x size %x\n", preq->memaddr_user, preq->memaddr_kernel, preq->memsize);

    return 0;
    
}


int xvdev_release_kernel_mem(PVETHDEV pVdev,void *parg)
{
    struct para_release_kernel_mem *preq = (struct para_release_kernel_mem *)parg;    

    PDEBUG("Release Kernel Mem vAddr %x pAddr %x size %x\n", preq->memaddr_user, preq->memaddr_kernel, preq->memsize);
    

    if(preq->memaddr_kernel)
    {
        free((unsigned long)preq->memaddr_kernel);
    }

    return 0;
}

int xvdev_get_rxbuf(PVETHDEV pVdev,unsigned int handle,unsigned recv_size,void **rxbuf)
{
    struct ioctl_param *pparam;
    
    pparam = &pVdev->xvdev_ioctl_param[handle];

    if(pparam->flag!= HANDLE_OPENED)
        return VDEV_IOCTL_PEER_NOT_OPENED;

    if(((pparam->recv_head+1) & RECV_BUF_MASK) == pparam->recv_tail)
        return VDEV_IOCTL_PEER_BUF_FULL;

    *rxbuf = pparam->recv_buf[pparam->recv_head];
    pparam->recv_buf_size[pparam->recv_head]=recv_size - sizeof(struct _VIXS_CDC_HDR);
    pparam->recv_head++;
    pparam->recv_head &= RECV_BUF_MASK;

    if(((pparam->recv_head+1) & RECV_BUF_MASK) == pparam->recv_tail)
        return VDEV_IOCTL_RX_BUF_FULL_NO_DROP;
    else 
        return 0;
}

int xvdev_is_rx_full(PVETHDEV pVdev,void* parg,int* head){

    struct para_recv_req  * preq = (struct para_recv_req *)parg;
    int handle;
    unsigned int recv_head;
    unsigned int recv_tail;
    struct ioctl_param *pparam;
    unsigned long flag;
    
    //PDEBUG("--->%s entry.\n",__FUNCTION__);

    handle = preq->handle;
    if((handle < 0) || (handle >= XVDEV_MAX_HANDLE))
        return -1;

    pparam = &pVdev->xvdev_ioctl_param[handle];
    
    if(HANDLE_OPENED != pparam->flag)
        return -1;

        recv_tail = pparam->recv_tail;
        recv_head = pparam->recv_head;

    *head = recv_head;
    return (recv_head + 1 & RECV_BUF_MASK) == recv_tail;
}

int xvdev_is_all_rx_full(PVETHDEV pVdev){

    int handle;
    unsigned int recv_head;
    unsigned int recv_tail;
    struct ioctl_param *pparam;
    unsigned long flag;
    int ret=0; 
    //PDEBUG("--->%s entry.\n",__FUNCTION__);

    for(handle=0;handle<XVDEV_MAX_HANDLE;handle++){
        pparam = &pVdev->xvdev_ioctl_param[handle];
    
        if(HANDLE_OPENED != pparam->flag)
            continue;
        recv_tail = pparam->recv_tail;
        recv_head = pparam->recv_head;
    
        ret |= (((recv_head +1 & RECV_BUF_MASK) == recv_tail)<<handle);
    }

    return ret;

}
void xvdev_recv_notify(PVETHDEV pVdev,unsigned int handle)
{
    //DBF_LOC;
    pVdev->xvdev_ioctl_param[handle].recv_flag = 1;
}

void xvdev_ioctl_init(PVETHDEV pVdev)
{
    DBF_LOC;
    memset(pVdev->xvdev_ioctl_param,0x0,sizeof(pVdev->xvdev_ioctl_param));
    pVdev->mtu=MTU_SIZE;
}

void xvdev_ioctl_release(PVETHDEV pVdev)
{
    int i;
    PDEBUG("--->%s entry.\n",__FUNCTION__);
    for(i=0;i<XVDEV_MAX_HANDLE;i++)
    {
        if(pVdev->xvdev_ioctl_param[i].flag != NULL_HANDLE);
            xvdev_release_handle(pVdev, i);
    }
}

int xvdev_cmd_open(PVDEV pVdev,void *parg)
{
    int i;
    struct para_open_req  * preq = (struct para_open_req *)parg;
    int handle;
    handle = preq->handle;
    if((handle < 0) || (handle >= XVDEV_MAX_HANDLE)){
        DBF("handle:%x\n",handle);
        preq->return_value=-1;
        return -1;
    }
    
    if(NULL_HANDLE != pVdev->xvdev_ioctl_param[handle].flag){
        DBF("pVdev->xvdev_ioctl_param[%d].flag:%d\n",handle,pVdev->xvdev_ioctl_param[handle].flag);
        preq->return_value=-1;
        return -1;
    }

	pVdev->xvdev_ioctl_param[handle].max_buf_size = pVdev->mtu;
    pVdev->xvdev_ioctl_param[handle].send_buf = malloc(pVdev->xvdev_ioctl_param[handle].max_buf_size + sizeof(struct _VIXS_CDC_HDR)) ;
    if(0 == pVdev->xvdev_ioctl_param[handle].send_buf)
        goto  ioctl_open_exit;
    
    pVdev->xvdev_ioctl_param[handle].send_buf_user = 
        map_kernel_to_user((unsigned int)pVdev->xvdev_ioctl_param[handle].send_buf,pVdev->xvdev_ioctl_param[handle].max_buf_size + sizeof(struct _VIXS_CDC_HDR));

    pVdev->xvdev_ioctl_param[handle].send_buf_user += sizeof (struct _VIXS_CDC_HDR);
 
    if(!pVdev->xvdev_ioctl_param[handle].send_buf_user){
        DBF_LOC;
        goto ioctl_open_exit; 
    }


    for(i=0;i<RECV_BUF_NUM;i++)
    {
        pVdev->xvdev_ioctl_param[handle].recv_buf[i] = (void *) malloc(pVdev->xvdev_ioctl_param[handle].max_buf_size + sizeof(struct _VIXS_CDC_HDR));
        if(0 == pVdev->xvdev_ioctl_param[handle].recv_buf[i]){
            DBF_LOC;
            goto ioctl_open_exit;
        }

        pVdev->xvdev_ioctl_param[handle].recv_buf_user[i] = 
            map_kernel_to_user((unsigned int)pVdev->xvdev_ioctl_param[handle].recv_buf[i] ,pVdev->xvdev_ioctl_param[handle].max_buf_size+sizeof(struct _VIXS_CDC_HDR));
        if(!pVdev->xvdev_ioctl_param[handle].recv_buf_user[i]){
            DBF_LOC;
            goto ioctl_open_exit; 
    }

        pVdev->xvdev_ioctl_param[handle].recv_buf_user[i] += sizeof( struct _VIXS_CDC_HDR);

    }
    preq->handle = handle;
    preq->input_buf = pVdev->xvdev_ioctl_param[handle].send_buf_user;
    preq->input_buf_phy = (unsigned int)pVdev->xvdev_ioctl_param[handle].send_buf;
    preq->input_maxsize = pVdev->xvdev_ioctl_param[handle].max_buf_size ;
    pVdev->xvdev_ioctl_param[handle].flag = HANDLE_OPENED;
    pVdev->xvdev_ioctl_param[handle].handle = handle;
    pVdev->xvdev_ioctl_param[handle].recv_head = 0;
    pVdev->xvdev_ioctl_param[handle].recv_tail = 0;
    pVdev->xvdev_ioctl_param[handle].recv_cur_addr = 0;
    pVdev->xvdev_ioctl_param[handle].pVdev =(PVDEV) pVdev;
    pVdev->p_type=preq->type;
    preq->return_value=0;
    return 0;
ioctl_open_exit:
    DBF_LOC;
    preq->return_value=-1;
    xvdev_release_handle(pVdev, handle);
    return -1;
}

int xvdev_cmd_close(PVDEV pVdev,void *parg)
{
    struct para_close_req  * preq = (struct para_close_req *)parg;
    if((preq->handle < 0) || (preq->handle >= XVDEV_MAX_HANDLE))
        return -1;

    if(HANDLE_OPENED == pVdev->xvdev_ioctl_param[preq->handle].flag)
        xvdev_release_handle(pVdev, preq->handle);
    
    return 0;
}
extern struct eth_dev l_ethdev;
int xvdev_cmd_send(void * dev , void *parg)
{
    struct para_send_req  * preq = (struct para_send_req *)parg;
    PVDEV pVdev;
    PVIXS_CDC_HDR pvixs_cdc_hdr;
	struct eth_dev		*priv = &l_ethdev;
    int handle;
    int len=0;
    int ret=0;
    pVdev =priv->pVdev;
    handle = preq->handle;
    if((handle < 0) || (handle >= XVDEV_MAX_HANDLE))
    {
        preq->return_value = VDEV_IOCTL_UNCORRECT_PARAMETER;
        DBF_LOC;
        return -1;
    }

    if(HANDLE_OPENED != pVdev->xvdev_ioctl_param[handle].flag)
    {
        DBF_LOC;
        
        preq->return_value = VDEV_IOCTL_UNCORRECT_PARAMETER;
        return -1;
    }

    if((preq->input_size == 0) || (preq->input_size > pVdev->xvdev_ioctl_param[handle].max_buf_size))
    {
        DBF("Input buffer size:%d buf:%x max:%d\n",preq->input_size,preq->input_buf,pVdev->xvdev_ioctl_param[handle].max_buf_size);
        preq->return_value = VDEV_IOCTL_UNCORRECT_PARAMETER;
        return -1;
    }
    pVdev->xvdev_ioctl_param[handle].dev = dev;
    pvixs_cdc_hdr = pVdev->xvdev_ioctl_param[handle].send_buf;
    pvixs_cdc_hdr->magic = VIXS_CDC_MAGIC;
    pvixs_cdc_hdr->handle = handle;
    pvixs_cdc_hdr->seq = pVdev->xvdev_ioctl_param[handle].seq++;
    pvixs_cdc_hdr->length =  preq->input_size + sizeof(struct _VIXS_CDC_HDR);
    len = pvixs_cdc_hdr->length;
    if(pvixs_cdc_hdr->length % 512)
        len++;
    ret = vixs_eth_start_xmit(preq,dev, len);
    preq->return_value = ret;
    //DBKT("ret:%d\n",ret);
    return ret;
}

int xvdev_cmd_recv(PVDEV pVdev,void *parg)
{

    struct para_recv_req  * preq = (struct para_recv_req *)parg;
    int handle;
    unsigned long flag;
    struct ioctl_param *pparam;
    int timeout;
    unsigned int recv_head;
    unsigned int recv_tail;
    
    //PDEBUG("--->%s entry.\n",__FUNCTION__);
    handle = preq->handle;
    if((handle < 0) || (handle >= XVDEV_MAX_HANDLE)){
        DBF_LOC;
        return -1;
    }

    pparam = &pVdev->xvdev_ioctl_param[handle];
    
    if(HANDLE_OPENED != pparam->flag){
        DBF_LOC;
        return -1;
    }


    if(preq->output_buf != 0){
        pparam->recv_tail++;
        pparam->recv_tail &= RECV_BUF_MASK;
        pparam->recv_cur_addr = 0;
        recv_tail = pparam->recv_tail;
        recv_head = pparam->recv_head;
        preq->output_buf = 0;
    }else{
        recv_tail = pparam->recv_tail;
        recv_head = pparam->recv_head;
    }	

    if(recv_tail != recv_head)
    {
        //DBF("head:%d tail:%d\n",recv_head,recv_tail); 
        preq->output_buf = pparam->recv_buf_user[recv_tail] ;
        preq->output_size = pparam->recv_buf_size[recv_tail];
        preq->output_buf_phy = (unsigned int)pparam->recv_buf[recv_tail];
            
        pparam->recv_cur_addr = preq->output_buf;
        return 0;
    }
    else
    {
        pparam->recv_flag = 0;		
        preq->return_value = VDEV_IOCTL_RECV_TIMEOUT;
        return -1;
    }
/*
    timeout = preq->timeout;
    pparam->recv_flag = 0;		
    recv_head = pparam->recv_head;
    recv_tail = pparam->recv_tail;
    if(recv_tail != recv_head)
    {
    
        preq->output_buf = pparam->recv_buf_user[recv_tail];
        preq->output_size = pparam->recv_buf_size[recv_tail];
        preq->output_buf_phy = (unsigned int)pparam->recv_buf[recv_tail];
        
        pparam->recv_cur_addr = preq->output_buf;
        return 0;
    }

    //DBF("%s\n","Can't receive correctly.");
    preq->return_value = VDEV_IOCTL_RECV_TIMEOUT;
    return -1;*/
}


int xvdev_change_mtu(PVDEV pVdev, int mtu){
    DBF_LOC;
    if (mtu > 0 && mtu <= MTU_SIZE){
        pVdev->mtu = mtu; 
        return 0;
    }else{
        return -1;
    }
}
static void ether_tx_complete (struct usb_ep *ep, struct usb_request *req)
{
    struct ioctl_param* ioctl = req->context;
	if(!req->status) {
        ioctl->send_flag = 1;
        inv_dcache_range(ioctl->send_buf, ioctl->send_buf+ioctl->max_buf_size+sizeof(struct _VIXS_CDC_HDR));
        usb_ep_free_request (ep, req);
	}else{
        DBF_LOC;
    }
}
static int vixs_eth_start_xmit (struct para_send_req *skb, struct net_device *net, int len)
{
	struct eth_dev		*dev = &l_ethdev;
	int			retval;
	struct usb_request	*req = 0;
	unsigned long		flags;
    PVDEV pVdev = dev->pVdev;
    int res;
	req = usb_ep_alloc_request (dev->in_ep,0);
    req->buf = pVdev->xvdev_ioctl_param[skb->handle].send_buf;
    req->length = len;
    req->complete = ether_tx_complete;
    req->context = &pVdev->xvdev_ioctl_param[skb->handle];
    
	retval = usb_ep_queue (dev->in_ep, req, 0);
    pVdev->xvdev_ioctl_param[skb->handle].send_flag = 0;
    return retval;
}
#endif//VIXS_CDC
