/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Aug 20, 2005
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/include/asm-arc/arcregs.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Sandeep Patil and Pradeep Sawlani.
 * Auxiliary register definitions and macros to read and write to them. 
 */

#ifndef	_ASM_ARC_ARCDEFS_H
#define	_ASM_ARC_ARCDEFS_H


/* These masks correspond to the status word(STATUS_32) bits */
#define	STATUS_H_SET			0x01		/* Mask for Halt bit */
#define	STATUS_E1_MASK			0x02		/* Mask for Interrupt 1 enable */
#define	STATUS_E2_MASK			0x04		/* Mask for Interrupt 2 enable */
#define STATUS_DISABLE_INTERRUPTS	0xFFFFFFF9	/* Mask to disable Interrupts  */


/* Auxiliary register values	*/
#define ARC_REG_STATUS32	0x0A /* status 32 register */ 		
#define ARC_REG_TIMER0_LIMIT	0x23 /* timer 0 limit */
#define ARC_REG_TIMER0_CTRL	0x22 /* timer 0 control */
#define ARC_REG_TIMER0_CNT   0x21 /* timer 0 count */

/* Icache related registers */
#define ARC_REG_IC_IVIC		0x10
#define ARC_REG_IC_CTRL		0x11
#define ARC_REG_IC_IVIL     	0x19

/* Dcache registers	*/
#define ARC_REG_DC_IVDC		0x47
#define ARC_REG_DC_CTRL		0x48
#define ARC_REG_DC_IVDL     0x4A
#define ARC_REG_DC_FLSH     0x4B
#define ARC_REG_DC_FLDL     0x4C

#define IM_SET_DC_CTRL 		0x40

/* Instruction cache related Auxiliary registers */
#define ARC_REG_I_CACHE_BUILD_REG 0x77
#define ARC_REG_D_CACHE_BUILD_REG 0x72

#define FS_FLAG             0x100

/* Inline macros for reading, writing into auxiliary registers	*/

#ifndef	__ASSEMBLY__
/* Read an auxiliary register */
#define	read_new_aux_reg(reg)						\
	({ unsigned int __ret;							\
	 	__asm__ __volatile__("lr	%0, [%1]":"=r"(__ret):"i"(reg));\
	 	__ret;								\
	 })
	
/* Write to an auxiliary register */

#define	write_new_aux_reg(reg, val)					\
	({ 									\
	 	__asm__ __volatile__ ("sr	%0, [%1]"::"r"(val),"i"(reg));		\
	 })

#endif	/* __ASSEMBLY__ */

#endif	/* _ASM_ARC_ARCDEFS_H */
