/*
 * Copyright (C) 2007 ViXS System Inc. (http://www.vixs.com)
 * Copyright (C) 2007 Alan Tong
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*-------------------------------------------------------------------------*/
#ifndef __VIXS_USBD_H__
#define __VIXS_USBD_H__

#include <common.h>
#include <asm/errno.h>
#include <linux/list.h>
#include <linux/usb/ch9.h>
#include <linux/usb/gadget.h>

#include <asm/byteorder.h>
#include <asm/io.h>

#include <asm/vixstypes.h>
/* XXX: we are not jaguar anymore. update this header? */
#include <asm/jaguar_usb.h>

/* ep0 transfer state */
#define WAIT_FOR_SETUP          0
#define DATA_STATE_XMIT         1
#define DATA_STATE_NEED_ZLP     2
#define WAIT_FOR_OUT_STATUS     3
#define DATA_STATE_RECV         4

#define EP_DIR_IN	1
#define EP_DIR_OUT	0

// USB Speed
#define VIXS_HIGH_SPEED 0
#define VIXS_FULL_SPEED 1
#define VIXS_LOW_SPEED  2

#define NUM_OF_EP 4
#define MAX_TRANS_DESC 4

#define VIXS_CONTROL_ADDR           0x00
#define VIXS_INTERRUPT_IN_ADDR      0x82
#define VIXS_BULK_IN_ADDR           0x81
#define VIXS_BULK_OUT_ADDR          0x03

#define VIXS_CONFIG                 0x01
#define VIXS_STATUS_INTERFACE       0x00
#define VIXS_DATA_INTERFACE         0x01
#define VIXS_STATUS_ALTER           0x00

#ifdef CONFIG_USB_ETH_RNDIS
#define VIXS_DATA_ALTER           0x00
#else
#define VIXS_DATA_ALTER           0x01
#endif

// Endpoint zero descriptor
#define USB_MAX_EP_FIFO             64
#define USB_MAX_CTRL_PAYLOAD        64
#define USB_MAX_BULK_PAYLOAD        512
#define USB_MAX_BULK_PAYLOAD_FULL   64


#define MAX_TRANSFER_BYTES          ((64*1024)-1)
#define VIXS_MMR_BASE               0xF0000000

/* ep a-f highspeed and fullspeed maxpacket, addresses
 * computed from ep->num
 */
#define REG_EP_MAXPKT(dev,num) (((num) + 1) * 0x10 + \
		(((dev)->gadget.speed == USB_SPEED_HIGH) ? 0 : 1))


/* DRIVER DATA STRUCTURES and UTILITIES */

struct vixs_ep {
    struct usb_ep                           ep;
    struct list_head                        queue;
    struct vixs                             *dev;
    struct usb_gadget                       *gadget;
    const struct usb_endpoint_descriptor    *desc;

    char                                    name[14];
    unsigned    num:8,
                fifo_size:12,
                stopped:1,
                is_in:1,
                responded:1;
};

struct vixs_request {
	struct usb_request		req;
	struct list_head		queue;
	struct vixs_ep          *ep;
	unsigned                insert;
	unsigned                mapped : 1,
		valid : 1;
};

struct vixs {
    struct usb_gadget           gadget;
    struct usb_gadget_driver    *driver;
    struct vixs_ep              ep[4];
    struct vixs_request         *status_req;
    struct device 		      *dev;

    unsigned                    enabled : 1,
                                protocol_stall : 1,
                                softconnect : 1,
                                got_irq : 1,
                                region : 1;

    unsigned int                max_ep;
    unsigned int                Pci_Interrupt_Line;
    unsigned int                Pci_HW_Interrupt_Line;

    unsigned int*               setup_packet; 
    unsigned int                inter_out_buffer[MAX_TRANS_DESC];
    unsigned int                inter_in_buffer[MAX_TRANS_DESC];
    unsigned int                recv_buffer_index;
    unsigned int                out_buffer_index;
    unsigned int                in_buffer_index;

	/* no spin lock */
//    spinlock_t                  lock;

    unsigned                    vbus_active:1;
    unsigned                    stopped:1;
    unsigned                    remote_wakeup:1;

    unsigned int                num_of_in_desc;
    unsigned int                num_of_out_desc;

    unsigned int                pending_in_request;
    unsigned int                pending_out_request;

    volatile uint8 *            pmmr;
    volatile uint8 *            pmmfb;


    uint_ptr                    physfb;
    uint                        fbsize;
    uint_ptr                    physmmr;
    uint                        mmrsize;

    // statistics...
    u32 max_pipes;      /* Device max pipes */
    u32 max_use_endpts;	/* Max endpointes to be used */
    u32 bus_reset;      /* Device is bus reseting */
    u32 resume_state;   /* USB state to resume */
    u32 usb_state;      /* USB current state */
    u32 usb_next_state; /* USB next state */
    u32 ep0_state;      /* Endpoint zero state */
    u32 ep0_dir;        /* Endpoint zero direction: can be  USB_DIR_IN or USB_DIR_OUT */
    u32 usb_sof_count;  /* SOF count */
    u32 errors;         /* USB ERRORs count */
    u8 device_address;  /* Device USB address */

    struct completion *done;    /* to make sure release() is done */
};

struct vixs_platform_data {
	unsigned int usbd_clk;
};


#define ep_index(EP)                ((EP)->desc->bEndpointAddress&0xF)
#define ep_maxpacket(EP)            ((EP)->ep.maxpacket)
#define ep_is_in(EP)                ((ep_index(EP) == 0) ? (EP->dev->ep0_dir == USB_DIR_IN ):((EP)->desc->bEndpointAddress & USB_DIR_IN)==USB_DIR_IN)

#define get_ep_by_pipe(udc, pipe)   ((pipe == 1)? &dev->ep[0]: &dev->ep[pipe])
#define get_pipe_by_windex(windex)  ((windex & USB_ENDPOINT_NUMBER_MASK) * 2 + ((windex & USB_DIR_IN) ? 1 : 0))
#define get_pipe_by_ep(EP)          (ep_index(EP) * 2 + ep_is_in(EP))

/*-------------------------------------------------------------------------*/

#define xprintk(dev,level,fmt,args...) \
	printk(level "%s %s: " fmt , driver_name , \
			pci_name(dev->pdev) , ## args)

#ifdef DEBUG
#undef DEBUG
#define DEBUG(dev,fmt,args...) \
	xprintk(dev , KERN_DEBUG , fmt , ## args)
#else
#define DEBUG(dev,fmt,args...) \
	do { } while (0)
#endif /* DEBUG */

#ifdef VERBOSE
#define VDEBUG DEBUG
#else
#define VDEBUG(dev,fmt,args...) \
	do { } while (0)
#endif	/* VERBOSE */

#define ERROR(dev,fmt,args...) \
	xprintk(dev , KERN_ERR , fmt , ## args)
#define WARN(dev,fmt,args...) \
	xprintk(dev , KERN_WARNING , fmt , ## args)
#define INFO(dev,fmt,args...) \
	xprintk(dev , KERN_INFO , fmt , ## args)


/**
 * container_of - cast a member of a structure out to the containing structure
 * @ptr:        the pointer to the member.
 * @type:       the type of the container struct this is embedded in.
 * @member:     the name of the member within the struct.
 *
 */
#define container_of(ptr, type, member) ({                      \
        const typeof( ((type *)0)->member ) *__mptr = (ptr);    \
        (type *)( (char *)__mptr - offsetof(type,member) );})

/*-------------------------------------------------------------------------*/

#define virt_to_phys(_buf) (_buf)

/*********** function prototypes ***********/
int vixs_usbd_probe (void);
int vixs_usbd_init (void);

#endif	/* __VIXS_USBD_H__ */
