/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 *  Copyright 2009 ViXS Systems Inc
 *	  Philip Yang <pyang@vixs.com>
 *
 */
#ifndef __ASM_MACH_GENERIC_KERNEL_ENTRY_H
#define __ASM_MACH_GENERIC_KERNEL_ENTRY_H

/* Sample BootStrap code:
 * Generic for all Release 2 or higher MIPS32 or MIPS64 processors
 * Initializes the register file, some Cop0 state, and
 * has sections for cache and TLB init if desired.  This code does not
 * preserve state for NMI or debugging of subsequent reset.  It must be
 * changed if those features are desired.
 *
 * This document is security clasification 2B - Published (Commercial)
 *
 * mips_start_of_legal_notice
 * ***************************************************************************
 * Copyright (c)  MIPS Technologies, Inc.  All
 * rights reserved.
 * 
 * Unpublished rights (if any) reserved under the copyright laws of the United
 * States of America and other countries.
 * 
 * This code is proprietary to MIPS Technologies, Inc. ("MIPS Technologies"). 
 * Any copying, reproducing, modifying or use of this code (in whole or in
 * part) that is not expressly permitted in writing by MIPS Technologies or an
 * authorized third party is strictly prohibited.	At a minimum, this code is
 * protected under unfair competition and copyright laws.	Violations thereof
 * may result in criminal penalties and fines.
 * 
 * MIPS Technologies reserves the right to change this code to improve
 * function, design or otherwise.	MIPS Technologies does not assume any
 * liability arising out of the application or use of this code, or of any
 * error or omission in such code.  Any warranties, whether express,
 * statutory, implied or otherwise, including but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose, are
 * excluded.  Except as expressly provided in any written license agreement
 * from MIPS Technologies or an authorized third party, the furnishing of this
 * code does not give recipient any license to any intellectual property
 * rights, including any patent rights, that cover this code.
 * 
 * This code shall not be exported, reexported, transferred, or released,
 * directly or indirectly, in violation of the law of any country or
 * international law, regulation, treaty, Executive Order, statute, amendments
 * or supplements thereto.  Should a conflict arise regarding the export,
 * reexport, transfer, or release of this code, the laws of the United States
 * of America shall be the governing law.
 * 
 * This code constitutes one or more of the following: commercial computer
 * software, commercial computer software documentation or other commercial
 * items.	If the user of this code, or any related documentation of any kind,
 * including related technical data or manuals, is an agency, department, or
 * other entity of the United States government ("Government"), the use,
 * duplication, reproduction, release, modification, disclosure, or transfer
 * of this code, or any related documentation of any kind, is restricted in
 * accordance with Federal Acquisition Regulation 12.212 for civilian agencies
 * and Defense Federal Acquisition Regulation Supplement 227.7202 for military
 * agencies.  The use of this code by the Government is further restricted in
 * accordance with the terms of the license agreement(s) and/or applicable
 * contract terms and conditions covering this code from MIPS Technologies or
 * an authorized third party.
 * 
 * 
 * 
 * ***************************************************************************
 * mips_end_of_legal_notice
 *
 */

#define CACHE_INIT
#define TLB_INIT

//#define DO_NOT_INIT_GPRS
#define DO_NOT_CHECK_FOR_CMP
#define DO_NOT_CHECK_FOR_MT

# Select a Hazard Barrier
#ifdef LIGHT_WEIGHT_HB
#define HB nop
#else
#define HB ehb
#endif

	.macro	kernel_entry_setup

	.set	noreorder
	.set	noat
	.set	mips32r2

____start:
#ifndef DO_NOT_INIT_GPRS

	# Initialize the register file
	# should not be required with good software practices
	or	$1, $0, $0
	or	$2, $0, $0
	or	$3, $0, $0
	or	$4, $0, $0
	or	$5, $0, $0
	or	$6, $0, $0
	or	$7, $0, $0
	or	$8, $0, $0
	or	$9, $0, $0
	or	$10, $0, $0
	or	$11, $0, $0
	or	$12, $0, $0
	or	$13, $0, $0
	or	$14, $0, $0
	or	$15, $0, $0
	or	$16, $0, $0
	or	$17, $0, $0
	or	$18, $0, $0
	or	$19, $0, $0
	or	$20, $0, $0
	or	$21, $0, $0
	or	$22, $0, $0
	or	$23, $0, $0
	or	$24, $0, $0
	or	$25, $0, $0
	or	$26, $0, $0
	or	$27, $0, $0
	or	$28, $0, $0
	or	$29, $0, $0
	or	$30, $0, $0
	or	$31, $0, $0

#endif /* DO_NOT_INIT_GPRS */

# Initialize Misc. Cop0 state

	# Read status register
	mfc0	$10, $12		# C0_Status
	# Set up Status register:
	# Disable Coprocessor Usable bits
	# Turn off Reduce Power bit
	# Turn off reverse endian
	# Leave BEV turned on
	# Clear TS, SR, NMI bits
	# Clear Interrupt masks
	# Clear User Mode
	# Clear ERL
	# Set EXL
	# Clear Interrupt Enable
	lui	$11, 0x0040
	ori	$11, 0xff02		#li	$11, 0x0040ff02	 # (M_StatusBEV | M_StatusIM | M_StatusEXL)
	mtc0	$11, $12		# C0_Status

	# Disable watch exceptions
	mtc0	$0, $18			# C0_WatchLo
	mtc0	$0, $18, 1		# C0_WatchLo1
	mtc0	$0, $18, 2		# C0_WatchLo2
	mtc0	$0, $18, 3		# C0_WatchLo3
	mtc0	$0, $18, 4		# C0_WatchLo4
	mtc0	$0, $18, 5		# C0_WatchLo5
	mtc0	$0, $18, 6		# C0_WatchLo6
	mtc0	$0, $18, 7		# C0_WatchLo7

	# Clear Watch Status bits
	li	$11, 0x7		# (M_WatchHiI | M_WatchHiR | M_WatchHiW)
	mtc0	$11, $19		# C0_WatchHi
	mtc0	$11, $19, 1		# C0_WatchHi1
	mtc0	$11, $19, 2		# C0_WatchHi2
	mtc0	$11, $19, 3		# C0_WatchHi3
	mtc0	$11, $19, 4		# C0_WatchHi4
	mtc0	$11, $19, 5		# C0_WatchHi5
	mtc0	$11, $19, 6		# C0_WatchHi6
	mtc0	$11, $19, 7		# C0_WatchHi7

	# Clear WP bit to avoid watch exception upon user code entry
	# Clear IV bit - Interrupts go to general exception vector
	# Clear software interrupts
	mtc0	$0, $13			# C0_Cause

	# Set KSeg0 to cacheable
	# Config.K0
	mfc0	$10, $16		# C0_Config
	li	$11, 0x3		# K_CacheAttrCN
	ins	$10, $11, 0, 3		# S_ConfigK0, W_ConfigK0
	mtc0	$10, $16		# C0_Config

	# Clear Count register
	mtc0	$0, $9			# C0_Count

	# Set compare to -1 to delay 1st count=compare
	# Also, clears timer interrupt
	li	$10, -1
	mtc0	$10, $11		# C0_Compare

	# Cache initialization routine
	# Long and needed on HW
	# Can be skipped if using magic simulation cache invalidate

#ifdef CACHE_INIT

	# Determine how big the I$ is
/*
 ************************************************************************
 *         C O N F I G 1   R E G I S T E R   ( 1 6, SELECT 1 )          *
 ************************************************************************
 *
 *  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |M|  MMU Size |  IS |  IL |  IA |  DS |  DL |  DA |Rsvd |W|C|E|F| Config1
 * | |           |     |     |     |     |     |     |     |R|A|P|P|
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */
	mfc0	$10, $16, 1		# C0_Config1

	# Isolate I$ Line Size
	ext	$11, $10, 19, 3		# S_Config1IL, W_Config1IL

	# Skip ahead if No I$
	beq	$11, $0, no_icache
	HB

	li	$14, 2
	sllv	$11, $14, $11		# Now have true I$ line size in bytes

	ext	$12, $10, 22, 3		# S_Config1IS, W_Config1IS
	li	$14, 64
	sllv	$12, $14, $12		# I$ Sets per way

	# Config1IA == I$ Assoc - 1
	ext	$13, $10, 16, 3		# S_Config1IA, W_Config1IA
	add	$13, 1

	mul	$12, $12, $13		# Total number of sets

	lui	$14, 0x8000		# Get a KSeg0 address for cacheops

	# Clear TagLo/TagHi registers
	mtc0	$0, $28			# C0_ITagLo
	mtc0	$0, $29			# C0_ITagHi

	move	$15, $12

	# Index Store Tag Cache Op
	# Will invalidate the tag entry, clear the lock bit, and clear the LRF bit
1:	cache	0x8, 0($14)		# ICIndexStTag
	add	$15, -1			# Decrement set counter

	bne	$15, $0, 1b
	add	$14, $11		# Get next line address

	# Now go through and invalidate the D$
no_icache:


	# Isolate D$ Line Size
	ext	$11, $10, 10, 3		# S_Config1DL, W_Config1DL

	# Skip ahead if No D$
	beq	$11, $0, no_dcache
	HB

	li	$14, 2
	sllv	$11, $14, $11		# Now have true D$ line size in bytes

	ext	$12, $10, 13, 3		# S_Config1DS, W_Config1DS
	li	$14, 64
	sllv	$12, $14, $12		# D$ Sets per way

	# Config1DA == D$ Assoc - 1
	ext	$13, $10, 7, 3		# S_Config1DA, W_Config1DA
	add	$13, 1

	mul	$12, $12, $13		# Get total number of sets

	lui	$14, 0x8000		# Get a KSeg0 address for cacheops

	# Clear TagLo/TagHi registers
	mtc0	$0, $28			# C0_TagLo
	mtc0	$0, $29			# C0_TagHi
	mtc0	$0, $28, 2		# C0_DTagLo
	mtc0	$0, $29, 2		# C0_DTagHi

	move	$15, $12

	# Index Store Tag Cache Op
	# Will invalidate the tag entry, clear the lock bit, and clear the LRF bit
1:	cache	0x9, 0($14)		# DCIndexStTag
	add	$15, -1			# Decrement set counter

	bne	$15, $0, 1b
	add	$14, $11		# Get next line address

	# Do not initialize the L2$ until we know if we are a CMP system.
	# Only CPU0 initializes L2$ for CMP.  For non-CMP we assume L2$
	# is not shared with other cores.

no_dcache:

#endif

	# Now both caches have been invalidated
	# Initialize the TLB

#ifdef TLB_INIT

	# Determine if we have a TLB
	mfc0	$11, $16		# C0_Config
	ext	$11, $11, 7, 3		# S_ConfigMT, W_ConfigMT

	li	$15, 0x1		# K_ConfigMT_TLBMMU

	bne	$11, $15, mmu_not_tlb
	HB

	mfc0	$10, $16, 1		# C0_Config1

	# Config1MMUSize == Number of TLB entries - 1
	ext	$11, $10, 25, 6		# S_Config1MMUSize, W_Config1MMUSize

	mtc0	$0, $2			# C0_EntryLo0
	mtc0	$0, $3			# C0_EntryLo1
	mtc0	$0, $5			# C0_PageMask
	mtc0	$0, $6			# C0_Wired

	li	$12, 0x80000000

1:
	mtc0	$11, $0			# C0_Index
	mtc0	$12, $10		# C0_EntryHi
	ehb
	tlbwi
	add	$12, (2<<13)		# Add 8K to the address to avoid TLB conflict with previous entry

	bne	$11, $0, 1b
	add	$11, -1

mmu_not_tlb:

#endif

#ifndef DO_NOT_INIT_GPRS

	# Initialize other register files
	# Good code should obviously not read registers before they are written
	# so this is not not normally needed. But in simulation certain ways of initializing
	# registers may not work (xor rN, rN, rN for one)

	# read number of shadow sets
	mfc0	$11, $12, 2			# C0_SRSCtl
	ext	$12, $11, 26, 4			# S_SRSCtlHSS, W_SRSCtlHSS

	# Initialize shadow sets
1:	beqz	$12, 10f			# Moving this branch to the end of the loop
						# would also re-initialize the primary register file
	ins	$11, $12, 6, 4			# S_SRSCtlPSS, W_SRSCtlPSS

	# set PSS to shadow set to be initialized
	mtc0	$11, $12, 2			# C0_SRSCtl

	wrpgpr	$1, $0
	wrpgpr	$2, $0
	wrpgpr	$3, $0
	wrpgpr	$4, $0
	wrpgpr	$5, $0
	wrpgpr	$6, $0
	wrpgpr	$7, $0
	wrpgpr	$8, $0
	wrpgpr	$9, $0
	wrpgpr	$10, $0
	wrpgpr	$11, $0
	wrpgpr	$12, $0
	wrpgpr	$13, $0
	wrpgpr	$14, $0
	wrpgpr	$15, $0
	wrpgpr	$16, $0
	wrpgpr	$17, $0
	wrpgpr	$18, $0
	wrpgpr	$19, $0
	wrpgpr	$20, $0
	wrpgpr	$21, $0
	wrpgpr	$22, $0
	wrpgpr	$23, $0
	wrpgpr	$24, $0
	wrpgpr	$25, $0
	wrpgpr	$26, $0
	wrpgpr	$27, $0
	wrpgpr	$28, $0
	wrpgpr	$29, $0
	wrpgpr	$30, $0
	wrpgpr	$31, $0

	b	1b
	add	$12, -1
10:

#endif /* DO_NOT_INIT_GPRS */

#define CPUNUM	$3

#ifndef DO_NOT_CHECK_FOR_CMP

#define GCMP	$8

	# Determine if there is a coherency manager present.
	# This address is the default configuration address.  If your
	# system was configured with a different address, you must
	# change this value.
	li	GCMP, 0xbfbf8000	# KSEG1(GCMPBASE)
	lw	$9, 0x0008(GCMP)	# GCR_BASE

	addiu	$10, $0, 0x8000		# li r10, M_GCR_BASE_GCR_BASE (~0x7fff)
	and	$9, $10

	lui	$10, 0xa000		# KSEG1BASE
	or	$10, $9

	# This code uses $3 as the CPU number to make descisions about
	# whether to execute code or not.  For CMP systems, we use the
	# real CPU number.  For non-CMP, pretend the CPU number is zero
	# to force the correct behavior.

	bne	$10, GCMP, nogcmp
	move	CPUNUM, $0

	# Change the CCA from non-coherent to coherent
	# Assumption: GCMP => coherent CCA is usable
	mfc0	$10, $16		# C0_Config

	ext	$11, $10, 0, 3		# S_ConfigK0, W_ConfigK0
	xori	$11, 3			# K_CacheAttrCN

	bnez	$11, 1f
	HB

	li	$11, 5			# K_CacheAttrCCS
	ins	$10, $11, 0, 3		# S_ConfigK0, W_ConfigK0

	mtc0	$10, $16		# C0_Config
1:
	# Check the CPU number
	mfc0	CPUNUM, $15, 1		# C0_EBase
	ext	CPUNUM, CPUNUM, 0, 10	# S_EBaseCPUNum, W_EBaseCPUNum

	# Only the Primary Core needs to initialise the Coherence Manager.
	bnez	CPUNUM, 1f
	HB

	# Initialise CM
	#
	# Rewrite the GCR_BASE register to initialise CM_DEFAULT_TARGET
	ins	$9, $0, 0, 2		# M_GCR_BASE_CM_DEFAULT_TARGET, S_GCR_BASE_CM_DEFAULT_TARGET
	sw	$9, 0x0008(GCMP)	# GCR_BASE

	# Disable the CM regions
	lui	$9, 0xffff
	sw	$9, 0x0090(GCMP)	# GCR_REG0_BASE
	sw	$9, 0x0098(GCMP)	# GCR_REG0_MASK
	sw	$9, 0x00a0(GCMP)	# GCR_REG1_BASE
	sw	$9, 0x00a8(GCMP)	# GCR_REG1_MASK
	sw	$9, 0x00b0(GCMP)	# GCR_REG2_BASE
	sw	$9, 0x00b8(GCMP)	# GCR_REG2_MASK
	sw	$9, 0x00c0(GCMP)	# GCR_REG3_BASE
	sw	$9, 0x00c8(GCMP)	# GCR_REG3_MASK

1:
	# Allow interventions from all other cores (including self)
	# For CPU 1 and higher, NOP if access not enabled in GCR_ACCESS.
	li	$9, 0xff
	sw	$9, 0x2008(GCMP)	# GCR_CL_COHERENCE
	ehb

#undef GCMP

#endif	/* DO_NOT_CHECK_FOR_CMP */

nogcmp:

	# L2 Cache initialization routine
	# Long and needed on HW
	# Can be skipped if using magic simulation cache invalidate

#ifdef CACHE_INIT

#ifndef DO_NOT_CHECK_FOR_CMP

	# Primary Core needs to initialise L2$
	bnez	CPUNUM, no_l2cache
	HB

#endif

	#
	# Now go through and initialize the L2$

	# Check L2 cache size
/*
 ************************************************************************
 *         C O N F I G 2   R E G I S T E R   ( 1 6, SELECT 2 )          *
 ************************************************************************
 *
 *  3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *  1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * |M| TU  |  TS   |  TL   |  TA   |  SU   |  SS   |  SL   |  SA   | Config2
 * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

	mfc0	$10, $16, 2		# C0_Config2

	# Isolate L2$ Line Size
	ext	$11, $10, 4, 4		# S_Config2SL, W_Config2SL

	# Skip ahead if No L2$
	beq	$11, $0, no_l2cache
	HB

	li	$14, 2
	sllv	$11, $14, $11		# Now have true L2$ line size in bytes

	# Isolate L2$ Sets per Way
	ext	$12, $10, 8, 4		# S_Config2SS, W_Config2SS
	li	$14, 64
	sllv	$12, $14, $12		# L2$ Sets per way

	# Isolate L2$ Associativity
	# L2$ Assoc (-1)
	ext	$13, $10, 0, 4		# S_Config2SA, W_Config2SA
	add	$13, 1

	mul	$12, $12, $13		# Get total number of sets

	lui	$14, 0x8000		# Get a KSeg0 address for cacheops

	# Clear L23TagLo/L23TagHi registers
	mtc0	$0, $28, 4
	mtc0	$0, $29, 4

	move	$15, $12

	# L2$ Index Store Tag Cache Op
	# Will invalidate the tag entry, clear the lock bit, and clear the LRF bit
1:	cache	0xB, 0($14)		# SCIndexStTag
	add	$15, -1			# Decrement set counter

	bne	$15, $0, 1b
	add	$14, $11		# Get next line address

no_l2cache:

	# Isolate L3$ Line Size
	ext	$11, $10, 20, 4		# S_Config2TL, W_Config2TL

	# Skip ahead if No L3$
	beq	$11, $0, no_l3cache
	HB

	li	$14, 2
	sllv	$11, $14, $11		# Now have true L3$ line size in bytes

	# Isolate L3$ Sets per Way
	ext	$12, $10, 24, 4		# S_Config2TS, W_Config2TS
	li	$14, 64
	sllv	$12, $14, $12		# L2$ Sets per way

	# Isolate L3$ Associativity
	# L3$ Assoc (-1)
	ext	$13, $10, 16, 4		# S_Config2TA, W_Config2TA
	add	$13, 1

	mul	$12, $12, $13		# Get total number of sets

	lui	$14, 0x8000		# Get a KSeg0 address for cacheops

	# Clear L23TagLo/L23TagHi registers
	mtc0	$0, $28, 4
	mtc0	$0, $29, 4

	move	$15, $12

	# L3$ Index Store Tag Cache Op
	# Will invalidate the tag entry, clear the lock bit, and clear the LRF bit
1:	cache	0xA, 0($14)		# TCIndexStTag
	add	$15, -1			# Decrement set counter

	bne	$15, $0, 1b
	add	$14, $11		# Get next line address

no_l3cache:

#endif	/* CACHE_INIT */

	# Caches have now been initialised, so shift to kseg0
	la	$8, 1f

	lui	$9, 0xe000		# KSEG_MSK
	not	$9			# ~KSEG_MSK
	and	$8, $9

	lui	$9, 0x8000		# KSEG0BASE
	or	$8, $9

	jr	$8			# KSEG0(1f)
	HB
1:
	#### From now on, we are executing cached

#ifndef DO_NOT_CHECK_FOR_MT

	# Config1 is required for MIPS32
	mfc0	$8, $16, 1		# C0_Config1
	bgez	$8, notmtcapable	# No Config2 register
	HB

	mfc0	$8, $16, 2		# C0_Config2
	bgez	$8, notmtcapable	# No Config3 register
	HB

	mfc0	$8, $16, 3		# C0_Config3
	and	$8, (1 << 2)		# M_Config3MT
	beqz	$8, notmtcapable
	HB

	# This core supports MT ASE
	#
	# Start up secondary VPE's
	.set	push
	.set	mt

startmtconfig:
	# This is TC0 bound to VPE0.  Therefore VPEConf0.MVP is set.

	# Enter config mode
	mfc0	$8, $0, 1		# C0_MVPCtl
	or	$8, (1 << 1)		# M_MVPCtlVPC
	mtc0	$8, $0, 1		# C0_MVPCtl
	ehb

#define NTCS	$10
#define NVPES	$11
#define TC	$12

	# Get number of TC's and VPE's
	mfc0	$8, $0, 2		# C0_MVPConf0
	ext	NTCS, $8, 0, 8		# S_MVPConf0PTC, W_MVPConf0PTC
	ext	NVPES, $8, 10, 4	# S_MVPConf0PVPE, W_MVPConf0PVPE

	# Initialise TC's/VPE's
	move	TC, $0
nexttc:
	# Select TCn
	mfc0	$8, $1, 1		# C0_VPECtl
	ins	$8, TC, 0, 8		# S_VPECtlTargTC, W_VPECtlTargTC
	mtc0	$8, $1, 1		# C0_VPECtl
	ehb

	# Bind TC to next VPE
	beqz	TC, nextvpe		# Don't rebind TC0
	HB

	# Halt all TC's other than TC0
	li	$8, 1			# M_TCHaltH
	mttc0	$8, $2, 4		# C0_TCHalt
	ehb

	slt	$9, NVPES, TC
	bnez	$9, 2f			# Bind spare TC's to VPElast
	move	$9, NVPES

	# Set XTC for active TC's
	mftc0	$8, $1, 2		# C0_VPEConf0
	ins	$8, TC, 21, 8		# S_VPEConf0XTC, W_VPEConf0XTC
	mttc0	$8, $1, 2		# C0_VPEConf0

	move	$9, TC
2:
	# Bind TC to a VPE
	mftc0	$8, $2, 2		# C0_TCBind
	ins	$8, $9, 0, 4		# S_TCBindCurVPE, W_TCBindCurVPE
	mttc0	$8, $2, 2		# C0_TCBind

	# Set up TCStatus register:
	# Disable Coprocessor Usable bits
	# Disable MDMX/DSP ASE
	# Clear Dirty TC
	# not dynamically allocatable
	# not allocated
	# Kernel mode
	# interrupt exempt
	# ASID 0
	li	$8, (1 << 10)		# M_TCStatusIXMT
	mttc0	$8, $2, 1		# C0_TCStatus

#ifndef DO_NOT_INIT_GPRS

	# Initialize the TC's register file
	# should not be required with good software practices
	mttgpr	$0, $1
	mttgpr	$0, $2
	mttgpr	$0, $3
	mttgpr	$0, $4
	mttgpr	$0, $5
	mttgpr	$0, $6
	mttgpr	$0, $7
	mttgpr	$0, $8
	mttgpr	$0, $9
	mttgpr	$0, $10
	mttgpr	$0, $11
	mttgpr	$0, $12
	mttgpr	$0, $13
	mttgpr	$0, $14
	mttgpr	$0, $15
	mttgpr	$0, $16
	mttgpr	$0, $17
	mttgpr	$0, $18
	mttgpr	$0, $19
	mttgpr	$0, $20
	mttgpr	$0, $21
	mttgpr	$0, $22
	mttgpr	$0, $23
	mttgpr	$0, $24
	mttgpr	$0, $25
	mttgpr	$0, $26
	mttgpr	$0, $27
	mttgpr	$0, $28
	mttgpr	$0, $29
	mttgpr	$0, $30
	mttgpr	$0, $31

#endif /* DO_NOT_INIT_GPRS */

nextvpe:
	slt	$9, NVPES, TC
	bnez	$9, donevpe		# No more VPE's
	HB

	# Disable multi-threading with TC's
	mftc0	$8, $1, 1		# C0_VPECtl
	ins	$8, $0, 15, 1		# S_VPECtlTE, W_VPECtlTE
	mttc0	$8, $1, 1		# C0_VPECtl

	beqz	TC, 1f
	HB

	# For VPE1..n
	# Clear VPA and set master VPE
	mftc0	$8, $1, 2		# C0_VPEConf0
	ins	$8, $0, 0, 1		# S_VPEConf0VPA, W_VPEConf0VPA
	or	$8, (1 << 1)		# M_VPEConf0MVP
	mttc0	$8, $1, 2		# C0_VPEConf0

	# Copy Status from the current TC to the target TC's Cop0 Status register
	mfc0	$8, $12			# C0_Status
	mttc0	$8, $12			# C0_Status

#ifndef OTHER_VPE_EPC_ADDR
#define OTHER_VPE_EPC_ADDR 0x12345678
#endif
	la	$8, OTHER_VPE_EPC_ADDR
	mttc0	$8, $14			# C0_EPC

	mttc0	$0, $13			# C0_Cause

	mfc0	$8, $16			# C0_Config
	mttc0	$8, $16			# C0_Config

	mftc0	$8, $15, 1		# C0_EBase
	ext	$8, $8, 0, 10		# S_EBaseCPUNum, W_EBaseCPUNum
	mttgpr	$8, CPUNUM

	# Finally... arrange for other VPE's to continue from OTHER_VPE_TCRESTART_ADDR
#ifndef OTHER_VPE_TCRESTART_ADDR
#define OTHER_VPE_TCRESTART_ADDR 0x12345678
#endif
	la	$8, OTHER_VPE_TCRESTART_ADDR
	mttc0	$8, $2, 3		# C0_TCRestart

	# Yes.. this is undoing all of the work done above... :)
	mftc0	$8, $2, 1		# C0_TCStatus
	ins	$8, $0, 10, 1		# S_TCStatusIXMT, W_TCStatusIXMT
	ori	$8, (1 << 13)		# M_TCStatusA
	mttc0	$8, $2, 1		# C0_TCStatus

	mttc0	$0, $2, 4		# C0_TCHalt

	mftc0	$8, $1, 2		# C0_VPEConf0
	ori	$8, 1			# M_VPEConf0VPA
	mttc0	$8, $1, 2		# C0_VPEConf0
1:

donevpe:
	addu	TC, 1
	sltu	$9, NTCS, TC
	beqz	$9, nexttc
	HB

	# Exit config mode
	mfc0	$8, $0, 1		# C0_MVPCtl
	ins	$8, $0, 1, 1		# S_MVPCtlVPC, W_MVPCtlVPC
	mtc0	$8, $0, 1		# C0_MVPCtl
	ehb

#undef NTCS
#undef NVPES
#undef TC

notmtcapable:

#endif	/* DO_NOT_CHECK_FOR_MT */

/*
return:
	# At this point the chip has been initialized
	# The address of the real code can be loaded into EPC
	# An ERet will clear EXL and jump to EPC

#ifndef INITIAL_EPC_ADDR
#define INITIAL_EPC_ADDR 0x80030120
#endif
	la	$11, INITIAL_EPC_ADDR
	mtc0	$11, $14
	ehb
	eret
*/

/*
 * The core has been initialized.  It is running the code at
 * INITIAL_EPC_ADDR in kernel mode with interrupts disabled.
 * If the core is multithreaded, it is still running with other TCs
 * and VPEs disabled.  Each VPE has at least one TC.  If there are
 * more TCs than VPEs the excess TCs are bound to the final VPE.
 * These TCs are set to run in kernel mode from OTHER_VPE_TCRESTART_ADDR.
 *
 * If this is a CMP system, the state of the system varies depending
 * on the CPU number.  For CPU0, it is the only core running.  The
 * others are still waiting to be released from reset state by CPU0.
 * CPUs other than CPU0 only reach the code at INITIAL_EPC_ADDR after
 * CPU0 releases them from reset.
 *
 * It is now the responsibility of the code at INITIAL_EPC_ADDR to
 * initialize the rest of the system.  This optionally includes enabling
 * other VPEs and TCs for a multi-threaded core, configuring the
 * Coherence Manager (CM) in a CMP system and releasing the other cores
 * from reset as needed.  Care should be taken that only CPU0 performs
 * the remainder of the global system initialization for CMP systems.
 */

    .set	at
    .endm

#endif /* __ASM_MACH_GENERIC_KERNEL_ENTRY_H */
