/*
 *  gpio.h: GPIO Support for VIPER.
 *
 *  Copyright 2009 ViXS Systems Inc
 *	  Philip Yang <pyang@vixs.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __ASM_VIPER_GPIO_H
#define __ASM_VIPER_GPIO_H

/* BIG FAT WARNING: races danger!
   No protections exist here. Current users are only early init code,
   when locking is not needed because no cuncurency yet exists there,
   and GPIO IRQ dispatcher, which does locking.
   However, if many uses will ever happen, proper locking will be needed
   - including locking between different uses
*/

#include <asm/io.h>
#include <asm/viper.h>



/* Initialize GPIO to a known state */
static inline void viper_gpio_init(void)
{
	return;
}

/* Select GPIO direction for a pin */
static inline void viper_gpio_select_input(unsigned int gpio, unsigned int pin)
{
	unsigned long flags;
	unsigned int reg;
//	local_irq_save(flags);
	reg = readl(XC_SOC_PROC_MMREG_BASE + gpio);
	writel(reg & ~(1<< pin), XC_SOC_PROC_MMREG_BASE + gpio);
//	local_irq_restore(flags);
	return;
}
static inline void viper_gpio_select_output(unsigned int gpio, unsigned int pin)
{
	unsigned long flags;
	unsigned int reg;
//	local_irq_save(flags);
	reg = readl(XC_SOC_PROC_MMREG_BASE + gpio);
	writel(reg | (1<< pin), XC_SOC_PROC_MMREG_BASE + gpio);
//	local_irq_restore(flags);
	return;
}

/* Select GPIO or alternate function for a pin
 * It is the caller's responsbility to avoid the race
 */
static inline void viper_gpio_select_function_io(unsigned int gpio, unsigned int mode)
{	
	writel(mode, XC_SOC_PROC_MMREG_BASE + gpio);
	return;
}
static inline void viper_gpio_select_function_alt(unsigned int gpio, unsigned int mode)
{
	writel(mode, XC_SOC_PROC_MMREG_BASE + gpio);	
	return;
}

/* Read GPIO pin */
static inline unsigned int viper_gpio_read(unsigned int gpio, unsigned int pin)
{
    return ((readl(XC_SOC_PROC_MMREG_BASE + gpio) & (1<<pin))? 1 : 0);
}

/* Write GPIO pin */
static inline void viper_gpio_set(unsigned int gpio, unsigned int pin)
{
	unsigned int reg;
	reg = readl(XC_SOC_PROC_MMREG_BASE + gpio);
	writel(reg | (1<< pin), XC_SOC_PROC_MMREG_BASE + gpio);	
	return;
}
static inline void viper_gpio_clear(unsigned int gpio, unsigned int pin)
{
	unsigned int reg;
	reg = readl(XC_SOC_PROC_MMREG_BASE + gpio);
	writel(reg & ~(1<< pin), XC_SOC_PROC_MMREG_BASE + gpio);
	return;
}
static inline void viper_gpio_set_prot(unsigned int gpio, unsigned int pin)
{
	unsigned long flags;
	unsigned int reg;
//	local_irq_save(flags);
	reg = readl(XC_SOC_PROC_MMREG_BASE + gpio);
	writel(reg | (1<< pin), XC_SOC_PROC_MMREG_BASE + gpio);
//	local_irq_restore(flags);
	return;
}
static inline void viper_gpio_clear_prot(unsigned int gpio, unsigned int pin)
{
	unsigned long flags;
	unsigned int reg;
//	local_irq_save(flags);
	reg = readl(XC_SOC_PROC_MMREG_BASE + gpio);
	writel(reg & ~(1<< pin), XC_SOC_PROC_MMREG_BASE + gpio);
//	local_irq_restore(flags);
	return;
}

static inline void viper_gpio_setup_irq(int when, unsigned int gpio, unsigned int pin)
{
}

/* Enable/disable GPIO interrupt */
static inline void viper_gpio_enable_irq(unsigned int gpio, unsigned int pin)
{
}
static inline void viper_gpio_disable_irq(unsigned int gpio, unsigned int pin)
{
}

/* Clear GPIO interrupt request */
static inline void viper_gpio_clear_irq(unsigned int gpio, unsigned int pin)
{
}

#endif
