/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) 
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/include/configs/aa3.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

#ifndef __CONFIG_H
#define __CONFIG_H
#include <cmd_confdefs.h>

#define CONFIG_BZIP2

#define CONFIG_ARC			1 		/* This is an ARC cpu */

/*
 *  Size of malloc pool
 */
#define CFG_MALLOC_LEN			0x800000 	//=8M//0x8000	/* Malloc size = 32k */

#define CFG_STACK_LEN			0x400000

#define CFG_UBOOT_SIZE			0x40000

/*
 *  Serial configuartion
 */
#define CONFIG_ARC_SERIAL_BAUD		115200	/* Baudrate for serial port */

/*
 *  Physical memory map
 */

/*#define ARC700				1			 For arc 700 */	
/*#define ARC5			 	1			 For arc 5 automatically defined in mkconfig*/
/*This clock speed is for u-boot */
#ifdef PLATFORM_AA5
#define CONFIG_ARC_CLK			24000000		/* 24MHz defaul clock speed */ 				  
#endif
#ifdef PLATFORM_XCODE3
//#define CONFIG_ARC_CLK			9380000		/* 9.38MHz clock speed */

#define CONFIG_ARC_CLK calculate_proc_clock()


#define MMREG_BASE			0xF0000000
#endif

#define CONFIG_NR_DRAM_BANKS    	1          		/* We have 1 bank of DRAM */
#define PHYS_SDRAM_1            	0x00000000 		/* SDRAM Bank #1 */

#if defined ARC5

	#define PHYS_SDRAM_1_SIZE	0x200000		/* 2 MB for arc 5 */
	#define UART_BASEADDR 		0x00FC1000		/* UART base address */
	#define VMAC_REG_BASEADDR 	0x00FC2000		/* EMAC base address */
	#define GMAC_REG_BASEADDR 	0xd0000000
	
#endif

#if defined ARC700

    #ifdef PLATFORM_AA5
	#define PHYS_SDRAM_1_SIZE      	0x400000 		/* 4 MB for arc 700*/
	#define UART_BASEADDR 		0xC0FC1000
	#define VMAC_REG_BASEADDR 	0xC0FC2000
	#endif
	
    #ifdef PLATFORM_XCODE3
	#define PHYS_SDRAM_1_SIZE      	0x400000 		/* 4 MB for arc 700*/
//	#define UART_BASEADDR 		(0xE0FC1000)
	#define UART_BASEADDR 		(0xF000F400)      // for XCode4
	#define VMAC_REG_BASEADDR 	0xC0FC2000
	#endif

	#define GMAC_REG_BASEADDR 	0x00002800

	/* for usb net on XC4 */
      	#define CONFIG_NET_MULTI
        #define CONFIG_USBNET_DEV_ADDR "00:02:03:04:05:06"
        #define CONFIG_USBNET_HOST_ADDR "0a:0b:0c:0d:0e:0f"
        #define CONFIG_USB_GADGET_DUALSPEED

#endif

/*
 *  Environment variables
 */
#define CONFIG_AUTO_COMPLETE		1			/* Auto complete commands */
#define CONFIG_HOSTNAME  		ViXS				
#define CONFIG_ETHADDR          	00:02:03:04:05:08	/* Ethernet address */

#define CONFIG_SERVERIP			10.10.10.1		/* Tftp server ip address */
#define CONFIG_IPADDR			10.10.10.10		/* Our ip address */
#define CONFIG_SERVERIP			10.10.10.1		/* Tftp server ip address */
#define CONFIG_GATEWAYIP		10.10.1.1
#define CONFIG_NETMASK			255.255.0.0

#define CONFIG_ETH0_RESET_PIN   6

#if defined ARC700
	#define CONFIG_LOADADDR			0x3000000	/* Default load address for net commands */	
#else 
	#define CONFIG_LOADADDR			0x0
#endif

#define CONFIG_BOOTDELAY 10
#define CONFIG_BOOTFILE uImage_arc_gz

#ifdef CFG_BOARD_XC4105
#define CONFIG_ETHACT				usb0
#define CONFIG_BOARD_ID				0x000a
#define CONFIG_BOOTCOMMAND "tftp 1000000 uImage_arc_gz;bootm 1000000\0"
#else
#define CONFIG_ETHACT				eth0
#define CONFIG_BOARD_ID				0x0084
#define CONFIG_BOOTCOMMAND "ext2load scsi 0 1000000 /boot/uImage_arc_gz;bootm 1000000"
#endif

//#define CONFIG_SHOW_BOOT_PROGRESS 1

#define CONFIG_BOOTARGS  "root=/dev/nfs rw nfsroot=172.18.2.155:/root/initramfs ip=dhcp"

/*#define CONFIG_BOOTDELAY 		5			
#define  CONFIG_BOOTFILE		arclinux 		
#define CONFIG_BOOTCOMMAND		\
	"tftpboot \"arclinux\";"			\
	"go 0x80002000"				*/

/*
 * Miscellaneous configurable options
 */

#define CONFIG_ENV_OVERWRITE		
#define CFG_LONGHELP
#define CFG_HZ 				CONFIG_ARC_CLK

#ifdef PLATFORM_AA5
#define CFG_PROMPT	 		"ArcAngel3# "
#endif
#ifdef PLATFORM_XCODE3
#define CFG_PROMPT	 		"ViXS XCode4# "
#endif

#define CFG_CBSIZE			256 			/* Size of console buffer */
#define CFG_PBSIZE 			(CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS             	16              	/* Max number of command args   */
#define CFG_BARGSIZE            	CFG_CBSIZE      	/* Boot Argument Buffer Size    */
#define CFG_LOAD_ADDR			CONFIG_LOADADDR		/* Default load address	for bootm command */ 


//The following defines have been moved to config.h
//#define CFG_ENV_IS_IN_NAND 		1	
//#define CFG_ENV_IS_IN_FLASH 		1	
//#define CFG_ENV_IS_NOWHERE


#define CFG_BAUDRATE_TABLE 		{ 57600 }
#define CFG_MEMTEST_START       	0x200 			/* Exception table starts at 0x0 */ /* memtest works on */
#define CFG_MEMTEST_END         	(PHYS_SDRAM_1_SIZE - (350 << 10)/* code + stack area */ - (64 << 10)/* malloc area */ )/* size of memory */   
#if defined ARC5
	#define CFG_DIRECT_FLASH_TFTP	1			/* Directly TFTP writes to flash memory */			
#endif

#if defined(CFG_ENV_IS_IN_NAND) /* Environment is in Nand Flash, we support only Samsung K9F2G08U0A by now*/
#define CFG_ENV_OFFSET	0x00100000
#define CFG_ENV_SIZE	2048
#else
#define CFG_ENV_SIZE            	1024			/* Max : 512 bytes total size of environment sector, fof Flash     */
#endif /* if defined(CFG_ENV_IS_IN_NAND) */

/*
 *  Flash options
 */
#define CFG_FLASH_XCODE3            1
//#define CFG_FLASH_M25P80
//#define CFG_FLASH_M25P128

//#define CFG_UBOOT_ON_FLASH 		0			 /*If flash memory is present and uboot is on the flash then 		flash = 1 automatically define in mkconfig*/

#define CFG_MAX_FLASH_BANKS     	1     			/* Max number of memory banks */
#define CFG_FLASH_BASE			0xF0000000 		 

//#define CFG_ENV_SIZE            	1024			/* Max : 512 bytes total size of environment sector     */
#define CFG_MAX_FLASH_SECT      	64  	 		/* Max number of sectors on one chip */

//#ifdef CFG_FLASH_M25P80
//#define CFG_ENV_ADDR			(CFG_FLASH_BASE + 0xf0000) /* use last sector for environments 256k i.e first sector occupied by u-boot second sector contains environment*/	
#define CFG_ENV_SECT_SIZE           0x10000
//#else
//#define CFG_ENV_ADDR			(CFG_FLASH_BASE + 0xfc0000) /* use last sector for environments 256k i.e first sector occupied by u-boot second sector contains environment*/	
//#define CFG_ENV_SECT_SIZE           0x40000

#define CFG_XCODE_MAX_IMAGE_SIZE        0x780000
#define CFG_XCODE_1ST_IMAGE_LOC (CFG_FLASH_BASE + 0x80000)
#define CFG_XCODE_2ND_IMAGE_LOC (CFG_XCODE_1ST_IMAGE_LOC + CFG_XCODE_MAX_IMAGE_SIZE)
#define CFG_XCODE_IMAGE_SEL_LOC (CFG_XCODE_2ND_IMAGE_LOC + CFG_XCODE_MAX_IMAGE_SIZE -1)
//#endif

/* #define CFG_FLASH_PROTECTION 		1 */ 		/* Hardware protection */

//#define CFG_FLASH_CFI           	1 			/* Flash memory is CFI compliant */
//#define CFG_FLASH_CFI_DRIVER    	1       		/* Use drivers/cfi_flash.c */
//#define CFG_FLASH_USE_BUFFER_WRITE 	1  			/* Use buffered writes (~10x faster) */


#endif /* __CONFIG_H */





/*
 * IDE options
 */
#ifdef PLATFORM_XCODE3

/*
 * AHCI options
 */
#ifdef CFG_SATA_ENABLED
#define CONFIG_SCSI_AHCI

#ifdef CONFIG_SCSI_AHCI
#define CFG_SCSI_MAX_SCSI_ID	1
#define CFG_SCSI_MAX_LUN	1
#define CFG_SCSI_MAX_DEVICE	(CFG_SCSI_MAX_SCSI_ID * CFG_SCSI_MAX_LUN)
#define CFG_SCSI_MAXDEVICE	CFG_SCSI_MAX_DEVICE

#define CONFIG_SCSI
#endif

#endif


//#define CFG_ATA_STRIDE          4		/* Width betwix addrs	*/
#define CONFIG_ATAPI            1

#define CONFIG_LBA48

#define CONFIG_DOS_PARTITION
#define CONFIG_ISO_PARTITION

#endif

/* 
 * Kernel Parameters 
 */

#define CONFIG_CMDLINE_TAG			1

#define CONFIG_CLK_SPEED_TAG         CONFIG_ARC_CLK    /* clock speed */

//# define CONFIG_SETUP_MEMORY_TAGS     0                 /* configuring bank */

#ifdef CONFIG_SETUP_MEMORY_TAGS  /* Memory size  */

#define CONFIG_MEMORY_SIZE  0x2000000
/*#define CONFIG_BANK0_BASE      0x0
#define CONFIG_BANK0_SIZE      0x400000

#define CONFIG_BANK1_BASE      0x400000
#define CONFIG_BANK1_SIZE      0x400000

#define CONFIG_BANK2_BASE      0x800000
#define CONFIG_BANK2_SIZE      0x400000*/

#endif /* CONFIG_SETUP_MEMORY_TAGS */


#define CONFIG_CACHE_TAG         1

#ifdef CONFIG_CACHE_TAG

#define CONFIG_ICACHE        1
#define CONFIG_DCACHE        1

#endif /* CONFIG_CACHE_TAG */

#define CONFIG_SERIAL_TAG 1

#ifdef CONFIG_SERIAL_TAG

#define CONFIG_UART0_BAUDRATE  CONFIG_ARC_SERIAL_BAUD

#endif 

#define CONFIG_VMAC_TAG 1

#define CONFIG_VMAC1_TAG 1

#define CONFIG_VMACUSB0_TAG 1

#ifdef CONFIG_VMAC_TAG
#define CONFIG_VMAC_ADDR   "00:02:03:04:05:08"
#endif

#ifdef CONFIG_VMAC1_TAG
#define CONFIG_VMAC1_ADDR  "00:20:30:40:50:60"
#endif

#ifdef PLATFORM_XCODE3

#define CONFIG_VMACUSB0_TAG 1

#ifdef CONFIG_VMACUSB0_TAG
#define CONFIG_VMACUSB0_ADDR  CONFIG_USBNET_DEV_ADDR
#endif

#endif

