/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) 
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/include/configs/xcode_viper.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

#ifndef __CONFIG_H
#define __CONFIG_H
#include <cmd_confdefs.h>

//#define CONFIG_BZIP2

/*
 *  Size of malloc pool
 */
#define CFG_MALLOC_LEN			0x800000 	//=8M//0x8000	/* Malloc size = 32k */

#define CFG_STACK_LEN			0x400000

#define CFG_UBOOT_SIZE			0x100000     //0x40000

/*
 *  Serial configuartion
 */
#define CONFIG_SERIAL_BAUD	115200	/* Baudrate for serial port */
#define NUM_PORTS		1	/* how many UART ports to enable on u-boot */
#ifdef VIPER_FPGA
#define UART_CLK_DIVIDER 8      // for FPGA testing, STANDALONE FPGA
#else
#define UART_CLK_DIVIDER 1      
#endif
#if NUM_PORTS==4
#define UART_BASEADDR	{(xcode_uart *)0xBFFFF400, (xcode_uart *)0xBFFFF450, (xcode_uart *)0xBFFFF500, (xcode_uart *)0xBFFFF550}
#elif NUM_PORTS==3
#define UART_BASEADDR	{(xcode_uart *)0xBFFFF400, (xcode_uart *)0xBFFFF450, (xcode_uart *)0xBFFFF500}
#elif NUM_PORTS==2
#define UART_BASEADDR	{(xcode_uart *)0xBFFFF400, (xcode_uart *)0xBFFFF450}
#else
#define UART_BASEADDR 	{(xcode_uart *)0xBFFFF400}
#endif

/*
 *  Physical memory map
 */

/*This clock speed is for u-boot */
#define MMREG_BASE			(0xBFFF0000UL)

/* DISPLAY SPLASH SCREEN */
#define CFG_DISPLAY_BLENDING	0
#define CFG_DISPLAY_FB_ADDR (0x88000000UL)

/* SECURE BOOT CONFIGURATION */
#ifdef CONFIG_SECUREBOOT
#define CFG_SECURITY_RSA2048		1
#define CFG_SECURITY_RSA1024		0
#define	CFG_SECURITY_AES_ECB		0
#define CFG_SECURITY_AES_CBC		0
#endif 
/*END CONFIG_SECUREBOOT */

#define CONFIG_NR_DRAM_BANKS    	1          		/* We have 1 bank of DRAM */
#define PHYS_SDRAM_1            	0x00000000 		/* SDRAM Bank #1 */
#define PHYS_SDRAM_1_SIZE      	0x400000 			/* 4 MB for arc 700*/

#define VMAC_REG_BASEADDR 	0xC0FC2000

#define GMAC_REG_BASEADDR 	0x00002800

/* for usb net on XC4 */
#define CONFIG_NET_MULTI
#define CONFIG_USBNET_DEV_ADDR 	"00:02:03:04:05:06"
#define CONFIG_USBNET_HOST_ADDR "0a:0b:0c:0d:0e:0f"
#define CONFIG_USB_GADGET_DUALSPEED

/*
 *  Environment variables
 */
#define CONFIG_AUTO_COMPLETE	1				/* Auto complete commands */
#define CONFIG_HOSTNAME  		ViXS				
#define CONFIG_ETHADDR          00:02:03:04:05:46		/* Ethernet address */


#define CONFIG_SERVERIP			172.19.1.159		/* Tftp server ip address */
#define CONFIG_IPADDR			172.19.1.234		/* Our ip address */
#define CONFIG_GATEWAYIP		172.19.1.1
#define CONFIG_NETMASK			255.255.0.0

#define CONFIG_ETH0_RESET_PIN   3

#define CONFIG_LOADADDR			0x81000000

#define CONFIG_BOOTDELAY 		5
#define CONFIG_ETHACT			eth0
#define CONFIG_BOARD_ID			0x00F6
#define CONFIG_POWER_MANAGEMENT0			0xFFFFFFFF
#define CONFIG_POWER_MANAGEMENT1			0xFFFFFFFF


#if defined(CFG_ENV_IS_IN_NAND)
#define CONFIG_CURRENT_ACTIVE	0
#define CONFIG_BOOTCOMMAND		nboot 81000000 0; bootm 81000000
#define CONFIG_RFSARGS			rw
#else
#define CONFIG_BOOTFILE 		/boot/uImage_mips_gz
#define CONFIG_BOOTCOMMAND 		ext2load scsi 0 81000000;bootm 81000000
#define CONFIG_BOOTARGS  		root=/dev/sda2 rw

// Multiboot enable 
#ifdef CONFIG_MULTIBOOT
#define CONFIG_BOOTFILE1		/boot/uImage_mips_gz
#define CONFIG_BOOTFILE2		/boot/uImage_mips_android_gz
#define CONFIG_BOOTFILE3		/boot/uImage_mips_gz_bd
#define CONFIG_BOOTFILE4 		/boot/uImage_mips_gz
#define CONFIG_BOOTFILE5 		/boot/uImage_mips_gz
#define CONFIG_BOOTARGS1		root=/dev/sda2 rw
#define CONFIG_BOOTARGS2  		root=/dev/sda5 rw rdinit=/none init=/init ip=dhcp console=ttyS0
#define CONFIG_BOOTARGS3  		root=/dev/sda6 rw
#define CONFIG_BOOTARGS4  		root=/dev/sda7 rw
#define CONFIG_BOOTARGS5  		root=/dev/sda2 rw
#define CONFIG_BOOTNAME			DEBUG
#define CONFIG_BOOTNAME_1		VIXS_SDK
#define CONFIG_BOOTNAME_2		Android
#define CONFIG_BOOTNAME_3		BDPlayer
#endif /* CONFIG_MULTIBOOT */
#endif

//#define CONFIG_BOOTARGS  		root=/dev/nfs rw nfsroot=172.19.1.159:/root/initramfs ip=dhcp
//#define CONFIG_SHOW_BOOT_PROGRESS 1

/*
 * Miscellaneous configurable options
 */

#define CONFIG_ENV_OVERWRITE		
#define CFG_LONGHELP

#define CFG_PROMPT	 		"XCode_Viper# "

#define CFG_CBSIZE			256 			/* Size of console buffer */
#define CFG_PBSIZE 			(CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS             	16              	/* Max number of command args   */
#define CFG_BARGSIZE            	CFG_CBSIZE      	/* Boot Argument Buffer Size    */
#define CFG_LOAD_ADDR			CONFIG_LOADADDR		/* Default load address	for bootm command */ 

#define CFG_BAUDRATE_TABLE 		{ 57600 }
#define CFG_MEMTEST_START       	0x200 			/* Exception table starts at 0x0 */ /* memtest works on */
#define CFG_MEMTEST_END         	(PHYS_SDRAM_1_SIZE - (350 << 10)/* code + stack area */ - (64 << 10)/* malloc area */ )/* size of memory */   


#define CFG_NAND_INTERLEAF_MODE 1

#if defined(CFG_ENV_IS_IN_NAND)
#define CFG_ENV_SIZE	2048			/* Max : 2048 bytes total size for NAND Flash */
#else
#define CFG_ENV_SIZE	1024			/* Max : 1024 bytes total size for SPI Flash */
#endif

/*
 *  Flash options
 */
#define CFG_FLASH_XCODE            	1

#define CFG_MAX_FLASH_BANKS     	1     			/* Max number of memory banks */
#define CFG_FLASH_BASE			0xF0000000 		 

#define CFG_MAX_FLASH_SECT      	1024			/* Max number of sectors on one chip */

#define CFG_ENV_SECT_SIZE           	0x10000         	/* it may not be accurate since the sector size depends on the SPI flash type*/


/* Linux Image configuration on SPI */
#define CFG_XCODE_MAX_IMAGE_SIZE        0x780000
#define CFG_XCODE_1ST_IMAGE_LOC (CFG_FLASH_BASE + 0x80000)
#define CFG_XCODE_2ND_IMAGE_LOC (CFG_XCODE_1ST_IMAGE_LOC + CFG_XCODE_MAX_IMAGE_SIZE)
#define CFG_XCODE_IMAGE_SEL_LOC (CFG_XCODE_2ND_IMAGE_LOC + CFG_XCODE_MAX_IMAGE_SIZE -1)





/*
 * AHCI options
 */
#define CFG_SATA_ENABLED
#ifdef CFG_SATA_ENABLED
#define CONFIG_SCSI_AHCI

#ifdef CONFIG_SCSI_AHCI
#define CFG_SCSI_MAX_SCSI_ID	1
#define CFG_SCSI_MAX_LUN	2 /* Logical Unit Number, usually is used to refer to the drive itself */
#define CFG_SCSI_MAX_DEVICE	(CFG_SCSI_MAX_SCSI_ID * CFG_SCSI_MAX_LUN)
#define CFG_SCSI_MAXDEVICE	CFG_SCSI_MAX_DEVICE

#define CONFIG_SCSI
#endif



//#define CFG_ATA_STRIDE          4		/* Width betwix addrs	*/
#define CONFIG_ATAPI            1

#define CONFIG_LBA48

#define CONFIG_DOS_PARTITION
#define CONFIG_ISO_PARTITION
#define CONFIG_EFI_PARTITION

#define CFG_64BIT_VSPRINTF

#endif
// Kernel image decryption detination
#define CFG_DECRYPT_LOCATION 0x84000000
/* 
 * Kernel Parameters 
 */

#define CONFIG_CMDLINE_TAG			1

#define CONFIG_CLK_SPEED_TAG         CONFIG_ARC_CLK    /* clock speed */

//# define CONFIG_SETUP_MEMORY_TAGS     0                 /* configuring bank */

#ifdef CONFIG_SETUP_MEMORY_TAGS  /* Memory size  */

#define CONFIG_MEMORY_SIZE  0x2000000

#endif /* CONFIG_SETUP_MEMORY_TAGS */


#define CONFIG_CACHE_TAG         1

#ifdef CONFIG_CACHE_TAG

#define CONFIG_ICACHE        1
#define CONFIG_DCACHE        1

#endif /* CONFIG_CACHE_TAG */

#define CONFIG_SERIAL_TAG 1

#ifdef CONFIG_SERIAL_TAG

#define CONFIG_UART0_BAUDRATE  CONFIG_SERIAL_BAUD

#endif 

#define CONFIG_VMAC_TAG 1

#define CONFIG_VMAC1_TAG 1

#define CONFIG_VMACUSB0_TAG 1

#ifdef CONFIG_VMAC_TAG
#define CONFIG_VMAC_ADDR   "00:02:03:04:05:08"
#endif

#ifdef CONFIG_VMAC1_TAG
#define CONFIG_VMAC1_ADDR  "00:20:30:40:50:60"
#endif

#ifdef PLATFORM_XCODE

#define CONFIG_VMACUSB0_TAG 1

#ifdef CONFIG_VMACUSB0_TAG
#define CONFIG_VMACUSB0_ADDR  CONFIG_USBNET_DEV_ADDR
#endif

#define XC_SOC_PROC_MMREG_BASE (0xBFFF0000)

/*
 * copy from mips boards config.h
 */
 
/* The following #defines are needed to get flash environment right */
#define CFG_MONITOR_BASE    TEXT_BASE
#define CFG_MONITOR_LEN     (192 << 10)

#define CFG_INIT_SP_OFFSET  (TEXT_BASE + CFG_UBOOT_SIZE + CFG_MALLOC_LEN + 4 + CFG_STACK_LEN)
#define CFG_MHZ         325

//#if (CFG_MHZ % 12) != 0
//#error "Invalid CPU frequency - must be multiple of 12!"
//#endif

#define CFG_HZ                  (CFG_MHZ * 1000000) /* FIXME causes overflow in net.c */

//#define CFG_SDRAM_BASE      	0x00000000     /* Physical addr */
#define CFG_SDRAM_BASE		0x80000000     /* KSEG0 Cached addr */
//#define CFG_SDRAM_BASE      	0xA0000000     /* KSEG1 UnCached addr */

#define MEM_SIZE 192

#define CFG_DCACHE_SIZE     65536
#define CFG_ICACHE_SIZE     65536
#define CFG_CACHELINE_SIZE  32

#define CFG_BOOTPARAMS_LEN  128*1024

#define CONFIG_BAUDRATE     CONFIG_SERIAL_BAUD 



#ifdef CFG_MINI_IMAGE
/* configuration with minimal size*/
#define CONFIG_COMMANDS ( CFG_CMD_NET|    \
											CFG_CMD_ENV)

#else /* environment variables are in Flash */
#define CONFIG_TERSE_MII 1
#define CONFIG_COMMANDS (CFG_CMD_NET|\
											CFG_CMD_ENV|\
											CFG_CMD_FLASH|\
											CFG_CMD_FAT|\
											CFG_CMD_DHCP|\
											CFG_CMD_PING|\
											CFG_CMD_NFS|\
											CFG_CMD_EXT2|\
											CFG_CMD_SCSI|\
											CFG_CMD_NAND|\
											CFG_CMD_MEMORY|\
											CFG_CMD_I2C|\
											CFG_CMD_USB|\
											CFG_CMD_RUN|\
											CFG_CMD_MII|\
											CFG_CMD_YUV|\
											CFG_CMD_MMC|\
											CFG_CMD_LZO)
#endif

#if (CONFIG_COMMANDS&CFG_CMD_I2C)
#define CONFIG_DRIVER_XCODE_I2C
#endif

#if (CONFIG_COMMANDS&CFG_CMD_USB)
#define CONFIG_USB_EHCI
#define CONFIG_USB_STORAGE
#endif

#if (CONFIG_COMMANDS& CFG_CMD_MMC)
#define CONFIG_MMC			1
#define CONFIG_GENERIC_MMC	1
#define CONFIG_XCODE_MMC	1
#define CONFIG_PARTITIONS	1
#define CFG_MMC_HIGHSPEED	1
#endif

#ifndef CONFIG_SYS_HZ
#define CONFIG_SYS_HZ 500000000
#endif

#endif

#endif /* __CONFIG_H */

