#ifndef _XVDRV_IF_H
#define _XVDRV_IF_H
#ifdef LINUX
#include <linux/ioctl.h>
#endif

enum 
{
    VDEV_IOCTL_PEER_NOT_OPENED = 1,
    VDEV_IOCTL_PEER_BUF_FULL,
    VDEV_IOCTL_MSG_FULL,
    VDEV_IOCTL_SEND_TIMEOUT,
    VDEV_IOCTL_RECV_TIMEOUT,
    VDEV_IOCTL_UNCORRECT_PARAMETER,
    VDEV_IOCTL_NO_MEM,
    VDEV_IOCTL_RX_BUF_FULL_NO_DROP,
    
};

typedef struct para_open_req
{
    unsigned int struct_size;
    unsigned int handle;
    unsigned int timeout;
    unsigned int return_value;
    unsigned int input_buf;
    unsigned int input_buf_phy;
    unsigned int input_maxsize;
    unsigned char type;
} IOCTLOPENCOMM, *PIOCTLOPENCOMM;

typedef struct para_close_req
{
    unsigned int struct_size;
    unsigned int handle;
    unsigned int timeout;
    unsigned int return_value;

}IOCTLCLOSECOMM, *PIOCTLCLOSECOMM;

typedef struct para_send_req
{
    unsigned int struct_size;
    unsigned int handle;
    unsigned int timeout;
    unsigned int return_value;
    unsigned int input_buf;
    unsigned int input_size;

}IOCTLSDEVSENDCOMM, *PIOCTLSDEVSENDCOMM;

typedef struct para_recv_req
{
    unsigned int struct_size;
    unsigned int handle;
    unsigned int timeout;
    unsigned int return_value;
    unsigned int output_buf;
    unsigned int output_buf_phy;
    unsigned int output_size;
}IOCTLRECVCOMM, *PIOCTLRECVCOMM;

typedef struct para_flush_cache_region
{
    unsigned int struct_size;
    unsigned int memaddr_virt;
    unsigned int memsize;
 
}IOCTLFLUSHCACHEREGION, *PIOCTLFLUSHCACHEREGION;

typedef struct para_invalid_cache_region
{
    unsigned int struct_size;
    unsigned int memaddr_phy;
    unsigned int memsize;
    
}IOCTLINVCACHEREGION, *PIOCTLINVCACHEREGION;


typedef struct para_alloc_kernel_mem
{
    unsigned int struct_size;
    unsigned int return_value;    
    unsigned int memaddr_user;
    unsigned int memaddr_kernel;
    unsigned int memaddr_bus;    
    unsigned int memsize;

 
}IOCTLALLOCKERNELMEM, *PIOCTLALLOCKERNELMEM;

typedef struct para_release_kernel_mem
{
    unsigned int struct_size;
    unsigned int return_value;    
    unsigned int memaddr_user;
    unsigned int memaddr_kernel;
    unsigned int memsize;
    
}IOCTLRELEASEKERNELMEM, *PIOCTLRELEASEKERNELMEM;


//Application interface
#ifndef LINUX
#define SIOCDEVPRIVATE 0
#endif

#define VIXS_CDC_CP 1
#define VIXS_CDC_NP 0
#ifndef VIXS_VCOMM_DRV
#define XVETH_CMD_OPEN SIOCDEVPRIVATE
#define XVETH_CMD_CLOSE (SIOCDEVPRIVATE + 1)
#define XVETH_CMD_SEND (SIOCDEVPRIVATE + 2)
#define XVETH_CMD_RECV (SIOCDEVPRIVATE + 3)
#define XVETH_CMD_REG_RD  (SIOCDEVPRIVATE + 4)
#define XVETH_CMD_WAKEUP  (SIOCDEVPRIVATE + 5)
#define XVETH_CMD_REG_WE  (SIOCDEVPRIVATE + 6)

#else

#define XVCOMM_IOCTL_MAGIC        'x'

#define XVCOMM_IOCTL_OPEN   _IOW(XVCOMM_IOCTL_MAGIC, 0, IOCTLOPENCOMM)
#define XVCOMM_IOCTL_CLOSE   _IOW(XVCOMM_IOCTL_MAGIC, 1, IOCTLCLOSECOMM)
#define XVCOMM_IOCTL_SEND _IOW(XVCOMM_IOCTL_MAGIC, 2, IOCTLSDEVSENDCOMM)
#define XVCOMM_IOCTL_RECV _IOW(XVCOMM_IOCTL_MAGIC, 3, IOCTLRECVCOMM)
#define XVCOMM_IOCTL_FLUSH_CACHE_REGION _IOW(XVCOMM_IOCTL_MAGIC, 4, IOCTLRECVCOMM)
#define XVCOMM_IOCTL_INVALID_CACHE_REGION _IOW(XVCOMM_IOCTL_MAGIC, 5, IOCTLRECVCOMM)
#define XVCOMM_IOCTL_ALLOC_KERNEL_MEM   _IOW(XVCOMM_IOCTL_MAGIC, 6, IOCTLRECVCOMM)
#define XVCOMM_IOCTL_RELEASE_KERNEL_MEM   _IOW(XVCOMM_IOCTL_MAGIC, 7, IOCTLRECVCOMM)

#define XVCOMM_IOCTL_MAXNR        8

#endif


#endif

