/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Aug 17 , 2005
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/lib_arc/arclinux.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */

#include <asm/setup.h>
#include <command.h>
#include <linux/types.h>
#include <image.h>
#include <asm/byteorder.h>
#include <common.h>

extern image_header_t header; /* from cmd_bootm.c */




#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
defined (CONFIG_CMDLINE_TAG) || \
defined (CONFIG_CLK_SPEED_TAG) || \
defined (CONFIG_CACHE_TAG) || \
defined (CONFIG_SERIAL_TAG) || \
defined (CONFIG_VMAC_TAG) || \
defined (CONFIG_VMAC1_TAG) || \
defined (CONFIG_INITRD_TAG) 

static void setup_start_tag (bd_t *bd);

# ifdef CONFIG_SETUP_MEMORY_TAGS
        static void setup_memory_tags (bd_t *bd,u32 size);
# endif
# ifdef CONFIG_CMDLINE_TAG
        static void setup_commandline_tag (bd_t *bd, char *commandline);
# endif
# ifdef CONFIG_CLK_SPEED_TAG
        static void setup_clk_speed_tag(bd_t *bd,u32 clk_speed);
# endif
#ifdef CONFIG_CACHE_TAG
        static void setup_cache_tag(bd_t *bd,u8 icache,u8 dcache); 
#endif

#ifdef CONFIG_VMAC_TAG
static void setup_vmac_tag(bd_t *bd,u8 addr[6]) ;
#endif

#ifdef CONFIG_VMAC1_TAG
static void setup_vmac1_tag(bd_t *bd,u8 addr[6]) ;
#endif

#ifdef CONFIG_VMACUSB0_TAG
static void setup_vmacusb0_tag (bd_t *bd,u8 addr[8]);
#endif

#ifdef CONFIG_SERIAL_TAG
static void setup_serial_tag(bd_t *bd,u8 serial_nr,u32 baudrate);
#endif
 
# ifdef CONFIG_INITRD_TAG
	static void setup_initrd_tag (bd_t *bd, ulong initrd_start,ulong initrd_end);
# endif
static void setup_end_tag (bd_t *bd);
static struct tag *params;

#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */







/* 
 * Last function before U-Boot passes control to kernel.
 */

void do_bootm_linux (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[],ulong addr, ulong *len_ptr, int verify)
{

        DECLARE_GLOBAL_DATA_PTR;		
	void (*kernel)();
	image_header_t *hdr = &header;
	ulong data,len=0,checksum;
	ulong initrd_start,initrd_end;
	char *temp;
	bd_t *bd = gd->bd;
       ulong temp1, temp2;

	kernel = (void (*)()) ntohl(hdr->ih_ep);

	printf("do_bootm_linux\n");
	//(*kernel)();
	
	/* halt the vmac controller */
	eth_halt();
	

	/*
	 * check for initrd image 
	 */

	if(argc >= 3) {
	  
	  addr = simple_strtoul(argv[2],NULL,16); 
	  printf("## Loading Ramdisk Image at %08lx.....\n",addr);

	  memcpy(&header,(char *)addr,sizeof(image_header_t)); /* copy header of uramdisk */

	  /* check magic number */
	  if(ntohl(hdr->ih_magic) != IH_MAGIC ) {

	    printf("Bad Magic Number\n");
	    goto noinitrd;

	  }
	  /* check header crc */
	  checksum = ntohl(hdr->ih_hcrc);
	  hdr->ih_hcrc = 0;
	  data = (ulong) &header;
	  len = sizeof(image_header_t);
	  
	  if(crc32(0,(char *)data,len) != checksum) {
	    
	    printf("Bad header checksum\n");
	    goto noinitrd;

	  }
  
	  print_image_hdr(hdr);
	  
	  /* check data crc */
	  data = addr + sizeof(image_header_t);
	  len = ntohl(hdr->ih_size);

	  if(verify) {
	    ulong csum = 0;
	    printf("   Verifying Checksum...");
	    csum = crc32(0,(char *)data,len);
	    if(csum != ntohl(hdr->ih_dcrc)) {
	      
	      printf("Bad Data Checksum\n");
	      goto noinitrd;
	    }
	    printf("OK\n");
	  }

	  
          /* check valid image */  
	  if( (hdr->ih_os != IH_OS_LINUX) ||
	      (hdr->ih_arch != IH_CPU_ARC) ||
	      (hdr->ih_type != IH_TYPE_RAMDISK)) {
	    
	    printf("No Linux ARC Ramdisk Image\n");
	    goto noinitrd;
	  }


	  /* move ramdisk image to its load address */
	  memmove((void *)ntohl(hdr->ih_load),(uchar *)data,len);
	  data = ntohl(hdr->ih_load);
	}
	else {
	  /* no initrd image */
	noinitrd:
	  len = data = 0;
	}
	
#ifdef DEBUG
	if(!data) {
	  printf("No initrd");
	}
#endif
	if(data) {
	  initrd_start = data;
	  initrd_end = initrd_start + len;
	}
	else {
	  initrd_start = initrd_end = 0;
	}



	
#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
defined (CONFIG_CMDLINE_TAG) || \
defined (CONFIG_CLK_SPEED_TAG) || \
defined (CONFIG_CACHE_TAG) ||  \
defined (CONFIG_SERIAL_TAG) || \
defined (CONFIG_VMAC_TAG) || \
defined (CONFIG_VMAC1_TAG) || \
defined (CONFIG_INITRD_TAG) 
    
	setup_start_tag(bd);
#ifdef CONFIG_SETUP_MEMORY_TAGS
          u32 size ;   
	  
	  temp = getenv("memsize");
	  size = temp ? simple_strtoul(temp,NULL,10) : CONFIG_MEMORY_SIZE;

	  setup_memory_tags(bd,size);
          /*temp = getenv("bank0");
	  bank_flag = temp ? (int)simple_strtoul(temp,NULL,10) : 1; 
	  if(bank_flag)
	    setup_memory_tags(bd,0,CONFIG_BANK0_BASE,CONFIG_BANK0_SIZE);

	  temp = getenv("bank1");
	  bank_flag = temp ? (int)simple_strtoul(temp,NULL,10) : 1; 
	  if(bank_flag)
	    setup_memory_tags(bd,1,CONFIG_BANK1_BASE,CONFIG_BANK1_SIZE);

          temp = getenv("bank2");
	  bank_flag = temp ? (int)simple_strtoul(temp,NULL,10) : 1; 
	  if(bank_flag)
	  setup_memory_tags(bd,2,CONFIG_BANK2_BASE,CONFIG_BANK2_SIZE); */



#endif

#ifdef CONFIG_CMDLINE_TAG
	char *cmdline = getenv("bootargs");  
	setup_commandline_tag(bd,cmdline);
#endif
#ifdef CONFIG_CLK_SPEED_TAG
	u32 clk_speed;

       clk_speed  = CONFIG_CLK_SPEED_TAG;
    
	setup_clk_speed_tag(bd,clk_speed);
#endif
#ifdef CONFIG_VMAC_TAG

	int i;
	u8 macaddr[6];
        ulong reg;
        u8 *s, *e;
        u8 tmp[64];
        //i = getenv_r ("macaddr", tmp, sizeof (tmp));
        i = getenv_r ("ethaddr", tmp, sizeof (tmp));

        s = (i > 0) ? tmp : CONFIG_VMAC_ADDR;

	
        for (reg = 0; reg < 6; ++reg) {
	  macaddr[reg] = s ? simple_strtoul (s, &e, 16) : 0;
          if (s)
	    s = (*e) ? e + 1 : e;
	} 
	setup_vmac_tag(bd,macaddr);
#endif
#ifdef CONFIG_VMAC1_TAG

        int i_vmac1;
        u8 macaddr_vmac1[6];
        ulong reg_vmac1;
        u8 *s_vmac1, *e_vmac1;
        u8 tmp_vmac1[64];
        //i_vmac1 = getenv_r ("mac1addr", tmp_vmac1, sizeof (tmp_vmac1));
        i_vmac1 = getenv_r ("eth1addr", tmp_vmac1, sizeof (tmp_vmac1));

       s_vmac1 = (i_vmac1 > 0) ? tmp_vmac1 : CONFIG_VMAC1_ADDR;

        for (reg_vmac1 = 0; reg_vmac1 < 6; ++reg_vmac1) {
          macaddr_vmac1[reg_vmac1] = s_vmac1 ? simple_strtoul (s_vmac1, &e_vmac1, 16) : 0;
          if (s_vmac1)
            s_vmac1 = (*e_vmac1) ? e_vmac1 + 1 : e_vmac1;
        }
        setup_vmac1_tag(bd,macaddr_vmac1);
#endif


#ifdef CONFIG_VMACUSB0_TAG

        int i_vmacusb0;
        u8 macaddr_vmacusb0[6];
        ulong reg_vmacusb0;
        u8 *s_vmacusb0, *e_vmacusb0;
        u8 tmp_vmacusb0[64];
        //i_vmac1 = getenv_r ("mac1addr", tmp_vmac1, sizeof (tmp_vmac1));
        i_vmacusb0 = getenv_r ("usb0addr", tmp_vmacusb0, sizeof (tmp_vmacusb0));

       s_vmacusb0 = (i_vmacusb0 > 0) ? tmp_vmacusb0 : CONFIG_VMACUSB0_ADDR;

        for (reg_vmacusb0 = 0; reg_vmacusb0 < 6; ++reg_vmacusb0) {
          macaddr_vmacusb0[reg_vmacusb0] = s_vmacusb0 ? simple_strtoul (s_vmacusb0, &e_vmacusb0, 16) : 0;
          if (s_vmacusb0)
            s_vmacusb0 = (*e_vmacusb0) ? e_vmacusb0 + 1 : e_vmacusb0;
        }
        setup_vmacusb0_tag(bd,macaddr_vmacusb0);
#endif


#ifdef CONFIG_CACHE_TAG
	u8 icache,dcache;
	
	temp = getenv("icache");
	icache = temp ? simple_strtoul(temp,NULL,10) : CONFIG_ICACHE;


	temp = getenv("dcache");
	
	dcache = temp ? simple_strtoul(temp,NULL,10) : CONFIG_DCACHE;

	setup_cache_tag(bd,icache,dcache);
#endif
#ifdef CONFIG_SERIAL_TAG


        u32 baudrate;
	
	temp = getenv("uart0");
	baudrate = temp ? simple_strtoul(temp,NULL,10) : CONFIG_UART0_BAUDRATE;

	setup_serial_tag(bd,0,baudrate);   
	    

#endif
#ifdef CONFIG_INITRD_TAG
	if(initrd_start && initrd_end)
	  setup_initrd_tag(bd,initrd_start,initrd_end);
#endif
	setup_end_tag(bd);

	disable_dcache();
	
	disable_icache();;
	
	printf("Starting kernel = %x\n", kernel);
	__asm__ __volatile__(	
			      "mov r0,0\n\t"
			      "lr  r1,[identity]\n\t"
			      "ld  r2,%0\n\t"
			      "ld  r3,%1\n\t"
			      "jl  [r3]\n\t"
			      ::"m"(bd->bi_boot_params),"m"(kernel)
			      :"r0","r1","r2");

#endif

	printf("Starting kernel kernel = %x\n", kernel);
	(*kernel)();
}

#if defined (CONFIG_SETUP_MEMORY_TAGS) || \
defined (CONFIG_CMDLINE_TAG) || \
defined (CONFIG_CLK_SPEED_TAG) || \
defined (CONFIG_CACHE_TAG) || \
defined (CONFIG_SERIAL_TAG) || \
defined (CONFIG_VMAC_ADDR)  || \
defined (CONFIG_VMAC1_ADDR)  || \
defined (CONFIG_VMACUSB0_ADDR)  || \
defined (CONFIG_INITRD_TAG) 


static void setup_start_tag (bd_t *bd)
{
	params = (struct tag *) bd->bi_boot_params;
	params->hdr.tag = ATAG_CORE;
	params->hdr.size = tag_size (tag_core);
	params = tag_next (params);
}


#ifdef CONFIG_SERIAL_TAG
static void setup_serial_tag (bd_t *bd,u8 serial_nr,u32 baudrate)
{
         printf("Setting up serial tag for uart%d with baudrate %d\n",serial_nr,baudrate); 
         params->hdr.tag = ATAG_SERIAL; 
	 params->hdr.size = tag_size(tag_serial);

	 params->u.serial.serial_nr = serial_nr;
	 params->u.serial.baudrate = baudrate;

	 
	 params = tag_next(params);


}

#endif

#ifdef CONFIG_VMAC_ADDR
static void setup_vmac_tag (bd_t *bd,u8 addr[8])
{
         ulong reg;
         printf("Setting up vmac tag with mac addr %d:%d:%d:%d:%d:%d\n",addr[0],addr[1],addr[2],addr[3],addr[4],addr[5]); 
 
         params->hdr.tag = ATAG_VMAC; 
	 params->hdr.size = tag_size(tag_vmac);

	 for(reg=0; reg < 6 ;reg++) {
	 
	   params->u.vmac.addr[reg] = addr[reg];

	 }
	
	 params = tag_next(params);
	



}
#endif

#ifdef CONFIG_VMAC1_ADDR
static void setup_vmac1_tag (bd_t *bd,u8 addr[8])
{
         ulong reg;
         printf("Setting up vmac1 tag with mac addr %d:%d:%d:%d:%d:%d\n",addr[0],addr[1],addr[2],addr[3],addr[4],addr[5]);

         params->hdr.tag = ATAG_VMAC1;
         params->hdr.size = tag_size(tag_vmac);

         for(reg=0; reg < 6 ;reg++) {

           params->u.vmac1.addr[reg] = addr[reg];

         }

         params = tag_next(params);




}
#endif

#ifdef CONFIG_VMACUSB0_ADDR
static void setup_vmacusb0_tag (bd_t *bd,u8 addr[8])
{
         ulong reg;
         printf("Setting up vmacusb0 tag with mac addr %d:%d:%d:%d:%d:%d\n",addr[0],addr[1],addr[2],addr[3],addr[4],addr[5]);

         params->hdr.tag = ATAG_VMACUSB0;
         params->hdr.size = tag_size(tag_vmac);

         for(reg=0; reg < 6 ;reg++) {

           params->u.vmacusb0.addr[reg] = addr[reg];

         }

         params = tag_next(params);




}
#endif



#ifdef CONFIG_SETUP_MEMORY_TAGS
static void setup_memory_tags (bd_t *bd,u32 size)
{

         printf("Setting up memory tag of size %x \n",size);
         params->hdr.tag = ATAG_MEM; 
	 params->hdr.size = tag_size(tag_mem32);


	 params->u.mem.size = size;
	 
	 params = tag_next(params);


}
#endif /* CONFIG_SETUP_MEMORY_TAGS */


#ifdef CONFIG_CACHE_TAG
static void setup_cache_tag(bd_t *bd,u8 icache,u8 dcache)
{
         
         printf("Setting up cache tag with icache %d dcache %d\n",icache,dcache);  
         params->hdr.tag = ATAG_CACHE;
	 params->hdr.size = tag_size(tag_cache);
	 
	 params->u.cache.icache = icache;
	 params->u.cache.dcache = dcache;
	 
	 params = tag_next(params);

}
#endif /* CONFIG_CACHE_TAG */



#ifdef CONFIG_CMDLINE_TAG
static void setup_commandline_tag (bd_t *bd, char *commandline)
{
	char *p;
	if (!commandline)
                 return;
 
         /* eat leading white space */
         for (p = commandline; *p == ' '; p++);
 
         /* skip non-existent command lines so the kernel will still
          * use its default command line.
          */
         if (*p == '\0')
                 return;
 
         params->hdr.tag = ATAG_CMDLINE;
         params->hdr.size =
                 (sizeof (struct tag_header) + strlen (p) + 1 + 4) >> 2;
 
         strcpy (params->u.cmdline.cmdline, p);
 
         params = tag_next (params);
}

#endif /* CONFIG_CMDLINE_TAG */

#ifdef CONFIG_CLK_SPEED_TAG
static void setup_clk_speed_tag(bd_t *bd, u32 clk_speed)
{
         printf("Setting up clock speed tag with value %d\n",clk_speed);
         params->hdr.tag = ATAG_CLK_SPEED;
	 params->hdr.size = tag_size (tag_clk_speed);

	 params->u.clk_speed.clk_speed_hz = clk_speed;
	 params = tag_next (params);
 
}
#endif /* CONFIG_CLK_SPEED_TAG */


#ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag (bd_t *bd, ulong initrd_start, ulong initrd_end)
{
        /* an ATAG_INITRD node tells the kernel where the compressed
         * ramdisk can be found. 
         */
         params->hdr.tag = ATAG_INITRD2;
         params->hdr.size = tag_size (tag_initrd);
 
         params->u.initrd.start = initrd_start;
         params->u.initrd.size = initrd_end - initrd_start;
 
         params = tag_next (params);
}
#endif/* CONFIG_INITRD_TAG */


static void setup_end_tag (bd_t *bd)
{
         params->hdr.tag = ATAG_NONE;
         params->hdr.size = 0;
}

#endif /* CONFIG_SETUP_MEMORY_TAGS || CONFIG_CMDLINE_TAG || CONFIG_INITRD_TAG */
 


