/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Aug 15 , 2005
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  uboot/lib_arc/board.c
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * Authors : Sandeep Patil and Pradeep Sawlani.
 */


#include <common.h>		/* Serial functions	*/ 
#include <linux/types.h>	/* ulong typedef    	*/
#include <asm/uboot-arc.h>	/* _start		*/
#include <asm/global_data.h>	/* For global data structre */
#include <version.h>
#include <config.h>
#include <net.h>
#include <devices.h>
//#include <asm/jaguar.h>
#include <asm/eagle.h>

#include <asm/vixs_usbd.h>

const char version_string[] =
	U_BOOT_VERSION" (" __DATE__ " - " __TIME__ ")";
	
static int display_banner (void)
{
	printf ("\n\n%s\n\n", version_string);
	
	/*Fix me --  print in little endian */
	printf ("U-Boot code: %x -> %x  BSS: -> %x\n",
		ntohl(_arcboot_start), ntohl(_bss_start), ntohl(_bss_end));
	return (0);
}


typedef int (init_fnc_t) (void);

int board_init(void);

init_fnc_t *init_sequence[] = {
	serial_init,            /* serial communications setup */
	cpu_init,		/* cpu specific initialisations*/
	board_init,
	//serial_init,            /* serial communications setup */
	//env_init,		/* intialise environment */
	console_init_f,         /* stage 1 init of console */
	display_banner,         /* say that we are here */
	NULL,
};


/*
 *    Begin and End of memory area for malloc(), and current "brk"
 */
static ulong mem_malloc_start = 0;
static ulong mem_malloc_end = 0;
static ulong mem_malloc_brk = 0;

static void mem_malloc_init (ulong dest_addr)
{
	mem_malloc_start = dest_addr;

	//printf("mem_malloc_start = %x\n", mem_malloc_start);
	
	mem_malloc_end = dest_addr + CFG_MALLOC_LEN;
	mem_malloc_brk = mem_malloc_start;
	memset ((void *) mem_malloc_start, 0xff, mem_malloc_end - mem_malloc_start);
}

void *sbrk (ptrdiff_t increment)
{
	ulong old = mem_malloc_brk;
	ulong new = old + increment;
	
	if ((new < mem_malloc_start) || (new > mem_malloc_end)) {
		return (NULL);
	}
	mem_malloc_brk = new;
	return ((void *) old);
}
static void display_flash_config (ulong size)
{
    puts ("Flash: ");
	print_size (size, "\n");
}


extern int arc_console_baud ;
gd_t *global_data;

#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))


static  int chartodigital(unsigned char * pchar)
{
	unsigned char sz = *pchar;
	
	if( (sz >= 0x30) && (sz <= 0x39))	 //0-9
	{
		sz -= 0x30;
	}
	else if(sz >= 0x61 && (sz <= 0x66)) //a-f
	{
		sz = sz - 0x61 + 0xa;
	}
	else if(sz >= 0x41 && (sz <= 0x46)) //A-F
	{
		sz = sz - 0x41 + 0xa;
	}
	else
		return -1;

	return sz;
}

static int atoi(unsigned char * pstring)
{
    unsigned int strLen, val = 0, i;
    unsigned char *ptr = pstring;

    strLen = strlen(pstring);

    for(i = 0; i < strLen; i++)
    {
        val = val*10 + chartodigital(ptr);
        ptr++;
    }

    return val;
}

/* xcode_monitor() in board/aa3/serial.c will monitor board status based on board id.
  * Here is the place to set the initial status if required.
  */
static void init_xcode_monitor(void)
{
	uint32	temp;

	temp = readl(MMREG_BASE + CG_DUMMY_REG);
	if((temp==0x2301) || (temp==0x2304))
	{
		/* set GPIO_E_CTRL to GPIO mode */
		temp = readl(MMREG_BASE + GPIO_E_CTRL);
		temp |= 0x80000000;
		writel(temp, MMREG_BASE + GPIO_E_CTRL);
		/* set low */
		temp = readl(MMREG_BASE + GPIO_E_OUT);
		temp &= 0xfffffffe;
		writel(temp, MMREG_BASE + GPIO_E_OUT);
		/* set GPIO to output mode */
		temp = readl(MMREG_BASE + GPIO_E_OE);
		temp |= 0x1;
		writel(temp, MMREG_BASE + GPIO_E_OE);
	}
}

void start_arcboot(void)
{
	DECLARE_GLOBAL_DATA_PTR;
	static gd_t gd_data;
	static bd_t bd_data;
	unsigned long size;
	unsigned long boardid;
	unsigned char *s;
	unsigned long reg, data;
	unsigned long arcclk, arcbaudrate;
	init_fnc_t **init_fnc_ptr;
	gd = global_data = &gd_data;	

	//put the PROC0 on reset
	reg = readl(MMREG_BASE + CG_RESET_REG);
	reg |= PROC2_RESET_MASK;
	writel(reg, MMREG_BASE + CG_RESET_REG);

	memset (gd, 0, sizeof (gd_t));
	gd->bd = &bd_data;
	memset (gd->bd, 0, sizeof (bd_t));

	for (init_fnc_ptr = init_sequence;*init_fnc_ptr;++init_fnc_ptr) {
		if ((*init_fnc_ptr)() != 0) {
			hang();
		}		
	}

	/* Setup malloc area */
	printf(" TEXT_BASE = 0x%x, CFG_UBOOT_SIZE = 0x%x\n",
	       TEXT_BASE, CFG_UBOOT_SIZE);
	//mem_malloc_init(TEXT_BASE - CFG_MALLOC_LEN);
	mem_malloc_init(TEXT_BASE + CFG_UBOOT_SIZE);
    #if defined(CFG_ENV_IS_IN_FLASH)
	size = flash_init();
    #elif defined(CFG_ENV_IS_IN_NAND)
	nand_probe(0);
    #else
       #error "Where is my uboot environment??"
    #endif
    
	env_init();
	env_relocate();

	display_flash_config(size);
	/* Test serial port */
#if 0	
	int i = cpu_init();
	i = serial_init();
	unsigned char c;
	for (;;) {
		serial_putc('s');	
		c=serial_getc(); 
		serial_putc(c);
		serial_putc('\n');
	}	
#endif	
	/* Test the udelay working	*/
/*	printf("\nwelcome to u-boot-1.1.3.");
	for(i=0;i <= 10;i++)
	{
		udelay(500000);
		printf(".");
	}
	*/
	
	/*
	 * Read board id env and setup boardid scratch register
	 */
	if(!getenv("Board_ID"))
	{
		printf("Board_ID: not set in cfg env. Set boardid to 0x00F6\n");
		boardid = 0x00F6;
	}
	else 
	{
		s = getenv("Board_ID");
		boardid = simple_strtoul(s,NULL,16);
	}

	// write boardid to scratch register for driver/fw/applications
	printf("Board_ID: 0x%08x\n", boardid);
	writel(boardid, MMREG_BASE + CG_DUMMY_REG);

#ifdef CFG_ETH_ENABLED
	/*IP Address */
	gd->bd->bi_ip_addr = getenv_IPaddr ("ipaddr");

	/* MAC Address */
	{
	int i;
	ulong reg;
	char *s, *e;
	uchar tmp[64];

	i = getenv_r ("ethaddr", tmp, sizeof (tmp));

	s = (i > 0) ? tmp : NULL;
	for (reg = 0; reg < 6; ++reg) {
		gd->bd->bi_enetaddr[reg] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}
	}
#endif
	devices_init ();        /* get the devices list going. */

	jumptable_init ();
	console_init_r ();

#ifdef CFG_ETH_ENABLED

#if (CONFIG_COMMANDS & CFG_CMD_NET)  
    //0-7 : dedicated GPIO0-7 //8-11: GPIO SCIO0-3
    {
	unsigned int pin_number = 0;
	unsigned int tmpReg = 0;

	if(getenv("eth0_reset_pin"))
	{
		pin_number = atoi(getenv("eth0_reset_pin"));
		printf("Use GPIO %x to reset eth0\n",pin_number);

		if(pin_number <= 7)
		{
			tmpReg = readl(MMREG_BASE + GPIO_DEDICATED_OUTEN)|(GPIO_DEDICATED_OUTEN0_MASK<<pin_number);
			writel(tmpReg, MMREG_BASE + GPIO_DEDICATED_OUTEN);
			tmpReg = readl(MMREG_BASE + GPIO_DEDICATED_OUT)&(~(GPIO_DEDICATED_OUT0_MASK<<pin_number));
			writel(tmpReg, MMREG_BASE + GPIO_DEDICATED_OUT);
			udelay(100000);
			tmpReg |= (GPIO_DEDICATED_OUT0_MASK<<pin_number);
			writel(tmpReg, MMREG_BASE + GPIO_DEDICATED_OUT);
		}
		else if(pin_number <= 11)
		{
			// for SCIO[0], this is GPIO_I[13]
			reg = MMREG_BASE + GPIO_I_OE;
			tmpReg = (0x2000<<(pin_number-8))|readl(reg);
			writel(tmpReg, reg); 
			// goes low
			reg = MMREG_BASE + GPIO_I_OUT;
			tmpReg = readl(reg)&(~(0x2000<<(pin_number-8)));
			writel(tmpReg, reg);
			// wait for about 100ms
			udelay(100000);
			// goes high
			tmpReg |= (0x2000<<(pin_number-8));
			writel(tmpReg, reg);        
		}
		else
		{
			printf("Invalid parameter for eth0_reset_pin\n");
		}
	}
    }

#ifdef CONFIG_NET_MULTI
    eth_initialize(gd->bd);
#endif
//	printf("going eth_init %p\n", eth_init);
    eth_init(gd->bd);
#endif //#if (CONFIG_COMMANDS & CFG_CMD_NET)
#endif //#ifdef CFG_ETH_ENABLED

#ifdef CFG_SATA_ENABLED
        scsi_init();
#endif

	init_xcode_monitor();
	for(;;) {
		main_loop();		
	}	
}

int calculate_proc_clock( void)
{
      u64 proc3clk_sel, div_int, div_frac;
      u32 proc3_clk_speed;
	proc3clk_sel = (readl(MMREG_BASE+CG_CLK_SRC_SEL6) & PROC3CLK_SRC_SEL_MASK) >> PROC3CLK_SRC_SEL_SHIFT;
	switch( proc3clk_sel >>4){
	case 0: //pll5
		div_int = (readl(MMREG_BASE+CG_PLL5_FREQ) & CG_PLL5_FREQ_FB_DIV_INT_MASK) >> CG_PLL5_FREQ_FB_DIV_INT_SHIFT;
              div_frac = (readl(MMREG_BASE+CG_PLL5_FREQ) & CG_PLL5_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL5_FREQ_FB_DIV_FRAC_SHIFT;
		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27;
		break;
	case 4: //pll1
		div_int = (readl(MMREG_BASE+CG_PLL1_FREQ) & CG_PLL1_FREQ_FB_DIV_INT_MASK) >> CG_PLL1_FREQ_FB_DIV_INT_SHIFT;
              div_frac = (readl(MMREG_BASE+CG_PLL1_FREQ) & CG_PLL1_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL1_FREQ_FB_DIV_FRAC_SHIFT;
		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27;
		break;
	case 8: //pll4
		div_int = (readl(MMREG_BASE+CG_PLL4_FREQ) & CG_PLL4_FREQ_FB_DIV_INT_MASK) >> CG_PLL4_FREQ_FB_DIV_INT_SHIFT;
              div_frac = (readl(MMREG_BASE+CG_PLL4_FREQ) & CG_PLL4_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL4_FREQ_FB_DIV_FRAC_SHIFT;
		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27;
		break;
        default:
            switch(proc3clk_sel)
             {
        	case 0xc0: //pll2
        		div_int = (readl(MMREG_BASE+CG_PLL2_FREQ) & CG_PLL2_FREQ_FB_DIV_INT_MASK) >> CG_PLL2_FREQ_FB_DIV_INT_SHIFT;
                      div_frac = (readl(MMREG_BASE+CG_PLL2_FREQ) & CG_PLL2_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL2_FREQ_FB_DIV_FRAC_SHIFT;
        		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27;
        		break;
        	case 0xc4://pll6/2
        		div_int = (readl(MMREG_BASE+CG_PLL6_FREQ) & CG_PLL6_FREQ_FB_DIV_INT_MASK) >> CG_PLL6_FREQ_FB_DIV_INT_SHIFT;
                      div_frac = (readl(MMREG_BASE+CG_PLL6_FREQ) & CG_PLL6_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL6_FREQ_FB_DIV_FRAC_SHIFT;
        		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27/2;
        		break;
        	case 0xc8: //pll3/2
        		div_int = (readl(MMREG_BASE+CG_PLL3_FREQ) & CG_PLL3_FREQ_FB_DIV_INT_MASK) >> CG_PLL3_FREQ_FB_DIV_INT_SHIFT;
                      div_frac = (readl(MMREG_BASE+CG_PLL3_FREQ) & CG_PLL3_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL3_FREQ_FB_DIV_FRAC_SHIFT;
        		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27 /2;
        		break;
        	case 0xcc: //pll0/2
        		div_int = (readl(MMREG_BASE+CG_PLL0_FREQ) & CG_PLL0_FREQ_FB_DIV_INT_MASK) >> CG_PLL0_FREQ_FB_DIV_INT_SHIFT;
                      div_frac = (readl(MMREG_BASE+CG_PLL0_FREQ) & CG_PLL0_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL0_FREQ_FB_DIV_FRAC_SHIFT;
        		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27 /2;
        		break;
        	case 0xcd: //pll7/2
        		div_int = (readl(MMREG_BASE+CG_PLL7_FREQ) & CG_PLL7_FREQ_FB_DIV_INT_MASK) >> CG_PLL7_FREQ_FB_DIV_INT_SHIFT;
                      div_frac = (readl(MMREG_BASE+CG_PLL7_FREQ) & CG_PLL7_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL7_FREQ_FB_DIV_FRAC_SHIFT;
        		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27 /2;
        		break;
        	case 0xce: //pll0
        		div_int = (readl(MMREG_BASE+CG_PLL0_FREQ) & CG_PLL0_FREQ_FB_DIV_INT_MASK) >> CG_PLL0_FREQ_FB_DIV_INT_SHIFT;
                      div_frac = (readl(MMREG_BASE+CG_PLL0_FREQ) & CG_PLL0_FREQ_FB_DIV_FRAC_MASK) >> CG_PLL0_FREQ_FB_DIV_FRAC_SHIFT;
        		proc3_clk_speed = (u32)(div_int*1000000  + ((div_frac*1000000)>>20))* 27 ;
        		break;
        	case 0xcf: //LPCLK
        	default:
        		proc3_clk_speed = 30000000;
        		break;
        	}
       }
	return proc3_clk_speed;
}

void hang()
{
	printf("please reset....\n");
	for(;;);
}
