/*
 * (C) Copyright 2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <malloc.h>
#include <devices.h>
#include <version.h>
#include <asm/global_data.h>	/* For global data structre */
#include <net.h>
#include <environment.h>
#include <asm/viper.h>


#if ( ((CFG_ENV_ADDR+CFG_ENV_SIZE) < CFG_MONITOR_BASE) || \
      (CFG_ENV_ADDR >= (CFG_MONITOR_BASE + CFG_MONITOR_LEN)) ) || \
    defined(CFG_ENV_IS_IN_NVRAM)
#define	TOTAL_MALLOC_LEN	(CFG_MALLOC_LEN + CFG_ENV_SIZE)
#else
#define	TOTAL_MALLOC_LEN	CFG_MALLOC_LEN
#endif

#undef DEBUG

gd_t *global_data;

extern int timer_init(void);

extern int incaip_set_cpuclk(void);

extern ulong uboot_end_data;
extern ulong uboot_end;

ulong monitor_flash_len;

const char version_string[] =
	U_BOOT_VERSION" (" __DATE__ " - " __TIME__ ")";

static char *failed = "*** failed ***\n";

extern int stdlog;


#define writel(b,addr) (*(volatile unsigned int *) (addr) = (b))
#define readl(addr) (*(volatile unsigned int *) (addr))


static  int chartodigital(unsigned char * pchar)
{
	unsigned char sz = *pchar;
	
	if( (sz >= 0x30) && (sz <= 0x39))	 //0-9
	{
		sz -= 0x30;
	}
	else if(sz >= 0x61 && (sz <= 0x66)) //a-f
	{
		sz = sz - 0x61 + 0xa;
	}
	else if(sz >= 0x41 && (sz <= 0x46)) //A-F
	{
		sz = sz - 0x41 + 0xa;
	}
	else
		return -1;

	return sz;
}

static int atoi(unsigned char * pstring)
{
    unsigned int strLen, val = 0, i;
    unsigned char *ptr = pstring;

    strLen = strlen(pstring);

    for(i = 0; i < strLen; i++)
    {
        val = val*10 + chartodigital(ptr);
        ptr++;
    }

    return val;
}

/*
 * Begin and End of memory area for malloc(), and current "brk"
 */
static ulong mem_malloc_start;
static ulong mem_malloc_end;
static ulong mem_malloc_brk;


/*
 * The Malloc area is immediately below the monitor copy in DRAM
 */
static void mem_malloc_init (ulong dest_addr)
{
	mem_malloc_start = dest_addr;

	//printf("mem_malloc_start = %x, size=0x%x\n", mem_malloc_start, CFG_MALLOC_LEN);
	
	mem_malloc_end = dest_addr + CFG_MALLOC_LEN;
	mem_malloc_brk = mem_malloc_start;
	memset ((void *) mem_malloc_start, 0xff, mem_malloc_end - mem_malloc_start);
}

void *sbrk (ptrdiff_t increment)
{
	ulong old = mem_malloc_brk;
	ulong new = old + increment;

	if ((new < mem_malloc_start) || (new > mem_malloc_end)) {
		return (NULL);
	}
	mem_malloc_brk = new;
	return ((void *) old);
}


static int init_func_ram (void)
{
	DECLARE_GLOBAL_DATA_PTR;

#ifdef	CONFIG_BOARD_TYPES
	int board_type = gd->board_type;
#else
	int board_type = 0;	/* use dummy arg */
#endif
	puts ("DRAM:  ");

	if ((gd->ram_size = initdram (board_type)) > 0) {
		print_size (gd->ram_size, "\n");
		return (0);
	}
	puts (failed);
	return (1);
}

static int display_banner(void)
{
	printf ("\n\n%s\n\n", version_string);
	return (0);
}

static void display_flash_config(ulong size)
{
	puts ("Flash: ");
	print_size (size, "\n");
}


static int init_baudrate (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	uchar tmp[64];	/* long enough for environment variables */
	int i = getenv_r ("baudrate", tmp, sizeof (tmp));

	gd->baudrate = (i > 0)
			? (int) simple_strtoul (tmp, NULL, 10)
			: CONFIG_BAUDRATE;

	return (0);
}


/*
 * Breath some life into the board...
 *
 * The first part of initialization is running from Flash memory;
 * its main purpose is to initialize the RAM so that we
 * can relocate the monitor code to RAM.
 */

/*
 * All attempts to come up with a "common" initialization sequence
 * that works for all boards and architectures failed: some of the
 * requirements are just _too_ different. To get rid of the resulting
 * mess of board dependend #ifdef'ed code we now make the whole
 * initialization sequence configurable to the user.
 *
 * The requirements for any new initalization function is simple: it
 * receives a pointer to the "global data" structure as it's only
 * argument, and returns an integer return code, where 0 means
 * "continue" and != 0 means "fatal error, hang the system".
 */
typedef int (init_fnc_t) (void);

int r4k_cache_init(void);

init_fnc_t *init_sequence[] = {
    timer_init,
	serial_init,            /* serial communications setup */
	r4k_cache_init,		/* cpu specific initialisations*/
	//board_init,
	//env_init,		/* intialise environment */
	console_init_f,         /* stage 1 init of console */
	display_banner,         /* say that we are here */
	NULL,
};

static void setup_gpio (unsigned int board_id)
{
	unsigned int reg;
	reg &= 0xFF00;

	switch(board_id) {
	case 0x1c00:
		/* Dedicate GPIO */
		reg = readl(MMREG_BASE + GPIO_DEDICATED_OUTEN);
		writel(reg | 0x3F, MMREG_BASE + GPIO_DEDICATED_OUTEN);	
		reg = readl(MMREG_BASE + GPIO_DEDICATED_OUT);
		writel(reg | 0x37, MMREG_BASE + GPIO_DEDICATED_OUT);
		/* GPIO Group C */
		reg = readl(MMREG_BASE + GPIO_C_CTRL);
		writel(reg | 0xA0000000, MMREG_BASE + GPIO_C_CTRL);	
		reg = readl(MMREG_BASE + GPIO_C_OE);
		writel(reg | 0x2C, MMREG_BASE + GPIO_C_OE);	
		reg = readl(MMREG_BASE + GPIO_C_OUT);
		writel(reg | 0x08, MMREG_BASE + GPIO_C_OUT);
		/* GPIO Group D */
		reg = readl(MMREG_BASE + GPIO_D_CTRL);
		writel(reg | 0xF0000000, MMREG_BASE + GPIO_D_CTRL);	
		reg = readl(MMREG_BASE + GPIO_D_OE);
		writel(reg | 0x790, MMREG_BASE + GPIO_D_OE);	
		reg = readl(MMREG_BASE + GPIO_D_OUT);
		writel(reg | 0x590, MMREG_BASE + GPIO_D_OUT);
		/* GPIO Group I */
		reg = readl(MMREG_BASE + GPIO_I_CTRL);
		writel(reg | 0x0C000000, MMREG_BASE + GPIO_I_CTRL);	
		reg = readl(MMREG_BASE + GPIO_I_OE);
		writel(reg | 0x0c00, MMREG_BASE + GPIO_I_OE);
		reg = readl(MMREG_BASE + GPIO_I_OUT);
		writel(reg | 0x0800, MMREG_BASE + GPIO_I_OUT);
		/* GPIO Group J */
		reg = readl(MMREG_BASE + GPIO_J_CTRL);
		writel(reg | 0xC0000000, MMREG_BASE + GPIO_J_CTRL);
		reg = readl(MMREG_BASE + GPIO_J_OE);
		writel(reg | 0x07FEBA, MMREG_BASE + GPIO_J_OE);	
		reg = readl(MMREG_BASE + GPIO_J_OUT);
		writel(reg | 0x03FE28, MMREG_BASE + GPIO_J_OUT);	
		break;

	case 0x1700:
		reg = readl(MMREG_BASE + GPIO_G_CTRL);
        reg |= GPIO_G_CTRL_GPIO_MODE_SEL_MASK;
        writel(reg, MMREG_BASE + GPIO_G_CTRL);
        reg = readl(MMREG_BASE + GPIO_G_OE);
        reg |= 0x1 << 4;       /*GPIO_G_4*/
        writel(reg, MMREG_BASE + GPIO_G_OE);
        reg = readl(MMREG_BASE + GPIO_G_OUT);
        reg |= 0x1 << 4;
        writel(reg, MMREG_BASE + GPIO_G_OUT);
		break;

	case 0x3300:
		reg = readl(MMREG_BASE + GPIO_J_CTRL);
		reg |= GPIO_J_CTRL_GPIO_MODE_SEL_MASK;
		writel(reg, MMREG_BASE + GPIO_J_CTRL);
		reg = readl(MMREG_BASE + GPIO_J_OE);
		reg = GPIO_J_OE_GPIO_OE_MASK;
		reg &= ~(0x1 | (0x1 << 10));
		writel(reg, MMREG_BASE + GPIO_J_OE);
		writel(reg, MMREG_BASE + GPIO_J_OUT);

		reg = readl(MMREG_BASE + GPIO_G_CTRL);
		reg |= GPIO_G_CTRL_GPIO_MODE_SEL_MASK;
		writel(reg, MMREG_BASE + GPIO_G_CTRL);
		reg = readl(MMREG_BASE + GPIO_G_OE);
		reg = GPIO_G_OE_GPIO_OE_MASK;
		writel(reg, MMREG_BASE + GPIO_G_OE);
		writel(reg, MMREG_BASE + GPIO_G_OUT);
		break;

	default:
		break;
	}
}

#if 0
void board_init_f(ulong bootflag)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd_t gd_data, *id;
	bd_t *bd;
	init_fnc_t **init_fnc_ptr;
	ulong addr, addr_sp, len = (ulong)&uboot_end - CFG_MONITOR_BASE;
#ifdef CONFIG_PURPLE
	void copy_code (ulong);
#endif

	/* Pointer is writable since we allocated a register for it.
	 */
	gd = &gd_data;
	/* compiler optimization barrier needed for GCC >= 3.4 */
	__asm__ __volatile__("": : :"memory");

	memset ((void *)gd, 0, sizeof (gd_t));

	for (init_fnc_ptr = init_sequence; *init_fnc_ptr; ++init_fnc_ptr) {
		if ((*init_fnc_ptr)() != 0) {
			hang ();
		}
	}

	/*
	 * Now that we have DRAM mapped and working, we can
	 * relocate the code and continue running from DRAM.
	 */
	addr = CFG_SDRAM_BASE + gd->ram_size;

	/* We can reserve some RAM "on top" here.
	 */

	/* round down to next 4 kB limit.
	 */
	addr &= ~(4096 - 1);
	debug ("Top of RAM usable for U-Boot at: %08lx\n", addr);

	/* Reserve memory for U-Boot code, data & bss
	 * round down to next 16 kB limit
	 */
	addr -= len;
	addr &= ~(16 * 1024 - 1);

	debug ("Reserving %ldk for U-Boot at: %08lx\n", len >> 10, addr);

	 /* Reserve memory for malloc() arena.
	 */
	addr_sp = addr - TOTAL_MALLOC_LEN;
	debug ("Reserving %dk for malloc() at: %08lx\n",
			TOTAL_MALLOC_LEN >> 10, addr_sp);

	/*
	 * (permanently) allocate a Board Info struct
	 * and a permanent copy of the "global" data
	 */
	addr_sp -= sizeof(bd_t);
	bd = (bd_t *)addr_sp;
	gd->bd = bd;
	debug ("Reserving %d Bytes for Board Info at: %08lx\n",
			sizeof(bd_t), addr_sp);

	addr_sp -= sizeof(gd_t);
	id = (gd_t *)addr_sp;
	debug ("Reserving %d Bytes for Global Data at: %08lx\n",
			sizeof (gd_t), addr_sp);

 	/* Reserve memory for boot params.
	 */
	addr_sp -= CFG_BOOTPARAMS_LEN;
	bd->bi_boot_params = addr_sp;
	debug ("Reserving %dk for boot params() at: %08lx\n",
			CFG_BOOTPARAMS_LEN >> 10, addr_sp);

	/*
	 * Finally, we set up a new (bigger) stack.
	 *
	 * Leave some safety gap for SP, force alignment on 16 byte boundary
	 * Clear initial stack frame
	 */
	addr_sp -= 16;
	addr_sp &= ~0xF;
	*((ulong *) addr_sp) = 0;
	addr_sp -= 4;
	*((ulong *) addr_sp) = 0;
	addr_sp -= 4;
	debug ("Stack Pointer at: %08lx\n", addr_sp);

	/*
	 * Save local variables to board info struct
	 */
	bd->bi_memstart	= CFG_SDRAM_BASE;	/* start of  DRAM memory */
	bd->bi_memsize	= gd->ram_size;		/* size  of  DRAM memory in bytes */
	bd->bi_baudrate	= gd->baudrate;		/* Console Baudrate */

	memcpy (id, (void *)gd, sizeof (gd_t));

	/* On the purple board we copy the code in a special way
	 * in order to solve flash problems
	 */
#ifdef CONFIG_PURPLE
	copy_code(addr);
#endif

	relocate_code (addr_sp, id, addr);

	/* NOTREACHED - relocate_code() does not return */
}
#endif

/************************************************************************
 *
 * This is the next part if the initialization sequence: we are now
 * running from RAM and have a "normal" C environment, i. e. global
 * data can be written, BSS has been cleared, the stack size in not
 * that critical any more, etc.
 *
 ************************************************************************
 */

void board_init_r  (gd_t *id, ulong dest_addr)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd_t gd_data;
	bd_t *bd, bd_data;
	init_fnc_t **init_fnc_ptr;

	cmd_tbl_t *cmdtp;
	ulong size;
	extern void malloc_bin_reloc (void);
#ifndef CFG_ENV_IS_NOWHERE
	extern char * env_name_spec;
#endif
	char *s, *e;
	int i;
	unsigned long boardid, temp;

    arc_write_uncached_32(0x80000004, 0x1);

	/* Kernel used these two register the determine the boot from PCI or Soc
 	 * clear these two dummy registers as zero two avoid kernel boot fail
 	 */ 
	writel(0x0, MMREG_BASE + CG1_DUMMY_REG);
	writel(0x0, MMREG_BASE + RBM_DUMMY_REG);

	/* Pointer is writable since we allocated a register for it.
	 */
	gd = global_data = &gd_data;
	arc_write_uncached_32(0x80000004, &gd_data);
	memset ((void *)gd, 0, sizeof (gd_t));
	gd->bd = &bd_data;
	memset (gd->bd, 0, sizeof (bd_t));	
	/* compiler optimization barrier needed for GCC >= 3.4 */
	__asm__ __volatile__("": : :"memory");

    arc_write_uncached_32(0x80000004, 0x2);

    arc_write_uncached_32(0x80000004, 0x3);
	for (init_fnc_ptr = init_sequence; *init_fnc_ptr; ++init_fnc_ptr) {
		if ((*init_fnc_ptr)() != 0) {
			hang ();
		}
	}
	arc_write_uncached_32(0x80000004, 0x4);
    
	debug ("Now running in RAM - U-Boot at: %08lx\n", dest_addr);

	printf(" TEXT_BASE = 0x%x, CFG_UBOOT_SIZE = 0x%x\n",
	       TEXT_BASE, CFG_UBOOT_SIZE);

	mem_malloc_init(TEXT_BASE + CFG_UBOOT_SIZE);

	arc_write_uncached_32(0x80000004, 0x5);
	
	monitor_flash_len = (ulong)&uboot_end_data - dest_addr;

	/*
	 * We have to relocate the command table manually
	 */
 	for (cmdtp = &__u_boot_cmd_start; cmdtp !=  &__u_boot_cmd_end; cmdtp++) {
		ulong addr;

		addr = (ulong) (cmdtp->cmd) + gd->reloc_off;
#if 0
		printf ("Command \"%s\": 0x%08lx => 0x%08lx\n",
				cmdtp->name, (ulong) (cmdtp->cmd), addr);
#endif
		cmdtp->cmd =
			(int (*)(struct cmd_tbl_s *, int, int, char *[]))addr;

		addr = (ulong)(cmdtp->name) + gd->reloc_off;
		cmdtp->name = (char *)addr;

		if (cmdtp->usage) {
			addr = (ulong)(cmdtp->usage) + gd->reloc_off;
			cmdtp->usage = (char *)addr;
		}
#ifdef	CFG_LONGHELP
		if (cmdtp->help) {
			addr = (ulong)(cmdtp->help) + gd->reloc_off;
			cmdtp->help = (char *)addr;
		}
#endif
	}

	/* there are some other pointer constants we must deal with */
#ifndef CFG_ENV_IS_NOWHERE
	env_name_spec += gd->reloc_off;
#endif

	#if defined(CFG_ENV_IS_IN_FLASH) /* SPI */
	size = flash_init();
	#elif defined(CFG_ENV_IS_IN_NAND)
	nand_probe(0);
	#else
		//#error "Where is my uboot environment??"
	#endif
 
	env_init();
	env_relocate();

	s = getenv("stdlog");
	if(s && !strncmp(s, "null", 4))
		stdlog = 0;
	else
		stdlog = 1;

	display_flash_config(size);

	/*
	 * Read board id env and setup boardid scratch register
	 */
	if(!getenv("Board_ID"))
	{
		printf("Board_ID: not set in cfg env. Set boardid to 0x0001\n");
		boardid = 0x0001;
	}
	else 
	{
		s = getenv("Board_ID");
		boardid = simple_strtoul(s,NULL,16);
	}

	// write boardid to scratch register for driver/fw/applications
	printf("Board_ID: 0x%08x\n", boardid);
	writel(boardid, MMREG_BASE + CG_DUMMY_REG);

	/* Initialization GPIO based on Board ID */
	setup_gpio(boardid);

	bd = gd->bd;
	bd->bi_flashstart = CFG_FLASH_BASE;
	bd->bi_flashsize = size;
#if CFG_MONITOR_BASE == CFG_FLASH_BASE
	bd->bi_flashoffset = monitor_flash_len;	/* reserved area for U-Boot */
#else
	bd->bi_flashoffset = 0;
#endif

	malloc_bin_reloc();

	/* board MAC address */
	s = getenv ("ethaddr");
	for (i = 0; i < 6; ++i) {
		bd->bi_enetaddr[i] = s ? simple_strtoul (s, &e, 16) : 0;
		if (s)
			s = (*e) ? e + 1 : e;
	}

	/* IP Address */
	bd->bi_ip_addr = getenv_IPaddr("ipaddr");

#if defined(CONFIG_PCI)
	/*
	 * Do pci configuration
	 */
	pci_init();
#endif

/** leave this here (after malloc(), environment and PCI are working) **/
	/* Initialize devices */
	devices_init ();

	jumptable_init ();

	/* Initialize the console (after the relocation and devices init) */
	console_init_r ();
/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/

	/* Initialize from environment */
	if ((s = getenv ("loadaddr")) != NULL) {
		load_addr = simple_strtoul (s, NULL, 16);
	}
#if (CONFIG_COMMANDS & CFG_CMD_NET)
	if ((s = getenv ("bootfile")) != NULL) {
		copy_filename (BootFile, s, sizeof (BootFile));
	}
#endif	/* CFG_CMD_NET */

#if defined(CONFIG_MISC_INIT_R)
	/* miscellaneous platform dependent initialisations */
	misc_init_r ();
#endif


#if (CONFIG_COMMANDS & CFG_CMD_NET)  
    //0-7 : dedicated GPIO0-7 //8-11: GPIO SCIO0-3
    {
	unsigned int pin_number = 0;
	unsigned int tmpReg = 0, reg;

	if(getenv("eth0_reset_pin"))
	{
		pin_number = atoi(getenv("eth0_reset_pin"));
		printf("Use GPIO %x to reset eth0\n",pin_number);

		if(pin_number <= 7)
		{
			tmpReg = readl(MMREG_BASE + GPIO_DEDICATED_OUTEN)|(GPIO_DEDICATED_OUTEN0_MASK<<pin_number);
			writel(tmpReg, MMREG_BASE + GPIO_DEDICATED_OUTEN);
			tmpReg = readl(MMREG_BASE + GPIO_DEDICATED_OUT)&(~(GPIO_DEDICATED_OUT0_MASK<<pin_number));
			writel(tmpReg, MMREG_BASE + GPIO_DEDICATED_OUT);
			udelay(20000);
			tmpReg |= (GPIO_DEDICATED_OUT0_MASK<<pin_number);
			writel(tmpReg, MMREG_BASE + GPIO_DEDICATED_OUT);
		}
		else if(pin_number <= 11)
		{
			// for SCIO[0], this is GPIO_I[13]
			reg = MMREG_BASE + GPIO_I_OE;
			tmpReg = (0x2000<<(pin_number-8))|readl(reg);
			writel(tmpReg, reg); 
			// goes low
			reg = MMREG_BASE + GPIO_I_OUT;
			tmpReg = readl(reg)&(~(0x2000<<(pin_number-8)));
			writel(tmpReg, reg);
			// wait for about 20ms
			udelay(20000);
			// goes high
			tmpReg |= (0x2000<<(pin_number-8));
			writel(tmpReg, reg);        
		}
		else
		{
			printf("Invalid parameter for eth0_reset_pin\n");
		}
		udelay(10000);
	}
    }
#endif


    // always enable Ethernet block clk because we need pass MAC address to kernel using ETH_DUMMY register
    
    // When using Gigabit ETHERNET
	writel(readl((unsigned long *)(CG_CLK_STOP0 + MMREG_BASE)) & ~ETHR125CLK_STOP_MASK, (unsigned long *)(CG_CLK_STOP0 + MMREG_BASE));

    // When using ETHERNET0
	writel(readl((unsigned long *)(CG_CLK_STOP0 + MMREG_BASE)) & ~ETHR0CLK_STOP_MASK, (unsigned long *)(CG_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH0_MCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH0_SCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));
	
    // When using ETHERNET1
	writel(readl((unsigned long *)(CG_CLK_STOP0 + MMREG_BASE)) & ~ETHR1CLK_STOP_MASK, (unsigned long *)(CG_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH1_MCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE)) & ~ETH1_SCLK_STOP_MASK, (unsigned long *)(CG_BLK_CLK_STOP0 + MMREG_BASE));
	
/*----- This Network Initialization was relocated in Net.c 
#if (CONFIG_COMMANDS & CFG_CMD_NET) && defined(CONFIG_NET_MULTI)
	puts ("Net:   ");
	eth_initialize(gd->bd);
#endif
	eth_init(gd->bd);
*/	
	extern int EthInitState;
	EthInitState = 0;

#if (CONFIG_COMMANDS&CFG_CMD_I2C)
        i2c_init(100, 0x15);
#endif

#if (CONFIG_COMMANDS&CFG_CMD_MMC)
    puts("MMC:   ");
    mmc_initialize(bd);
#endif

	dma_init();

    /* The power_management_config0/1 env determine which block will be 
     * turned on after kernel boot. Here we just put this config env variable 
     * into dummy register, and we have a script to read this dummy
     * and load driver specifically after kernel boot.
     */
    //first enable dma clock before writing DMA_DUMMY_REG
	writel(readl((unsigned long *)(CG1_CLK_STOP0 + MMREG_BASE)) & ~DMACLK_STOP_MASK, (unsigned long *)(CG1_CLK_STOP0 + MMREG_BASE));
	writel(readl((unsigned long *)(CG1_BLK_CLK_STOP1 + MMREG_BASE)) & ~DMA_DMACLK_STOP_MASK, (unsigned long *)(CG1_BLK_CLK_STOP1 + MMREG_BASE));
	writel(readl((unsigned long *)(CG1_BLK_CLK_STOP0 + MMREG_BASE)) & ~DMA_MCLK_STOP_MASK, (unsigned long *)(CG1_BLK_CLK_STOP0 + MMREG_BASE));

	s = getenv ("power_management_config0");
    if(s)
        writel(simple_strtoul(s,NULL,16), (unsigned long *)(DMA_DUMMY_REG + MMREG_BASE)); 
    else //No such env, set the default value
        writel(0xFFFFFFFF, (unsigned long *)(DMA_DUMMY_REG + MMREG_BASE)); 

	s = getenv ("power_management_config1");
    if(s)
        writel(simple_strtoul(s,NULL,16), (unsigned long *)(SEQ_CH0_DUMMY_REG + MMREG_BASE)); 
    else
        writel(0xFFFFFFFF, (unsigned long *)(SEQ_CH0_DUMMY_REG + MMREG_BASE)); 


	/* main_loop() can return to retry autoboot, if so just run it again. */
	for (;;) {
		main_loop ();
	}

	/* NOTREACHED - no way out of command loop except booting */
}

void hang (void)
{
    arc_write_uncached_32(0x80000004, 0x99999999);
	puts ("### ERROR ### Please RESET the board ###\n");
	for (;;);
}
