/*
 * (C) Copyright 2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <zlib.h>
#include <asm/byteorder.h>
#include <asm/addrspace.h>
#include <asm/viper.h>

//#define	LINUX_MAX_ENVS		256
//#define	LINUX_MAX_ARGS		256
#define LINUX_MAX_ARG_ITEM  32
#define LINUX_MAX_ENV_ITEM  32

#ifdef CONFIG_SHOW_BOOT_PROGRESS
# include <status_led.h>
# define SHOW_BOOT_PROGRESS(arg)	show_boot_progress(arg)
#else
# define SHOW_BOOT_PROGRESS(arg)
#endif

#if (CONFIG_COMMANDS & CFG_CMD_I2C)
extern int i2c_read (uchar chip, uint addr, int alen, uchar * buffer, int len);
#endif

extern image_header_t header;           /* from cmd_bootm.c */

const char * kernel_env[] =
{
	"memsize",
	"usb0addr",
	"initrd_start",
	"initrd_size",
	"baudrate",       //"uart0",
	"icache",
	"dcache",
	"stdlog",
	"eth0_reset_pin",
	NULL,
};


extern int do_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);

static int	linux_argc;
static char **	linux_argv;

static int linux_envc;
static char * linux_envp;

//static void linux_params_init (ulong start, char * commandline);
//static void linux_env_set (char * env_name, char * env_val);

/*
 * To set bootargs:
 * Multiple boot mode, bootargs was determined by boot mode:
 * 	1. SDK boot 	  - default: /dev/sda2
 *	2. Andorid boot - default: /dev/sda4
 * for Nand Flash 
 * if( (current_active is defined) && (bootargs "root=" is not defined) )
 * {
 * 		if current_active ==0
 * 			root=/dev/mtdblock6
 *		else
 *			root=/dev/mtdblock7
 * }
 * Note: either  bootargs or androidargs  "root=" will over write nandflash bootargs
 */

static int setup_kernel_args(char ** argv,int max_item)
{
	int argc=0;
	char * arg=NULL;
	char * value=NULL, *opt;
	char *pcur, *pstr;	
	int cur;

	//assert(argc<max_item);
#ifdef CONFIG_MULTIBOOT
	unsigned long boot_mode = 0, multi_boot = 0;
	value = getenv("multiboot");
	if (NULL!=value)
		multi_boot = simple_strtoul(value, NULL, 16);
	else
		multi_boot = 0;
	
	if (multi_boot) {
#if (CONFIG_COMMANDS&CFG_CMD_I2C)
		i2c_read(0x50, 0x3000, 0x02, (uchar*)(&boot_mode), 0x04);
#endif
		boot_mode &= 0xff;
		debug("Multi-boot, Mode: %d\n",(int)boot_mode);
		switch (boot_mode) {
		case 0x00:
			value = getenv ("bootargs");
			break;
			
		case 0x01:
			value = getenv ("bootargs_1");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x02:
			value = getenv ("bootargs_2");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x03:
			value = getenv ("bootargs_3");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x04:
			value = getenv ("bootargs_4");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x05:
			value = getenv ("bootargs_5");
			if(NULL==value)
				value = getenv("bootargs");
			break;

		case 0x06:
			value = getenv ("bootargs_6");
			break;
			
		case 0x07:
			value = getenv ("bootargs_7");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x08:
			value = getenv ("bootargs_8");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x09:
			value = getenv ("bootargs_9");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x0a:
			value = getenv ("bootargs_10");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		case 0x0b:
			value = getenv ("bootargs_11");
			if(NULL==value)
				value = getenv("bootargs");
			break;
			
		default:
			value = getenv ("bootargs");			
			break;
		}
	} else {
		value = getenv("bootargs");
	}
    if(NULL!=value)
    {
    	argv[argc] = value;
        argc++;
    }
#else
#ifdef CFG_ENV_IS_IN_NAND
	/* Process current_active and rfsargs */
	pcur = getenv ("current_active");	
	value = getenv ("bootargs");
	opt = getenv("rfsargs");
	if (pcur != NULL) {
		if ((value != NULL) && (strncmp(value, "root=", 5) == 0)) {
			argv[argc]= value;
		} else {
			arg = (char*)malloc(32);
			strcpy (arg, "root=");
			if (*pcur == 0x30) {			
				strcat(arg, "/dev/mtdblock6");
			} else {
				strcat(arg, "/dev/mtdblock7");
			}

			if (opt == NULL) {
				strcat(arg, " rw");
			} else {
				if (strncmp(opt, "rw", 2) == 0)
					strcat(arg, " rw");
				else
					strcat(arg, " ro");
			}
			argv[argc] = arg;
		}
		argc++;
	} else {
		if ((value != NULL) && (strncmp(value, "root=", 5) == 0)) {
			argv[argc]=value;
		} else {
			arg = (char*)malloc(32);
			strcpy (arg, "root=");
			strcat(arg, "/dev/mtdblock6");
			
			if (opt == NULL) {
				strcat(arg, " ro");
			} else {
				if (strncmp(opt, "rw", 2) == 0)
					strcat(arg, " rw");
				else
					strcat(arg, " ro");
			}
			argv[argc] = arg;
		}
		argc++;
	}	
#else	
	value = getenv("bootargs");
    if(NULL!=value)
    {
    	argv[argc] = value;
        argc++;
    }
#endif
#endif

	//assert(argc<max_item);
	value = getenv("ethaddr");
	if(NULL!=value)
	{
		arg=(char *)malloc(strlen(value) + 20);
		strcpy(arg,"ethaddr=");
		strcat(arg,value);
		argv[argc] = arg;
		argc++;	
	}

	//assert(argc<max_item);
	value = getenv("eth1addr");
	if(NULL!=value)
	{
		arg=(char *)malloc(strlen(value) + 20);
		strcpy(arg,"eth1addr=");
		strcat(arg,value);
		argv[argc] = arg;
		argc++;	
	}

	value = getenv("sata_clkfreq");
	if(NULL!=value)
	{
		arg=(char *)malloc(strlen(value) + 20);
		strcpy(arg,"sata_clkfreq=");
		strcat(arg,value);
		argv[argc] = arg;
		argc++;	
	}

	value = getenv("sata_ssmode");
	if(NULL!=value)
	{
		arg=(char *)malloc(strlen(value) + 20);
		strcpy(arg,"sata_ssmode=");
		strcat(arg,value);
		argv[argc] = arg;
		argc++;	
	}

	value = getenv("sata0_limit_speed");
	if(NULL!=value)
	{
		arg=(char *)malloc(strlen(value) + 25);
		strcpy(arg,"sata0_limit_speed=");
		strcat(arg,value);
		argv[argc] = arg;
		argc++;	
	}

	value = getenv("sata1_limit_speed");
	if(NULL!=value)
	{
		arg=(char *)malloc(strlen(value) + 25);
		strcpy(arg,"sata1_limit_speed=");
		strcat(arg,value);
		argv[argc] = arg;
		argc++;	
	}

	return argc;
}

static int setup_kernel_envs(char ** envp,int max_item)
{
	int envc=0;
	char * env=NULL;
	char * value;
	//const char * p = kernel_env[0];
	int i;
	
	for(i=0;(envc < max_item) && (kernel_env[i] != NULL);i++)
	{
		value = getenv(kernel_env[i]);
		if(NULL!=value)
		{
			env=(char *)malloc(strlen(value) + strlen(kernel_env[i]) + 1);
			strcpy(env,kernel_env[i]);
			strcat(env,"=");
			strcat(env,value);
			envp[envc] = env;
			envc++;	
		}
	}

	return envc;
}
void do_bootm_linux (cmd_tbl_t * cmdtp, int flag, int argc, char *argv[],
		     ulong addr, ulong * len_ptr, int verify)
{
	DECLARE_GLOBAL_DATA_PTR;

	ulong len = 0, checksum;
	ulong initrd_start, initrd_end;
	ulong data;
	//void (*theKernel) (int, char **, char **, int *);
	void (*theKernel) (int,char *,int, char *);
	image_header_t *hdr = &header;
	char * *kargv;
	int kargc;
	int kenvc;
	char * *kenvp;
	char **p;

	theKernel = (void (*) (char *)) ntohl (hdr->ih_ep);

	kargv = (char **)malloc(sizeof(char *)*LINUX_MAX_ARG_ITEM);
	if(kargv)
	{
		kargc = setup_kernel_args(kargv,LINUX_MAX_ARG_ITEM);
	}
	else
	{
		printf("No memory for kernel arguments");
	}

	
	for(p = & kargv;(argc>=3) && (argc<LINUX_MAX_ARG_ITEM);kargc++,argc--)
	{
		p[kargc]=argv[argc-1];
	}

#ifdef DEBUG
	if(kargc)
	{
		int i;
		char **p = &kargv;
		printf("kernel boot arguments: ");
		for(i=0;i<kargc;i++)
			printf("%s ",p[i]);
		printf("\n");
	}
#endif

	//set envirnoments to kernel
	kenvp = (char * *)malloc(sizeof(char *)*LINUX_MAX_ENV_ITEM);
	if(kenvp)
	{
		kenvc = setup_kernel_envs(kenvp,LINUX_MAX_ENV_ITEM);
	}
	else
	{
		printf("No memory for kernel environments");
	}

	#ifdef DEBUG
	if(kenvc)
	{
		int i;
		char **p = &kenvp;
		printf("kernel boot environments: ");
		for(i=0;i<kenvc;i++)
			printf("%s ",p[i]);
		printf("\n");
	}
	#endif
		
	*((unsigned long *)(MMREG_BASE + ETH_DUMMY_REG)) = 0x55aabeef; //flag which indicates arguments is from u-boot;
	flush_dcache_all();
	theKernel (kargc,kargv,kenvc,kenvp);

}

