/*
 * (C) Copyright 2002
 * Detlev Zundel, DENX Software Engineering, dzu@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * BMP handling routines
 */

#include <common.h>
#include <command.h>
#include <zlib.h>
#include <image.h>
#include <linux/string.h>
#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/viper.h>

#if (CONFIG_COMMANDS & CFG_CMD_YUV)
#define	fb_writel(a , d)	writel(d , XC_SOC_PROC_MMREG_BASE + a)
#define	fb_readl(a)			readl(XC_SOC_PROC_MMREG_BASE + a)

typedef struct {
    unsigned int     reg;
    unsigned int     value;
    unsigned int     mask;
} comp_reg_entry_t;


extern int  gunzip (void *, int, unsigned char *, unsigned long *);

static int yuv_display (ulong addr);

static int video_display_yuv_1080p (unsigned long offset);
static int video_display_yuv_480p (unsigned long offset);
static int video_display_yuv_576p (unsigned long offset);

#define DISP_HDMI_REG_WRITE(reg, value) \
    fb_writel(DISP_HDMI_WRDATA,(value)&DISP_HDMI_WRDATA_WRDATA_MASK); \
    fb_writel(DISP_HDMI_ADDR, (DISP_HDMI_ADDR_WOP_MASK |((reg)&DISP_HDMI_ADDR_ADDR_MASK))); \
    REG_POLL(DISP_HDMI_STATUS, (DISP_HDMI_STATUS_BUSY_MASK | SFRRAM_BUSY_MASK | UPRAM_BUSY_MASK), \
   (DISP_HDMI_STATUS_BUSY_MASK | SFRRAM_BUSY_MASK | UPRAM_BUSY_MASK));

#define DISP_HDMI_REG_READ(reg, value) \
    fb_writel(DISP_HDMI_ADDR, (((reg)&DISP_HDMI_ADDR_ADDR_MASK))); \
	udelay(200);\
	value = fb_readl(DISP_HDMI_RDDATA);

#define REG_POLL(reg, bit_mask, value) \
timeout = 1000000; \
while (timeout --) \
{ \
    if ((fb_readl(reg) & bit_mask) == value) \
        break; \
}


/* Data table for 480p component VENC initialization ... */
static const comp_reg_entry_t comp480p60 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30250000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20430000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20960000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20058000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31930000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30338000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31DB0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30338000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x332D0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30540000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20588000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x317F8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31DB0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30520000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x21BC8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31DB0000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30338000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20150002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30540000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20588000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20600006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x74000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x395E7FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x6500F8, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0xB, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x268, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    //{DISP_HD_VENC0_YCMPN, 0x03C00093, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    //{DISP_HD_VENC0_CB2GAIN, 0x80450045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000C0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000181, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    //{DISP_HD_VENC0_DVE_VSYNC_START, 0x5a7ff003, 0xFFFFFFFF},
    //{DISP_HD_VENC0_DVE_VSYNC_END, 0x037ff006, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
};


/* Data table for 576p component VENC initialization ... */
static const comp_reg_entry_t comp576p50 [] = {
    {DISP_HD_VENC0_CONT_STATUS, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20040000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x30240000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000001, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20460000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000002, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0FFF8006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x206B8007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000004, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x207E8003, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000005, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20960000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000006, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x04000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000007, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x20040000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000008, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x31940000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000009, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x2FFF8000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x33298000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x24000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000000F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000010, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000011, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000012, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000013, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000014, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000015, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000016, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000017, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000018, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000019, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000001F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000020, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000021, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000022, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000023, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000024, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000025, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000026, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000027, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000028, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000029, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000002F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000031, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000032, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000033, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000034, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000035, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000036, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000037, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000038, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000039, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000003F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000040, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000041, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000042, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000043, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000044, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000045, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000046, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000047, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000048, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000049, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000004F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000050, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000051, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000052, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000053, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000054, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000055, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000056, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000057, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000058, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000059, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000005F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000060, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000061, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000063, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000064, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000065, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000066, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000067, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000068, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000069, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000006F, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000070, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000071, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000072, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000073, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000074, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000075, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000076, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000077, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000078, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000079, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000007F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_FREQ, 0x21F07C1F, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB_PHASE, 0x1040C0D0, 0xFFFFFFFF},
    {DISP_HD_VENC0_CMPST_GAIN, 0x395E7FF3, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLACK, 0x27, 0xFFFFFFFF},
    {DISP_HD_VENC0_BLANK, 0x920133, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC1, 0x0, 0xFFFFFFFF},
    {DISP_HD_VENC0_SYNC2, 0x268, 0xFFFFFFFF},
    //{DISP_HD_VENC0_YCMPN, 0x03C00093, 0xFFFFFFFF},
    {DISP_HD_VENC0_YCMPN, 0x03C000D0, 0xFFFFFFFF},
    //{DISP_HD_VENC0_CB2GAIN, 0x80450045, 0xFFFFFFFF},
    {DISP_HD_VENC0_CB2GAIN, 0x80620062, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000000A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000100, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000101, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000102, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00000248, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000103, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000002E4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000104, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000020E0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000105, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000106, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002142, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000107, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002163, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000108, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021A1, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x00000109, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000021C5, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010A, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002202, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010B, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002227, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010C, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x00002328, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010D, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x000023A4, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010E, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRDATA, 0x0003FFE0, 0xFFFFFFFF},
    {DISP_HD_VENC0_WRADDR, 0x0000010F, 0xFFFFFFFF},
    {DISP_HD_VENC0_FIELDBNDRY, 0x106, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS1, 0x040a03fb, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FYCMPS3, 0x0000006b, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS0, 0x00000000, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS1, 0x02fef8f6, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618, 0xFFFFFFFF},
    {DISP_HD_VENC0_COEF_FUVCMPS3, 0x00000030, 0xFFFFFFFF},
    {DISP_HD_VENC0_CONT_STATUS, 0x3, 0xFFFFFFFF},
    {DISP_OUTPUT_MUX, 0x13, 0x13},
    {0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF}
}; 

void hdmi_reg_write( u32 addr, u32 data )
{
    unsigned long flags = 0;
    int           timeout = 0;

    udelay(300);
    /* make sure block is idle before changing registers */
    //REG_POLL( DISP_HDMI_STATUS, (DISP_HDMI_STATUS_BUSY_MASK | SFRRAM_BUSY_MASK | UPRAM_BUSY_MASK),
    //                            (DISP_HDMI_STATUS_BUSY_MASK | SFRRAM_BUSY_MASK | UPRAM_BUSY_MASK) );
    ///* update registers */
    DISP_HDMI_REG_WRITE( addr, data );
    udelay(300);
	return;
}
unsigned long hdmi_reg_read( u32 addr )
{
    unsigned long flags = 0;
    int           timeout = 0;
	unsigned long rvalue;
	udelay(300);
    /* make sure block is idle before changing registers */
    //REG_POLL( DISP_HDMI_STATUS, (DISP_HDMI_STATUS_BUSY_MASK | SFRRAM_BUSY_MASK | UPRAM_BUSY_MASK),
    //                            (DISP_HDMI_STATUS_BUSY_MASK | SFRRAM_BUSY_MASK | UPRAM_BUSY_MASK) );
    ///* update registers */
    DISP_HDMI_REG_READ( addr, rvalue);
    udelay(300);
    return rvalue;
}



void    prog_component_venc ( comp_reg_entry_t *pscript)
{
    int i   =   0;
    unsigned int reg, mask, value;
    unsigned int tmp_sft_reset, temp;
    
    
    reg     =   pscript[i].reg;
    mask    =   pscript[i].mask;
    value   =   pscript[i].value;
    //D_DEBUG("reg    =   0x%X\n", reg);
    //D_DEBUG("mask    =   0x%X\n", mask);
    //D_DEBUG("value    =   0x%X\n", value);
    
    
    tmp_sft_reset   =   fb_readl(DISP_SFT_RESET);
    /* Clear out the HD VENC reset and the OUTPUT_MUX reset ... */
    tmp_sft_reset&= (~(HD_VENC_SFT_RSTN_MASK | OUTPUT_MUX_SFT_RSTN_MASK));
    fb_writel(DISP_SFT_RESET, tmp_sft_reset);
    
    while (1)
    {
        /* 
        *   It is a good idea to put safeguards here to limit where in the register space
        *   we touch ...
        */
        
        if (reg != 0xFFFFFFFF)
        {
            temp    =   fb_readl(reg);
            //D_DEBUG("REG[0x%X] = 0x%X\n", reg, temp);
            //D_DEBUG("MASK = 0x%X\n", mask);
            temp    &=  (~mask);
            temp|=value;
            //D_DEBUG("REG[0x%X]_NEW = 0x%X\n", reg, temp);
            fb_writel(reg, temp);
            i++;
            reg     =   pscript[i].reg;
            mask    =   pscript[i].mask;
            value   =   pscript[i].value;
            //D_DEBUG("reg    =   0x%X\n", reg);
            //D_DEBUG("mask    =   0x%X\n", mask);
            //D_DEBUG("value    =   0x%X\n", value);
        }
        else
        {
            break;
        
        }
        
        
        
    }
    
    /* De-assert the reset and the VENC and OUTPUT_MUX */
    tmp_sft_reset|= (HD_VENC_SFT_RSTN_MASK | OUTPUT_MUX_SFT_RSTN_MASK);
    
    fb_writel(DISP_SFT_RESET, tmp_sft_reset);
    
    /* Power on Component DAC's ... */
    fb_writel(DISP_VDAC_SET0_CNTL, 0x80000015);
    //REG_WRITE(DISP_VDAC_SET1_CNTL, 0x80000080);
    fb_writel(DISP_VDAC01_CNTL, 0xA600A600);
		
    temp = fb_readl(DISP_VDAC23_CNTL);
		
    temp|= 0xA600; /* Enable the HD Pr DAC*/
		
    fb_writel(DISP_VDAC23_CNTL, temp);
    //REG_WRITE(DISP_VDAC45_CNTL, 0xA600A600);
    //REG_WRITE(DISP_VDAC_CABLE_CONNECT, 0xA6000000);
    fb_writel(DISP_VDAC_BIAS_CNTL, 0x10);
    
    return;
}


/*
 * Subroutine:  do_yuv
 *
 * Description: Handler for 'bmp' command..
 *
 * Inputs:	argv[1] contains the subcommand
 *
 * Return:      None
 *
 */
int do_yuv(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong addr = 0;

	switch (argc) {
	case 2:
		if(load_addr & 0x80000000)
			addr = load_addr;
		else
			printf("Usage:\n%s\n",cmdtp->usage);
		break;

	case 3:
		addr = simple_strtoul(argv[2], NULL, 16);
		break;

	default:
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	/* Allow for short names
	 * Adjust length if more sub-commands get added
	 */
	if (strncmp(argv[1],"display",1) == 0) {		
	    return (yuv_display(addr));
	} else {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}
}

U_BOOT_CMD(
	yuv,	3,	1,	do_yuv,
	"yuv     - manipulate YUV Video data\n",	
	"display <imageAddr> - display image source from imageAddr\n"
);

static inline void xcode_disp_1080p_setup (void)
{
	unsigned int tmp;

	tmp = fb_readl(DISP_HD_VID1_MEM_RCLI_CTRL);
	tmp &= ~(DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_MASK | \
				DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_MASK);
	tmp |= (0x40 << DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_SHIFT) | \
			(0x20 << DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_SHIFT);
	fb_writel(DISP_HD_VID1_MEM_RCLI_CTRL, tmp);

	tmp = fb_readl(DISP_HD_VID0_MEM_RCLI_CTRL);
	tmp &= ~(DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_MASK | \
				DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_MASK);
	tmp |= (0x40 << DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_SHIFT) | \
			(0x20 << DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_SHIFT);
	fb_writel(DISP_HD_VID0_MEM_RCLI_CTRL, tmp);

	tmp = fb_readl(DISP_HD_GRA1_FORMAT);
	tmp &= ~(DISP_HD_GRA1_FORMAT_MAX_REQ_SIZE_MASK);
	tmp |= (0x80 << DISP_HD_GRA1_FORMAT_MAX_REQ_SIZE_SHIFT);
	fb_writel(DISP_HD_GRA1_FORMAT, tmp);

	tmp = fb_readl(DISP_HD_GRA0_FORMAT);
	tmp &= ~(DISP_HD_GRA0_FORMAT_MAX_REQ_SIZE_MASK);
	tmp |= (0x20 << DISP_HD_GRA0_FORMAT_MAX_REQ_SIZE_SHIFT);
	fb_writel(DISP_HD_GRA0_FORMAT, tmp);

	tmp=fb_readl(DISP_HD_VENC0_6562_CNTRL);
	tmp &= ~(DISP_HD_VENC0_6562_CNTRL_VBI_OUT_VENC_MASK |\
			DISP_HD_VENC0_6562_CNTRL_VBI_ON_Y_MASK 		|\
			DISP_HD_VENC0_6562_CNTRL_VANC_OUT_VENC_MASK |\
			DISP_HD_VENC0_6562_CNTRL_VANC_ON_Y_MASK 	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_OUT_VENC_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_ON_Y_MASK		|\
			DISP_HD_VENC0_6562_CNTRL_VBI_OUT_656_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_OUT_656_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_VANC_OUT_656_MASK);

	tmp |= ((0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_ON_Y_SHIFT)		|\
			(0x0 << DISP_HD_VENC0_6562_CNTRL_VANC_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VANC_ON_Y_SHIFT) 		|\
			(0x0 << DISP_HD_VENC0_6562_CNTRL_HANC_OUT_VENC_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_HANC_ON_Y_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_HANC_OUT_656_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VANC_OUT_656_SHIFT));
	fb_writel(DISP_HD_VENC0_6562_CNTRL, tmp);

	tmp=fb_readl(DISP_HD_VENC0_DVE_CNTRL);	
	tmp &= ~(DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK 	|\
			DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK 		|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_VENC_MASK 	|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_ON_Y_MASK 		|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_VENC_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_ON_Y_MASK		|\
			DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_656_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_656_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_656_MASK);

	tmp |= ((0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_SHIFT)			|\
			(0x0 << DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VANC_ON_Y_SHIFT) 		|\
			(0x0 << DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_VENC_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_HANC_ON_Y_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_656_SHIFT));
	fb_writel(DISP_HD_VENC0_DVE_CNTRL, tmp);

	tmp=fb_readl(DISP_HD_COMP_CTRL);	
	fb_writel(DISP_HD_COMP_CTRL, tmp | DISP_HD_COMP_CTRL_COMP_EN_MASK);

	tmp=fb_readl(DISP_SD_VENC0_DVE_CNTRL);
	tmp &= ~(DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK 	|\
			DISP_SD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK 		|\
			DISP_SD_VENC0_DVE_CNTRL_VANC_OUT_VENC_MASK 	|\
			DISP_SD_VENC0_DVE_CNTRL_VANC_ON_Y_MASK 		|\
			DISP_SD_VENC0_DVE_CNTRL_HANC_OUT_VENC_MASK	|\
			DISP_SD_VENC0_DVE_CNTRL_HANC_ON_Y_MASK		|\
			DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_656_MASK	|\
			DISP_SD_VENC0_DVE_CNTRL_HANC_OUT_656_MASK	|\
			DISP_SD_VENC0_DVE_CNTRL_VANC_OUT_656_MASK);

	tmp |= ((0x1 << DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_SD_VENC0_DVE_CNTRL_VBI_ON_Y_SHIFT)			|\
			(0x0 << DISP_SD_VENC0_DVE_CNTRL_VANC_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_SD_VENC0_DVE_CNTRL_VANC_ON_Y_SHIFT) 		|\
			(0x0 << DISP_SD_VENC0_DVE_CNTRL_HANC_OUT_VENC_SHIFT)	|\
			(0x1 << DISP_SD_VENC0_DVE_CNTRL_HANC_ON_Y_SHIFT)		|\
			(0x1 << DISP_SD_VENC0_DVE_CNTRL_VBI_OUT_656_SHIFT)		|\
			(0x1 << DISP_SD_VENC0_DVE_CNTRL_HANC_OUT_656_SHIFT)		|\
			(0x1 << DISP_SD_VENC0_DVE_CNTRL_VANC_OUT_656_SHIFT));
	fb_writel(DISP_SD_VENC0_DVE_CNTRL, tmp);

	tmp=fb_readl(DISP_SD_COMP_CTRL);	
	fb_writel(DISP_SD_COMP_CTRL, tmp | DISP_SD_COMP_CTRL_COMP_EN_MASK);

	fb_writel(DISP_INT_STATUS, 0xffffffff);

	return;
}


static void xcode_disp_comp_480p_60hz (void)
{
    prog_component_venc(&comp480p60);
    return;
}

static void xcode_disp_comp_576p_50hz (void)
{
    prog_component_venc(&comp576p50);
    return;
}

static void xcode_disp_hdmi_480p_60hz (void)
{
	unsigned int	tmp;
	/* Perform reset of both HDCP/HDMI engines ... */
	hdmi_reg_write( 0x4002, 0xff);
	
	tmp	=	hdmi_reg_read(0x5001);
	
	/* Clear the LSB of the register (to perform reset) */
	tmp &= (~0x1);		
	hdmi_reg_write(0x5001, tmp);
	udelay(50000); 
	hdmi_reg_write( 0x3000 , 0xfe); 
	hdmi_reg_write( 0x4001 , 0x0); 
	hdmi_reg_write( 0x4000 , 0x1); 
	//hdmi_reg_write( 0x4001 , 0x40); 
	hdmi_reg_write( 0x5000 , 0x20); 
	hdmi_reg_write( 0x801 , 0x40); 
	hdmi_reg_write( 0x3000 , 0x80); 
	hdmi_reg_write( 0x3000 , 0xfe); 
	hdmi_reg_write( 0x3001 , 0x20); 
	hdmi_reg_write( 0x3001 , 0x0); 
	hdmi_reg_write( 0x10e0 , 0x10); 
	hdmi_reg_write( 0x803 , 0x0); 
	//hdmi_reg_write( 0x5009 , 0x50); 
	
	hdmi_reg_write( 0x1001 , 0xd0); 
	hdmi_reg_write( 0x1002 , 0x2); 
	hdmi_reg_write( 0x1003 , 0x8A); 
	hdmi_reg_write( 0x1004 , 0x0); 
	hdmi_reg_write( 0x1005 , 0xe0); 
	hdmi_reg_write( 0x1006 , 0x1); 
	hdmi_reg_write( 0x1007 , 0x2d); 
	hdmi_reg_write( 0x1008 , 0x10); 
	hdmi_reg_write( 0x1009 , 0x0); 
	hdmi_reg_write( 0x100a , 0x3e); 
	hdmi_reg_write( 0x100b , 0x0); 
	hdmi_reg_write( 0x100c , 0x9); 
	hdmi_reg_write( 0x100d , 0x6); 
    
    /* Registers are not used by H/W*/
	//hdmi_reg_write( 0x100e , 0x60); 
	//hdmi_reg_write( 0x100f , 0xea); 
	//hdmi_reg_write( 0x1010 , 0x0); 
    
    /* No need to initialize the CSC matrix */
	// hdmi_reg_write( 0x4102 , 0x0); 
	// hdmi_reg_write( 0x4102 , 0x20); 
	// hdmi_reg_write( 0x4103 , 0x0); 
	// hdmi_reg_write( 0x4104 , 0x6a); 
	// hdmi_reg_write( 0x4105 , 0x18); 
	// hdmi_reg_write( 0x4106 , 0x75); 
	// hdmi_reg_write( 0x4107 , 0x71); 
	// hdmi_reg_write( 0x4108 , 0x1); 
	// hdmi_reg_write( 0x4109 , 0x3); 
	// hdmi_reg_write( 0x410a , 0x20); 
	// hdmi_reg_write( 0x410b , 0x0); 
	// hdmi_reg_write( 0x410c , 0x2b); 
	// hdmi_reg_write( 0x410d , 0xe5); 
	// hdmi_reg_write( 0x410e , 0x0); 
	// hdmi_reg_write( 0x410f , 0x0); 
	// hdmi_reg_write( 0x4110 , 0x7e); 
	// hdmi_reg_write( 0x4111 , 0xa1); 
	// hdmi_reg_write( 0x4112 , 0x20); 
	// hdmi_reg_write( 0x4113 , 0x0); 
	// hdmi_reg_write( 0x4114 , 0x7d); 
	// hdmi_reg_write( 0x4115 , 0xac); 
	// hdmi_reg_write( 0x4116 , 0x36); 
	// hdmi_reg_write( 0x4117 , 0x61); 
	// hdmi_reg_write( 0x4118 , 0x7e); 
	// hdmi_reg_write( 0x4119 , 0x62); 
	hdmi_reg_write( 0x4004 , 0x0); 
	hdmi_reg_write( 0x1019 , 0x2); 
	hdmi_reg_write( 0x4101 , 0x41); 
	hdmi_reg_write( 0x802 , 0x0); 
	hdmi_reg_write( 0x804 , 0x46); 
	hdmi_reg_write( 0x200 , 0x9); 
	hdmi_reg_write( 0x1012 , 0x20); 
	hdmi_reg_write( 0x1013 , 0x1); 
	hdmi_reg_write( 0x1014 , 0xb); 
	hdmi_reg_write( 0x1015 , 0x16); 
	hdmi_reg_write( 0x1016 , 0x21); 
	hdmi_reg_write( 0x101a , 0x68);     /* Set to 0x58 for 4:3 and set to 0x48 for None */
	hdmi_reg_write( 0x101b , 0x0); 
	hdmi_reg_write( 0x101c , 0x3);     /* VIC for 576p 16:9, 576p 4:3 = 0x11 */ 
	hdmi_reg_write( 0x101d , 0x0); 
	hdmi_reg_write( 0x101e , 0x0); 
	hdmi_reg_write( 0x101f , 0x0); 
	hdmi_reg_write( 0x1020 , 0x0); 
	hdmi_reg_write( 0x1021 , 0x0); 
	hdmi_reg_write( 0x1022 , 0x0); 
	hdmi_reg_write( 0x1023 , 0x0); 
	hdmi_reg_write( 0x1024 , 0x0); 
	hdmi_reg_write( 0x1029 , 0xaa); 
	hdmi_reg_write( 0x1030 , 0x0); 
	hdmi_reg_write( 0x1031 , 0x0); 
	hdmi_reg_write( 0x1062 , 0xa); 
	hdmi_reg_write( 0x1073 , 0xf); 
	hdmi_reg_write( 0x1074 , 0x3); 
	hdmi_reg_write( 0x1075 , 0x1); 
	hdmi_reg_write( 0x1092 , 0x3); 
	hdmi_reg_write( 0x10b3 , 0x18); 
	hdmi_reg_write( 0x10b4 , 0x0); 
	hdmi_reg_write( 0x10b5 , 0x10); 
	hdmi_reg_write( 0x10b7 , 0xf); 
	hdmi_reg_write( 0x10b8 , 0x0); 
	hdmi_reg_write( 0x10b9 , 0x0); 
	hdmi_reg_write( 0x10ba , 0x0); 
	hdmi_reg_write( 0x10bb , 0x0); 
	hdmi_reg_write( 0x10bc , 0x0); 
	hdmi_reg_write( 0x10bd , 0x0); 
	hdmi_reg_write( 0x10bd , 0x0); 
	hdmi_reg_write( 0x10bf , 0x0); 
	hdmi_reg_write( 0x1101 , 0x0); 
	hdmi_reg_write( 0x1103 , 0x10); 
	hdmi_reg_write( 0x4100 , 0x0); 
	hdmi_reg_write( 0x201 , 0x0); 
	hdmi_reg_write( 0x202 , 0x0); 
	hdmi_reg_write( 0x203 , 0x0); 
	hdmi_reg_write( 0x204 , 0x0); 
	hdmi_reg_write( 0x205 , 0x0); 
	hdmi_reg_write( 0x206 , 0x0); 
	hdmi_reg_write( 0x207 , 0x0); 
	hdmi_reg_write( 0x1032 , 0x0); 
	hdmi_reg_write( 0x1033 , 0x0); 
	hdmi_reg_write( 0x1034 , 0x0); 
	hdmi_reg_write( 0x1035 , 0x0); 
	hdmi_reg_write( 0x1036 , 0x0); 
	hdmi_reg_write( 0x1037 , 0x0); 
	hdmi_reg_write( 0x1038 , 0x0); 
	hdmi_reg_write( 0x1039 , 0x0); 
	hdmi_reg_write( 0x103a , 0x0); 
	hdmi_reg_write( 0x103b , 0x0); 
	hdmi_reg_write( 0x103c , 0x0); 
	hdmi_reg_write( 0x103d , 0x0); 
	hdmi_reg_write( 0x103e , 0x0); 
	hdmi_reg_write( 0x103f , 0x0); 
	hdmi_reg_write( 0x1040 , 0x0); 
	hdmi_reg_write( 0x1041 , 0x0); 
	hdmi_reg_write( 0x1042 , 0x0); 
	hdmi_reg_write( 0x1043 , 0x0); 
	hdmi_reg_write( 0x1044 , 0x0); 
	hdmi_reg_write( 0x1045 , 0x0); 
	hdmi_reg_write( 0x1046 , 0x0); 
	hdmi_reg_write( 0x1047 , 0x0); 
	hdmi_reg_write( 0x1048 , 0x0); 
	hdmi_reg_write( 0x1049 , 0x0); 
	hdmi_reg_write( 0x1011 , 0xc); 
	hdmi_reg_write( 0x104a , 0x56); 
	hdmi_reg_write( 0x104b , 0x69); 
	hdmi_reg_write( 0x104c , 0x58); 
	hdmi_reg_write( 0x104d , 0x53); 
	hdmi_reg_write( 0x104e , 0x20); 
	hdmi_reg_write( 0x104f , 0x49); 
	hdmi_reg_write( 0x1050 , 0x6e); 
	hdmi_reg_write( 0x1051 , 0x63); 
	hdmi_reg_write( 0x1052 , 0x58); 
	hdmi_reg_write( 0x1053 , 0x63); 
	hdmi_reg_write( 0x1054 , 0x6f); 
	hdmi_reg_write( 0x1055 , 0x64); 
	hdmi_reg_write( 0x1056 , 0x65); 
	hdmi_reg_write( 0x1057 , 0x34); 
	hdmi_reg_write( 0x1058 , 0x20); 
	hdmi_reg_write( 0x1059 , 0x53); 
	hdmi_reg_write( 0x105a , 0x44); 
	hdmi_reg_write( 0x105b , 0x4b); 
	hdmi_reg_write( 0x105c , 0x20); 
	hdmi_reg_write( 0x105d , 0x44); 
	hdmi_reg_write( 0x105e , 0x45); 
	hdmi_reg_write( 0x105f , 0x4d); 
	hdmi_reg_write( 0x1060 , 0x4f); 
	hdmi_reg_write( 0x1061 , 0x20); 
	hdmi_reg_write( 0x1101 , 0x0); 
	hdmi_reg_write( 0x1093 , 0x47); 
	hdmi_reg_write( 0x1094 , 0xffffffa7); 
	hdmi_reg_write( 0x1095 , 0xffffff89); 
	hdmi_reg_write( 0x1096 , 0xffffffaa); 
	hdmi_reg_write( 0x1097 , 0x7); 
	hdmi_reg_write( 0x1098 , 0x0); 
	hdmi_reg_write( 0x1099 , 0xffffffff); 
	hdmi_reg_write( 0x109a , 0xffffff87); 
	hdmi_reg_write( 0x109b , 0xfffffff1); 
	hdmi_reg_write( 0x109c , 0x11); 
	hdmi_reg_write( 0x109d , 0xffffffff); 
	hdmi_reg_write( 0x109e , 0x5); 
	hdmi_reg_write( 0x109f , 0xffffffa5); 
	hdmi_reg_write( 0x10a0 , 0xffffffbc); 
	hdmi_reg_write( 0x10a1 , 0xffffffb5); 
	hdmi_reg_write( 0x10a2 , 0x7); 
	hdmi_reg_write( 0x10a3 , 0x54); 
	hdmi_reg_write( 0x10a4 , 0xfffffff7); 
	hdmi_reg_write( 0x10a5 , 0xffffffff); 
	hdmi_reg_write( 0x10a6 , 0x41); 
	hdmi_reg_write( 0x10a7 , 0x0); 
	hdmi_reg_write( 0x10a8 , 0xffffffdc); 
	hdmi_reg_write( 0x10a9 , 0xffffffc7); 
	hdmi_reg_write( 0x10aa , 0x79); 
	hdmi_reg_write( 0x10ab , 0x44); 
	hdmi_reg_write( 0x10ac , 0xfffffff1); 
	hdmi_reg_write( 0x10ad , 0xf); 
	hdmi_reg_write( 0x10ae , 0xffffffaa); 
	hdmi_reg_write( 0x10af , 0xffffffab); 
	hdmi_reg_write( 0x10b0 , 0x12); 
	hdmi_reg_write( 0x10b1 , 0xa); 
	hdmi_reg_write( 0x10b2 , 0xffffffff); 
	hdmi_reg_write( 0x10b6 , 0xf0); 
	hdmi_reg_write( 0x1011 , 0xc); 
	hdmi_reg_write( 0x4001 , 0x40); 
//	hdmi_reg_write( 0x5000 , 0x28); 

    /* This code is reading out the EDID, not necessary in our case ... */
#if 0    
	hdmi_reg_write( 0x7e0a , 0x0); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x10); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x11); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x12); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x13); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x14); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x15); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x16); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x17); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x18); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x19); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x20); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x21); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x22); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x23); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x24); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x25); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x26); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x27); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x28); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x29); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x30); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x31); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x32); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x33); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x34); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x35); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x36); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x37); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x38); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x39); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x40); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x41); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x42); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x43); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x44); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x45); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x46); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x47); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x48); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x49); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x50); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x51); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x52); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x53); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x54); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x55); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x56); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x57); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x58); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x59); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x60); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x61); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x62); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x63); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x64); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x65); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x66); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x67); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x68); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x69); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x70); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x71); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x72); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x73); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x74); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x75); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x76); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x77); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x78); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x79); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x80); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x81); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x82); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x83); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x84); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x85); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x86); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x87); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x88); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x89); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x90); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x91); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x92); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x93); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x94); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x95); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x96); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x97); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x98); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x99); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xaa); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xab); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xac); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xad); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xae); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xaf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xba); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbe); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xca); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xce); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xda); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xde); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xea); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xeb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xec); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xed); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xee); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xef); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfa); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfe); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xff); 
	hdmi_reg_write( 0x7e04 , 0x1); 
#endif    
	
    hdmi_reg_write(0x3100, 0x2F);
    hdmi_reg_write(0x3101, 0x10);
    hdmi_reg_write(0x3102, 0x03);
    hdmi_reg_write(0x3200, 0x00);
    hdmi_reg_write(0x3201, 0x10);
    hdmi_reg_write(0x3202, 0x00);
    hdmi_reg_write(0x3203, 0x93);
    hdmi_reg_write(0x3204, 0x69);
    hdmi_reg_write(0x3205, 0x00);
    hdmi_reg_write(0x3206, 0x00);
    hdmi_reg_write(0x3300, 0x80);
    hdmi_reg_write(0x3301, 0x10);
    hdmi_reg_write(0x3302, 0x03);
    hdmi_reg_write(0x3400, 0x00);
    hdmi_reg_write(0x3403, 0x07);
    hdmi_reg_write(0x3404, 0x00);
    
    
    
    
    hdmi_reg_write( 0x10b3 , 0x18); 
	hdmi_reg_write( 0x4004 , 0x0); 
//	hdmi_reg_write( 0x5000 , 0x20); 
	hdmi_reg_write( 0x1018 , 0x1); 
	hdmi_reg_write( 0x1200 , 0x0); 
	hdmi_reg_write( 0x3006 , 0xf1);
    hdmi_reg_write( 0x1000 , 0x10);
    udelay(200);    
    hdmi_reg_write( 0x4005,  0x01);     /* Pulls HDMI PHY out of reset ... */
    udelay(50000);
    hdmi_reg_write( 0x1000, 0x98);
    
    
    
	return;
}

static void xcode_disp_hdmi_576p_50hz (void)
{
	unsigned int tmp;
	
	hdmi_reg_write(0x4002, 0xFF);
	tmp	=	hdmi_reg_read(0x5001);
	tmp&=(~0x1);
	hdmi_reg_write(0x5001, tmp);
	udelay(50000);

    hdmi_reg_write( 0x3000 , 0xfe); 
	hdmi_reg_write( 0x4001 , 0x0); 
	hdmi_reg_write( 0x4000 , 0x1); 
	//hdmi_reg_write( 0x4001 , 0x40); 
	hdmi_reg_write( 0x5000 , 0x20); 
	hdmi_reg_write( 0x801 , 0x40); 
	hdmi_reg_write( 0x3000 , 0x80); 
	hdmi_reg_write( 0x3000 , 0xfe); 
	hdmi_reg_write( 0x3001 , 0x20); 
	hdmi_reg_write( 0x3001 , 0x0); 
	hdmi_reg_write( 0x10e0 , 0x10); 
	hdmi_reg_write( 0x803 , 0x0); 
	//hdmi_reg_write( 0x5009 , 0x50); 
	
	hdmi_reg_write( 0x1001 , 0xd0); 
	hdmi_reg_write( 0x1002 , 0x2); 
	hdmi_reg_write( 0x1003 , 0x90); 
	hdmi_reg_write( 0x1004 , 0x0); 
	hdmi_reg_write( 0x1005 , 0x40); 
	hdmi_reg_write( 0x1006 , 0x2); 
	hdmi_reg_write( 0x1007 , 0x31); 
	hdmi_reg_write( 0x1008 , 0x0C); 
	hdmi_reg_write( 0x1009 , 0x0); 
	hdmi_reg_write( 0x100a , 0x40); 
	hdmi_reg_write( 0x100b , 0x0); 
	hdmi_reg_write( 0x100c , 0x5); 
	hdmi_reg_write( 0x100d , 0x5); 
    
    /* Registers are not used by H/W*/
	//hdmi_reg_write( 0x100e , 0x60); 
	//hdmi_reg_write( 0x100f , 0xea); 
	//hdmi_reg_write( 0x1010 , 0x0); 
    
    /* No need to initialize the CSC matrix */
	// hdmi_reg_write( 0x4102 , 0x0); 
	// hdmi_reg_write( 0x4102 , 0x20); 
	// hdmi_reg_write( 0x4103 , 0x0); 
	// hdmi_reg_write( 0x4104 , 0x6a); 
	// hdmi_reg_write( 0x4105 , 0x18); 
	// hdmi_reg_write( 0x4106 , 0x75); 
	// hdmi_reg_write( 0x4107 , 0x71); 
	// hdmi_reg_write( 0x4108 , 0x1); 
	// hdmi_reg_write( 0x4109 , 0x3); 
	// hdmi_reg_write( 0x410a , 0x20); 
	// hdmi_reg_write( 0x410b , 0x0); 
	// hdmi_reg_write( 0x410c , 0x2b); 
	// hdmi_reg_write( 0x410d , 0xe5); 
	// hdmi_reg_write( 0x410e , 0x0); 
	// hdmi_reg_write( 0x410f , 0x0); 
	// hdmi_reg_write( 0x4110 , 0x7e); 
	// hdmi_reg_write( 0x4111 , 0xa1); 
	// hdmi_reg_write( 0x4112 , 0x20); 
	// hdmi_reg_write( 0x4113 , 0x0); 
	// hdmi_reg_write( 0x4114 , 0x7d); 
	// hdmi_reg_write( 0x4115 , 0xac); 
	// hdmi_reg_write( 0x4116 , 0x36); 
	// hdmi_reg_write( 0x4117 , 0x61); 
	// hdmi_reg_write( 0x4118 , 0x7e); 
	// hdmi_reg_write( 0x4119 , 0x62); 
	hdmi_reg_write( 0x4004 , 0x0); 
	hdmi_reg_write( 0x1019 , 0x2); 
	hdmi_reg_write( 0x4101 , 0x41); 
	hdmi_reg_write( 0x802 , 0x0); 
	hdmi_reg_write( 0x804 , 0x46); 
	hdmi_reg_write( 0x200 , 0x9); 
	hdmi_reg_write( 0x1012 , 0x20); 
	hdmi_reg_write( 0x1013 , 0x1); 
	hdmi_reg_write( 0x1014 , 0xb); 
	hdmi_reg_write( 0x1015 , 0x16); 
	hdmi_reg_write( 0x1016 , 0x21); 
	hdmi_reg_write( 0x101a , 0x68);     /* Set to 0x58 for 4:3 and set to 0x48 for None */
	hdmi_reg_write( 0x101b , 0x0); 
	hdmi_reg_write( 0x101c , 0x12);     /* VIC for 576p 16:9, 576p 4:3 = 0x11 */ 
	hdmi_reg_write( 0x101d , 0x0); 
	hdmi_reg_write( 0x101e , 0x0); 
	hdmi_reg_write( 0x101f , 0x0); 
	hdmi_reg_write( 0x1020 , 0x0); 
	hdmi_reg_write( 0x1021 , 0x0); 
	hdmi_reg_write( 0x1022 , 0x0); 
	hdmi_reg_write( 0x1023 , 0x0); 
	hdmi_reg_write( 0x1024 , 0x0); 
	hdmi_reg_write( 0x1029 , 0xaa); 
	hdmi_reg_write( 0x1030 , 0x0); 
	hdmi_reg_write( 0x1031 , 0x0); 
	hdmi_reg_write( 0x1062 , 0xa); 
	hdmi_reg_write( 0x1073 , 0xf); 
	hdmi_reg_write( 0x1074 , 0x3); 
	hdmi_reg_write( 0x1075 , 0x1); 
	hdmi_reg_write( 0x1092 , 0x3); 
	hdmi_reg_write( 0x10b3 , 0x18); 
	hdmi_reg_write( 0x10b4 , 0x0); 
	hdmi_reg_write( 0x10b5 , 0x10); 
	hdmi_reg_write( 0x10b7 , 0xf); 
	hdmi_reg_write( 0x10b8 , 0x0); 
	hdmi_reg_write( 0x10b9 , 0x0); 
	hdmi_reg_write( 0x10ba , 0x0); 
	hdmi_reg_write( 0x10bb , 0x0); 
	hdmi_reg_write( 0x10bc , 0x0); 
	hdmi_reg_write( 0x10bd , 0x0); 
	hdmi_reg_write( 0x10bd , 0x0); 
	hdmi_reg_write( 0x10bf , 0x0); 
	hdmi_reg_write( 0x1101 , 0x0); 
	hdmi_reg_write( 0x1103 , 0x10); 
	hdmi_reg_write( 0x4100 , 0x0); 
	hdmi_reg_write( 0x201 , 0x0); 
	hdmi_reg_write( 0x202 , 0x0); 
	hdmi_reg_write( 0x203 , 0x0); 
	hdmi_reg_write( 0x204 , 0x0); 
	hdmi_reg_write( 0x205 , 0x0); 
	hdmi_reg_write( 0x206 , 0x0); 
	hdmi_reg_write( 0x207 , 0x0); 
	hdmi_reg_write( 0x1032 , 0x0); 
	hdmi_reg_write( 0x1033 , 0x0); 
	hdmi_reg_write( 0x1034 , 0x0); 
	hdmi_reg_write( 0x1035 , 0x0); 
	hdmi_reg_write( 0x1036 , 0x0); 
	hdmi_reg_write( 0x1037 , 0x0); 
	hdmi_reg_write( 0x1038 , 0x0); 
	hdmi_reg_write( 0x1039 , 0x0); 
	hdmi_reg_write( 0x103a , 0x0); 
	hdmi_reg_write( 0x103b , 0x0); 
	hdmi_reg_write( 0x103c , 0x0); 
	hdmi_reg_write( 0x103d , 0x0); 
	hdmi_reg_write( 0x103e , 0x0); 
	hdmi_reg_write( 0x103f , 0x0); 
	hdmi_reg_write( 0x1040 , 0x0); 
	hdmi_reg_write( 0x1041 , 0x0); 
	hdmi_reg_write( 0x1042 , 0x0); 
	hdmi_reg_write( 0x1043 , 0x0); 
	hdmi_reg_write( 0x1044 , 0x0); 
	hdmi_reg_write( 0x1045 , 0x0); 
	hdmi_reg_write( 0x1046 , 0x0); 
	hdmi_reg_write( 0x1047 , 0x0); 
	hdmi_reg_write( 0x1048 , 0x0); 
	hdmi_reg_write( 0x1049 , 0x0); 
	hdmi_reg_write( 0x1011 , 0xc); 
	hdmi_reg_write( 0x104a , 0x56); 
	hdmi_reg_write( 0x104b , 0x69); 
	hdmi_reg_write( 0x104c , 0x58); 
	hdmi_reg_write( 0x104d , 0x53); 
	hdmi_reg_write( 0x104e , 0x20); 
	hdmi_reg_write( 0x104f , 0x49); 
	hdmi_reg_write( 0x1050 , 0x6e); 
	hdmi_reg_write( 0x1051 , 0x63); 
	hdmi_reg_write( 0x1052 , 0x58); 
	hdmi_reg_write( 0x1053 , 0x63); 
	hdmi_reg_write( 0x1054 , 0x6f); 
	hdmi_reg_write( 0x1055 , 0x64); 
	hdmi_reg_write( 0x1056 , 0x65); 
	hdmi_reg_write( 0x1057 , 0x34); 
	hdmi_reg_write( 0x1058 , 0x20); 
	hdmi_reg_write( 0x1059 , 0x53); 
	hdmi_reg_write( 0x105a , 0x44); 
	hdmi_reg_write( 0x105b , 0x4b); 
	hdmi_reg_write( 0x105c , 0x20); 
	hdmi_reg_write( 0x105d , 0x44); 
	hdmi_reg_write( 0x105e , 0x45); 
	hdmi_reg_write( 0x105f , 0x4d); 
	hdmi_reg_write( 0x1060 , 0x4f); 
	hdmi_reg_write( 0x1061 , 0x20); 
	hdmi_reg_write( 0x1101 , 0x0); 
	hdmi_reg_write( 0x1093 , 0x47); 
	hdmi_reg_write( 0x1094 , 0xffffffa7); 
	hdmi_reg_write( 0x1095 , 0xffffff89); 
	hdmi_reg_write( 0x1096 , 0xffffffaa); 
	hdmi_reg_write( 0x1097 , 0x7); 
	hdmi_reg_write( 0x1098 , 0x0); 
	hdmi_reg_write( 0x1099 , 0xffffffff); 
	hdmi_reg_write( 0x109a , 0xffffff87); 
	hdmi_reg_write( 0x109b , 0xfffffff1); 
	hdmi_reg_write( 0x109c , 0x11); 
	hdmi_reg_write( 0x109d , 0xffffffff); 
	hdmi_reg_write( 0x109e , 0x5); 
	hdmi_reg_write( 0x109f , 0xffffffa5); 
	hdmi_reg_write( 0x10a0 , 0xffffffbc); 
	hdmi_reg_write( 0x10a1 , 0xffffffb5); 
	hdmi_reg_write( 0x10a2 , 0x7); 
	hdmi_reg_write( 0x10a3 , 0x54); 
	hdmi_reg_write( 0x10a4 , 0xfffffff7); 
	hdmi_reg_write( 0x10a5 , 0xffffffff); 
	hdmi_reg_write( 0x10a6 , 0x41); 
	hdmi_reg_write( 0x10a7 , 0x0); 
	hdmi_reg_write( 0x10a8 , 0xffffffdc); 
	hdmi_reg_write( 0x10a9 , 0xffffffc7); 
	hdmi_reg_write( 0x10aa , 0x79); 
	hdmi_reg_write( 0x10ab , 0x44); 
	hdmi_reg_write( 0x10ac , 0xfffffff1); 
	hdmi_reg_write( 0x10ad , 0xf); 
	hdmi_reg_write( 0x10ae , 0xffffffaa); 
	hdmi_reg_write( 0x10af , 0xffffffab); 
	hdmi_reg_write( 0x10b0 , 0x12); 
	hdmi_reg_write( 0x10b1 , 0xa); 
	hdmi_reg_write( 0x10b2 , 0xffffffff); 
	hdmi_reg_write( 0x10b6 , 0xf0); 
	hdmi_reg_write( 0x1011 , 0xc); 
	hdmi_reg_write( 0x4001 , 0x40); 
	//hdmi_reg_write( 0x5000 , 0x28); 

    /* This code is reading out the EDID, not necessary in our case ... */
#if 0    
	hdmi_reg_write( 0x7e0a , 0x0); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x10); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x11); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x12); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x13); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x14); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x15); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x16); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x17); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x18); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x19); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x1f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x20); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x21); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x22); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x23); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x24); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x25); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x26); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x27); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x28); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x29); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x2f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x30); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x31); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x32); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x33); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x34); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x35); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x36); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x37); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x38); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x39); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x3f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x40); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x41); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x42); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x43); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x44); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x45); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x46); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x47); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x48); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x49); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x4f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x50); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x51); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x52); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x53); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x54); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x55); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x56); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x57); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x58); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x59); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x5f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x60); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x61); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x62); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x63); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x64); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x65); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x66); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x67); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x68); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x69); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x6f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x70); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x71); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x72); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x73); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x74); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x75); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x76); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x77); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x78); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x79); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x7f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x80); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x81); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x82); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x83); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x84); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x85); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x86); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x87); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x88); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x89); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x8f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x90); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x91); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x92); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x93); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x94); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x95); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x96); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x97); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x98); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x99); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9a); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9b); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9c); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9d); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9e); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0x9f); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xa9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xaa); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xab); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xac); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xad); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xae); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xaf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xb9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xba); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbe); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xbf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xc9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xca); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xce); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xcf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xd9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xda); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xde); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xdf); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xe9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xea); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xeb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xec); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xed); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xee); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xef); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf0); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf1); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf2); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf3); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf4); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf5); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf6); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf7); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf8); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xf9); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfa); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfb); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfc); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfd); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xfe); 
	hdmi_reg_write( 0x7e04 , 0x1); 
	hdmi_reg_write( 0x7e00 , 0x50); 
	hdmi_reg_write( 0x7e01 , 0xff); 
	hdmi_reg_write( 0x7e04 , 0x1); 
#endif    
	
    hdmi_reg_write(0x3100, 0x2F);
    hdmi_reg_write(0x3101, 0x10);
    hdmi_reg_write(0x3102, 0x03);
    hdmi_reg_write(0x3200, 0x00);
    hdmi_reg_write(0x3201, 0x10);
    hdmi_reg_write(0x3202, 0x00);
    hdmi_reg_write(0x3203, 0x93);
    hdmi_reg_write(0x3204, 0x69);
    hdmi_reg_write(0x3205, 0x00);
    hdmi_reg_write(0x3206, 0x00);
    hdmi_reg_write(0x3300, 0x80);
    hdmi_reg_write(0x3301, 0x10);
    hdmi_reg_write(0x3302, 0x03);
    hdmi_reg_write(0x3400, 0x00);
    hdmi_reg_write(0x3403, 0x07);
    hdmi_reg_write(0x3404, 0x00);
    
    
    
    
    hdmi_reg_write( 0x10b3 , 0x18); 
	hdmi_reg_write( 0x4004 , 0x0); 
	//hdmi_reg_write( 0x5000 , 0x20); 
	hdmi_reg_write( 0x1018 , 0x1); 
	hdmi_reg_write( 0x1200 , 0x0); 
	hdmi_reg_write( 0x3006 , 0xf1);
    hdmi_reg_write( 0x1000 , 0x10);
    udelay(200);    
    hdmi_reg_write( 0x4005,  0x01);     /* Pulls HDMI PHY out of reset ... */
    udelay(50000);
    hdmi_reg_write( 0x1000, 0x98);
    
    
    
	return;

}


static inline void xcode_disp_480p_setup (void)
{
	unsigned int tmp;

	tmp = fb_readl(DISP_HD_VID1_MEM_RCLI_CTRL);
	tmp &= ~(DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_MASK | \
				DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_MASK);
	tmp |= (0x40 << DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_SHIFT) | \
			(0x20 << DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_SHIFT);
	fb_writel(DISP_HD_VID1_MEM_RCLI_CTRL, tmp);

	tmp = fb_readl(DISP_HD_VID0_MEM_RCLI_CTRL);
	tmp &= ~(DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_MASK | \
				DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_MASK);
	tmp |= (0x40 << DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_SHIFT) | \
			(0x20 << DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_SHIFT);
	fb_writel(DISP_HD_VID0_MEM_RCLI_CTRL, tmp);

	tmp = fb_readl(DISP_HD_GRA1_FORMAT);
	tmp &= ~(DISP_HD_GRA1_FORMAT_MAX_REQ_SIZE_MASK);
	tmp |= (0x80 << DISP_HD_GRA1_FORMAT_MAX_REQ_SIZE_SHIFT);
	fb_writel(DISP_HD_GRA1_FORMAT, tmp);

	tmp = fb_readl(DISP_HD_GRA0_FORMAT);
	tmp &= ~(DISP_HD_GRA0_FORMAT_MAX_REQ_SIZE_MASK);
	tmp |= (0x20 << DISP_HD_GRA0_FORMAT_MAX_REQ_SIZE_SHIFT);
	fb_writel(DISP_HD_GRA0_FORMAT, tmp);

	tmp=fb_readl(DISP_HD_VENC0_6562_CNTRL);
	tmp &= ~(DISP_HD_VENC0_6562_CNTRL_VBI_OUT_VENC_MASK |\
			DISP_HD_VENC0_6562_CNTRL_VBI_ON_Y_MASK 		|\
			DISP_HD_VENC0_6562_CNTRL_VANC_OUT_VENC_MASK |\
			DISP_HD_VENC0_6562_CNTRL_VANC_ON_Y_MASK 	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_OUT_VENC_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_ON_Y_MASK		|\
			DISP_HD_VENC0_6562_CNTRL_VBI_OUT_656_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_OUT_656_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_VANC_OUT_656_MASK);

	tmp |= ((0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_ON_Y_SHIFT)		|\
			(0x0 << DISP_HD_VENC0_6562_CNTRL_VANC_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VANC_ON_Y_SHIFT) 		|\
			(0x0 << DISP_HD_VENC0_6562_CNTRL_HANC_OUT_VENC_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_HANC_ON_Y_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_HANC_OUT_656_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VANC_OUT_656_SHIFT));
	fb_writel(DISP_HD_VENC0_6562_CNTRL, tmp);

	tmp=fb_readl(DISP_HD_VENC0_DVE_CNTRL);	
	tmp &= ~(DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK 	|\
			DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK 		|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_VENC_MASK 	|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_ON_Y_MASK 		|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_VENC_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_ON_Y_MASK		|\
			DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_656_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_656_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_656_MASK);

	tmp |= ((0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_SHIFT)			|\
			(0x0 << DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VANC_ON_Y_SHIFT) 		|\
			(0x0 << DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_VENC_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_HANC_ON_Y_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_656_SHIFT));
	fb_writel(DISP_HD_VENC0_DVE_CNTRL, tmp);

	tmp=fb_readl(DISP_HD_COMP_CTRL);	
	fb_writel(DISP_HD_COMP_CTRL, tmp | DISP_HD_COMP_CTRL_COMP_EN_MASK);

	fb_writel(DISP_INT_STATUS, 0xffffffff);

	return;
}



static inline void xcode_disp_576p_setup (void)
{
	unsigned int tmp;

	tmp = fb_readl(DISP_HD_VID1_MEM_RCLI_CTRL);
	tmp &= ~(DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_MASK | \
				DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_MASK);
	tmp |= (0x40 << DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_SHIFT) | \
			(0x20 << DISP_HD_VID1_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_SHIFT);
	fb_writel(DISP_HD_VID1_MEM_RCLI_CTRL, tmp);

	tmp = fb_readl(DISP_HD_VID0_MEM_RCLI_CTRL);
	tmp &= ~(DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_MASK | \
				DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_MASK);
	tmp |= (0x40 << DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_Y_SHIFT) | \
			(0x20 << DISP_HD_VID0_MEM_RCLI_CTRL_MAX_REQ_SIZE_C_SHIFT);
	fb_writel(DISP_HD_VID0_MEM_RCLI_CTRL, tmp);

	tmp = fb_readl(DISP_HD_GRA1_FORMAT);
	tmp &= ~(DISP_HD_GRA1_FORMAT_MAX_REQ_SIZE_MASK);
	tmp |= (0x80 << DISP_HD_GRA1_FORMAT_MAX_REQ_SIZE_SHIFT);
	fb_writel(DISP_HD_GRA1_FORMAT, tmp);

	tmp = fb_readl(DISP_HD_GRA0_FORMAT);
	tmp &= ~(DISP_HD_GRA0_FORMAT_MAX_REQ_SIZE_MASK);
	tmp |= (0x20 << DISP_HD_GRA0_FORMAT_MAX_REQ_SIZE_SHIFT);
	fb_writel(DISP_HD_GRA0_FORMAT, tmp);

	tmp=fb_readl(DISP_HD_VENC0_6562_CNTRL);
	tmp &= ~(DISP_HD_VENC0_6562_CNTRL_VBI_OUT_VENC_MASK |\
			DISP_HD_VENC0_6562_CNTRL_VBI_ON_Y_MASK 		|\
			DISP_HD_VENC0_6562_CNTRL_VANC_OUT_VENC_MASK |\
			DISP_HD_VENC0_6562_CNTRL_VANC_ON_Y_MASK 	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_OUT_VENC_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_ON_Y_MASK		|\
			DISP_HD_VENC0_6562_CNTRL_VBI_OUT_656_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_HANC_OUT_656_MASK	|\
			DISP_HD_VENC0_6562_CNTRL_VANC_OUT_656_MASK);

	tmp |= ((0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_ON_Y_SHIFT)		|\
			(0x0 << DISP_HD_VENC0_6562_CNTRL_VANC_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VANC_ON_Y_SHIFT) 		|\
			(0x0 << DISP_HD_VENC0_6562_CNTRL_HANC_OUT_VENC_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_HANC_ON_Y_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VBI_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_HANC_OUT_656_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_6562_CNTRL_VANC_OUT_656_SHIFT));
	fb_writel(DISP_HD_VENC0_6562_CNTRL, tmp);

	tmp=fb_readl(DISP_HD_VENC0_DVE_CNTRL);	
	tmp &= ~(DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_MASK 	|\
			DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_MASK 		|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_VENC_MASK 	|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_ON_Y_MASK 		|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_VENC_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_ON_Y_MASK		|\
			DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_656_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_656_MASK	|\
			DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_656_MASK);

	tmp |= ((0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_ON_Y_SHIFT)			|\
			(0x0 << DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_VENC_SHIFT) 	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VANC_ON_Y_SHIFT) 		|\
			(0x0 << DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_VENC_SHIFT)	|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_HANC_ON_Y_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VBI_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_HANC_OUT_656_SHIFT)		|\
			(0x1 << DISP_HD_VENC0_DVE_CNTRL_VANC_OUT_656_SHIFT));
	fb_writel(DISP_HD_VENC0_DVE_CNTRL, tmp);

	tmp=fb_readl(DISP_HD_COMP_CTRL);	
	fb_writel(DISP_HD_COMP_CTRL, tmp | DISP_HD_COMP_CTRL_COMP_EN_MASK);

	fb_writel(DISP_INT_STATUS, 0xffffffff);

	return;
}

static inline void xcode_disp_ablend_setup (void)
{
	// Enable advanced blending
	fb_writel(DISP_HD_COMP_ALPHA_MODE, 0x80800000);
	return;
}

static inline void xcode_disp_init_1080p(void)
{
	unsigned int tmp, val;	
	tmp = fb_readl(CG_RESET_REG);
	fb_writel(CG_RESET_REG , tmp | DISP_RESET_MASK);
	udelay(100);
	fb_writel(CG_RESET_REG , tmp &~ DISP_RESET_MASK);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0x0);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0xffff);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0xffff);
	
	fb_writel(DISP_OUTPUT_MUX , 0xd);

#if 0
	// Reference Clock Source Select for 1080P
	CG1_CLK_SRC_SEL7 \ VSCLK_SRC_SEL = 0x00
	CG1_CLK_SRC_SEL4 \ DAC0CLK_SRC_SEL = 0x00
	CG1_CLK_SRC_SEL5 \ DISP_656CLK_SRC_SEL = 0x40
	CG1_CLK_SRC_SEL5 \ PCLK_SRC_SEL = 0x40
	CG1_CLK_SRC_SEL5 \ VHDCLK_SRC_SEL = 0x40
#endif

	tmp = fb_readl(CG1_CLK_STOP0);
	val = (VSCLK_STOP_MASK | DISP_656CLK_STOP_MASK	\
			| VSDCLK_STOP_MASK | VHDCLK_STOP_MASK	\
			| DAC0CLK_STOP_MASK | DAC1CLK_STOP_MASK	\
			| PCLK_STOP_MASK);
	fb_writel(CG1_CLK_STOP0 , tmp & ~val);
	
	tmp = fb_readl(CG1_BLK_CLK_STOP0);
	fb_writel(CG1_BLK_CLK_STOP0 , tmp & ~DISPCLK_STOP_MASK);

	return;
}

static int video_display_yuv_1080p (unsigned long offset)
{
	unsigned int tmp, delay = 1;
	
	xcode_disp_init_1080p();
	
	printf("Dispaly 1920 x 1080 YUV image at %x\n", (unsigned)offset);
	fb_writel(DISP_HD_VID0_Y_ADR, 0x0);
	fb_writel(DISP_HIST_ADR, 0x0);
	fb_writel(DISP_HD_VID0_CB_ADR, 0xa8c000);
	fb_writel(DISP_HD_VID0_CR_ADR, 0xfd2000);
	fb_writel(DISP_HD_VID1_Y_ADR, 0x1779600);
	fb_writel(DISP_HD_VID1_CB_ADR, 0x2205600);
	fb_writel(DISP_HD_VID1_CR_ADR, 0x274b600);
	fb_writel(DISP_HD_GRA0_ADR, offset & 0x7FFFFFFF);
	fb_writel(DISP_HD_GRA1_ADR, 0x6c91600);
	fb_writel(DISP_HD_VBI0_ADR, 0x1518000);
	fb_writel(DISP_HD_HANC0_ADR, 0x156c600);
	fb_writel(DISP_SD_GRA0_ADR, 0xac91600);
	fb_writel(DISP_SD_VBI0_ADR, 0x1518000);
	fb_writel(DISP_SD_HANC0_ADR, 0x156c600);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x0);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x0);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x0);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30550001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20588001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x206b8001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20960001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x4000008);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x8);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31820000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x9);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xa);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x332f0000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xb);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xc);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xd);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xe);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xf);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30350000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x12);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31e20000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x13);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x14);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x15);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x16);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x17);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x18);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30350000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x19);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x332d0000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x20);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30550001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x21);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20588001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x22);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x23);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x206b8001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x24);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x25);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20960001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x26);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x4000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x27);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x28);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31820000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x29);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31e20000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x30);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30550001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x31);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x32);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x33);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20960001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x34);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c20001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x35);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31e20001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x36);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x37);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x38);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30350000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x39);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20150002);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x40);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30550000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x41);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20588000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x42);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600006);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x43);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x206b8007);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x44);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x207e8003);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x45);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x74000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x46);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x47);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x48);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x49);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x50);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x51);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x52);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x53);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x54);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x55);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x56);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x57);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x58);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x59);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x60);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x61);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x62);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x63);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x64);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x65);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x66);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x67);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x68);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x69);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x70);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x71);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x72);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x73);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x74);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x75);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x76);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x77);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x78);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x79);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x100);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x101);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x102);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x103);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x104);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x105);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x106);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x107);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x108);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x109);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x110);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x111);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x112);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x113);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x114);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x115);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x116);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x117);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x118);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x119);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11f);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x0);
	fb_writel(DISP_SFT_RESET, 0xef77);
	fb_writel(DISP_SFT_RESET, 0xef77);
	fb_writel(DISP_SFT_RESET, 0xffff);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x0);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x0);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30550001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20588001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x206b8001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20960001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x4000008);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x8);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x31820000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x9);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xa);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x332f0000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xb);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xc);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xd);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xe);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xf);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30350000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x12);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x31e20000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x13);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x14);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x15);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x16);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x17);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x18);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30350000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x19);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x332d0000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x20);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30550001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x21);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20588001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x22);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x23);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x206b8001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x24);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x25);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20960001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x26);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x4000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x27);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x28);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x31820000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x29);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x21c20000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x31e20000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x30);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30550001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x31);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x32);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x33);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20960001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x34);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x21c20001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x35);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x31e20001);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x36);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x37);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x38);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30350000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x39);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20150002);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x40);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30550000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x41);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20588000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x42);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20600006);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x43);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x206b8007);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x44);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x207e8003);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x45);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x74000000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x46);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x47);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x48);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x49);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x50);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x51);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x52);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x53);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x54);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x55);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x56);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x57);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x58);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x59);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x60);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x61);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x62);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x63);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x64);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x65);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x66);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x67);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x68);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x69);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x70);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x71);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x72);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x73);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x74);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x75);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x76);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x77);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x78);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x79);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x100);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x101);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x102);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x103);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x104);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x105);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x106);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x107);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x108);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x109);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x110);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x111);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x112);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x113);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x114);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x115);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x116);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x117);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x118);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x119);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11f);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x0);
	fb_writel(DISP_SFT_RESET, 0xef77);
	fb_writel(DISP_SFT_RESET, 0xef77);
	fb_writel(DISP_SFT_RESET, 0xffff);
	fb_writel(DISP_HD_GRA0_SIZE, 0x438780);
	fb_writel(DISP_HD_GRA0_SCALED_SIZE, 0x438780);
	fb_writel(DISP_HD_GRA0_VSCA_INIT, 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_START, 0x1);
	fb_writel(DISP_HD_GRA0_VSCA_INV_RATIO, 0xffc3519);
	fb_writel(DISP_HD_GRA0_HSCA_INV_RATIO, 0x10000000);
	fb_writel(DISP_HD_GRA0_BUF, 0x7e901e00);
	fb_writel(DISP_HD_GRA0_SAMPLES, 0x1e00);
	fb_writel(DISP_HD_GRA0_FORMAT, 0x2012007);
	fb_writel(DISP_HD_GRA0_FORMAT1, 0x1000006c);
	fb_writel(DISP_HD_RAM_ADDR, 0x1100000);
	fb_writel(DISP_HD_RAM_ADDR, 0x1110000);
	fb_writel(DISP_SD_GRA0_FORMAT, 0x20811087);
	fb_writel(DISP_SD_GRA0_SIZE, 0x1e02d0);
	fb_writel(DISP_SD_GRA0_SCALED_SIZE, 0x1e02d0);
	fb_writel(DISP_SD_GRA0_HSCA_INV_RATIO, 0x10000000);
	fb_writel(DISP_SD_GRA0_BUF, 0xa8c0b40);
	fb_writel(DISP_SD_GRA0_SAMPLES, 0xb40);
	fb_writel(DISP_SD_GRA0_FORMAT1, 0x1000018);
	fb_writel(DISP_SD_CC, 0x0);
	fb_writel(DISP_SD_OCC_C11, 0x10210839);
	fb_writel(DISP_SD_OCC_C13, 0x2000323);
	fb_writel(DISP_SD_OCC_C21, 0xf6b0fb44);
	fb_writel(DISP_SD_OCC_C23, 0x10000e0c);
	fb_writel(DISP_SD_OCC_C31, 0xf4390e0c);
	fb_writel(DISP_SD_OCC_C33, 0x1000fdba);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x0);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x0);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30548001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20588001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x206b8001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x208d0001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x4000008);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x8);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31818000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x9);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xa);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x332e8000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xb);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xc);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xd);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xe);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0xf);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30348000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x11);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x12);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31e18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x13);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x14);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x15);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x16);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x17);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x18);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30348000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x19);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x332e8000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x1f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x20);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30548001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x21);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20588001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x22);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x23);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x206b8001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x24);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x25);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20960001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x26);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x4000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x27);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x28);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31818000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x29);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31e18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x2f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x30);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30548000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x31);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x32);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20820001);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x33);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20960000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x34);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21c18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x35);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x31e18000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x36);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x37);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x38);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30348000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x39);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x24000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x3f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20148002);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x40);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x30548000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x41);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20588000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x42);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20600006);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x43);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x206b8007);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x44);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x207e8003);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x45);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x74000000);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x46);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x47);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x48);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x49);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x4f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x50);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x51);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x52);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x53);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x54);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x55);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x56);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x57);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x58);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x59);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x5f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x60);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x61);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x62);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x63);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x64);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x65);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x66);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x67);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x68);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x69);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x6f);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x70);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x71);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x72);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x73);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x74);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x75);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x76);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x77);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x78);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x79);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x7f);
	fb_writel(DISP_SD_VENC0_CB_FREQ, 0x21f07c1f);
	fb_writel(DISP_SD_VENC0_CB_PHASE, 0x1040bad0);
	fb_writel(DISP_SD_VENC0_CMPST_GAIN, 0x395e7ff3);
	fb_writel(DISP_SD_VENC0_BLACK, 0x27);
	fb_writel(DISP_SD_VENC0_BLANK, 0x3200f8);
	fb_writel(DISP_SD_VENC0_SYNC1, 0xb);
	fb_writel(DISP_SD_VENC0_SYNC2, 0x268);
	fb_writel(DISP_SD_VENC0_YCMPN, 0x3c00093);
	fb_writel(DISP_SD_VENC0_CB2GAIN, 0x80450045);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x62);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x100);
	fb_writel(DISP_SD_VENC0_WRDATA, 0xc1);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x101);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x122);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x102);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x2c4);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x103);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20c0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x104);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x20e6);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x105);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x2122);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x106);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x2143);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x107);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x2181);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x108);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21a5);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x109);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x21e2);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10a);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x2207);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10b);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x2384);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10c);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x3ffe0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10d);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x3ffe0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10e);
	fb_writel(DISP_SD_VENC0_WRDATA, 0x3ffe0);
	fb_writel(DISP_SD_VENC0_WRADDR, 0x10f);
	fb_writel(DISP_SD_VENC0_FIELDBNDRY, 0x106);
	fb_writel(DISP_SD_VENC0_COEF_FYCMPS0, 0x0);
	fb_writel(DISP_SD_VENC0_COEF_FYCMPS1, 0x40a03fb);
	fb_writel(DISP_SD_VENC0_COEF_FYCMPS2, 0x4e13eef2);
	fb_writel(DISP_SD_VENC0_COEF_FYCMPS3, 0x6b);
	fb_writel(DISP_SD_VENC0_COEF_FUVCMPS0, 0x0);
	fb_writel(DISP_SD_VENC0_COEF_FUVCMPS1, 0x2fef8f6);
	fb_writel(DISP_SD_VENC0_COEF_FUVCMPS2, 0x302c2618);
	fb_writel(DISP_SD_VENC0_COEF_FUVCMPS3, 0x30);
	fb_writel(DISP_SD_VENC0_CONT_STATUS, 0x3);
	fb_writel(DISP_SD_VENC0_DCS_CONFIG, 0x4);
	fb_writel(DISP_SD_VENC0_DCS_CONFIG, 0x80210104);
	fb_writel(DISP_SD_VENC0_DCS_CMPNT, 0x3e0002c);
	fb_writel(DISP_SD_VENC0_DCS_CMPNT_ICLK227K, 0x450081);
	fb_writel(DISP_SD_VENC0_DCS_CMPNT_ICLK397K, 0x3f00e3);
	fb_writel(DISP_SD_VENC0_DCS_CONFIG_CMST, 0x2);
	fb_writel(DISP_SD_VENC0_DCS_CONFIG_CMST, 0x80210102);
	fb_writel(DISP_SD_VENC0_DCS_CMST, 0x3ad0028);
	fb_writel(DISP_SD_VENC0_DCS_CMST_ICLK227K, 0x450081);
	fb_writel(DISP_SD_VENC0_DCS_CMST_ICLK397K, 0x3f00e3);
	fb_writel(DISP_SD_VENC0_DCS_START_LINE, 0x4);
	fb_writel(DISP_SD_COMP_SIZE, 0x811e02d0);
	fb_writel(DISP_SD_COMP_GLOBAL_SIZE, 0x20d035a);
	fb_writel(DISP_SD_COMP_START_VBI_POS, 0xffffff);
	fb_writel(DISP_SD_COMP_STOP_VBI_POS, 0xffffff);
	fb_writel(DISP_SD_COMP_START_FLD_POS, 0x11e017);
	fb_writel(DISP_SD_COMP_START_PIX_POS, 0x8a);
	fb_writel(DISP_SD_COMP_START_HANC_POS, 0x0);
	fb_writel(DISP_SD_COMP_STOP_HANC_POS, 0x0);
	fb_writel(DISP_SD_VBI0_HANC0_FORMAT, 0x80808000);
	fb_writel(DISP_SD_VENC0_DVE_FIELD_BNDRY, 0x10a0004);
	fb_writel(DISP_SD_VENC0_DVE_VBLANK_START_BNDRY, 0x1080001);
	fb_writel(DISP_SD_VENC0_DVE_VBLANK_END_BNDRY, 0x11c0015);
	fb_writel(DISP_SD_VENC0_DVE_CNTRL, 0x20);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0x0);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20138000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x0);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30298000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x403f8000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x208a0000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3);
	fb_writel(DISP_HD_VENC0_WRDATA, 0xfff8000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20138000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x8);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x30298000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x9);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x403f8000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xa);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x206b8000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xb);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x34498000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xc);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2fff8000);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xd);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xe);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0xf);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x11);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x12);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x13);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x14);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x15);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x16);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x17);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x18);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x19);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x1f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x20);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x21);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x22);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x23);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x24);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x25);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x26);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x27);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x28);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x29);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x2f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x30);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x31);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x32);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x33);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x34);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x35);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x36);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x37);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x38);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x39);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x3f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x40);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x41);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x42);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x43);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x44);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x45);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x46);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x47);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x48);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x49);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x4f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x50);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x51);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x52);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x53);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x54);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x55);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x56);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x57);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x58);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x59);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x5f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x60);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x61);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x62);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x63);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x64);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x65);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x66);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x67);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x68);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x69);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x6f);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x70);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x71);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x72);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x73);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x74);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x75);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x76);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x77);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x78);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x79);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x7f);
	fb_writel(DISP_HD_VENC0_CB_FREQ, 0x21f07c1f);
	fb_writel(DISP_HD_VENC0_CB_PHASE, 0x1040c0d0);
	fb_writel(DISP_HD_VENC0_CMPST_GAIN, 0x39487ff3);
	fb_writel(DISP_HD_VENC0_BLACK, 0x27);
	fb_writel(DISP_HD_VENC0_BLANK, 0x40013f);
	fb_writel(DISP_HD_VENC0_SYNC1, 0xb);
	fb_writel(DISP_HD_VENC0_SYNC2, 0x273);
	fb_writel(DISP_HD_VENC0_YCMPN, 0x3c000bf);
	fb_writel(DISP_HD_VENC0_CB2GAIN, 0x805a005a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0xa1);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x100);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x3ffe0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x101);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x142);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x102);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x248);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x103);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2e4);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x104);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x20e0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x105);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2106);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x106);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2142);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x107);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2163);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x108);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x21a1);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x109);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x21c5);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10a);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2202);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10b);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2227);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10c);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x2328);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10d);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x23a4);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10e);
	fb_writel(DISP_HD_VENC0_WRDATA, 0x3ffe0);
	fb_writel(DISP_HD_VENC0_WRADDR, 0x10f);
	fb_writel(DISP_HD_VENC0_FIELDBNDRY, 0x106);
	fb_writel(DISP_HD_VENC0_COEF_FYCMPS0, 0x0);
	fb_writel(DISP_HD_VENC0_COEF_FYCMPS1, 0x40a03fb);
	fb_writel(DISP_HD_VENC0_COEF_FYCMPS2, 0x4e13eef2);
	fb_writel(DISP_HD_VENC0_COEF_FYCMPS3, 0x6b);
	fb_writel(DISP_HD_VENC0_COEF_FUVCMPS0, 0x0);
	fb_writel(DISP_HD_VENC0_COEF_FUVCMPS1, 0x2fef8f6);
	fb_writel(DISP_HD_VENC0_COEF_FUVCMPS2, 0x302c2618);
	fb_writel(DISP_HD_VENC0_COEF_FUVCMPS3, 0x30);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_START, 0x5a7ff002);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_END, 0x37ff005);
	fb_writel(DISP_OUTPUT_MUX, 0x1d);
	fb_writel(DISP_HD_VENC0_DCS_CONFIG, 0x40);
	fb_writel(DISP_HD_VENC0_DCS_CONFIG, 0x801e0140);
	fb_writel(DISP_HD_VENC0_DCS_CMPNT, 0x4fc002c);
	fb_writel(DISP_HD_VENC0_DCS_CMPNT_ICLK227K, 0x450081);
	fb_writel(DISP_HD_VENC0_DCS_CMPNT_ICLK397K, 0x3f00e3);
	fb_writel(DISP_HD_VENC0_DCS_CONFIG_CMST, 0x2);
	fb_writel(DISP_HD_VENC0_DCS_CONFIG_CMST, 0x80210102);
	fb_writel(DISP_HD_VENC0_DCS_CMST, 0x3ad0028);
	fb_writel(DISP_HD_VENC0_DCS_CMST_ICLK227K, 0x450081);
	fb_writel(DISP_HD_VENC0_DCS_CMST_ICLK397K, 0x3f00e3);
	fb_writel(DISP_HD_VENC0_DCS_START_LINE, 0x1);
	fb_writel(DISP_HD_VENC0_CONT_STATUS, 0xb);
	fb_writel(DISP_HD_COMP_SIZE, 0x438780);
	fb_writel(DISP_HD_COMP_GLOBAL_SIZE, 0x4650898);
	fb_writel(DISP_HD_COMP_START_VBI_POS, 0xffffff);
	fb_writel(DISP_HD_COMP_STOP_VBI_POS, 0xffffff);
	fb_writel(DISP_HD_COMP_START_FLD_POS, 0xfff02a);
	fb_writel(DISP_HD_COMP_START_PIX_POS, 0x118);
	fb_writel(DISP_HD_COMP_START_HANC_POS, 0x0);
	fb_writel(DISP_HD_COMP_STOP_HANC_POS, 0x0);
	fb_writel(DISP_HD_VBI0_HANC0_FORMAT, 0x8080800a);
	fb_writel(DISP_HD_VENC0_DVE_FIELD_BNDRY, 0xffff0001);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_START_BNDRY, 0xffff0462);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_END_BNDRY, 0xffff002a);
	fb_writel(DISP_HD_VENC0_DVE_CNTRL, 0x34);
	fb_writel(DISP_HD_VENC0_6562_FIELD_BNDRY, 0xffff0001);
	fb_writel(DISP_HD_VENC0_6562_VBLANK_START_BNDRY, 0xffff0462);
	fb_writel(DISP_HD_VENC0_6562_VBLANK_END_BNDRY, 0xffff002a);
	fb_writel(DISP_HD_VENC0_6562_CNTRL, 0x34);
	fb_writel(DISP_HD_COMP_ALPHA_MODE, 0x3);
	fb_writel(DISP_HD_CC, 0x0);
	fb_writel(DISP_HD_GRA0_XVYCC_CSC1_C0, 0x13a705d7);
	fb_writel(DISP_HD_GRA0_XVYCC_CSC1_C1, 0xfcd901fb);
	fb_writel(DISP_HD_GRA0_XVYCC_CSC1_C2, 0xdbdf568);
	fb_writel(DISP_HD_GRA0_XVYCC_CSC1_C3, 0xf3840dbd);
	fb_writel(DISP_HD_GRA0_XVYCC_CSC1_C4, 0xfebd);
	fb_writel(DISP_HD_OCC_C11, 0x2000);
	fb_writel(DISP_HD_OCC_C13, 0x2000000);
	fb_writel(DISP_HD_OCC_C21, 0x20000000);
	fb_writel(DISP_HD_OCC_C23, 0x10000000);
	fb_writel(DISP_HD_OCC_C31, 0x0);
	fb_writel(DISP_HD_OCC_C33, 0x10002000);
	fb_writel(DISP_HD_OUTPUT_LIMIT, 0x10f010eb);
	fb_writel(DISP_HD_GRA0_SIZE, 0x438780);
	fb_writel(DISP_HD_GRA0_SCALED_SIZE, 0x438780);
	fb_writel(DISP_HD_GRA0_VSCA_INIT, 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_START, 0x1);
	fb_writel(DISP_HD_GRA0_VSCA_INV_RATIO, 0x10000000);
	fb_writel(DISP_HD_GRA0_HSCA_INV_RATIO, 0x10000000);
	fb_writel(DISP_HD_GRA0_BUF, 0x7e901e00);
	fb_writel(DISP_HD_GRA0_SAMPLES, 0x1e00);
	fb_writel(DISP_HD_GRA0_FORMAT, 0x20812287);
	fb_writel(DISP_HD_GRA0_FORMAT1, 0x1b);
	fb_writel(DISP_SD_CC, 0x7);
	fb_writel(DISP_SD_OCC_C11, 0x2000);
	fb_writel(DISP_SD_OCC_C13, 0x2000000);
	fb_writel(DISP_SD_OCC_C21, 0x20000000);
	fb_writel(DISP_SD_OCC_C23, 0x10000000);
	fb_writel(DISP_SD_OCC_C31, 0x0);
	fb_writel(DISP_SD_OCC_C33, 0x10002000);

	tmp = fb_readl(DISP_DEBUG_CONTROL);
	fb_writel(DISP_DEBUG_CONTROL, tmp | DEBUG_ENABLE_MASK);

	fb_writel(DISP_HD_VID0_CMD0_FIFO, 0x60070100);
	fb_writel(DISP_HD_VID0_CMD1_FIFO, 0x60030100);
	fb_writel(DISP_HD_VID0_CMD2_FIFO, 0x60030100);
	fb_writel(DISP_HD_VID0_CMD3_FIFO, 0xe0270201);
	fb_writel(DISP_HD_VID1_CMD0_FIFO, 0xe0070100);
	fb_writel(DISP_HD_GRA0_CMD0_FIFO, 0xe0000000);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO, 0xe0000000);
	fb_writel(DISP_SD_CMD0_FIFO, 0x80003040);
	fb_writel(DISP_SD_CMD1_FIFO, 0xc0003040);

	xcode_disp_1080p_setup();
		
	fb_writel(DISP_HD_VID0_CMD0_FIFO, 0x70201);
	fb_writel(DISP_HD_VID0_CMD1_FIFO, 0x30201);
	fb_writel(DISP_HD_VID0_CMD2_FIFO, 0x30201);
	fb_writel(DISP_HD_VID0_CMD3_FIFO, 0x80170201);
	fb_writel(DISP_HD_VID1_CMD0_FIFO, 0x80070201);
	fb_writel(DISP_HD_GRA0_CMD0_FIFO, 0x80000000);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO, 0x80000001);
	fb_writel(DISP_SD_CMD0_FIFO, 0x3081);
	fb_writel(DISP_SD_CMD1_FIFO, 0x40007081);
	fb_writel(DISP_HD_VENC0_DVE_FIELD_BNDRY, 0x7900001);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_START_BNDRY, 0x8000461);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_END_BNDRY, 0x8000029);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_START, 0x6E800465);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_END, 0x02800005);
	fb_writel(DISP_HD_VENC0_DVE_HSYNC, 0x0086005a);
	fb_writel(DISP_HD_VENC0_DVE_CNTRL, 0x1E4);

	tmp = fb_readl(RBM_PADU_CTRL);
	tmp &= ~PADU_CTRL_HDMI_VO_MASK;
	fb_writel(RBM_PADU_CTRL, tmp);

	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80004000);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x80);
	fb_writel(DISP_HDMI_ADDR, 0x80003100);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x40);
	fb_writel(DISP_HDMI_ADDR, 0x80000801);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80004005);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xfe);
	fb_writel(DISP_HDMI_ADDR, 0x80003000);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x20);
	fb_writel(DISP_HDMI_ADDR, 0x80003001);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80003001);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2f);
	fb_writel(DISP_HDMI_ADDR, 0x80003100);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x10);
	fb_writel(DISP_HDMI_ADDR, 0x80003101);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x03);
	fb_writel(DISP_HDMI_ADDR, 0x80003102);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x80);
	fb_writel(DISP_HDMI_ADDR, 0x80003200);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2d);
	fb_writel(DISP_HDMI_ADDR, 0x80003201);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80003202);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xf3);
	fb_writel(DISP_HDMI_ADDR, 0x80003203);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x6f);
	fb_writel(DISP_HDMI_ADDR, 0x80003204);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x06);
	fb_writel(DISP_HDMI_ADDR, 0x80003205);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80003206);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x80);
	fb_writel(DISP_HDMI_ADDR, 0x80003300);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x10);
	fb_writel(DISP_HDMI_ADDR, 0x80003301);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x03);
	fb_writel(DISP_HDMI_ADDR, 0x80003302);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80003400);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x07);
	fb_writel(DISP_HDMI_ADDR, 0x80003403);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80003404);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x07);
	fb_writel(DISP_HDMI_ADDR, 0x80007001);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007002);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x80);
	fb_writel(DISP_HDMI_ADDR, 0x80007003);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x07);
	fb_writel(DISP_HDMI_ADDR, 0x80007004);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x18);
	fb_writel(DISP_HDMI_ADDR, 0x80007005);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80007006);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x58);
	fb_writel(DISP_HDMI_ADDR, 0x80007007);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007008);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2c);
	fb_writel(DISP_HDMI_ADDR, 0x80007009);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000700a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x38);
	fb_writel(DISP_HDMI_ADDR, 0x8000700b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x04);
	fb_writel(DISP_HDMI_ADDR, 0x8000700c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2d);
	fb_writel(DISP_HDMI_ADDR, 0x8000700d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x04);
	fb_writel(DISP_HDMI_ADDR, 0x8000700e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x05);
	fb_writel(DISP_HDMI_ADDR, 0x8000700f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007000);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xc1);
	fb_writel(DISP_HDMI_ADDR, 0x80007101);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007102);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0c);
	fb_writel(DISP_HDMI_ADDR, 0x80007103);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007104);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x18);
	fb_writel(DISP_HDMI_ADDR, 0x80007105);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x06);
	fb_writel(DISP_HDMI_ADDR, 0x80007110);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007111);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80007112);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x21);
	fb_writel(DISP_HDMI_ADDR, 0x80007113);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x03);
	fb_writel(DISP_HDMI_ADDR, 0x80007114);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007115);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x03);
	fb_writel(DISP_HDMI_ADDR, 0x80007116);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007117);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x80);
	fb_writel(DISP_HDMI_ADDR, 0x80007118);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007119);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80007100);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x40);
	fb_writel(DISP_HDMI_ADDR, 0x80000801);
	fb_writel(DISP_HDMI_ADDR, 0x80003004);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x10);
	fb_writel(DISP_HDMI_ADDR, 0x800010e0);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000802);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000803);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x47);
	fb_writel(DISP_HDMI_ADDR, 0x80000804);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x9c);
	fb_writel(DISP_HDMI_ADDR, 0x80001000);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x80);
	fb_writel(DISP_HDMI_ADDR, 0x80001001);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x07);
	fb_writel(DISP_HDMI_ADDR, 0x80001002);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x18);
	fb_writel(DISP_HDMI_ADDR, 0x80001003);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80001004);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x38);
	fb_writel(DISP_HDMI_ADDR, 0x80001005);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x04);
	fb_writel(DISP_HDMI_ADDR, 0x80001006);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2d);
	fb_writel(DISP_HDMI_ADDR, 0x80001007);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x58);
	fb_writel(DISP_HDMI_ADDR, 0x80001008);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001009);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2c);
	fb_writel(DISP_HDMI_ADDR, 0x8000100a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000100b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x04);
	fb_writel(DISP_HDMI_ADDR, 0x8000100c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x05);
	fb_writel(DISP_HDMI_ADDR, 0x8000100d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x60);
	fb_writel(DISP_HDMI_ADDR, 0x8000100e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xea);
	fb_writel(DISP_HDMI_ADDR, 0x8000100f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001010);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0c);
	fb_writel(DISP_HDMI_ADDR, 0x80001011);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x20);
	fb_writel(DISP_HDMI_ADDR, 0x80001012);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80001013);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0b);
	fb_writel(DISP_HDMI_ADDR, 0x80001014);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x16);
	fb_writel(DISP_HDMI_ADDR, 0x80001015);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x21);
	fb_writel(DISP_HDMI_ADDR, 0x80001016);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001018);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x02);
	fb_writel(DISP_HDMI_ADDR, 0x80001019);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x28);
	fb_writel(DISP_HDMI_ADDR, 0x8000101a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000101b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x10);
	fb_writel(DISP_HDMI_ADDR, 0x8000101c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000101d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000101e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000101f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001020);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001021);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001022);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001023);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001024);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001025);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001026);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x1f);
	fb_writel(DISP_HDMI_ADDR, 0x80001027);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001028);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xaa);
	fb_writel(DISP_HDMI_ADDR, 0x80001029);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001030);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001031);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0a);
	fb_writel(DISP_HDMI_ADDR, 0x80001062);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80001063);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x80001065);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001066);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x30);
	fb_writel(DISP_HDMI_ADDR, 0x80001067);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001068);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80001069);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x42);
	fb_writel(DISP_HDMI_ADDR, 0x8000106a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x86);
	fb_writel(DISP_HDMI_ADDR, 0x8000106b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x31);
	fb_writel(DISP_HDMI_ADDR, 0x8000106c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x75);
	fb_writel(DISP_HDMI_ADDR, 0x8000106d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x03);
	fb_writel(DISP_HDMI_ADDR, 0x8000106e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000106f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0f);
	fb_writel(DISP_HDMI_ADDR, 0x80001073);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x03);
	fb_writel(DISP_HDMI_ADDR, 0x80001074);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80001075);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x03);
	fb_writel(DISP_HDMI_ADDR, 0x80001092);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x1f);
	fb_writel(DISP_HDMI_ADDR, 0x800010b3);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010b4);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x10);
	fb_writel(DISP_HDMI_ADDR, 0x800010b5);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0f);
	fb_writel(DISP_HDMI_ADDR, 0x800010b7);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010b8);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010b9);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010ba);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010bb);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010bc);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010bd);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010be);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010bf);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001101);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x10);
	fb_writel(DISP_HDMI_ADDR, 0x80001103);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004100);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x41);
	fb_writel(DISP_HDMI_ADDR, 0x80004101);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x20);
	fb_writel(DISP_HDMI_ADDR, 0x80004102);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004103);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004104);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004105);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004106);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004107);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004108);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004109);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000410a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000410b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x20);
	fb_writel(DISP_HDMI_ADDR, 0x8000410c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000410d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000410e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000410f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004110);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004111);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004112);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004113);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004114);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004115);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x20);
	fb_writel(DISP_HDMI_ADDR, 0x80004116);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004117);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004118);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004119);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x09);
	fb_writel(DISP_HDMI_ADDR, 0x80000200);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80004004);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000201);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000202);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000203);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000204);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000205);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000206);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80000207);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x80001032);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x41);
	fb_writel(DISP_HDMI_ADDR, 0x80001033);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xa4);
	fb_writel(DISP_HDMI_ADDR, 0x80001034);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xfd);
	fb_writel(DISP_HDMI_ADDR, 0x80001035);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x99);
	fb_writel(DISP_HDMI_ADDR, 0x80001036);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x59);
	fb_writel(DISP_HDMI_ADDR, 0x80001037);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x31);
	fb_writel(DISP_HDMI_ADDR, 0x80001038);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x4c);
	fb_writel(DISP_HDMI_ADDR, 0x80001039);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xaa);
	fb_writel(DISP_HDMI_ADDR, 0x8000103a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xab);
	fb_writel(DISP_HDMI_ADDR, 0x8000103b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x87);
	fb_writel(DISP_HDMI_ADDR, 0x8000103c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x08);
	fb_writel(DISP_HDMI_ADDR, 0x8000103d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000103e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x1f);
	fb_writel(DISP_HDMI_ADDR, 0x8000103f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xc3);
	fb_writel(DISP_HDMI_ADDR, 0x80001040);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xd4);
	fb_writel(DISP_HDMI_ADDR, 0x80001041);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x57);
	fb_writel(DISP_HDMI_ADDR, 0x80001042);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x89);
	fb_writel(DISP_HDMI_ADDR, 0x80001043);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x14);
	fb_writel(DISP_HDMI_ADDR, 0x80001044);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x80001045);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xaf);
	fb_writel(DISP_HDMI_ADDR, 0x80001046);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x15);
	fb_writel(DISP_HDMI_ADDR, 0x80001047);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xe8);
	fb_writel(DISP_HDMI_ADDR, 0x80001048);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x7d);
	fb_writel(DISP_HDMI_ADDR, 0x80001049);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x43);
	fb_writel(DISP_HDMI_ADDR, 0x8000104a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x48);
	fb_writel(DISP_HDMI_ADDR, 0x8000104b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x49);
	fb_writel(DISP_HDMI_ADDR, 0x8000104c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x50);
	fb_writel(DISP_HDMI_ADDR, 0x8000104d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x49);
	fb_writel(DISP_HDMI_ADDR, 0x8000104e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x44);
	fb_writel(DISP_HDMI_ADDR, 0x8000104f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x45);
	fb_writel(DISP_HDMI_ADDR, 0x80001050);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x41);
	fb_writel(DISP_HDMI_ADDR, 0x80001051);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x48);
	fb_writel(DISP_HDMI_ADDR, 0x80001052);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x31);
	fb_writel(DISP_HDMI_ADDR, 0x80001053);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x33);
	fb_writel(DISP_HDMI_ADDR, 0x80001054);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x54);
	fb_writel(DISP_HDMI_ADDR, 0x80001055);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x43);
	fb_writel(DISP_HDMI_ADDR, 0x80001056);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x54);
	fb_writel(DISP_HDMI_ADDR, 0x80001057);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x52);
	fb_writel(DISP_HDMI_ADDR, 0x80001058);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x4c);
	fb_writel(DISP_HDMI_ADDR, 0x80001059);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x30);
	fb_writel(DISP_HDMI_ADDR, 0x8000105a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x56);
	fb_writel(DISP_HDMI_ADDR, 0x8000105b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x31);
	fb_writel(DISP_HDMI_ADDR, 0x8000105c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x5f);
	fb_writel(DISP_HDMI_ADDR, 0x8000105d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x3b);
	fb_writel(DISP_HDMI_ADDR, 0x8000105e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2d);
	fb_writel(DISP_HDMI_ADDR, 0x8000105f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x29);
	fb_writel(DISP_HDMI_ADDR, 0x80001060);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x5f);
	fb_writel(DISP_HDMI_ADDR, 0x80001061);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x80001076);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x07);
	fb_writel(DISP_HDMI_ADDR, 0x80001077);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x99);
	fb_writel(DISP_HDMI_ADDR, 0x80001078);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xc5);
	fb_writel(DISP_HDMI_ADDR, 0x80001079);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x47);
	fb_writel(DISP_HDMI_ADDR, 0x8000107a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x1f);
	fb_writel(DISP_HDMI_ADDR, 0x8000107b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x8000107c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xda);
	fb_writel(DISP_HDMI_ADDR, 0x8000107d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x49);
	fb_writel(DISP_HDMI_ADDR, 0x8000107e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x86);
	fb_writel(DISP_HDMI_ADDR, 0x8000107f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xf2);
	fb_writel(DISP_HDMI_ADDR, 0x80001080);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001081);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001082);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001083);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001084);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001085);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001086);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001087);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001088);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001089);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000108a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000108b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000108c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000108d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000108e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000108f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001090);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001091);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x47);
	fb_writel(DISP_HDMI_ADDR, 0x80001093);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xa7);
	fb_writel(DISP_HDMI_ADDR, 0x80001094);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x89);
	fb_writel(DISP_HDMI_ADDR, 0x80001095);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xaa);
	fb_writel(DISP_HDMI_ADDR, 0x80001096);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x07);
	fb_writel(DISP_HDMI_ADDR, 0x80001097);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001098);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x80001099);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x87);
	fb_writel(DISP_HDMI_ADDR, 0x8000109a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xf1);
	fb_writel(DISP_HDMI_ADDR, 0x8000109b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x11);
	fb_writel(DISP_HDMI_ADDR, 0x8000109c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x8000109d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x05);
	fb_writel(DISP_HDMI_ADDR, 0x8000109e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xa5);
	fb_writel(DISP_HDMI_ADDR, 0x8000109f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xbc);
	fb_writel(DISP_HDMI_ADDR, 0x800010a0);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xb5);
	fb_writel(DISP_HDMI_ADDR, 0x800010a1);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x07);
	fb_writel(DISP_HDMI_ADDR, 0x800010a2);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x54);
	fb_writel(DISP_HDMI_ADDR, 0x800010a3);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xf7);
	fb_writel(DISP_HDMI_ADDR, 0x800010a4);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x800010a5);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x41);
	fb_writel(DISP_HDMI_ADDR, 0x800010a6);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800010a7);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xdc);
	fb_writel(DISP_HDMI_ADDR, 0x800010a8);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xc7);
	fb_writel(DISP_HDMI_ADDR, 0x800010a9);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x79);
	fb_writel(DISP_HDMI_ADDR, 0x800010aa);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x44);
	fb_writel(DISP_HDMI_ADDR, 0x800010ab);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xf1);
	fb_writel(DISP_HDMI_ADDR, 0x800010ac);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0f);
	fb_writel(DISP_HDMI_ADDR, 0x800010ad);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xaa);
	fb_writel(DISP_HDMI_ADDR, 0x800010ae);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xab);
	fb_writel(DISP_HDMI_ADDR, 0x800010af);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x12);
	fb_writel(DISP_HDMI_ADDR, 0x800010b0);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0a);
	fb_writel(DISP_HDMI_ADDR, 0x800010b1);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x800010b2);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x5a);
	fb_writel(DISP_HDMI_ADDR, 0x80001120);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x2a);
	fb_writel(DISP_HDMI_ADDR, 0x8000111f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xb1);
	fb_writel(DISP_HDMI_ADDR, 0x8000111e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x9c);
	fb_writel(DISP_HDMI_ADDR, 0x8000111d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xfd);
	fb_writel(DISP_HDMI_ADDR, 0x8000111c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xe8);
	fb_writel(DISP_HDMI_ADDR, 0x8000111b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x15);
	fb_writel(DISP_HDMI_ADDR, 0x8000111a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xaf);
	fb_writel(DISP_HDMI_ADDR, 0x80001119);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x80001118);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x14);
	fb_writel(DISP_HDMI_ADDR, 0x80001117);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x89);
	fb_writel(DISP_HDMI_ADDR, 0x80001116);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x57);
	fb_writel(DISP_HDMI_ADDR, 0x80001115);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xd4);
	fb_writel(DISP_HDMI_ADDR, 0x80001114);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xc3);
	fb_writel(DISP_HDMI_ADDR, 0x80001113);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x1f);
	fb_writel(DISP_HDMI_ADDR, 0x80001112);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80001111);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x08);
	fb_writel(DISP_HDMI_ADDR, 0x80001110);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x87);
	fb_writel(DISP_HDMI_ADDR, 0x8000110f);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xab);
	fb_writel(DISP_HDMI_ADDR, 0x8000110e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xaa);
	fb_writel(DISP_HDMI_ADDR, 0x8000110d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x4c);
	fb_writel(DISP_HDMI_ADDR, 0x8000110c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x31);
	fb_writel(DISP_HDMI_ADDR, 0x8000110b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x59);
	fb_writel(DISP_HDMI_ADDR, 0x8000110a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x99);
	fb_writel(DISP_HDMI_ADDR, 0x80001109);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xfd);
	fb_writel(DISP_HDMI_ADDR, 0x80001108);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xa4);
	fb_writel(DISP_HDMI_ADDR, 0x80001107);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x41);
	fb_writel(DISP_HDMI_ADDR, 0x80001106);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xff);
	fb_writel(DISP_HDMI_ADDR, 0x80001105);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x84);
	fb_writel(DISP_HDMI_ADDR, 0x8000501a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x84);
	fb_writel(DISP_HDMI_ADDR, 0x80005017);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x8b);
	fb_writel(DISP_HDMI_ADDR, 0x80005018);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x82);
	fb_writel(DISP_HDMI_ADDR, 0x80005019);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x5d);
	fb_writel(DISP_HDMI_ADDR, 0x8000500b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x8000500c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x15);
	fb_writel(DISP_HDMI_ADDR, 0x80005009);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x40);
	fb_writel(DISP_HDMI_ADDR, 0x8000500a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x8000500d);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x08);
	fb_writel(DISP_HDMI_ADDR, 0x8000500e);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x01);
	fb_writel(DISP_HDMI_ADDR, 0x80005016);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800052b9);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x800052ba);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x00);
	fb_writel(DISP_HDMI_ADDR, 0x80005016);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x8f);
	fb_writel(DISP_HDMI_ADDR, 0x80005017);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x87);
	fb_writel(DISP_HDMI_ADDR, 0x80005018);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x87);
	fb_writel(DISP_HDMI_ADDR, 0x80005019);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x15);
	fb_writel(DISP_HDMI_ADDR, 0x80005009);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x40);
	fb_writel(DISP_HDMI_ADDR, 0x8000500a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x44);
	fb_writel(DISP_HDMI_ADDR, 0x8000500b);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x0);
	fb_writel(DISP_HDMI_ADDR, 0x8000500c);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x81);
	fb_writel(DISP_HDMI_ADDR, 0x8000501a);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0xA1);
	fb_writel(DISP_HDMI_ADDR, 0x80005000);
	udelay(delay);
	fb_writel(DISP_HDMI_WRDATA, 0x05);
	fb_writel(DISP_HDMI_ADDR, 0x80005001);
	udelay(delay);
#if CFG_DISPLAY_BLENDING
	xcode_disp_ablend_setup();
#endif
	return 0;
}

static inline void xcode_disp_init_480p(void)
{
	unsigned int tmp, val;	

	tmp = fb_readl(CG_RESET_REG);
	fb_writel(CG_RESET_REG , tmp | DISP_RESET_MASK);
	udelay(100);
	fb_writel(CG_RESET_REG , tmp &~ DISP_RESET_MASK);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0x0);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0xffff);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0xffff);		
	fb_writel(DISP_OUTPUT_MUX , 0xf);

	// Reference Clock Source Select for 480P
	tmp = fb_readl(CG1_CLK_SRC_SEL7);
	tmp &= ~VSCLK_SRC_SEL_MASK;
	tmp |= 0x0 << VSCLK_SRC_SEL_SHIFT;
	fb_writel(CG1_CLK_SRC_SEL7, tmp);
		
	//CG1_CLK_SRC_SEL7 \ VSCLK_SRC_SEL = 0x00
	tmp = fb_readl(CG1_CLK_SRC_SEL4);
	tmp &= ~DAC0CLK_SRC_SEL_MASK;
	tmp |= 0x40 << DAC0CLK_SRC_SEL_SHIFT;
	fb_writel(CG1_CLK_SRC_SEL4, tmp);
	//CG1_CLK_SRC_SEL4 \ DAC0CLK_SRC_SEL = 0x40
	
	tmp = fb_readl(CG1_CLK_SRC_SEL5);
	tmp &= ~(DISP_656CLK_SRC_SEL_MASK | PCLK_SRC_SEL_MASK | VHDCLK_SRC_SEL_MASK);
	tmp |= 	((0xc0 << DISP_656CLK_SRC_SEL_SHIFT) | \
			(0xc0 << PCLK_SRC_SEL_SHIFT) | \
			(0xc0 << VHDCLK_SRC_SEL_SHIFT));
	fb_writel(CG1_CLK_SRC_SEL5, tmp);		
	//CG1_CLK_SRC_SEL5 \ 	DISP_656CLK_SRC_SEL = 0xC0
	//CG1_CLK_SRC_SEL5 \ PCLK_SRC_SEL = 0xC0);
	//CG1_CLK_SRC_SEL5 \ VHDCLK_SRC_SEL = 0xC0);
	//
	tmp = fb_readl(CG1_CLK_STOP0);
	val = (VSCLK_STOP_MASK | DISP_656CLK_STOP_MASK	\
			| VSDCLK_STOP_MASK | VHDCLK_STOP_MASK	\
			| DAC0CLK_STOP_MASK | DAC1CLK_STOP_MASK \
			| PCLK_STOP_MASK);
	fb_writel(CG1_CLK_STOP0 , tmp & ~val);
	
	tmp = fb_readl(CG1_BLK_CLK_STOP0);
	fb_writel(CG1_BLK_CLK_STOP0 , tmp & ~DISPCLK_STOP_MASK);
	return;
}



static inline void xcode_disp_init_576p(void)
{
	unsigned int tmp, val;	

	tmp = fb_readl(CG_RESET_REG);
	fb_writel(CG_RESET_REG , tmp | DISP_RESET_MASK);
	udelay(100);
	fb_writel(CG_RESET_REG , tmp &~ DISP_RESET_MASK);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0x0);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0xffff);
	udelay(100);
	fb_writel(DISP_SFT_RESET , 0xffff);		
	fb_writel(DISP_OUTPUT_MUX , 0xf);

	// Reference Clock Source Select for 576P
	tmp = fb_readl(CG1_CLK_SRC_SEL7);
	tmp &= ~VSCLK_SRC_SEL_MASK;
	tmp |= 0x0 << VSCLK_SRC_SEL_SHIFT;
	fb_writel(CG1_CLK_SRC_SEL7, tmp);
		
	//CG1_CLK_SRC_SEL7 \ VSCLK_SRC_SEL = 0x00
	tmp = fb_readl(CG1_CLK_SRC_SEL4);
	tmp &= ~DAC0CLK_SRC_SEL_MASK;
	tmp |= 0x40 << DAC0CLK_SRC_SEL_SHIFT;
	fb_writel(CG1_CLK_SRC_SEL4, tmp);
	//CG1_CLK_SRC_SEL4 \ DAC0CLK_SRC_SEL = 0x40
	
	tmp = fb_readl(CG1_CLK_SRC_SEL5);
	tmp &= ~(DISP_656CLK_SRC_SEL_MASK | PCLK_SRC_SEL_MASK | VHDCLK_SRC_SEL_MASK);
	tmp |= 	((0xc0 << DISP_656CLK_SRC_SEL_SHIFT) | \
			(0xc0 << PCLK_SRC_SEL_SHIFT) | \
			(0xc0 << VHDCLK_SRC_SEL_SHIFT));
	fb_writel(CG1_CLK_SRC_SEL5, tmp);		
	//CG1_CLK_SRC_SEL5 \ 	DISP_656CLK_SRC_SEL = 0xC0
	//CG1_CLK_SRC_SEL5 \ PCLK_SRC_SEL = 0xC0);
	//CG1_CLK_SRC_SEL5 \ VHDCLK_SRC_SEL = 0xC0);
	//
	tmp = fb_readl(CG1_CLK_STOP0);
	val = (VSCLK_STOP_MASK | DISP_656CLK_STOP_MASK	\
			| VSDCLK_STOP_MASK | VHDCLK_STOP_MASK	\
			| DAC0CLK_STOP_MASK | DAC1CLK_STOP_MASK \
			| PCLK_STOP_MASK);
	fb_writel(CG1_CLK_STOP0 , tmp & ~val);
	
	tmp = fb_readl(CG1_BLK_CLK_STOP0);
	fb_writel(CG1_BLK_CLK_STOP0 , tmp & ~DISPCLK_STOP_MASK);
	return;
}

static int video_display_yuv_480p (unsigned long offset)
{
	unsigned int tmp, delay=5;

	printf("Display 720 x 480 YUV image at %x\n", (unsigned)offset);
	xcode_disp_init_480p();
	fb_writel(DISP_HD_VID0_Y_ADR , 0x0);
	fb_writel(DISP_HIST_ADR , 0x0);
	fb_writel(DISP_HD_VID0_CB_ADR , 0xa8c000);
	fb_writel(DISP_HD_VID0_CR_ADR , 0xfd2000);
	fb_writel(DISP_HD_VID1_Y_ADR , 0x1779600);
	fb_writel(DISP_HD_VID1_CB_ADR , 0x2205600);
	fb_writel(DISP_HD_VID1_CR_ADR , 0x274b600);
	fb_writel(DISP_HD_GRA0_ADR , offset & 0x7FFFFFFF);
	fb_writel(DISP_HD_GRA1_ADR , 0x6c91600);
	fb_writel(DISP_HD_VBI0_ADR , 0x1518000);
	fb_writel(DISP_HD_HANC0_ADR , 0x156c600);
	fb_writel(DISP_SD_GRA0_ADR , 0xac91600);
	fb_writel(DISP_SD_VBI0_ADR , 0x1518000);
	fb_writel(DISP_SD_HANC0_ADR , 0x156c600);
	
	fb_writel(DISP_SFT_RESET , 0xef77);
	fb_writel(DISP_SFT_RESET , 0xef77);
	fb_writel(DISP_SFT_RESET , 0xffff);
	
	
	fb_writel(DISP_SFT_RESET , 0xef77);
	fb_writel(DISP_SFT_RESET , 0xef77);
	fb_writel(DISP_SFT_RESET , 0xffff);
	fb_writel(DISP_HD_GRA0_SIZE , 0x1e02d0);
	fb_writel(DISP_HD_GRA0_SCALED_SIZE , 0x1e02d0);
	fb_writel(DISP_HD_GRA0_VSCA_INIT , 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_START , 0x1);
	fb_writel(DISP_HD_GRA0_VSCA_INV_RATIO , 0xff77777);
	fb_writel(DISP_HD_GRA0_HSCA_INV_RATIO , 0x10000000);
	fb_writel(DISP_HD_GRA0_BUF , 0x15180b40);
	fb_writel(DISP_HD_GRA0_SAMPLES , 0xb40);
	fb_writel(DISP_HD_GRA0_FORMAT , 0x2012087);
	fb_writel(DISP_HD_GRA0_FORMAT1 , 0x1000001b);
	fb_writel(DISP_HD_CC , (fb_readl(DISP_HD_CC) & 0xFFFFFFED) | 0x2);
	fb_writel(DISP_HD_RAM_ADDR , 0x1100000);
	fb_writel(DISP_HD_RAM_ADDR , 0x1110000);
	fb_writel(DISP_HD_VENC0_FIELDBNDRY , 0x106);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_START , 0x5a7ff003);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_END , 0x37ff006);
	fb_writel(DISP_HD_COMP_SIZE , 0x1e02d0);
	fb_writel(DISP_HD_COMP_GLOBAL_SIZE , 0x20d035a);
	fb_writel(DISP_HD_COMP_START_VBI_POS , 0xffffff);
	fb_writel(DISP_HD_COMP_STOP_VBI_POS , 0xffffff);
	fb_writel(DISP_HD_COMP_START_FLD_POS , 0xfff02d);
	fb_writel(DISP_HD_COMP_START_PIX_POS , 0x8a);
	fb_writel(DISP_HD_COMP_START_HANC_POS , 0x0);
	fb_writel(DISP_HD_COMP_STOP_HANC_POS , 0x0);
	fb_writel(DISP_HD_VBI0_HANC0_FORMAT , 0x8080800a);
	fb_writel(DISP_HD_VENC0_DVE_FIELD_BNDRY , 0xffff0001);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_START_BNDRY , 0xffff020b);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_END_BNDRY , 0xffff002b);
	fb_writel(DISP_HD_VENC0_DVE_CNTRL , 0x22);
	
	fb_writel(DISP_HD_COMP_ALPHA_MODE , 0x3);
	fb_writel(DISP_HD_COMP_GRA0_POS , 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_INIT , 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_START , 0x20001);

	fb_writel(DISP_DEBUG_CONTROL, fb_readl(DISP_DEBUG_CONTROL) | DEBUG_ENABLE_MASK);

	fb_writel(DISP_HD_VID0_CMD0_FIFO , 0x60070000);
	fb_writel(DISP_HD_VID0_CMD1_FIFO , 0x60030100);
	fb_writel(DISP_HD_VID0_CMD2_FIFO , 0x60030100);
	fb_writel(DISP_HD_VID0_CMD3_FIFO , 0xe0270000);
	fb_writel(DISP_HD_VID1_CMD0_FIFO , 0xe0070100);
	fb_writel(DISP_HD_GRA0_CMD0_FIFO , 0xe0000000);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO , 0xe0000000);

	xcode_disp_480p_setup();	

	fb_writel(DISP_HD_VID0_CMD0_FIFO , 0x70101);
	fb_writel(DISP_HD_VID0_CMD1_FIFO , 0x30201);
	fb_writel(DISP_HD_VID0_CMD2_FIFO , 0x30201);
	fb_writel(DISP_HD_VID0_CMD3_FIFO , 0x80170101);
	fb_writel(DISP_HD_VID1_CMD0_FIFO , 0x80070201);
	fb_writel(DISP_HD_GRA0_CMD0_FIFO , 0x80000001);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO , 0x80000001);
	fb_writel(DISP_SD_CMD0_FIFO , 0x3081);
	fb_writel(DISP_SD_CMD1_FIFO , 0x40007081);
	fb_writel(DISP_HD_VENC0_DVE_FIELD_BNDRY, 0x3ff020C);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_START_BNDRY, 0x3ff020A);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_END_BNDRY, 0x3ff002A);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_START, 0x6E3ff006);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_END, 0x023ff00C);
	/* TODO: Review the setting inside DISP_HD_VENC0_DVE_HSYNC */
    fb_writel(DISP_HD_VENC0_DVE_HSYNC, 0x00520014);
	fb_writel(DISP_HD_VENC0_DVE_CNTRL, 0x1E4);

	fb_writel(DISP_HD_GRA0_CMD0_FIFO , 0xe0000000);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO , 0xe0000000);
	tmp = fb_readl(RBM_PADU_CTRL);
	tmp &= ~PADU_CTRL_HDMI_VO_MASK;
	fb_writel(RBM_PADU_CTRL, tmp);

    
#if CFG_DISPLAY_BLENDING
	xcode_disp_ablend_setup();
#endif

	xcode_disp_hdmi_480p_60hz();
    xcode_disp_comp_480p_60hz();
	return 0;
}


static int video_display_yuv_576p (unsigned long offset)
{
	unsigned int tmp, delay=5;

	printf("Display 720 x 576 YUV image at %x\n", (unsigned)offset);
	xcode_disp_init_576p();
	fb_writel(DISP_HD_VID0_Y_ADR , 0x0);
	fb_writel(DISP_HIST_ADR , 0x0);
	fb_writel(DISP_HD_VID0_CB_ADR , 0xa8c000);
	fb_writel(DISP_HD_VID0_CR_ADR , 0xfd2000);
	fb_writel(DISP_HD_VID1_Y_ADR , 0x1779600);
	fb_writel(DISP_HD_VID1_CB_ADR , 0x2205600);
	fb_writel(DISP_HD_VID1_CR_ADR , 0x274b600);
	fb_writel(DISP_HD_GRA0_ADR , offset & 0x7FFFFFFF);
	fb_writel(DISP_HD_GRA1_ADR , 0x6c91600);
	fb_writel(DISP_HD_VBI0_ADR , 0x1518000);
	fb_writel(DISP_HD_HANC0_ADR , 0x156c600);
	
	fb_writel(DISP_SFT_RESET , 0xef77);
	fb_writel(DISP_SFT_RESET , 0xef77);
	fb_writel(DISP_SFT_RESET , 0xffff);
    
    /* This part changes for 576p */
	fb_writel(DISP_HD_GRA0_SIZE , 0x2402d0);
	fb_writel(DISP_HD_GRA0_SCALED_SIZE , 0x2402d0);
	fb_writel(DISP_HD_GRA0_VSCA_INIT , 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_START , 0x1);
	//fb_writel(DISP_HD_GRA0_VSCA_INV_RATIO , 0xff77777);
	fb_writel(DISP_HD_GRA0_VSCA_INV_RATIO , 0x10000000);
	fb_writel(DISP_HD_GRA0_HSCA_INV_RATIO , 0x10000000);
	fb_writel(DISP_HD_GRA0_BUF , 0x19500B40);
	fb_writel(DISP_HD_GRA0_SAMPLES , 0xb40);
	fb_writel(DISP_HD_GRA0_FORMAT , 0x2012087);
	fb_writel(DISP_HD_GRA0_FORMAT1 , 0x1000001b);
	fb_writel(DISP_HD_CC , (fb_readl(DISP_HD_CC) & 0xFFFFFFED) | 0x2);
	fb_writel(DISP_HD_RAM_ADDR , 0x1100000);
	fb_writel(DISP_HD_RAM_ADDR , 0x1110000);


	fb_writel(DISP_HD_VENC0_DVE_VSYNC_START , 0x18fff271);
	fb_writel(DISP_HD_VENC0_DVE_VSYNC_END , 0x1fff005);
	fb_writel(DISP_HD_VENC0_CONT_STATUS , 0x3);
	fb_writel(DISP_HD_COMP_SIZE , 0x2402d0);
	fb_writel(DISP_HD_COMP_GLOBAL_SIZE , 0x2710360);
	fb_writel(DISP_HD_COMP_START_VBI_POS , 0xffffff);
	fb_writel(DISP_HD_COMP_STOP_VBI_POS , 0xffffff);
	fb_writel(DISP_HD_COMP_START_FLD_POS , 0xfff02d);
	fb_writel(DISP_HD_COMP_START_PIX_POS , 0x90);
	fb_writel(DISP_HD_COMP_START_HANC_POS , 0x0);
	fb_writel(DISP_HD_COMP_STOP_HANC_POS , 0x0);
	fb_writel(DISP_HD_VBI0_HANC0_FORMAT , 0x8080800a);
	fb_writel(DISP_HD_VENC0_DVE_FIELD_BNDRY , 0xffff0001);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_START_BNDRY , 0xffff026C);
	fb_writel(DISP_HD_VENC0_DVE_VBLANK_END_BNDRY , 0xffff002c);
	fb_writel(DISP_HD_VENC0_DVE_CNTRL , 0x22);
	
	fb_writel(DISP_HD_COMP_ALPHA_MODE , 0x3);
	fb_writel(DISP_HD_COMP_GRA0_POS , 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_INIT , 0x0);
	fb_writel(DISP_HD_GRA0_VSCA_START , 0x20001);

	fb_writel(DISP_DEBUG_CONTROL, fb_readl(DISP_DEBUG_CONTROL) | DEBUG_ENABLE_MASK);

	fb_writel(DISP_HD_VID0_CMD0_FIFO , 0x60070000);
	fb_writel(DISP_HD_VID0_CMD1_FIFO , 0x60030100);
	fb_writel(DISP_HD_VID0_CMD2_FIFO , 0x60030100);
	fb_writel(DISP_HD_VID0_CMD3_FIFO , 0xe0270000);
	fb_writel(DISP_HD_VID1_CMD0_FIFO , 0xe0070100);
	fb_writel(DISP_HD_GRA0_CMD0_FIFO , 0xe0000000);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO , 0xe0000000);

	xcode_disp_576p_setup();	

	fb_writel(DISP_HD_VID0_CMD0_FIFO , 0x70101);
	fb_writel(DISP_HD_VID0_CMD1_FIFO , 0x30201);
	fb_writel(DISP_HD_VID0_CMD2_FIFO , 0x30201);
	fb_writel(DISP_HD_VID0_CMD3_FIFO , 0x80170101);
	fb_writel(DISP_HD_VID1_CMD0_FIFO , 0x80070201);
	fb_writel(DISP_HD_GRA0_CMD0_FIFO , 0x80000001);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO , 0x80000001);
	fb_writel(DISP_SD_CMD0_FIFO , 0x3081);
	fb_writel(DISP_SD_CMD1_FIFO , 0x40007081);
    
    /* Already been programmed ... */
	// fb_writel(DISP_HD_VENC0_DVE_FIELD_BNDRY, 0x3ff020C);
	// fb_writel(DISP_HD_VENC0_DVE_VBLANK_START_BNDRY, 0x3ff020A);
	// fb_writel(DISP_HD_VENC0_DVE_VBLANK_END_BNDRY, 0x3ff002A);
	// fb_writel(DISP_HD_VENC0_DVE_VSYNC_START, 0x6E3ff006);
	// fb_writel(DISP_HD_VENC0_DVE_VSYNC_END, 0x023ff00C);
	
    fb_writel(DISP_HD_VENC0_DVE_HSYNC, 0x00500010);
	fb_writel(DISP_HD_VENC0_DVE_CNTRL, 0x1E4);

	fb_writel(DISP_HD_GRA0_CMD0_FIFO , 0xe0000000);
	fb_writel(DISP_HD_GRA1_CMD0_FIFO , 0xe0000000);
	tmp = fb_readl(RBM_PADU_CTRL);
	tmp &= ~PADU_CTRL_HDMI_VO_MASK;
	fb_writel(RBM_PADU_CTRL, tmp);


    /* HDMI Tx is being programmed twice in this control path, why?*/
    fb_writel(DISP_HDMI_WRDATA, 0xfe);
	fb_writel(DISP_HDMI_ADDR, 0x80003000);
	udelay(delay);
    
    
#if CFG_DISPLAY_BLENDING
	xcode_disp_ablend_setup();
#endif
    
	xcode_disp_hdmi_576p_50hz();
    xcode_disp_comp_576p_50hz();
	return 0;
}

/*
 * Subroutine:  yuv_display
 *
 * Description: Display bmp file located in memory
 *
 * Inputs:	addr		address of the yuv file
 *			size		size of image if compressed
 * Return:      None
 *
 */
static int yuv_display(ulong addr)
{
	image_header_t *hdr = (image_header_t *)addr;
	unsigned int fb_addr = CFG_DISPLAY_FB_ADDR;
	unsigned int size, ret_len;


	if(ntohl(hdr->ih_magic) != IH_MAGIC){
		debug("Bad Magic Number [%x] <-> [%x], Please check image and reload\n", (unsigned)ntohl(hdr->ih_magic), IH_MAGIC);
		printf("RAW image, Display directly\n");
		fb_addr = (unsigned int)addr;
	} else {

		size = 0x800000;
		switch(hdr->ih_comp) {
		case IH_COMP_NONE:
			fb_addr = (unsigned int)(addr) + 64;
			break;

		case IH_COMP_GZIP:
			printf ("	Uncompressing... ");
			fb_addr = ntohl(hdr->ih_load);
			if (gunzip ((void*)fb_addr, (ulong*)&ret_len,
				(unsigned)((uchar *)addr + 64), (ulong*)&size) != 0) {
				puts ("GUNZIP ERROR\n");			
			} else {
				printf("Done\n");
			}
			break;

		default:
			printf("Not supported compressed format\n");
			break;
		}
	}

	flush_dcache_all();
	if(!strncmp(hdr->ih_name, "SPLASH_1080P", 12))
		return (video_display_yuv_1080p(fb_addr));
	else if(!strncmp(hdr->ih_name, "SPLASH_480P", 11))
		return (video_display_yuv_480p(fb_addr));
	else
		return (video_display_yuv_576p(fb_addr)); 
}

#endif /* (CONFIG_COMMANDS & CFG_CMD_BMP) */
